@@ -10,11 +10,13 @@
 #   include	<time.h>
 
 #   include	<bitmap.h>
-#   include	<docFont.h>
+#   include	<utilDocFont.h>
+#   include	<psFont.h>
 #   include	"docShape.h"
-#   include	<appGeo.h>
+#   include	<geo2DInteger.h>
 #   include	<sioGeneral.h>
 #   include	<utilMemoryBuffer.h>
+#   include	<utilTextAttributeAdmin.h>
 
 #   include	"docParaProperties.h"
 #   include	"docRowProperties.h"
@@ -26,6 +28,8 @@
 #   include	"docObject.h"
 #   include	"docSelect.h"
 #   include	"docExternalItem.h"
+#   include	"docListTable.h"
+#   include	"docListNumberTree.h"
 
 /************************************************************************/
 /*									*/
@@ -59,22 +63,32 @@ typedef enum DocParticuleKind
     DOCkindXE,
     DOCkindLINEBREAK,
     DOCkindPAGEBREAK,
+    DOCkindCOLUMNBREAK,
     DOCkindCHFTNSEP,
+    DOCkindCHFTNSEPC,
 
     DOCkind_COUNT
     } DocParticuleKind;
 
+#   define DOCkindBETWEEN_TEXT(k) \
+	( \
+	    (k) == DOCkindFIELDSTART	|| \
+	    (k) == DOCkindFIELDEND	|| \
+	    (k) == DOCkindNOTE		   \
+	)
+
+#   define	DOCkind_BITS	6
+
 typedef struct TextParticule
     {
     unsigned int	tpStroff:16;
-    unsigned int	tpStrlen:12;
+    unsigned int	tpStrlen:14;
     unsigned int	tpX0:12;
     unsigned int	tpPixelsWide:12;
-    unsigned int	tpKind:6;
+    unsigned int	tpKind:DOCkind_BITS;
     unsigned int	tpInField:1;
-    short int		tpPhysicalFont;
     short int		tpObjectNumber;
-    TextAttribute	tpTextAttribute;
+    int			tpTextAttributeNumber;
     } TextParticule;
 
 /************************************************************************/
@@ -90,6 +104,13 @@ typedef struct TextParticule
 /*	last line of a page.						*/
 /*  5)  Screen geometry.						*/
 /*									*/
+/*  6)  The width of the column for which the line was last formatted.	*/
+/*	When the line is shifted to a column with the same width, it	*/
+/*	is sufficient to change the coordinates of the line and its	*/
+/*	contents during a reformat of the document. When the column	*/
+/*	width is diffrerent, the line must be reformatted. (and most	*/
+/*	probably subsequent lines as well).				*/
+/*									*/
 /************************************************************************/
 
 typedef struct TextLine
@@ -111,23 +132,16 @@ typedef struct TextLine
 
     short int		tlX0Pixels;
     short int		tlX1Pixels;
+
+    short int		tlColumnWidthTwips;			/*  6  */
     } TextLine;
 
 /************************************************************************/
 /*									*/
-/*  Information on Lists.						*/
+/*  A Paragraph.							*/
 /*									*/
 /************************************************************************/
 
-typedef struct DocumentList
-    {
-    int			dlListId;
-    int			dlListTemplateId;
-    int			dlListIsSimple;
-    int			dlRestartForEverySection;
-    char *		dlListName;
-    } DocumentList;
-
 typedef struct BufferPara
     {
     unsigned char *		btString;
@@ -151,10 +165,7 @@ typedef struct BufferPara
     int				btDescentTwips;
     int				btLeadingTwips;
 
-    int				btSpaceAboveLinesTwips;
     const BorderProperties *	btBorderAboveParagraph;
-
-    int				btSpaceBelowLinesTwips;
     const BorderProperties *	btBorderBelowParagraph;
     } BufferPara;
 
@@ -226,9 +237,48 @@ typedef struct BufferSect
 					    BIU.biuSect.bsUseForPage
     } BufferSect;
 
+/************************************************************************/
+/*									*/
+/*  Row specific information.						*/
+/*									*/
+/*  1)  The rtf file format does not really support tables. Keep an	*/
+/*	administration of the set of rows that looks like a table.	*/
+/*  2)  Properties of the row.						*/
+/*  3)  Cells with a rowspan can protrude below the actual row: To	*/
+/*	optimize position lookups and to delimit areas to redraw	*/
+/*	correctly, the position below all 'rowspan' columns is		*/
+/*	remembered.							*/
+/*  4)  If a row is a table header, it might be repeated on every page	*/
+/*	where the table is continued. When the row is drawn on a	*/
+/*	different page. A separate administration must be kept to	*/
+/*	handle the case where a row starts on a new page and the header	*/
+/*	appears above it. Additionally we need to know on what page the	*/
+/*	header apprears in the normal text flow.			*/
+/*  6)  The top position of the table header row that appears above an	*/
+/*	ordinary row. (This must be kept with ordinary rows at the top	*/
+/*	of a page.)							*/
+/*  7)  A flag indicating that this row is preceded by a table header	*/
+/*	that does not appear at its home position.			*/
+/*									*/
+/************************************************************************/
+
 typedef struct BufferRow
     {
-    RowProperties		bguRowProperties;
+				/*  1  */
+    int				brTableFirst;
+    int				brTablePast;
+    int				brTableFirstIsHeader;
+
+				/*  2  */
+    RowProperties		brRowProperties;
+
+				/*  3  */
+    LayoutPosition		brBelowAllPosition;
+
+				/*  4,6  */
+    LayoutPosition		brAboveHeaderPosition;
+				/*  4,7  */
+    int				brPrecededByHeader;
     } BufferRow;
 
 /************************************************************************/
@@ -305,20 +355,27 @@ typedef enum ItemLevel
 			/*  * biGroupChildren			*/
 			/*  * biGroupChildCount			*/
 			/****************************************/
-#	define	biRowProperties		BIU.biuRow.bguRowProperties
+
+#	define	biRowTableFirst		BIU.biuRow.brTableFirst
+#	define	biRowTablePast		BIU.biuRow.brTablePast
+#	define	biRowTableFirstIsHeader	BIU.biuRow.brTableFirstIsHeader
+#	define	biRowPrecededByHeader	BIU.biuRow.brPrecededByHeader
+#	define	biRowBelowAllPosition	BIU.biuRow.brBelowAllPosition
+#	define	biRowAboveHeaderPosition \
+					BIU.biuRow.brAboveHeaderPosition
+
+#	define	biRowProperties		BIU.biuRow.brRowProperties
+
 #	define	biRowHasTableParagraphs	biRowProperties.rpHasTableParagraphs
 #	define	biRowHalfGapWidthTwips	biRowProperties.rpHalfGapWidthTwips
-#	define	biRowHalfGapWidthPixels \
-					biRowProperties.rpHalfGapWidthPixels
+#	define	biRowHalfGapWidthPixels biRowProperties.rpHalfGapWidthPixels
 #	define	biRowLeftIndentTwips	biRowProperties.rpLeftIndentTwips
 #	define	biRowLeftIndentPixels	biRowProperties.rpLeftIndentPixels
 #	define	biRowHeightTwips	biRowProperties.rpHeightTwips
 #	define	biRowIsTableHeader	biRowProperties.rpIsTableHeader
 #	define	biRowKeepOnOnePage	biRowProperties.rpKeepOnOnePage
-#	define	biRowHasHorizontalBorders \
-					biRowProperties.rpHasHorizontalBorders
-#	define	biRowHasVerticalBorders \
-					biRowProperties.rpHasVerticalBorders
+#	define	biRowKeepWithNext	biRowProperties.rpKeepWithNext
+#	define	biRowAutofit		biRowProperties.rpAutofit
 #	define	biRowCells		biRowProperties.rpCells
 #	define	biRowCellCount		biRowProperties.rpCellCount
 #	define	biRowAlignment		biRowProperties.rpAlignment
@@ -341,9 +398,9 @@ typedef enum ItemLevel
 	*/
 
     DOClevPARA,
-			/********************************/
-			/*  Paragraph.			*/
-			/********************************/
+			/****************************************/
+			/*  Paragraph.				*/
+			/****************************************/
 #	define	biParaString		BIU.biuPara.btString
 #	define	biParaStrlen		BIU.biuPara.btStrlen
 #	define	biParaParticuleCount	BIU.biuPara.btParticuleCount
@@ -354,7 +411,18 @@ typedef enum ItemLevel
 #	define	biParaLines		BIU.biuPara.btLines
 #	define	biParaShapeCount	BIU.biuPara.btShapeCount
 #	define	biParaShapes		BIU.biuPara.btShapes
+
+#	define	biParaAscentTwips	BIU.biuPara.btAscentTwips
+#	define	biParaDescentTwips	BIU.biuPara.btDescentTwips
+#	define	biParaLeadingTwips	BIU.biuPara.btLeadingTwips
+
+#	define	biParaBorderAboveParagraph \
+					BIU.biuPara.btBorderAboveParagraph
+#	define	biParaBorderBelowParagraph \
+					BIU.biuPara.btBorderBelowParagraph
+
 #	define	biParaProperties	BIU.biuPara.btProperties
+
 #	define	biParaInTable		biParaProperties.ppInTable
 #	define	biParaLineSpacingIsMultiple \
 					biParaProperties.ppLineSpacingIsMultiple
@@ -366,13 +434,13 @@ typedef enum ItemLevel
 #	define	biParaSpaceAfterTwips	biParaProperties.ppSpaceAfterTwips
 #	define	biParaLineSpacingTwips	biParaProperties.ppLineSpacingTwips
 #	define	biParaAlignment		biParaProperties.ppAlignment
-#	define	biParaTabStops		biParaProperties.ppTabStops
-#	define	biParaTabCount		biParaProperties.ppTabCount
+#	define	biParaTabStopList	biParaProperties.ppTabStopList
 #	define	biParaKeepOnPage	biParaProperties.ppKeepOnPage
 #	define	biParaKeepWithNext	biParaProperties.ppKeepWithNext
 #	define	biParaShadingLevel	biParaProperties.ppShadingLevel
 #	define	biParaShadingPattern	biParaProperties.ppShadingPattern
 #	define	biParaOutlineLevel	biParaProperties.ppOutlineLevel
+#	define	biParaListLevel		biParaProperties.ppListLevel
 #	define	biParaTopBorder		biParaProperties.ppTopBorder
 #	define	biParaBottomBorder	biParaProperties.ppBottomBorder
 #	define	biParaLeftBorder	biParaProperties.ppLeftBorder
@@ -381,29 +449,37 @@ typedef enum ItemLevel
 #	define	biParaBetweenBorder	biParaProperties.ppBetweenBorder
 #	define	biParaBar		biParaProperties.ppBar
 #	define	biParaStyle		biParaProperties.ppStyle
+#	define	biParaListOverride	biParaProperties.ppListOverride
 #	define	biParaWidowControl	biParaProperties.ppWidowControl
 
-#	define	biParaAscentTwips	BIU.biuPara.btAscentTwips
-#	define	biParaDescentTwips	BIU.biuPara.btDescentTwips
-#	define	biParaLeadingTwips	BIU.biuPara.btLeadingTwips
-
-#	define	biParaSpaceAboveLinesTwips \
-					BIU.biuPara.btSpaceAboveLinesTwips
-#	define	biParaBorderAboveParagraph \
-					BIU.biuPara.btBorderAboveParagraph
-#	define	biParaSpaceBelowLinesTwips \
-					BIU.biuPara.btSpaceBelowLinesTwips
-#	define	biParaBorderBelowParagraph \
-					BIU.biuPara.btBorderBelowParagraph
-
+#	define	biParaTabStopCount	biParaTabStopList.tslTabStopCount
+#	define	biParaTabStops		biParaTabStopList.tslTabStops
     DOClevTEXT
-			/********************************/
-			/*  Handeled inside TEXT.	*/
-			/********************************/
+			/****************************************/
+			/*  Handeled inside TEXT.		*/
+			/****************************************/
     } ItemLevel;
 
+/************************************************************************/
+/*									*/
+/*  A BufferItem.							*/
+/*									*/
+/*  A BufferItem is a node in what we fashionably call the document	*/
+/*  object model.							*/
+/*									*/
+/*  1)  Fields for managing the hierarchy.				*/
+/*  2)  Geometry information that is set by the formatter.		*/
+/*	The insets are an amount of space that the item claims for	*/
+/*	itself. The content of the item (children, or lines) appears	*/
+/*	between the insets. Typically, the insets are used for spacing	*/
+/*	or borders.							*/
+/*  3)  Information that is specific to the kind of item.		*/
+/*									*/
+/************************************************************************/
+
 typedef struct BufferItem
     {
+				/*  1  */
     short int			biLevel;
     unsigned char		biInExternalItem;
     struct BufferItem *		biParent;
@@ -412,14 +488,18 @@ typedef struct BufferItem
     short int			biNumberInParent;
     int				biLeftParagraphs;
 
+				/*  2  */
     LayoutPosition		biTopPosition;
     LayoutPosition		biBelowPosition;
+    int				biTopInsetTwips;
+    int				biBottomInsetTwips;
 
-    union	BIU
+				/*  3  */
+    union			BufferItemUnion
 	{
-	BufferPara	biuPara;
-	BufferSect	biuSect;
-	BufferRow	biuRow;
+	BufferPara		biuPara;
+	BufferSect		biuSect;
+	BufferRow		biuRow;
 	} BIU;
 
     } BufferItem;
@@ -432,22 +512,27 @@ typedef struct BufferItem
 
 typedef struct BufferDocument
     {
-    BufferItem		bdItem;
-    DocumentProperties	bdProperties;
+    TextAttributeList		bdTextAttributeList;
 
-    DocumentStyleSheet	bdStyleSheet;
-    DocumentFieldList	bdFieldList;
+    BufferItem			bdItem;
+    DocumentProperties		bdProperties;
 
-    DocumentNote *	bdNotes;
-    int			bdNoteCount;
+    DocumentStyleSheet		bdStyleSheet;
+    DocumentFieldList		bdFieldList;
 
-    ExternalItem	bdEiFtnsep;
-    ExternalItem	bdEiFtnsepc;
-    ExternalItem	bdEiFtncn;
+    DocumentNote *		bdNotes;
+    int				bdNoteCount;
 
-    ExternalItem	bdEiAftnsep;
-    ExternalItem	bdEiAftnsepc;
-    ExternalItem	bdEiAftncn;
+    ListNumberTreeNode *	bdListNumberTrees;
+    int				bdListNumberTreeCount;
+
+    ExternalItem		bdEiFtnsep;
+    ExternalItem		bdEiFtnsepc;
+    ExternalItem		bdEiFtncn;
+
+    ExternalItem		bdEiAftnsep;
+    ExternalItem		bdEiAftnsepc;
+    ExternalItem		bdEiAftncn;
     } BufferDocument;
 
 /************************************************************************/
@@ -474,6 +559,8 @@ typedef struct DocumentStatistics
 
 extern BufferDocument *	docPlainReadFile( SimpleInputStream *	sis,
 				    int *			pMxL,
+				    int				docCharset,
+				    int				ansiCodepage,
 				    const DocumentGeometry *	dg );
 
 extern BufferDocument *	docRtfReadFile(	SimpleInputStream *	sis,
@@ -482,9 +569,13 @@ extern BufferDocument *	docRtfReadFile(	SimpleInputStream *	sis,
 extern BufferDocument *	docLeafReadFile(	SimpleInputStream *	sis );
 extern BufferDocument *	docWordReadFile(	SimpleInputStream *	sis );
 
-extern BufferDocument * docNewFile(	const char *		fontName,
-					int			fontSize,
-					const DocumentGeometry * dg );
+extern BufferDocument * docNewFile(
+				TextAttribute *			taDefault,
+				const char *			defaultFont,
+				int				docCharset,
+				int				ansiCodepage,
+				const PostScriptFontList *	psfl,
+				const DocumentGeometry * 	dg );
 
 extern int docRtfSaveDocument(	SimpleOutputStream *		sos,
 				BufferDocument *		bd,
@@ -543,36 +634,6 @@ extern BufferItem * docInsertItem(	const BufferDocument *	bd,
 					int			n,
 					ItemLevel		level );
 
-extern BufferItem * docCopyParaItem(	BufferDocument *	bdTo,
-					const BufferDocument *	bdFrom,
-					int *			fieldMap,
-					unsigned int *		pFieldUpd,
-					BufferItem *		biCellTo,
-					int			n,
-					BufferItem *		biParaFrom,
-					int			inTable,
-					const char *		refFilename );
-
-extern BufferItem * docCopyRowItem(	BufferDocument *	bdTo,
-					const BufferDocument *	bdFrom,
-					int *			fieldMap,
-					unsigned int *		pFieldUpd,
-					BufferItem *		biSectTo,
-					int			n,
-					BufferItem *		biRowFrom,
-					int			inTable,
-					const char *		refFilename );
-
-extern BufferItem * docCopySectItem(	BufferDocument *	bdTo,
-					const BufferDocument *	bdFrom,
-					int *			fieldMap,
-					unsigned int *		pFieldUpd,
-					BufferItem *		biParentTo,
-					int			n,
-					BufferItem *		biSectFrom,
-					const SelectionScope *	ss,
-					const char *		refFilename );
-
 extern void docDeleteItems(		BufferDocument *	bd,
 					BufferItem *		bi,
 					int			first,
@@ -583,7 +644,7 @@ extern TextParticule *	docInsertTextParticule(	BufferItem *	bi,
 						int		off,
 						int		len,
 						int		kind,
-						TextAttribute	ta	);
+						int		textAttrNr );
 
 extern TextParticule * docCopyParticule(	BufferItem *		bi,
 						int			n,
@@ -592,6 +653,13 @@ extern TextParticule * docCopyParticule(	BufferItem *		bi,
 						int			kind,
 						const TextParticule *	from );
 
+extern int docSplitTextParticule(
+				TextParticule **		pTpPart,
+				TextParticule **		pTpNext,
+				BufferItem *			paraBi,
+				int				part,
+				int				stroff );
+
 extern void docDeleteParticules(	BufferItem *	bi,
 					int		first,
 					int		count );
@@ -618,25 +686,21 @@ extern BufferItem * docPrevSection(	BufferItem *	bi );
 
 extern int docNextPosition(	DocumentPosition *	dp );
 
-extern int docPrevPosition(	DocumentPosition *	dp,
-				int			lastOne );
+extern int docPrevPosition(	DocumentPosition *	dp );
 
-extern int docNextWord(		DocumentPosition *	dp,
-				int			lastOne );
-extern int docPrevWord(		DocumentPosition *	dp,
-				int			lastOne );
+extern int docNextWord(		DocumentPosition *	dp );
+extern int docPrevWord(		DocumentPosition *	dp );
 
-extern int docParaBegin(	DocumentPosition *	dp,
-				int			lastOne );
-extern int docParaEnd(		DocumentPosition *	dp,
-				int			lastOne );
+extern int docParaBegin(	DocumentPosition *	dp );
+extern int docParaEnd(		DocumentPosition *	dp );
 
-extern int docNextLine(		TextParticule **	pTp,
-				TextLine **		pTl,
-				DocumentPosition *	dp );
-extern int docPrevLine(		TextParticule **	pTp,
-				TextLine **		pTl,
-				DocumentPosition *	dp );
+extern int docLineDown(		TextLine **		pTl,
+				DocumentPosition *	dp,
+				int			line );
+
+extern int docLineUp(		TextLine **		pTl,
+				DocumentPosition *	dp,
+				int			line );
 
 extern int docFirstPosition(	DocumentPosition *	dp,
 				BufferItem *		bi );
@@ -657,25 +721,29 @@ extern void docSetParaSelection( DocumentSelection *	ds,
 				int			length );
 
 extern int docComparePositions(	const DocumentPosition *	dp1,
-				const DocumentPosition *	dp2,
-				int				mindPart );
+				const DocumentPosition *	dp2 );
 
 extern int docCompareItemPositions(	const BufferItem *	bi1,
 					const BufferItem *	bi2 );
 
 extern int docSetDocumentPosition(	DocumentPosition *	dp,
 					BufferItem *		bi,
-					int			stroff,
-					int			lastOne );
+					int			stroff );
 
 extern int docFindParticule(		int *			pPart,
 					const BufferItem *	bi,
 					int			stroff,
 					int			lastOne );
 
-extern int docFindLineOfParticule(	int *			pLine,
-					const BufferItem *	bi,
-					int			part );
+extern int docFindParticuleOfPosition(
+				int *				pPart,
+				const DocumentPosition *	dp,
+				int				lastOne );
+
+extern int docFindLineOfPosition(
+				int *				pLine,
+				const DocumentPosition *	dp,
+				int				lastOne );
 
 extern void docDeleteItem(		BufferDocument *	bd,
 					BufferItem *		bi );
@@ -708,13 +776,16 @@ extern const char * docKindStr(		int			kind );
 extern const char * docLevelStr(	int			level );
 extern const char * docExternalKindStr(	int			level );
 extern const char * docFieldKindStr(	int			kind );
-extern const char * docAttributeStr(	TextAttribute		ta );
+extern const char * docAttributeStr(	const TextAttribute *	ta );
 
-extern TextParticule *	docInsertObject(	BufferItem *		bi,
+extern void docListNotes(	const BufferDocument *	bd );
+
+extern TextParticule *	docInsertObject(	BufferDocument *	bd,
+						BufferItem *		paraBi,
 						InsertedObject **	pIo,
 						int			n,
 						int			off,
-						TextAttribute		ta );
+						const TextAttribute *	ta );
 
 extern void docCleanParticuleObject(	BufferItem *	bi,
 					TextParticule *	tp );
@@ -737,8 +808,6 @@ extern int docAddToFieldInst(	DocumentField *		df,
 				const unsigned char *	bytes,
 				int			size );
 
-extern void docInitCellProperties(	CellProperties *	cp );
-
 extern int docDelimitTable(	BufferItem *		paraBi,
 				BufferItem **		pSectBi,
 				int *			pCol,
@@ -746,9 +815,15 @@ extern int docDelimitTable(	BufferItem *		paraBi,
 				int *			pRow,
 				int *			pRow1 );
 
+extern void docTableDetermineCellspans(	int *			pRowspan,
+					int *			pColspan,
+					const BufferItem *	rowBi,
+					int			col );
+
 extern int docGetBookmarkForPosition(
 				const BufferDocument *		bd,
 				const DocumentPosition *	dp,
+				DocumentSelection *		dsBookmark,
 				int *				pPart,
 				int *				pEndPart,
 				const char **			pMarkName,
@@ -757,6 +832,7 @@ extern int docGetBookmarkForPosition(
 extern int docGetHyperlinkForPosition(
 				const BufferDocument *		bd,
 				const DocumentPosition *	dp,
+				DocumentSelection *		dsHyperlink,
 				int *				pStartPart,
 				int *				pEndPart,
 				const char **			pFileName,
@@ -764,48 +840,26 @@ extern int docGetHyperlinkForPosition(
 				const char **			pMarkName,
 				int *				pMarkSize );
 
-extern DocumentField *	docFindBookmarkField( const DocumentFieldList * dfl,
-					const char *		markName,
-					int			markSize );
-
-extern int docCopyParticuleData(	BufferDocument *	bdTo,
-					const BufferDocument *	bdFrom,
-					int *			fieldMap,
-					unsigned int *		pFieldUpd,
-					BufferItem *		biTo,
-					const BufferItem *	biFrom,
-					TextParticule *		tpTo,
-					const TextParticule *	tpFrom );
-
-extern int docCopyParticules(	BufferDocument *	bdTo,
-				const BufferDocument *	bdFrom,
-				int *			fieldMap,
-				unsigned int *		pFieldUpd,
-				BufferItem *		biTo,
-				const BufferItem *	biFrom,
-				int			partTo,
-				int			partFrom,
-				int			countFrom,
-				int *			pParticulesInserted,
-				int *			pCharactersCopied,
-				const char *		refFileName );
-
 extern void docInitShape(	DrawingShape *	ds );
 extern void docCleanShape(	DrawingShape *	ds );
 
 extern DrawingShape * docClaimShape(	int *			pNr,
 					BufferItem *		bi );
 
-extern int docParticuleInField(		BufferItem *	bi,
-					int		part );
+extern int docPositionInField(		const DocumentPosition *	dp,
+					const BufferDocument *		bd );
 
-extern int docSaveParticules(	BufferItem *		bi,
-				TextAttribute		ta,
+extern int docSaveParticules(	BufferDocument *	bd,
+				BufferItem *		paraBi,
+				const TextAttribute *	ta,
+				const unsigned char *	map,
 				const unsigned char *	text,
 				int			len );
 
-extern int docSaveSpecialParticule(	BufferItem *		bi,
-					TextAttribute		ta,
+extern int docSaveSpecialParticule(	BufferDocument *	bd,
+					BufferItem *		paraBi,
+					int			atBegin,
+					const TextAttribute *	ta,
 					int			kind );
 
 extern void docLogRectangle(	const char *			label,
@@ -819,6 +873,8 @@ extern void docLogRectangles(	const char *			label1,
 extern void docInitLayoutPosition(	LayoutPosition *	lp );
 
 extern void docLineSelection(	DocumentSelection *	dsLine,
+				int *			pPartLineBegin,
+				int *			pPartLineEnd,
 				const BufferItem *	bi,
 				int			line );
 
@@ -850,32 +906,6 @@ extern BufferItem * docGetSelectionRoot(
 
 extern void docInitParagraphNumber(	ParagraphNumber *	pn );
 
-extern TextParticule *	docInsertParticule(	BufferItem *	bi,
-						int		n,
-						int		off,
-						int		len,
-						int		kind );
-
-extern int docCopyFieldRelative(	BufferDocument *	bdTo,
-					const BufferDocument *	bdFrom,
-					int *			fieldMap,
-					unsigned int *		pFieldUpd,
-					BufferItem *		biTo,
-					const BufferItem *	biFrom,
-					TextParticule *		tpTo,
-					const TextParticule *	tpFrom,
-					const char *		refFileName,
-					int			refFileSize );
-
-extern void docFieldRefreshFlags(	BufferItem *		bi,
-					const BufferDocument *	bd );
-
-extern void docAdaptBookmarkName(	int *		pLength,
-					char *		mark );
-
-extern void docInitDocumentList(	DocumentList *	dl );
-extern void docCleanDocumentList(	DocumentList *	dl );
-
 extern int docRecalculateTextLevelFields(	RecalculateFields *	rf,
 						BufferItem *		bi );
 
@@ -885,18 +915,13 @@ extern int docRecalculateTextLevelFieldsInExternalItem(
 					const BufferItem *	sectBi,
 					int			page );
 
-extern int docRedivideStringInParticules(	BufferItem *	bi,
-						int		strOff,
-						int		strLen,
-						int		part,
-						int		partsFree,
-						TextAttribute	ta );
-
-extern int docBalanceFieldSelection(	int *			pBeginMoved,
-					int *			pEndMoved,
-					const BufferDocument *	bd,
-					DocumentPosition *	dpBegin,
-					DocumentPosition *	dpEnd );
+extern int docRedivideStringInParticules(
+					BufferItem *	bi,
+					int		strOff,
+					int		strLen,
+					int		part,
+					int		partsFree,
+					int		textAttributeNumber );
 
 extern int docSplitFieldInstructions(	const MemoryBuffer *	mb,
 				FieldInstructionsComponent *	fic,
@@ -917,13 +942,14 @@ extern BufferItem * docMakeExternalparagraph(
 				BufferDocument *		bd,
 				ExternalItem *			ei,
 				const BufferItem *		bodyBi,
-				TextAttribute			ta,
+				int				txtAttrNr,
 				int				noteIndex,
 				int				inExtIt );
 
-extern BufferItem * docInsertEmptyParagraph(	BufferDocument *	bd,
-						BufferItem *		bi,
-						TextAttribute		ta );
+extern BufferItem * docInsertEmptyParagraph(
+				BufferDocument *	bd,
+				BufferItem *		bi,
+				int			textAttributeNumber );
 
 extern int docIntersectTableRectangle(	TableRectangle *		tr,
 					const TableRectangle *		tr1,
@@ -943,8 +969,13 @@ extern int docTableRectangleSelection(	DocumentSelection *	ds,
 					BufferDocument *	bd,
 					const TableRectangle *	tr );
 
-extern int docMergeDocumentFontsIntoList( DocumentFontList *	dflTo,
-					BufferDocument *	bdFrom );
+extern int docMergeDocumentLists(
+				int **				pFontMap,
+				int **				pColorMap,
+				int **				pListStyleMap,
+				BufferDocument *		bdTo,
+				BufferDocument *		bdFrom,
+				const PostScriptFontList *	psfl );
 
 extern int docFindBookmarkInDocument(	DocumentSelection *	ds,
 					BufferDocument *	bd,
@@ -952,9 +983,11 @@ extern int docFindBookmarkInDocument(	DocumentSelection *	ds,
 					int			markSize );
 
 extern int docSurroundTextSelectionByField(
-				DocumentSelection *		dsField,
+				DocumentField **		pDf,
+				BufferDocument *		bd,
+				int *				pStartPart,
+				int *				pEndPart,
 				const DocumentSelection *	ds,
-				int				fieldNumber,
 				const PropertyMask *		taSetMask,
 				const TextAttribute *		taSet );
 
@@ -966,7 +999,7 @@ extern void docUnionParaSelections(	DocumentSelection *		ds,
 					const DocumentSelection *	ds1,
 					const DocumentSelection *	ds2 );
 
-extern int *	docAllocateFieldMap(	const BufferDocument *	bdFrom );
+extern int * docAllocateFieldMap(	const BufferDocument *	bdFrom );
 
 extern int docParaStringReplace(	int *			pStroffShift,
 					BufferItem *		bi,
@@ -978,22 +1011,50 @@ extern int docParaStringReplace(	int *			pStroffShift,
 extern int docInsertNote(		DocumentNote **		pDn,
 					BufferDocument *	bd,
 					BufferItem *		paraBi,
-					int			stroff );
+					int			stroff,
+					int			autoNumber );
 
 extern int docGetNote(			DocumentNote **		pDn,
 					const BufferDocument *	bd,
 					const BufferItem *	paraBi,
 					int			stroff );
 
-extern int docMakeNote(		DocumentNote **			pDn,
-				BufferDocument *		bd,
-				const DocumentPosition *	dp,
-				int				extItKind );
+extern int docMakeNote(	DocumentNote **			pDn,
+			BufferDocument *		bd,
+			BufferItem *			paraBi,
+			int				part,
+			int				stroff,
+			int				extItKind );
 
 int docGetSelectedNote(		DocumentNote **			pDn,
+				int *				pFieldNr,
+				unsigned char *			fixedText,
+				int				fixedTextSize,
 				BufferDocument *		bd,
 				const DocumentSelection *	ds );
 
+extern void docNoteGetTextAtHead(	unsigned char *		fixedText,
+					int *			pFixedLen,
+					int			fixedTextSize,
+					const BufferItem *	paraBi );
+
+extern void docNoteGetTextBefore(	unsigned char *		fixedText,
+					int *			pFixedLen,
+					int *			pTextAttrNr,
+					int			fixedTextSize,
+					const BufferItem *	paraBi,
+					int			stroff );
+
+extern int docDelimitNoteReference(
+				int *			pFieldNr,
+				int *			pPartBegin,
+				int *			pPartEnd,
+				int *			pStroffBegin,
+				int *			pStroffEnd,
+				const BufferItem *	paraBi,
+				int			stroff,
+				const BufferDocument *	bd );
+
 extern void docRenumberNotes(		int *			pChanged,
 					BufferDocument *	bd );
 
@@ -1016,20 +1077,39 @@ extern int docGetRootOfSelectionScope(	ExternalItem **		pEi,
 					const SelectionScope *	ss );
 
 extern int docGetTopOfColumn(	DocumentPosition *		dp,
+				int *				pPartTop,
 				BufferDocument *		bd,
 				int				page,
 				int				column );
 
+extern int docGetFirstInColumnForItem(
+				DocumentPosition *		dp,
+				int *				pLineTop,
+				int *				pPartTop,
+				BufferItem *			bi,
+				int				page,
+				int				column );
+
 extern int docGetBottomOfColumn(DocumentPosition *		dp,
+				int *				pPartBot,
 				BufferDocument *		bd,
 				int				page,
 				int				column );
 
-extern BufferItem * docInsertRowItem(	BufferDocument *	bd,
-					BufferItem *		sectBi,
-					int			n,
-					const RowProperties *	rp,
-					TextAttribute		ta );
+extern int docGetLastInColumnForItem(
+				DocumentPosition *		dp,
+				int *				pLineBot,
+				int *				pPartBot,
+				BufferItem *			bi,
+				int				page,
+				int				column );
+
+extern BufferItem * docInsertRowItem(
+				BufferDocument *	bd,
+				BufferItem *		sectBi,
+				int			n,
+				const RowProperties *	rp,
+				int			textAttributeNumber );
 
 extern int docCopyRowColumnAttributes(	BufferItem *		rowBi,
 					const BufferItem *	refRowBi );
@@ -1112,4 +1192,113 @@ extern int docCountParticulesInField(	const BufferItem *	bi,
 					int			part,
 					int			partUpto );
 
+extern int docGetSolidRgbShadeOfItem(	int *			pIsSolid,
+					int *			pRed,
+					int *			pGreen,
+					int *			pBlue,
+					const BufferDocument *	bd,
+					const ItemShading *	is );
+
+extern void docSectDelimitTables(	BufferItem *		sectBi );
+
+extern void docGetCellTopBorder(
+			const BorderProperties **	pBpTop,
+			int *				pUseAbove,
+			const BufferItem *		rowBi,
+			int				col,
+			int				atTopOfPage );
+
+extern void docGetCellBottomBorder(
+			const BorderProperties **	pBpButtom,
+			int *				pUseBelow,
+			const BufferItem *		rowBi,
+			int				col,
+			int				atBottomOfPage );
+
+extern int docTextAttributeNumber(	BufferDocument *	bd,
+					const TextAttribute *	ta );
+
+extern int docInsertAdminParticule(	BufferDocument *	bd,
+					BufferItem *		paraBi,
+					int			off,
+					int			n,
+					int			objectNumber,
+					const TextAttribute *	ta,
+					int			kind );
+
+extern int docBalanceFieldSelection(	int *			pBeginMoved,
+					int *			pEndMoved,
+					const BufferDocument *	bd,
+					DocumentPosition *	dpBegin,
+					DocumentPosition *	dpEnd );
+
+extern int docGetListOfParagraph(	ListOverride **		pLo,
+					ListNumberTreeNode **	pRoot,
+					DocumentList **		pDl,
+					int			ls,
+					BufferDocument *	bd );
+
+extern int docGetListOfOverride(	ListOverride *			lo,
+					const DocumentListTable *	dlt );
+
+extern int docListGetFormatPath(int *				startPath,
+				int *				formatPath,
+				const DocumentListLevel **	pDll,
+				int				ilvl,
+				const DocumentList *		dl,
+				const ListOverride *		lo );
+
+extern int docInsertListtextField(	BufferItem *		paraBi,
+					BufferDocument *	bd );
+
+extern int docRemoveParagraphFromList(	BufferItem *		paraBi,
+					BufferDocument *	bd );
+
+extern int docParaHeadFieldKind(	const BufferItem *	paraBi,
+					const BufferDocument *	bd );
+
+extern int docDelimitParaHeadField(
+				int *			pFieldNr,
+				int *			pPartBegin,
+				int *			pPartEnd,
+				int *			pStroffBegin,
+				int *			pStroffEnd,
+				const BufferItem *	paraBi,
+				const BufferDocument *	bd );
+
+extern int docFindListOfSelection(
+				int *				pLs,
+				int *				pLevel,
+				int *				pMultiList,
+				int *				pMultiLevel,
+				int *				pParaNr,
+				const DocumentSelection *	ds );
+
+extern int docApplyListRuler(	const ListNumberTreeNode *	root,
+				const DocumentList *		dl,
+				const ListOverride *		lo,
+				BufferDocument *		bd );
+
+extern void docGetSelectionAttributes(
+				BufferDocument *		bd,
+				const DocumentSelection *	ds,
+				PropertyMask *			pUpdMask,
+				TextAttribute *			pTaNew );
+
+extern int docInsertParaHeadField(
+				BufferItem *			paraBi,
+				BufferDocument *		bd,
+				int				fieldKind,
+				int				textAttrNr );
+
+extern int docGetNotePosition(	DocumentPosition *		dp,
+				BufferDocument *		bd,
+				const DocumentNote *		dn );
+
+extern int docNextSimilarRoot(	DocumentPosition *		dp,
+				BufferDocument *		bd );
+
+extern int docPrevSimilarRoot(	DocumentPosition *		dp,
+				BufferDocument *		bd );
+
 #   endif
