@@ -14,9 +14,60 @@
 #   include	<appDebugon.h>
 
 #   include	"docBuf.h"
+#   include	"docEdit.h"
 
 #   define	CHECK_ITEMS	0
 
+#if 0
+
+Schema to illustrate the numbering of paragraphs and the 
+administration that is kept for that purpose
+=========================================================
+
+bi->biLeftParagraphs is the number of paragraphs to the left 
+of this item in its immediate parent plus the number of paragraph
+descendants. I.E: For items with children, the number of paragraphs 
+in the children is included in bi->biLeftParagraphs. A paragraph 
+includes itself in its bi->biLeftParagraphs.
+
+Numbers of the paragraphs and the value of bi->biLeftParagraphs:
+
+    1     2     3     4     5     6     7     8     9    10    11    12
+    +     +     +     +     +     +     +     +     +     +     +     +
+    1     2     3     4     1     2     3     4     5     1     2     3
+    |     |     |     |     |     |     |     |     |     |     |     |
+    4-----+-----+-----+     9-----+-----+-----+-----+     12----+-----+
+    |                       |                             |
+    12----------------------+-----------------------------+
+    |
+    *
+
+Deleting 8 yeilds:
+
+    1     2     3     4     5     6     7           8     9    10    11
+    +     +     +     +     +     +     +           +     +     +     +
+    1     2     3     4     1     2     3           4     1     2     3
+    |     |     |     |     |     |     |           |     |     |     |
+    4-----+-----+-----+     8-----+-----+-----+-----+     11----+-----+
+    |                       |                             |
+    11----------------------+-----------------------------+
+    |
+    *
+
+So after deleting/inserting paragraphs: Descend to the root of the 
+tree. In every parent adapt bi->biLeftParagraphs of all direct 
+children to the right of the child that we come from. Set
+bi->biLeftParagraphs of the parent to the value in its right most 
+child. Continue with the parent of the parent until we have reached 
+the root.
+
+NOTE: I am educated as a biologist. My trees have their root at the 
+    bottom, not at the top like those of computer scientists that 
+    turn the world upside down. (I must admit that the metaphor 
+    of parents and descendants has the computer science orientation.)
+
+#endif
+
 /************************************************************************/
 /*									*/
 /*  Free a BufferItem.							*/
@@ -174,7 +225,15 @@ void docInitItem(	BufferItem *		bi,
 	    break;
 
 	case DOClevROW:
+	    bi->biRowTableFirst= -1;
+	    bi->biRowTablePast= -1;
+	    bi->biRowTableFirstIsHeader= 0;
+	    bi->biRowPrecededByHeader= 0;
+
 	    docInitRowProperties( &(bi->biRowProperties) );
+
+	    docInitLayoutPosition( &(bi->biRowBelowAllPosition) );
+	    docInitLayoutPosition( &(bi->biRowAboveHeaderPosition) );
 	    break;
 
 	case DOClevPARA:
@@ -197,14 +256,13 @@ void docInitItem(	BufferItem *		bi,
 	    bi->biParaDescentTwips= 0;
 	    bi->biParaLeadingTwips= 0;
 
-	    bi->biParaSpaceAboveLinesTwips= 0;
 	    bi->biParaBorderAboveParagraph= (const BorderProperties *)0;
-	    bi->biParaSpaceBelowLinesTwips= 0;
 	    bi->biParaBorderBelowParagraph= (const BorderProperties *)0;
 
 	    docInitParagraphProperties( &(bi->biParaProperties) );
 
 	    break;
+
 	default:
 	    bi->biLevel= DOClevOUT;
 	    bi->biParent= (BufferItem *)0;
@@ -219,6 +277,9 @@ void docInitItem(	BufferItem *		bi,
     docInitLayoutPosition( &(bi->biTopPosition) );
     docInitLayoutPosition( &(bi->biBelowPosition) );
 
+    bi->biTopInsetTwips= 0;
+    bi->biBottomInsetTwips= 0;
+
     return;
     }
 
@@ -229,9 +290,10 @@ void docInitItem(	BufferItem *		bi,
 /*									*/
 /************************************************************************/
 
-static void docSetSectExternalParents(		BufferItem *	sectBi,
-						int		n )
+static void docSetSectHeadFootScopes(		BufferItem *	sectBi )
     {
+    int		n= sectBi->biNumberInParent;
+
     if  ( sectBi->biSectHeader.eiItem )
 	{
 	sectBi->biSectHeader.eiItem->
@@ -283,6 +345,7 @@ void docDeleteItems(	BufferDocument *	bd,
 			int			first,
 			int			count )
     {
+    int		n;
     int		f;
     int		c;
     int		paragraphsDeleted= 0;
@@ -307,8 +370,6 @@ void docDeleteItems(	BufferDocument *	bd,
     if  ( count <= 0 )
 	{ LDEB(count); return;	}
 
-    bi->biChildCount -= count;
-
     if  ( count > 0 )
 	{
 	f= 0;
@@ -320,9 +381,11 @@ void docDeleteItems(	BufferDocument *	bd,
 	paragraphsDeleted= c- f;
 	}
 
-    f= first; c= count;
-    while( c > 0 )
-	{ docFreeItem( bd, bi->biChildren[f] ); f++; c--; }
+    n= first+ count- 1;
+    while( n >= first )
+	{ docFreeItem( bd, bi->biChildren[n] ); n--; }
+
+    bi->biChildCount -= count;
 
     f= first;
     while( f < bi->biChildCount )
@@ -331,9 +394,8 @@ void docDeleteItems(	BufferDocument *	bd,
 	bi->biChildren[f]->biNumberInParent= f;
 	bi->biChildren[f]->biLeftParagraphs -= paragraphsDeleted;
 
-	if  ( bi->biLevel == DOClevSECT		&&
-	      bi->biInExternalItem != DOCinBODY	)
-	    { docSetSectExternalParents( bi, f ); }
+	if  ( bi->biChildren[f]->biLevel == DOClevSECT )
+	    { docSetSectHeadFootScopes( bi->biChildren[f] ); }
 
 	f++;
 	}
@@ -439,9 +501,8 @@ BufferItem * docInsertItem(	const BufferDocument *	bd,
 
 	freshChildren[i]->biNumberInParent= i;
 
-	if  ( freshChildren[i]->biLevel == DOClevSECT		&&
-	      freshChildren[i]->biInExternalItem != DOCinBODY	)
-	    { docSetSectExternalParents( freshChildren[i], i ); }
+	if  ( freshChildren[i]->biLevel == DOClevSECT )
+	    { docSetSectHeadFootScopes( freshChildren[i] ); }
 	}
 
     freshChildren[n]= bi;
@@ -486,9 +547,10 @@ BufferItem * docInsertItem(	const BufferDocument *	bd,
 /*									*/
 /************************************************************************/
 
-BufferItem * docInsertEmptyParagraph(		BufferDocument *	bd,
-						BufferItem *		bi,
-						TextAttribute		ta )
+BufferItem * docInsertEmptyParagraph(
+				BufferDocument *	bd,
+				BufferItem *		bi,
+				int			textAttributeNumber )
     {
     if  ( bi->biLevel < DOClevSECT )
 	{ LDEB(bi->biLevel); return (BufferItem *)0;	}
@@ -519,9 +581,14 @@ BufferItem * docInsertEmptyParagraph(		BufferDocument *	bd,
 	    { XDEB(bi); return (BufferItem *)0;   }
 	}
 
-    if  ( ! docInsertTextParticule( bi, 0, 0, 0, DOCkindTEXT, ta ) )
+    if  ( ! docInsertTextParticule( bi, 0, 0, 0,
+				    DOCkindTEXT, textAttributeNumber ) )
 	{ LDEB(1); return (BufferItem *)0;	}
 
+    if  ( docInflateTextString( bi, 0 ) )
+	{ LDEB(1); return (BufferItem *)0;	}
+    bi->biParaString[bi->biParaStrlen]= '\0';
+
     return bi;
     }
 
@@ -531,31 +598,29 @@ BufferItem * docInsertEmptyParagraph(		BufferDocument *	bd,
 /*									*/
 /************************************************************************/
 
-BufferItem * docCopyParaItem(	BufferDocument *	bdTo,
-				const BufferDocument *	bdFrom,
-				int *			fieldMap,
-				unsigned int *		pFieldUpd,
+BufferItem * docCopyParaItem(	DocumentCopyJob *	dcj,
+				EditOperation *		eo,
 				BufferItem *		biCellTo,
 				int			n,
 				BufferItem *		biParaFrom,
-				int			inTable,
-				const char *		refFileName )
+				int			inTable )
     {
-    BufferItem *	insBi;
+    BufferItem *	biParaTo;
 
-    int			partTo= 0;
+    const int		partTo= 0;
     int			partFrom= 0;
+
     int			particulesInserted= 0;
     int			charactersCopied= 0;
 
     PropertyMask	ppChgMask;
     PropertyMask	ppUpdMask;
 
-    insBi= docInsertItem( bdTo, biCellTo, n, DOClevPARA );
-    if  ( ! insBi )
-	{ XDEB(insBi); return insBi;	}
+    biParaTo= docInsertItem( dcj->dcjBdTo, biCellTo, n, DOClevPARA );
+    if  ( ! biParaTo )
+	{ XDEB(biParaTo); return biParaTo;	}
 
-    insBi->biParaInTable= inTable;
+    biParaTo->biParaInTable= inTable;
 
     PROPmaskCLEAR( &ppChgMask );
 
@@ -563,41 +628,84 @@ BufferItem * docCopyParaItem(	BufferDocument *	bdTo,
     PROPmaskFILL( &ppUpdMask, PPprop_COUNT );
     PROPmaskUNSET( &ppUpdMask, PPpropIN_TABLE );
 
-    if  ( docUpdParaProperties( &ppChgMask, &insBi->biParaProperties,
-				&ppUpdMask, &biParaFrom->biParaProperties ) )
-	{ LDEB(1); return (BufferItem *)0;	}
-
-    if  ( docCopyParticules( bdTo, bdFrom, fieldMap, pFieldUpd,
-			insBi, biParaFrom,
-			partTo, partFrom, biParaFrom->biParaParticuleCount,
-			&particulesInserted, &charactersCopied, refFileName ) )
+    if  ( biParaFrom->biParaListOverride > 0 )
 	{
-	LDEB(biParaFrom->biParaParticuleCount);
-	docDeleteItem( bdTo, insBi ); return (BufferItem *)0;
+	int		bulletFieldNr= -1;
+	int		bulletPartBegin= -1;
+	int		bulletPartEnd= -1;
+	int		bulletStroffBegin= -1;
+	int		bulletStroffEnd= -1;
+
+	if  ( docDelimitParaHeadField( &bulletFieldNr,
+			&bulletPartBegin, &bulletPartEnd,
+			&bulletStroffBegin, &bulletStroffEnd,
+			biParaFrom, dcj->dcjBdFrom ) )
+	    { LDEB(1);	}
+
+	if  ( partFrom <= bulletPartEnd )
+	    { partFrom= bulletPartEnd+ 1;	}
 	}
 
-    return insBi;
+    if  ( partFrom < biParaFrom->biParaParticuleCount )
+	{
+	if  ( docCopyParticules( dcj, eo, biParaTo, biParaFrom, partTo,
+			partFrom, biParaFrom->biParaParticuleCount- partFrom,
+			&particulesInserted, &charactersCopied ) )
+	    {
+	    LDEB(biParaFrom->biParaParticuleCount);
+	    docDeleteItem( dcj->dcjBdTo, biParaTo ); return (BufferItem *)0;
+	    }
+	}
+    else{
+	int			textAttributeNumberTo;
+	const TextParticule *	tpFrom;
+
+	tpFrom= biParaFrom->biParaParticules+
+				    biParaFrom->biParaParticuleCount- 1;
+
+	textAttributeNumberTo= docMapTextAttributeNumber( dcj,
+					    tpFrom->tpTextAttributeNumber );
+	if  ( textAttributeNumberTo < 0 )
+	    { LDEB(textAttributeNumberTo); return (BufferItem *)0;	}
+
+	if  ( ! docInsertTextParticule( biParaTo, 0, 0, 0,
+					DOCkindTEXT, textAttributeNumberTo ) )
+	    { LDEB(1); return (BufferItem *)0;	}
+
+	if  ( docInflateTextString( biParaTo, 0 ) )
+	    { LDEB(1); return (BufferItem *)0;	}
+	biParaTo->biParaString[biParaTo->biParaStrlen]= '\0';
+	}
+
+    if  ( docEditUpdParaProperties( eo, &ppChgMask, biParaTo,
+				&ppUpdMask, &(biParaFrom->biParaProperties),
+				dcj->dcjColorMap, dcj->dcjListStyleMap ) )
+	{ LDEB(1); return (BufferItem *)0;	}
+
+    if  ( biParaTo->biParaListOverride > 0 )
+	{ dcj->dcjBulletsCopied++;	}
+
+    return biParaTo;
     }
 
-BufferItem * docCopyRowItem(	BufferDocument *	bdTo,
-				const BufferDocument *	bdFrom,
-				int *			fieldMap,
-				unsigned int *		pFieldUpd,
+BufferItem * docCopyRowItem(	DocumentCopyJob *	dcj,
+				EditOperation *		eo,
+				int *			pParasCopied,
 				BufferItem *		sectBiTo,
 				int			n,
 				BufferItem *		rowBiFrom,
-				int			inTable,
-				const char *		refFileName )
+				int			inTable )
     {
     BufferItem *	biRowTo;
     int			col;
+    int			parasCopied= 0;
 
-    biRowTo= docInsertItem( bdTo, sectBiTo, n, DOClevROW );
+    biRowTo= docInsertItem( dcj->dcjBdTo, sectBiTo, n, DOClevROW );
     if  ( ! biRowTo )
 	{ XDEB(biRowTo); return biRowTo;	}
 
-    if  ( docCopyRowProperties( &biRowTo->biRowProperties,
-						&rowBiFrom->biRowProperties ) )
+    if  ( docCopyRowProperties( &(biRowTo->biRowProperties),
+			    &(rowBiFrom->biRowProperties), dcj->dcjColorMap ) )
 	{ LDEB(1); return (BufferItem *)0;	}
 
     for ( col= 0; col < rowBiFrom->biChildCount; col++ )
@@ -607,34 +715,33 @@ BufferItem * docCopyRowItem(	BufferDocument *	bdTo,
 	int		para;
 
 	biCellFrom= rowBiFrom->biChildren[col];
-	biCellTo= docInsertItem( bdTo, biRowTo, col, DOClevCELL );
+	biCellTo= docInsertItem( dcj->dcjBdTo, biRowTo, col, DOClevCELL );
 
 	if  ( ! biCellTo )
-	    { XDEB(biCellTo); return biCellTo;	}
+	    { XDEB(biCellTo); return (BufferItem *)0;	}
 
 	for ( para= 0; para < biCellFrom->biChildCount; para++ )
 	    {
 	    BufferItem *	biParaFrom= biCellFrom->biChildren[para];
 
-	    if  ( ! docCopyParaItem( bdTo, bdFrom, fieldMap, pFieldUpd,
-				    biCellTo, para, biParaFrom,
-				    inTable, refFileName ) )
+	    if  ( ! docCopyParaItem( dcj, eo,
+				    biCellTo, para, biParaFrom, inTable ) )
 		{ LDEB(para); return (BufferItem *)0;	}
+
+	    parasCopied++;
 	    }
 	}
 
+    *pParasCopied= parasCopied;
     return biRowTo;
     }
 
-BufferItem * docCopySectItem(	BufferDocument *	bdTo,
-				const BufferDocument *	bdFrom,
-				int *			fieldMap,
-				unsigned int *		pFieldUpd,
+BufferItem * docCopySectItem(	DocumentCopyJob *	dcj,
+				EditOperation *		eo,
 				BufferItem *		parentBiTo,
 				int			n,
 				BufferItem *		biSectFrom,
-				const SelectionScope *	ss,
-				const char *		refFileName )
+				const SelectionScope *	ss )
     {
     BufferItem *		sectBiTo;
     int				row;
@@ -648,7 +755,7 @@ BufferItem * docCopySectItem(	BufferDocument *	bdTo,
 	    return (BufferItem *)0;
 	    }
 
-	sectBiTo= docInsertItem( bdTo, parentBiTo, n, DOClevSECT );
+	sectBiTo= docInsertItem( dcj->dcjBdTo, parentBiTo, n, DOClevSECT );
 	if  ( ! sectBiTo )
 	    { XDEB(sectBiTo); return (BufferItem *)0;	}
 	}
@@ -659,7 +766,7 @@ BufferItem * docCopySectItem(	BufferDocument *	bdTo,
 	if  ( ! sectBiTo )
 	    { XDEB(sectBiTo); return (BufferItem *)0;	}
 
-	docInitItem( sectBiTo, (BufferItem *)0, bdTo, n,
+	docInitItem( sectBiTo, (BufferItem *)0, dcj->dcjBdTo, n,
 				DOClevSECT, ss->ssInExternalItem );
 	}
 
@@ -669,7 +776,7 @@ BufferItem * docCopySectItem(	BufferDocument *	bdTo,
 					&(biSectFrom->biSectProperties) ) )
 	{
 	LDEB(1);
-	docDeleteItem( bdTo, sectBiTo );
+	docDeleteItem( dcj->dcjBdTo, sectBiTo );
 	return (BufferItem *)0;
 	}
 
@@ -677,16 +784,15 @@ BufferItem * docCopySectItem(	BufferDocument *	bdTo,
 	{
 	BufferItem *	rowBiFrom= biSectFrom->biChildren[row];
 	BufferItem *	rowBiTo;
+	int		parasCopied;
 
-	rowBiTo= docCopyRowItem( bdTo, bdFrom, fieldMap, pFieldUpd,
-					    sectBiTo, row, rowBiFrom,
-					    rowBiFrom->biRowHasTableParagraphs,
-					    refFileName );
-
+	rowBiTo= docCopyRowItem( dcj, eo, &parasCopied,
+					sectBiTo, row, rowBiFrom,
+					rowBiFrom->biRowHasTableParagraphs );
 	if  ( ! rowBiTo )
 	    {
 	    XDEB(rowBiTo);
-	    docDeleteItem( bdTo, sectBiTo );
+	    docDeleteItem( dcj->dcjBdTo, sectBiTo );
 	    return (BufferItem *)0;
 	    }
 	}
@@ -704,7 +810,7 @@ BufferItem * docInsertRowItem(	BufferDocument *	bd,
 				BufferItem *		sectBi,
 				int			n,
 				const RowProperties *	rp,
-				TextAttribute		ta )
+				int			textAttributeNumber )
     {
     int				col;
 
@@ -718,7 +824,7 @@ BufferItem * docInsertRowItem(	BufferDocument *	bd,
 	{
 	BufferItem *	paraBi;
 
-	paraBi= docInsertEmptyParagraph( bd, rowBi, ta );
+	paraBi= docInsertEmptyParagraph( bd, rowBi, textAttributeNumber );
 	if  ( ! paraBi )
 	    {
 	    XDEB(paraBi);
@@ -730,7 +836,8 @@ BufferItem * docInsertRowItem(	BufferDocument *	bd,
 	rowBi->biRowHasTableParagraphs= 1;
 	}
 
-    if  ( docCopyRowProperties( &(rowBi->biRowProperties), rp ) )
+    if  ( docCopyRowProperties( &(rowBi->biRowProperties),
+						    rp, (const int *)0 ) )
 	{
 	LDEB(1);
 	docDeleteItem( bd, rowBi );
@@ -740,6 +847,16 @@ BufferItem * docInsertRowItem(	BufferDocument *	bd,
     return rowBi;
     }
 
+/************************************************************************/
+/*									*/
+/*  Step in insert row actions: Copy attributes.			*/
+/*									*/
+/*  NOTE: Copying the text attribute numbers means that the rows MUST	*/
+/*	be in the same document. The same applies for the color numbers	*/
+/*	in the paragraph properties.					*/
+/*									*/
+/************************************************************************/
+
 int docCopyRowColumnAttributes(	BufferItem *		rowBi,
 				const BufferItem *	refRowBi )
     {
@@ -761,6 +878,9 @@ int docCopyRowColumnAttributes(	BufferItem *		rowBi,
 	PropertyMask		ppChgMask;
 	PropertyMask		ppUpdMask;
 
+	const int * const	colorMap= (const int *)0;
+	const int * const	listStyleMap= (const int *)0;
+
 	if  ( docFirstPosition( &dp, rowBi->biChildren[col] ) )
 	    { continue;	}
 	if  ( docFirstPosition( &dpRef, refRowBi->biChildren[col] ) )
@@ -772,13 +892,14 @@ int docCopyRowColumnAttributes(	BufferItem *		rowBi,
 	PROPmaskFILL( &ppUpdMask, PPprop_COUNT );
 
 	if  ( docUpdParaProperties( &ppChgMask, &(dp.dpBi->biParaProperties),
-				&ppUpdMask, &(dpRef.dpBi->biParaProperties) ) )
+				&ppUpdMask, &(dpRef.dpBi->biParaProperties),
+				colorMap, listStyleMap ) )
 	    { LDEB(1); return -1;	}
 
 	tp= dp.dpBi->biParaParticules;
 	tpRef= dpRef.dpBi->biParaParticules;
 
-	tp->tpTextAttribute= tpRef->tpTextAttribute;
+	tp->tpTextAttributeNumber= tpRef->tpTextAttributeNumber;
 	}
 
     return 0;
@@ -825,8 +946,8 @@ static int docSplitGroupItemLow(	const BufferDocument *	bd,
 		return -1;
 		}
 
-	    if  ( docCopyRowProperties( &newBi->biRowProperties,
-						&oldBi->biRowProperties ) )
+	    if  ( docCopyRowProperties( &(newBi->biRowProperties),
+				&(oldBi->biRowProperties), (const int *)0 ) )
 		{ LDEB(1); return -1;	}
 	    break;
 
@@ -963,6 +1084,9 @@ int docNumberOfParagraph(	const BufferItem *	bi )
 
 	if  ( bi->biParent && bi->biNumberInParent > 0 )
 	    {
+	    if  ( bi->biNumberInParent >= bi->biParent->biChildCount )
+		{ LLDEB(bi->biNumberInParent,bi->biParent->biChildCount); }
+
 	    n += bi->biParent->biChildren[
 				bi->biNumberInParent- 1]->biLeftParagraphs;
 	    }
