@@ -15,6 +15,8 @@
 #   undef	y0
 #   undef	y1
 
+#   include	<utilMatchFont.h>
+
 #   include	<appImage.h>
 #   include	"docLayout.h"
 #   include	<appWinMeta.h>
@@ -22,77 +24,6 @@
 
 #   include	<appDebugon.h>
 
-/************************************************************************/
-/*									*/
-/*  Find the position of the next tab.					*/
-/*									*/
-/*  1)  The automatic tab indent implied by the hanging indent.		*/
-/*									*/
-/************************************************************************/
-
-static int docPsTabStop(	const BufferItem *		bi,
-				int				x0Geometry,
-				int				x0TextLines,
-				int				tabInterval,
-				int *				pTab,
-				int *				pX,
-				int				xPosition )
-    {
-    int				tab= -1;
-    const ParagraphProperties *	pp= &(bi->biParaProperties);
-    const TabStop *		ts= pp->ppTabStops;
-
-    /*  1  */
-    if  ( xPosition < x0TextLines			&&
-	  ( pp->ppTabCount == 0			||
-	    ts->tsAlignment != DOCtaLEFT	)	)
-	{ *pX= x0TextLines; *pTab= tab; return 0; }
-
-    if  ( pp->ppTabCount == 0 )
-	{
-	if  ( tabInterval == 0 )
-	    {
-	    LLDEB(pp->ppTabCount,tabInterval);
-	    *pX= xPosition; *pTab= tab; return 0;
-	    }
-	else{
-	    *pTab= tab;
-
-	    xPosition -= x0Geometry;
-	    *pX= ( xPosition+ 1 )/ tabInterval;
-	    *pX *= tabInterval;
-	    *pX += tabInterval;
-	    *pX += x0Geometry;
-	    return 0;
-	    }
-	}
-    else{
-	for ( tab= 0; tab < pp->ppTabCount; ts++, tab++ )
-	    {
-	    if  ( ts->tsTwips > xPosition- x0Geometry )
-		{ break;	}
-	    }
-
-	if  ( tab >= pp->ppTabCount )
-	    {
-	    if  ( tabInterval > 0 )
-		{
-		xPosition -= x0Geometry;
-		*pX= ( xPosition+ 1 )/ tabInterval;
-		*pX *= tabInterval;
-		*pX += tabInterval;
-		*pX += x0Geometry;
-		return 0;
-		}
-	    else{ LDEB(tabInterval); return 1;	}
-	    }
-
-	*pTab= tab;
-	*pX= ts->tsTwips+ x0Geometry;
-	return 0;
-	}
-    }
-
 /************************************************************************/
 /*									*/
 /*  Get descent/ascent for the font of a special particule.		*/
@@ -100,27 +31,33 @@ static int docPsTabStop(	const BufferItem *		bi,
 /************************************************************************/
 
 static int docLayoutSpecialExtents(
+				const BufferDocument *		bd,
 				const BufferItem *		bi,
 				int *				pWordAscent,
 				int *				pWordDescent,
-				AppPhysicalFontList *		apfl,
+				const PostScriptFontList *	psfl,
 				const TextParticule *		tp )
     {
-    AfmFontInfo *	afi;
-    int			encoding;
-    TextAttribute	ta;
+    const DocumentProperties *	dp= &(bd->bdProperties);
+    const DocumentFontList *	dfl= &(dp->dpFontList);
 
-    int			size;
+    AfmFontInfo *		afi;
+    int				encoding;
+    TextAttribute		ta;
 
-    ta= tp->tpTextAttribute;
-    afi= docPsPrintGetAfi( &encoding, apfl, tp->tpPhysicalFont );
+    int				sizeTwips;
+
+    utilGetTextAttributeByNumber( &ta, &(bd->bdTextAttributeList),
+						tp->tpTextAttributeNumber );
+
+    afi= utilPsGetAfi( &encoding, dfl, psfl, &ta );
     if  ( ! afi )
 	{ XDEB(afi); return -1;	}
 
-    size= 10* tp->tpTextAttribute.taFontSizeHalfPoints;
+    sizeTwips= 10* ta.taFontSizeHalfPoints;
 
-    *pWordAscent= ( size* afi->afiFontBBox.abbTop+ 500 ) / 1000;
-    *pWordDescent= *pWordAscent- size;
+    *pWordAscent= ( sizeTwips* afi->afiFontBBox.abbTop+ 500 ) / 1000;
+    *pWordDescent= *pWordAscent- sizeTwips;
 
     return 0;
     }
@@ -132,18 +69,165 @@ static int docLayoutSpecialExtents(
 /*									*/
 /************************************************************************/
 
-static int docLayoutWord(	const BufferItem *		bi,
+static int docLayoutSegmentedStringExtents(
+				AfmBBox *		abbRet,
+				const unsigned char *	s,
+				const int *		segments,
+				int			segmentCount,
+				int			sizeTwips,
+				int			withKerning,
+				int			encoding,
+				const AfmFontInfo *	afi )
+    {
+    int			seg;
+    int			scapsSize= ( 8* sizeTwips )/ 10;
+
+    int			x= 0;
+    int			wide;
+
+    AfmBBox		abb;
+    int			done= 0;
+
+    for ( seg= 0; seg < segmentCount; seg++ )
+	{
+	if  ( segments[2* seg+ 0] > 0 )
+	    {
+	    wide= psCalculateStringExtents( &abb, s, segments[2* seg+ 0],
+				    sizeTwips, withKerning, encoding, afi );
+
+	    if  ( done )
+		{
+		abbRet->abbRight= x+ abb.abbRight;
+
+		if  ( abbRet->abbTop < abb.abbTop )
+		    { abbRet->abbTop=  abb.abbTop;	}
+		if  ( abbRet->abbBottom < abb.abbBottom )
+		    { abbRet->abbBottom=  abb.abbBottom;	}
+		}
+	    else{ *abbRet= abb;	}
+
+	    s += segments[2* seg+ 0];
+	    x += wide;
+	    done= 1;
+	    }
+
+	if  ( segments[2* seg+ 1] > 0 )
+	    {
+	    wide= psCalculateStringExtents( &abb, s, segments[2* seg+ 1],
+				    scapsSize, withKerning, encoding, afi );
+
+	    if  ( done )
+		{
+		abbRet->abbRight= x+ abb.abbRight;
+
+		if  ( abbRet->abbTop < abb.abbTop )
+		    { abbRet->abbTop=  abb.abbTop;	}
+		if  ( abbRet->abbBottom < abb.abbBottom )
+		    { abbRet->abbBottom=  abb.abbBottom;	}
+		}
+	    else{ *abbRet= abb;	}
+
+	    s += segments[2* seg+ 1];
+	    x += wide;
+	    done= 1;
+	    }
+	}
+
+    return x;
+    }
+
+static int docLayoutStringExtents(	int *			pWidth,
+					int *			pDecWidth,
+					AfmBBox *		abb,
+					const TextAttribute *	ta,
+					int			sizeTwips,
+					int			encoding,
+					const BufferDocument *	bd,
+					const AfmFontInfo *	afi,
+					const unsigned char *	printString,
+					int			len )
+    {
+    unsigned char *		upperString= (unsigned char *)0;
+    int *			segments= (int *)0;
+    int				segmentCount= 0;
+
+    int				rval= 0;
+    const int			withKerning= 0;
+
+    int				dec;
+    int				d;
+
+    int				width;
+    int				decWidth;
+
+    if  ( ta->taSmallCaps || ta->taCapitals )
+	{
+	if  ( docMakeCapsString( &upperString, &segments, &segmentCount,
+						bd, ta, printString, len ) )
+	    { LDEB(len); return -1;	}
+
+	printString= upperString;
+	}
+
+    if  ( ta->taSmallCaps && ! ta->taCapitals )
+	{
+	width= docLayoutSegmentedStringExtents( abb, printString,
+				    segments, segmentCount,
+				    sizeTwips, withKerning, encoding, afi );
+	if  ( width < 0 )
+	    { LLDEB(ta->taFontNumber,width);	}
+	}
+    else{
+	width= psCalculateStringExtents( abb, printString, len,
+				    sizeTwips, withKerning, encoding, afi );
+	if  ( width < 0 )
+	    { LLDEB(ta->taFontNumber,width);	}
+	}
+
+    dec= -1;
+    for ( d= 0; d < len; d++ )
+	{
+	if  ( printString[d] == '.' || printString[d] == ','	)
+	    { dec= d;	}
+	}
+
+    if  ( dec >= 0 )
+	{
+	AfmBBox		abbDec;
+
+	decWidth= psCalculateStringExtents( &abbDec, printString, dec,
+				    sizeTwips, withKerning, encoding, afi );
+	if  ( decWidth < 0 )
+	    { LDEB(decWidth);	}
+	}
+    else{ decWidth= width;	}
+
+    *pWidth= width;
+    *pDecWidth= decWidth;
+
+    if  ( upperString )
+	{ free( upperString );	}
+    if  ( segments )
+	{ free( segments );	}
+
+    return rval;
+    }
+
+static int docLayoutWord(	const BufferDocument *		bd,
+				const BufferItem *		bi,
+				int				part,
 				int *				pX1,
 				int *				pVisibleX1,
 				int *				pWordAscent,
 				int *				pWordDescent,
-				const DocumentFontList *	dfl,
-				AppPhysicalFontList *		apfl,
-				const TextParticule *		tp,
+				const PostScriptFontList *	psfl,
 				ParticuleData *			pd,
 				int				particuleCount,
 				int				x0 )
     {
+    const DocumentProperties *	dp= &(bd->bdProperties);
+    const DocumentFontList *	dfl= &(dp->dpFontList);
+
     int				particuleAscent;
     int				particuleDescent;
     int				fontAscent;
@@ -157,66 +241,56 @@ static int docLayoutWord(	const BufferItem *		bi,
 
     int				len= 0;
     int				xFrom= x0;
-    const unsigned char *	from= bi->biParaString+ tp->tpStroff;
 
     int				x1= x0;
     int				visibleX1= x0;
 
-    int				physicalFont= tp->tpPhysicalFont;
+    const TextParticule *	tp= bi->biParaParticules+ part;
+    const unsigned char *	from= bi->biParaString+ tp->tpStroff;
+    int				textAttrNumber= tp->tpTextAttributeNumber;
+
+    TextAttribute		taPr;
+    TextAttribute		taNx;
 
     accepted= 0;
     while( accepted < particuleCount && tp->tpKind == DOCkindTEXT )
 	{
-	AfmFontInfo *	afi;
-	int		encoding;
-	TextAttribute	ta;
+	const AfmFontInfo *	afi;
+	int			encoding;
+	TextAttribute		ta;
 
-	ta= tp->tpTextAttribute;
-	afi= docPsPrintGetAfi( &encoding, apfl, tp->tpPhysicalFont );
+	utilGetTextAttributeByNumber( &ta, &(bd->bdTextAttributeList),
+						tp->tpTextAttributeNumber );
+
+	afi= utilPsGetAfi( &encoding, dfl, psfl, &ta );
 	if  ( ! afi )
-	    { XDEB(afi); return -1;	}
+	    {
+	    LDEB(psfl->psflFamilyCount);
+	    LDEB(docNumberOfParagraph(bi));
+	    LDEB(tp->tpTextAttributeNumber);
+	    SXDEB((char *)bi->biParaString+tp->tpStroff,afi);
 
-	while( accepted < particuleCount		&&
-	       tp->tpPhysicalFont == physicalFont	&&
-	       tp->tpKind == DOCkindTEXT		)
+	    return -1;
+	    }
+
+	while( accepted < particuleCount			&&
+	       tp->tpTextAttributeNumber == textAttrNumber	&&
+	       tp->tpKind == DOCkindTEXT			)
 	    {
 	    AfmBBox		abb;
-	    const int		withKerning= 0;
-	    int			size;
+	    int			sizeTwips;
 
-	    int			dec;
-	    int			d;
+	    sizeTwips= 10* ta.taFontSizeHalfPoints;
 
-	    size= 10* tp->tpTextAttribute.taFontSizeHalfPoints;
-
-	    if  ( tp->tpTextAttribute.taSuperSub == DOCfontSUPERSCRIPT ||
-		  tp->tpTextAttribute.taSuperSub == DOCfontSUBSCRIPT   )
-		{ size= SUPERSUB_SIZE( size ); }
+	    if  ( ta.taSuperSub == DOCfontSUPERSCRIPT ||
+		  ta.taSuperSub == DOCfontSUBSCRIPT   )
+		{ sizeTwips= SUPERSUB_SIZE( sizeTwips ); }
 
 	    len += tp->tpStrlen;
 
-	    width= psCalculateStringExtents( &abb, from, len,
-					    size, withKerning, encoding, afi );
-	    if  ( width < 0 )
-		{ LDEB(width);	}
-
-	    dec= -1;
-	    for ( d= 0; d < len; d++ )
-		{
-		if  ( from[d] == '.' || from[d] == ','	)
-		    { dec= d;	}
-		}
-
-	    if  ( dec >= 0 )
-		{
-		AfmBBox		abbDec;
-
-		decWidth= psCalculateStringExtents( &abbDec, from, dec,
-					    size, withKerning, encoding, afi );
-		if  ( decWidth < 0 )
-		    { LDEB(decWidth);	}
-		}
-	    else{ decWidth= width;	}
+	    if  ( docLayoutStringExtents( &width, &decWidth, &abb,
+			    &ta, sizeTwips, encoding, bd, afi, from, len ) )
+		{ LDEB(1); return -1;	}
 
 	    x1= xFrom+ width;
 	    visibleX1= xFrom+ ( abb.abbRight- abb.abbLeft );
@@ -236,8 +310,8 @@ static int docLayoutWord(	const BufferItem *		bi,
 
 	    particuleAscent= abb.abbTop;
 	    particuleDescent= abb.abbBottom;
-	    fontAscent= ( size* afi->afiFontBBox.abbTop+ 500 ) / 1000;
-	    fontDescent= fontAscent- size;
+	    fontAscent= ( sizeTwips* afi->afiFontBBox.abbTop+ 500 ) / 1000;
+	    fontDescent= fontAscent- sizeTwips;
 
 	    if  ( wordAscent < particuleAscent	&& 
 		  wordAscent < fontAscent	)
@@ -247,21 +321,23 @@ static int docLayoutWord(	const BufferItem *		bi,
 		  wordDescent > fontDescent		)
 		{ wordDescent=  fontDescent; }
 
-	    accepted++; tp++; pd++;
+	    accepted++; part++; tp++; pd++;
 
 	    if  ( ( len > 0 && from[len- 1] == ' ' )	||
 		  accepted >= particuleCount		)
 		{ break;	}
 
-	    if  ( ! tp->tpTextAttribute.taFontIsSlanted	&&
-		  ta.taFontIsSlanted			)
+	    utilGetTextAttributeByNumber( &taNx, &(bd->bdTextAttributeList),
+						tp->tpTextAttributeNumber );
+
+	    if  ( ! taNx.taFontIsSlanted	&&
+		  ta.taFontIsSlanted		)
 		{
 		int		extra;
 
-		if  ( ta.taFontSizeHalfPoints <
-				    tp->tpTextAttribute.taFontSizeHalfPoints )
-		    { extra= ta.taFontSizeHalfPoints;			}
-		else{ extra= tp->tpTextAttribute.taFontSizeHalfPoints;	}
+		if  ( ta.taFontSizeHalfPoints < taNx.taFontSizeHalfPoints )
+		    { extra= ta.taFontSizeHalfPoints;		}
+		else{ extra= taNx.taFontSizeHalfPoints;		}
 
 		extra = (int)( -afi->afiTanItalicAngle* 10* extra );
 
@@ -273,33 +349,45 @@ static int docLayoutWord(	const BufferItem *		bi,
 	      accepted >= particuleCount		)
 	    { break;	}
 
-	ta= tp->tpTextAttribute;
-	physicalFont= tp->tpPhysicalFont;
+	utilGetTextAttributeByNumber( &ta, &(bd->bdTextAttributeList),
+						tp->tpTextAttributeNumber );
+
+	textAttrNumber= tp->tpTextAttributeNumber;
 	from= bi->biParaString+ tp->tpStroff;
 	xFrom= x0;
 	len= 0;
 	}
 
-    if  ( accepted < particuleCount			&&
-	  ! tp->tpTextAttribute.taFontIsSlanted		&&
-	  tp[-1].tpTextAttribute.taFontIsSlanted	)
+    if  ( part < bi->biParaParticuleCount )
 	{
-	int		extra;
-	AfmFontInfo *	afi;
-	int		encoding;
+	utilGetTextAttributeByNumber( &taPr, &(bd->bdTextAttributeList),
+						tp[-1].tpTextAttributeNumber );
+	utilGetTextAttributeByNumber( &taNx, &(bd->bdTextAttributeList),
+						tp[0 ].tpTextAttributeNumber );
 
-	if  ( tp[-1].tpTextAttribute.taFontSizeHalfPoints	<
-	      tp->tpTextAttribute.taFontSizeHalfPoints	)
-	    { extra= tp[-1].tpTextAttribute.taFontSizeHalfPoints; }
-	else{ extra= tp->tpTextAttribute.taFontSizeHalfPoints; }
+	if  ( accepted < particuleCount	&&
+	      ! taNx.taFontIsSlanted	&&
+	      taPr.taFontIsSlanted	)
+	    {
+	    int				extra;
+	    AfmFontInfo *		afi;
+	    int				encoding;
 
-	afi= docPsPrintGetAfi( &encoding, apfl, tp[-1].tpPhysicalFont );
-	if  ( ! afi )
-	    { XDEB(afi); return -1;	}
+	    const TextAttribute *	taExtra;
 
-	extra= (int) ( -afi->afiTanItalicAngle* 10* extra );
+	    if  ( taPr.taFontSizeHalfPoints < taNx.taFontSizeHalfPoints )
+		{ taExtra= &taPr; }
+	    else{ taExtra= &taNx; }
 
-	x1 += extra;
+	    afi= utilPsGetAfi( &encoding, dfl, psfl, taExtra );
+	    if  ( ! afi )
+		{ XDEB(afi); return -1;	}
+
+	    extra= (int) ( -afi->afiTanItalicAngle*
+					10* taExtra->taFontSizeHalfPoints );
+
+	    x1 += extra;
+	    }
 	}
 
     *pWordAscent= wordAscent;
@@ -322,21 +410,23 @@ static int docLayoutWord(	const BufferItem *		bi,
 #   define	PSfoundLINEBREAK	3
 #   define	PSfoundPAGEBREAK	4
 
-static int docPsLayoutText(	const BufferItem *		bi,
+static int docPsLayoutText(	const BufferDocument *		bd,
+				const BufferItem *		bi,
+				int				part,
 				int *				pFound,
 				int *				pWordCount,
 				int *				pX1,
 				int *				pTextAscent,
 				int *				pTextDescent,
-				int				x1TextLines,
-				const DocumentFontList *	dfl,
-				AppPhysicalFontList *		apfl,
-				const TextParticule *		tp,
+				const ParagraphFrame *		pf,
+				const PostScriptFontList *	psfl,
 				ParticuleData *			pd,
 				int				particuleCount,
 				int				acceptAtLeast,
 				int				x0 )
     {
+    int				x1TextLines= pf->pfX1TextLinesTwips;
+
     int				textAscent= 0;
     int				textDescent= 0;
 
@@ -344,6 +434,8 @@ static int docPsLayoutText(	const BufferItem *		bi,
     int				found= PSfoundNOTHING;
     int				wordCount= 0;
 
+    const TextParticule *	tp= bi->biParaParticules+ part;
+
     accepted= 0;
     while( accepted < particuleCount && found == PSfoundNOTHING )
 	{
@@ -359,9 +451,9 @@ static int docPsLayoutText(	const BufferItem *		bi,
 
 	    case DOCkindTEXT:
 	    textCase:
-		done= docLayoutWord( bi, &x1, &visibleX1,
+		done= docLayoutWord( bd, bi, part, &x1, &visibleX1,
 						&wordAscent, &wordDescent,
-						dfl, apfl, tp, pd,
+						psfl, pd,
 						particuleCount- accepted, x0 );
 		if  ( done < 1 )
 		    { LDEB(done); return -1;	}
@@ -382,7 +474,8 @@ static int docPsLayoutText(	const BufferItem *		bi,
 		if  ( textDescent > wordDescent )
 		    { textDescent=  wordDescent;	}
 
-		wordCount++; accepted += done; tp += done; pd += done;
+		wordCount++;
+		accepted += done; part += done; tp += done; pd += done;
 		x0= x1;
 		break;
 
@@ -406,7 +499,7 @@ static int docPsLayoutText(	const BufferItem *		bi,
 		    if  ( accepted < acceptAtLeast )
 			{ acceptAtLeast++;	}
 
-		    accepted++; tp++; pd++;
+		    accepted++; part++; tp++; pd++;
 		    }
 
 		if  ( accepted < particuleCount		&&
@@ -435,7 +528,7 @@ static int docPsLayoutText(	const BufferItem *		bi,
 		pd->pdWhiteUnits= 0;
 		pd->pdCorrectBy= 0;
 
-		accepted++; tp++; pd++; break;
+		accepted++; part++; tp++; pd++; break;
 
 	    case DOCkindOBJECT:
 		{
@@ -444,7 +537,9 @@ static int docPsLayoutText(	const BufferItem *		bi,
 
 		io= bi->biParaObjects+ tp->tpObjectNumber;
 
-		width= ( io->ioScaleX* io->ioTwipsWide )/ 100;
+		docLayoutScaleObjectToFitParagraphFrame( io, pf );
+
+		width= ( io->ioScaleX* io->ioTwipsWide )/ 100.0;
 
 		if  ( x0+ width >= x1TextLines		&&
 		      accepted >= acceptAtLeast		)
@@ -457,7 +552,7 @@ static int docPsLayoutText(	const BufferItem *		bi,
 		    return accepted;
 		    }
 
-		wordAscent= ( io->ioScaleY* io->ioTwipsHigh )/ 100;
+		wordAscent= ( io->ioScaleY* io->ioTwipsHigh )/ 100.0;
 		if  ( textAscent < wordAscent )
 		    { textAscent=  wordAscent;	}
 
@@ -474,10 +569,11 @@ static int docPsLayoutText(	const BufferItem *		bi,
 		pd->pdCorrectBy= 0;
 		x0 += width;
 
-		accepted++; tp++; pd++; break;
+		accepted++; part++; tp++; pd++; break;
 		}
 
 	    case DOCkindCHFTNSEP:
+	    case DOCkindCHFTNSEPC:
 		{
 		int	width= 2880;
 
@@ -492,8 +588,8 @@ static int docPsLayoutText(	const BufferItem *		bi,
 		    return accepted;
 		    }
 
-		if  ( docLayoutSpecialExtents( bi, &wordAscent, &wordDescent,
-								apfl, tp ) )
+		if  ( docLayoutSpecialExtents( bd, bi,
+				    &wordAscent, &wordDescent, psfl, tp ) )
 		    { LDEB(1); return -1;	}
 
 		if  ( textAscent < wordAscent )
@@ -514,11 +610,12 @@ static int docPsLayoutText(	const BufferItem *		bi,
 		pd->pdCorrectBy= 0;
 		x0 += width;
 
-		accepted++; tp++; pd++; break;
+		accepted++; part++; tp++; pd++; break;
 		}
 
 	    case DOCkindLINEBREAK:
 	    case DOCkindPAGEBREAK:
+	    case DOCkindCOLUMNBREAK:
 		pd->pdAfi= (AfmFontInfo *)0;
 		pd->pdX0= x0;
 		pd->pdWidth= 0;
@@ -531,12 +628,12 @@ static int docPsLayoutText(	const BufferItem *		bi,
 		pd->pdWhiteUnits= 0;
 		pd->pdCorrectBy= 0;
 
-		if  ( tp->tpKind == DOCkindPAGEBREAK	&&
+		if  ( tp->tpKind != DOCkindLINEBREAK	&&
 		      ! bi->biParaInTable		)
 		    { found= PSfoundPAGEBREAK;	}
 		else{ found= PSfoundLINEBREAK;	}
 
-		accepted++; tp++; pd++; break;
+		accepted++; part++; tp++; pd++; break;
 
 	    default:
 		LDEB(tp->tpKind); return -1;
@@ -565,21 +662,23 @@ static int docPsLayoutText(	const BufferItem *		bi,
 /*									*/
 /************************************************************************/
 
-static int docLayoutParticules( const BufferItem *		bi,
+static int docLayoutParticules( const BufferDocument *		bd,
+				const BufferItem *		bi,
+				int				part,
 				int *				pFound,
 				int *				pWordCount,
 				int *				pLineAscent,
 				int *				pLineDescent,
-				int				tabInterval,
 				const ParagraphFrame *		pf,
-				const DocumentFontList *	dfl,
-				AppPhysicalFontList *		apfl,
-				const TextParticule *		tp,
+				const PostScriptFontList *	psfl,
 				ParticuleData *			pd,
 				int				particuleCount,
 				int				acceptAtLeast,
 				int				x0 )
     {
+    const DocumentProperties *	dp= &(bd->bdProperties);
+    const DocumentFontList *	dfl= &(dp->dpFontList);
+
     int				lineAscent= 0;
     int				lineDescent= 0;
 
@@ -591,8 +690,11 @@ static int docLayoutParticules( const BufferItem *		bi,
     int				tabX1= 0;
     int				tabParticule= -1;
 
-    const TextParticule *	tpp= tp;
+    const TextParticule *	tpp= bi->biParaParticules+ part;
     ParticuleData *		pdd= pd;
+    TabStop			ts;
+
+    TextAttribute		ta;
 
     accepted= 0;
     while( accepted < particuleCount )
@@ -615,9 +717,9 @@ static int docLayoutParticules( const BufferItem *		bi,
 	if  ( accepted >= acceptAtLeast )
 	    { acceptAtLeast= 0;	}
 
-	done= docPsLayoutText( bi, &found, &textWordCount, &x1,
+	done= docPsLayoutText( bd, bi, part, &found, &textWordCount, &x1,
 				    &textAscent, &textDescent,
-				    pf->pfX1TextLinesTwips, dfl, apfl, tpp, pdd,
+				    pf, psfl, pdd,
 				    particuleCount- accepted,
 				    acceptAtLeast, x0 );
 	if  ( done < 0 )
@@ -732,7 +834,7 @@ static int docLayoutParticules( const BufferItem *		bi,
 	    }
 
 	wordCount += textWordCount;
-	accepted += done; tpp += done; pdd += done;
+	accepted += done; part += done; tpp += done; pdd += done;
 	x0= x1;
 
 	switch( found )
@@ -763,13 +865,17 @@ static int docLayoutParticules( const BufferItem *		bi,
 		LDEB(found); return -1;
 	    }
 
-	afi= docPsPrintGetAfi( &encoding, apfl, tpp->tpPhysicalFont );
+	utilGetTextAttributeByNumber( &ta, &(bd->bdTextAttributeList),
+						tpp->tpTextAttributeNumber );
+
+	afi= utilPsGetAfi( &encoding, dfl, psfl, &ta );
 	if  ( ! afi )
 	    { XDEB(afi); return -1;	}
 
 	tab= -1;
-	if  ( docPsTabStop( bi, pf->pfX0GeometryTwips, pf->pfX0TextLinesTwips,
-						tabInterval, &tab, &x1, x0 ) )
+	if  ( docNextTabStop( &ts, &x1, &tab, &(bi->biParaTabStopList),
+				pf->pfX0GeometryTwips, pf->pfX0TextLinesTwips,
+				dp->dpTabIntervalTwips, x0 ) )
 	    {
 	    if  ( accepted < 1 )
 		{ LLDEB(particuleCount,accepted);	}
@@ -781,8 +887,8 @@ static int docLayoutParticules( const BufferItem *		bi,
 	    }
 
 	if  ( tab >= 0 )
-	    { tabKind= bi->biParaTabStops[tab].tsAlignment;	}
-	else{ tabKind= DOCtaLEFT;			}
+	    { tabKind= ts.tsAlignment;	}
+	else{ tabKind= DOCtaLEFT;	}
 	tabX0= x0;
 	tabX1= x1;
 	tabParticule= accepted;
@@ -802,7 +908,7 @@ static int docLayoutParticules( const BufferItem *		bi,
 	if  ( tabKind == DOCtaLEFT )
 	    { x0= x1;	}
 
-	accepted++; tpp++; pdd++;
+	accepted++; part++; tpp++; pdd++;
 
 	}
 
@@ -962,30 +1068,29 @@ static void docPsJustifyLine(	const BufferItem *	bi,
 /*									*/
 /************************************************************************/
 
-int docLayoutLineBox(	TextLine *			tl,
+int docLayoutLineBox(	const BufferDocument *		bd,
+			TextLine *			tl,
 			const BufferItem *		bi,
 			int				part,
-			int				tabInterval,
-			const DocumentFontList *	dfl,
-			AppPhysicalFontList *		apfl,
-			const TextParticule *		tp,
+			const PostScriptFontList *	psfl,
 			ParticuleData *			pd,
 			const ParagraphFrame *		pf )
     {
-    int			lineAscent;
-    int			lineDescent;
-    int			ascent;
-    int			descent;
-    int			xShift;
+    int				lineAscent;
+    int				lineDescent;
+    int				ascent;
+    int				descent;
+    int				xShift;
 
-    int			accepted;
-    int			found;
-    int			wordCount;
+    int				accepted;
+    int				found;
+    int				wordCount;
 
-    int			x0;
-    int			visibleX1Twips;
+    int				x0;
+    int				visibleX1Twips;
 
-    unsigned char *	from;
+    const TextParticule *	tp= bi->biParaParticules+ part;
+    unsigned char *		from;
 
     if  ( part == 0 )
 	{ x0= pf->pfX0FirstLineTwips;	}
@@ -994,9 +1099,9 @@ int docLayoutLineBox(	TextLine *			tl,
     from= bi->biParaString+ tp->tpStroff;
 
     /*  1  */
-    accepted= docLayoutParticules( bi, &found, &wordCount,
+    accepted= docLayoutParticules( bd, bi, part, &found, &wordCount,
 				    &lineAscent, &lineDescent,
-				    tabInterval, pf, dfl, apfl, tp, pd,
+				    pf, psfl, pd,
 				    bi->biParaParticuleCount- part, 1, x0 );
 
     if  ( accepted <= 0 )
