@@ -15,6 +15,7 @@
 
 #   include	<appUnit.h>
 #   include	"docBuf.h"
+#   include	"docEdit.h"
 
 /************************************************************************/
 /*									*/
@@ -35,6 +36,7 @@ void docInitNote(		DocumentNote *		dn )
     dn->dnStroff= -1;
 
     dn->dnExternalItemKind= DOCinFOOTNOTE;
+    dn->dnAutoNumber= 1;
 
     return;
     }
@@ -58,6 +60,9 @@ void docCleanNote(		BufferDocument *	bd,
 /*	is that of its successor, that usually will be shifted away.	*/
 /*  2)  When the previous position is an empty hole, use it in stead	*/
 /*	of shifting the successors away.				*/
+/*  3)  Make a guess about the note number. If the guess is wrong, it	*/
+/*	fixed later. Fixing can involve a reformat, so making not too	*/
+/*	bad a guess here does help.					*/
 /*									*/
 /************************************************************************/
 
@@ -92,7 +97,8 @@ static int docGetNoteIndex(		int *			pFound,
 int docInsertNote(			DocumentNote **		pDn,
 					BufferDocument *	bd,
 					BufferItem *		paraBi,
-					int			stroff )
+					int			stroff,
+					int			autoNumber )
     {
     int			paraNr;
     int			noteIndex;
@@ -147,10 +153,16 @@ int docInsertNote(			DocumentNote **		pDn,
 	}
 
     docInitNote( fresh+ noteIndex );
-    fresh[noteIndex].dnNoteNumber= noteIndex;	/*  Wrong: fixed later */
+
+    /*  3  */
+    if  ( autoNumber )
+	{ fresh[noteIndex].dnNoteNumber= noteIndex;	}
+    else{ fresh[noteIndex].dnNoteNumber= 0;		}
+
     fresh[noteIndex].dnSectNr= sectBi->biNumberInParent;
     fresh[noteIndex].dnParaNr= paraNr;
     fresh[noteIndex].dnStroff= stroff;
+    fresh[noteIndex].dnAutoNumber= autoNumber;
 
     bd->bdNoteCount++;
 
@@ -191,18 +203,7 @@ int docGetNote(		DocumentNote **		pDn,
 	noteIndex= docGetNoteIndex( &found, bd, paraNr, stroff );
 	if  ( ! found )
 	    {
-#	    if 0
-
-	    int			i;
-	    DocumentNote *	dn;
-
-	    LLLDEB(paraNr,stroff,found);
-	    dn= bd->bdNotes;
-	    for ( i= 0; i < bd->bdNoteCount; dn++, i++ )
-		{ LLLDEB(i,dn->dnParaNr,dn->dnStroff); }
-
-#	    endif
-
+	    /* LLLDEB(paraNr,stroff,found); docListNotes( bd ); */
 	    return -1;
 	    }
 
@@ -385,25 +386,34 @@ void docRenumberNotes(		int *			pChanged,
 	if  ( dn->dnParaNr < 0 )
 	    { continue;	}
 
+	if  ( ! dn->dnAutoNumber )
+	    {
+	    if  ( dn->dnNoteNumber != 0 )
+		{ *pChanged= 1;	}
+
+	    dn->dnNoteNumber= 0;
+	    continue;
+	    }
+
 	/****/
 	np= &(dp->dpFootnoteProperties);
 
-	if  ( np->npRestart == DPftnRST_PER_SECTION	&&
+	if  ( np->npRestart == FTN_RST_PER_SECTION	&&
 	      dn->dnSectNr != sect			)
 	    { footN= 0;	}
 
-	if  ( np->npRestart == DPftnRST_PER_PAGE	&&
+	if  ( np->npRestart == FTN_RST_PER_PAGE	&&
 	      dn->dnReferringPage != page		)
 	    { footN= 0;	}
 
 	/****/
 	np= &(dp->dpEndnoteProperties);
 
-	if  ( np->npRestart == DPftnRST_PER_SECTION	&&
+	if  ( np->npRestart == FTN_RST_PER_SECTION	&&
 	      dn->dnSectNr != sect			)
 	    { endN= 0;	}
 
-	if  ( np->npRestart == DPftnRST_PER_PAGE	&&
+	if  ( np->npRestart == FTN_RST_PER_PAGE	&&
 	      dn->dnReferringPage != page		)
 	    { endN= 0;	}
 
@@ -424,7 +434,7 @@ void docRenumberNotes(		int *			pChanged,
 		break;
 
 	    default:
-		LDEB(dn->dnExternalItemKind);
+		LLDEB(i,dn->dnExternalItemKind);
 		break;
 	    }
 
@@ -440,6 +450,9 @@ void docRenumberNotes(		int *			pChanged,
 /*  Adapt references from notes to the text because the number of	*/
 /*  paragraphs changed in the document.					*/
 /*									*/
+/*  1)  Paranoia check. Skipping the paragraphs before the forst one	*/
+/*	also skips the deleted paragraphs with dn->dnParaNr == -1.	*/
+/*									*/
 /************************************************************************/
 
 void docShiftNoteReferences(		BufferDocument *	bd,
@@ -452,11 +465,17 @@ void docShiftNoteReferences(		BufferDocument *	bd,
     DocumentNote *	dn;
     int			i;
 
+    /*  1  */
+    if  ( paraFrom < 1 )
+	{ LDEB(paraFrom); return;	}
+
     dn= bd->bdNotes;
     for ( i= 0; i < bd->bdNoteCount; dn++, i++ )
 	{
+	/*  1  */
 	if  ( dn->dnParaNr < paraFrom )
 	    { continue;	}
+
 	if  ( dn->dnParaNr > paraFrom )
 	    {
 	    if  ( sectShift == 0 && paraShift == 0 )
@@ -464,6 +483,13 @@ void docShiftNoteReferences(		BufferDocument *	bd,
 
 	    dn->dnSectNr += sectShift;
 	    dn->dnParaNr += paraShift;
+
+	    if  ( dn->dnExternalItem.eiItem )
+		{
+		dn->dnExternalItem.eiItem->
+			biSectSelectionScope.ssSectNrExternalTo= dn->dnSectNr;
+		}
+
 	    continue;
 	    }
 
@@ -474,6 +500,13 @@ void docShiftNoteReferences(		BufferDocument *	bd,
 	    dn->dnSectNr += sectShift;
 	    dn->dnParaNr += paraShift;
 	    dn->dnStroff += stroffShift;
+
+	    if  ( dn->dnExternalItem.eiItem )
+		{
+		dn->dnExternalItem.eiItem->
+			biSectSelectionScope.ssSectNrExternalTo= dn->dnSectNr;
+		}
+
 	    continue;
 	    }
 	}
@@ -500,80 +533,85 @@ void docShiftNoteReferences(		BufferDocument *	bd,
 
 int docMakeNote(	DocumentNote **			pDn,
 			BufferDocument *		bd,
-			const DocumentPosition *	dp,
+			BufferItem *			paraBi,
+			int				part,
+			int				stroff,
 			int				extItKind )
     {
     TextParticule *	tp;
     TextAttribute	ta;
+    int			textAttributeNumberPlain;
+    int			textAttributeNumberSuper;
 
     DocumentNote *	dn;
-    BufferItem *	paraBi;
 
-    DocumentField *	df;
-    int			fieldNumber;
     int			noteIndex;
 
     int			stroffShift= 0;
+    const int		autoNumber= 1;
 
-    if  ( dp->dpBi->biInExternalItem != DOCinBODY )
-	{ LDEB(dp->dpBi->biInExternalItem); return -1;	}
+    BufferItem *	noteParaBi;
+
+    if  ( paraBi->biInExternalItem != DOCinBODY )
+	{ LDEB(paraBi->biInExternalItem); return -1;	}
 
     /*  1  */
-    noteIndex= docInsertNote( &dn, bd, dp->dpBi, dp->dpStroff );
+    noteIndex= docInsertNote( &dn, bd, paraBi, stroff, autoNumber );
     if  ( noteIndex < 0 )
 	{ LDEB(noteIndex); return -1;	}
 
     dn->dnExternalItemKind= extItKind;
 
-    if  ( dp->dpParticule > 0 )
-	{ tp= dp->dpBi->biParaParticules+ dp->dpParticule- 1;	}
-    else{ tp= dp->dpBi->biParaParticules+ dp->dpParticule;	}
+    if  ( part > 0 )
+	{ tp= paraBi->biParaParticules+ part- 1;	}
+    else{ tp= paraBi->biParaParticules+ part;		}
 
-    ta= tp->tpTextAttribute;
+    utilGetTextAttributeByNumber( &ta,
+		&(bd->bdTextAttributeList), tp->tpTextAttributeNumber );
+    ta.taSuperSub= DOCfontREGULAR;
+    textAttributeNumberPlain=
+		utilTextAttributeNumber( &(bd->bdTextAttributeList), &ta );
 
     /*  2  */
-    tp= docInsertTextParticule( dp->dpBi, dp->dpParticule+ 1,
-				    dp->dpStroff, 0, DOCkindNOTE, ta );
+    tp= docInsertTextParticule( paraBi, part+ 1,
+			stroff, 0, DOCkindNOTE, textAttributeNumberPlain );
     if  ( ! tp )
-	{ LDEB(dp->dpBi->biParaParticuleCount); return -1; }
+	{ LDEB(paraBi->biParaParticuleCount); return -1; }
 
     /*  4  */
-    paraBi= docMakeExternalparagraph( bd, &(dn->dnExternalItem),
-				    dp->dpBi, ta, noteIndex, DOCinFOOTNOTE );
-    if  ( ! paraBi )
-	{ XDEB(paraBi); return -1;	}
+    noteParaBi= docMakeExternalparagraph( bd, &(dn->dnExternalItem),
+					paraBi, textAttributeNumberPlain,
+					noteIndex, DOCinFOOTNOTE );
+    if  ( ! noteParaBi )
+	{ XDEB(noteParaBi); return -1;	}
+
+    {
+    TextAttribute	taSet;
+
+    taSet= ta;
+
+    taSet.taSuperSub= DOCfontSUPERSCRIPT;
+
+    textAttributeNumberSuper=
+		utilTextAttributeNumber( &(bd->bdTextAttributeList), &taSet );
+    }
 
     /*  5  */
-    df= docClaimField( &fieldNumber, &(bd->bdFieldList) );
-    if  ( ! df )
-	{ XDEB(df); return -1;	}
-
-    df->dfKind= DOCfkCHFTN;
-
-    tp= docInsertTextParticule( paraBi, 1, 0, 0, DOCkindFIELDEND, ta );
-    if  ( ! tp )
-	{ LDEB(paraBi->biParaParticuleCount); return -1; }
-
-    tp= paraBi->biParaParticules;
-    tp[0].tpKind= DOCkindFIELDSTART;
-    tp[0].tpObjectNumber= fieldNumber;
-    tp[1].tpObjectNumber= fieldNumber;
-
-    tp[0].tpTextAttribute.taSuperSub= DOCfontSUPERSCRIPT;
-    tp[1].tpTextAttribute.taSuperSub= DOCfontSUPERSCRIPT;
+    if  ( docInsertParaHeadField( noteParaBi, bd,
+				    DOCfkCHFTN, textAttributeNumberSuper ) )
+	{ LDEB(1); return -1;	}
 
     /*  6  */
-    if  ( docParaStringReplace( &stroffShift, paraBi,
-				paraBi->biParaStrlen, paraBi->biParaStrlen,
-				(const unsigned char *)" ", 1 ) )
-	{ LDEB(paraBi->biParaStrlen); return -1; }
+    if  ( docParaStringReplace( &stroffShift, noteParaBi,
+			    noteParaBi->biParaStrlen, noteParaBi->biParaStrlen,
+			    (const unsigned char *)" ", 1 ) )
+	{ LDEB(noteParaBi->biParaStrlen); return -1; }
 
-    tp= docInsertTextParticule( paraBi, paraBi->biParaParticuleCount,
-				paraBi->biParaStrlen- 1, 1, DOCkindTEXT, ta );
+    tp= docInsertTextParticule( noteParaBi, noteParaBi->biParaParticuleCount,
+			    noteParaBi->biParaStrlen- 1, 1,
+			    DOCkindTEXT, textAttributeNumberPlain );
     if  ( ! tp )
-	{ LDEB(paraBi->biParaParticuleCount); return -1; }
-
-    bd->bdFieldList.dflFieldCount++;
+	{ LDEB(noteParaBi->biParaParticuleCount); return -1; }
 
     *pDn= dn; return 0;
     }
@@ -603,47 +641,259 @@ int docDeleteNoteOfParticule(		BufferDocument *	bd,
 /*									*/
 /*  Get the note corresponding to a selection.				*/
 /*									*/
+/*  1)  Get the note. Outside the note itself, only notes that are	*/
+/*	exactly the end position of the selection are found. In the	*/
+/*	note itself, the note is returned.				*/
+/*  2)  In body?							*/
+/*  3)  If the selection spans more than one paragraph, it does not	*/
+/*	qualify as the selection of exactly the bote refrence.		*/
+/*  4)  For a range selection, see whether the selection exactly covers	*/
+/*	the note number.						*/
+/*  5)  Look for the beginning of a note field at the beginning of the	*/
+/*	selection.							*/
+/*  6)  Does the same note field end at the end of the selection?	*/
+/*  A)  Look for a note number field that ends at the IBar.		*/
+/*  B)  Look for the beginning of the field.				*/
+/*  a)  Look for a note number field at the beginning of the note item.	*/
+/*  b)  Look for the end of the field.					*/
+/*									*/
 /************************************************************************/
 
+void docNoteGetTextBefore(	unsigned char *		fixedText,
+				int *			pFixedLen,
+				int *			pTextAttrNr,
+				int			fixedTextSize,
+				const BufferItem *	paraBi,
+				int			stroff )
+    {
+    int				part;
+    const TextParticule *	tp= paraBi->biParaParticules+ part;
+
+    if  ( docFindParticule( &part, paraBi, stroff, 1 ) )
+	{ LDEB(stroff); fixedText[0]= '\0'; return;	}
+
+    while( part >= 0 )
+	{
+	if  ( tp->tpStroff+ tp->tpStrlen < stroff )
+	    { break;	}
+
+	if  ( tp->tpKind == DOCkindTEXT		&&
+	      tp->tpStroff < stroff		)
+	    {
+	    int		from= tp->tpStroff;
+	    int		len= tp->tpStrlen;
+
+	    if  ( len > fixedTextSize )
+		{ len= fixedTextSize; from += len- fixedTextSize; }
+
+	    *pFixedLen= len;
+	    *pTextAttrNr= tp->tpTextAttributeNumber;
+
+	    strncpy( (char *)fixedText,
+			(char *)paraBi->biParaString+ from, len )[len]= '\0';
+	    return;
+	    }
+
+	part--; tp--;
+	}
+
+    fixedText[0]= '\0';
+    return;
+    }
+
+void docNoteGetTextAtHead(	unsigned char *		fixedText,
+				int *			pFixedLen,
+				int			fixedTextSize,
+				const BufferItem *	paraBi )
+    {
+    const int			partB= 0;
+    const int			stroff= 0;
+    int				part= partB;
+    const TextParticule *	tp= paraBi->biParaParticules+ part;
+
+    while( part >= 0 )
+	{
+	if  ( tp->tpStroff > stroff )
+	    { break;	}
+
+	if  ( tp->tpKind == DOCkindTEXT			&&
+	      tp->tpStroff+ tp->tpStrlen > stroff	)
+	    {
+	    int		len= tp->tpStrlen;
+
+	    if  ( len > fixedTextSize )
+		{ len= fixedTextSize;	}
+
+	    *pFixedLen= len;
+	    strncpy( (char *)fixedText,
+		(char *)paraBi->biParaString+ tp->tpStroff, len )[len]= '\0';
+	    return;
+	    }
+
+	part++; tp++;
+	}
+
+    *pFixedLen= 0;
+    fixedText[0]= '\0';
+    return;
+    }
+
+int docDelimitNoteReference(	int *			pFieldNr,
+				int *			pPartBegin,
+				int *			pPartEnd,
+				int *			pStroffBegin,
+				int *			pStroffEnd,
+				const BufferItem *	paraBi,
+				int			stroff,
+				const BufferDocument *	bd )
+    {
+    const TextParticule *	tp;
+
+    int				fieldNr= -1;
+    int				partBegin= -1;
+    int				partEnd= -1;
+    int				stroffBegin= -1;
+    int				stroffEnd= -1;
+
+    if  ( docFindParticule( &partEnd, paraBi, stroff, 1 ) )
+	{ LDEB(stroff); return -1;	}
+
+    /*  A  */
+    tp= paraBi->biParaParticules+ partEnd;
+    while( partEnd >= 0			&&
+	   tp->tpStroff == stroff	)
+	{
+	if  ( tp->tpKind == DOCkindFIELDEND )
+	    {
+	    const DocumentField *	df;
+
+	    df= bd->bdFieldList.dflFields+ tp->tpObjectNumber;
+	    if  ( df->dfKind == DOCfkCHFTN )
+		{
+		fieldNr= tp->tpObjectNumber;
+		stroffEnd= tp->tpStroff;
+		break;
+		}
+	    }
+
+	partEnd--; tp--;
+	}
+
+    /*  B  */
+    if  ( fieldNr >= 0 )
+	{
+	partBegin= partEnd;
+	partBegin--; tp--;
+
+	while( partBegin >= 0 )
+	    {
+	    if  ( tp->tpKind == DOCkindFIELDSTART	&&
+		  tp->tpObjectNumber == fieldNr		)
+		{
+		stroffBegin= tp->tpStroff;
+		break;
+		}
+
+	    partBegin--; tp--;
+	    }
+
+	if  ( partBegin < 0 )
+	    { fieldNr= -1;	}
+	}
+
+    if  ( fieldNr < 0 )
+	{ return 1;	}
+
+    *pFieldNr= fieldNr;
+    *pPartBegin= partBegin;
+    *pPartEnd= partEnd;
+    *pStroffBegin= stroffBegin;
+    *pStroffEnd= stroffEnd;
+
+    return 0;
+    }
+
 int docGetSelectedNote(		DocumentNote **			pDn,
+				int *				pFieldNr,
+				unsigned char *			fixedText,
+				int				fixedTextSize,
 				BufferDocument *		bd,
 				const DocumentSelection *	ds )
     {
     DocumentNote *		dn;
     int				noteIndex;
 
+    const BufferItem *		paraBi;
+
+    int				fieldNr= -1;
+    int				partBegin= -1;
+    int				partEnd= -1;
+    int				stroffBegin= -1;
+    int				stroffEnd= -1;
+
+    /*  1  */
     noteIndex= docGetNote( &dn, bd, ds->dsEnd.dpBi, ds->dsEnd.dpStroff );
     if  ( noteIndex < 0 )
 	{ return noteIndex;	}
 
-    if  ( ds->dsSelectionScope.ssInExternalItem != DOCinFOOTNOTE	&&
-	  ds->dsSelectionScope.ssInExternalItem != DOCinENDNOTE		&&
-	  ! docIsIBarSelection( ds )					)
+    /*  2  */
+    if  ( ds->dsSelectionScope.ssInExternalItem == DOCinFOOTNOTE	||
+	  ds->dsSelectionScope.ssInExternalItem == DOCinENDNOTE		)
 	{
-	const TextParticule *		tpBegin;
-	const TextParticule *		tpEnd;
-	const DocumentField *		dfBegin;
+	DocumentPosition	dpBegin;
 
+	if  ( docFirstPosition( &dpBegin, dn->dnExternalItem.eiItem ) )
+	    { LDEB(1); return -1;	}
+
+	paraBi= dpBegin.dpBi;
+
+	if  ( docDelimitParaHeadField( &fieldNr, &partBegin, &partEnd,
+				    &stroffBegin, &stroffEnd, paraBi, bd ) )
+	    {
+	    if  ( fixedTextSize > 0 )
+		{
+		int	fixedLen= 0;
+
+		docNoteGetTextAtHead( fixedText, &fixedLen, fixedTextSize,
+								    paraBi );
+		}
+	    }
+	else{
+	    if  ( fixedTextSize > 0 )
+		{ fixedText[0]= '\0'; }
+	    }
+	}
+    else{
+	paraBi= ds->dsBegin.dpBi;
+
+	/*  3  */
 	if  ( ! docIsParaSelection( ds ) )
 	    { return -1;	}
 
-	tpBegin= ds->dsBegin.dpBi->biParaParticules+ ds->dsBegin.dpParticule;
-	if  ( tpBegin->tpKind != DOCkindFIELDSTART )
-	    { return -1;	}
+	if  ( ! docDelimitNoteReference( &fieldNr, &partBegin, &partEnd,
+		&stroffBegin, &stroffEnd, paraBi, ds->dsEnd.dpStroff, bd ) )
+	    {
+	    if  ( ! docIsIBarSelection( ds )		&&
+		  ds->dsBegin.dpStroff != stroffBegin	)
+		{ return 1;	}
+	    }
 
-	dfBegin= bd->bdFieldList.dflFields+ tpBegin->tpObjectNumber;
-	if  ( dfBegin->dfKind != DOCfkCHFTN )
-	    { return -1;	}
+	if  ( fixedTextSize > 0 )
+	    {
+	    if  ( fieldNr >= 0 )
+		{ fixedText[0]= '\0'; }
+	    else{
+		int	fixedLen= 0;
+		int	textAttrNr;
 
-	tpEnd= ds->dsEnd.dpBi->biParaParticules+ ds->dsEnd.dpParticule;
-	if  ( tpEnd->tpKind != DOCkindFIELDEND )
-	    { return -1;	}
-
-	if  ( tpEnd->tpObjectNumber != tpBegin->tpObjectNumber )
-	    { return -1;	}
+		docNoteGetTextBefore( fixedText, &fixedLen, &textAttrNr,
+				fixedTextSize, paraBi, ds->dsEnd.dpStroff );
+		}
+	    }
 	}
 
     *pDn= dn;
+    *pFieldNr= fieldNr;
     return noteIndex;
     }
 
@@ -652,12 +902,33 @@ int docGetSelectedNote(		DocumentNote **			pDn,
 /*  Verify that a certain kind of notes separator exists. Make it if	*/
 /*  not.								*/
 /*									*/
+/*  1)  Correctly handeled by docTextAttributeNumber().			*/
+/*									*/
 /************************************************************************/
 
 int docCheckNoteSeparatorItem(		BufferDocument *	bd,
 					int			extItKind )
     {
     ExternalItem *	ei;
+    int			particuleKind;
+
+    switch( extItKind )
+	{
+	case DOCinFTNSEP:
+	case DOCinAFTNSEP:
+	    particuleKind= DOCkindCHFTNSEP;
+	    break;
+
+	case DOCinFTNSEPC:
+	case DOCinAFTNSEPC:
+	    particuleKind= DOCkindCHFTNSEPC;
+	    break;
+
+	default:
+	    LDEB(extItKind);
+	    particuleKind= DOCkindCHFTNSEP;
+	    break;
+	}
 
     ei= docDocumentNoteSeparator( bd, extItKind );
     if  ( ! ei )
@@ -666,15 +937,20 @@ int docCheckNoteSeparatorItem(		BufferDocument *	bd,
     if  ( ! ei->eiItem )
 	{
 	const int		noteIndex= -1;
+	int			attributeNumber;
 
 	BufferItem *	bi;
 	TextAttribute	ta;
 
-	docInitTextAttribute( &ta );
-	ta.taFontNumber= 0;
+	utilInitTextAttribute( &ta );
+	ta.taFontNumber= -1; /*  1  */
 	ta.taFontSizeHalfPoints= 24;
 
-	bi= docMakeExternalparagraph( bd, ei, &(bd->bdItem), ta,
+	attributeNumber= docTextAttributeNumber( bd, &ta );
+	if  ( attributeNumber < 0 )
+	    { LDEB(attributeNumber); return -1;	}
+
+	bi= docMakeExternalparagraph( bd, ei, &(bd->bdItem), attributeNumber,
 						    noteIndex, extItKind );
 	if  ( ! bi )
 	    { XDEB(bi);	}
@@ -684,7 +960,7 @@ int docCheckNoteSeparatorItem(		BufferDocument *	bd,
 	    else{
 		TextParticule *	tp= bi->biParaParticules;
 
-		tp->tpKind= DOCkindCHFTNSEP;
+		tp->tpKind= particuleKind;
 		tp->tpStrlen= 1;
 		bi->biParaStrlen++;
 		}
@@ -694,3 +970,25 @@ int docCheckNoteSeparatorItem(		BufferDocument *	bd,
     return 0;
     }
 
+/************************************************************************/
+/*									*/
+/*  Return the position in the document body where a note is		*/
+/*  referenced.								*/
+/*									*/
+/************************************************************************/
+
+int docGetNotePosition(		DocumentPosition *		dp,
+				BufferDocument *		bd,
+				const DocumentNote *		dn )
+    {
+    BufferItem *	paraBi;
+
+    paraBi= docGetParagraphByNumber( &(bd->bdItem), dn->dnParaNr );
+    if  ( ! paraBi )
+	{ LXDEB(dn->dnParaNr,paraBi); return -1;	}
+
+    if  ( docSetDocumentPosition( dp, paraBi, dn->dnStroff ) )
+	{ LLDEB(paraBi->biParaStrlen,dn->dnStroff); return -1;	}
+
+    return 0;
+    }
