@@ -52,8 +52,12 @@ static int docPlainSaveParaItem(	SimpleOutputStream *		sos,
 
     if  ( ds && ds->dsBegin.dpBi == bi )
 	{
-	part= ds->dsBegin.dpParticule;
+	const int	lastOne= 1;
+
 	stroff= ds->dsBegin.dpStroff;
+
+	if  ( docFindParticuleOfPosition( &part, &(ds->dsBegin), lastOne ) )
+	    { LDEB(stroff); return -1;	}
 	}
 
     tp= bi->biParaParticules+ part;
@@ -65,6 +69,11 @@ static int docPlainSaveParaItem(	SimpleOutputStream *		sos,
 
     while( stroff < upto && part < bi->biParaParticuleCount )
 	{
+	TextAttribute		ta;
+
+	utilGetTextAttributeByNumber( &ta, &(bd->bdTextAttributeList),
+						tp->tpTextAttributeNumber );
+
 	switch( tp->tpKind )
 	    {
 	    case DOCkindTAB:
@@ -88,11 +97,26 @@ static int docPlainSaveParaItem(	SimpleOutputStream *		sos,
 			}
 		    }
 
-		if  ( ! tp->tpTextAttribute.taIsUnderlined )
+		if  ( ! ta.taTextIsUnderlined )
 		    { paragraphUnderlined= 0;	}
 
 		while( stroff < upto && stroff < tp->tpStroff+ tp->tpStrlen )
-		    { sioOutPutCharacter( *s, sos ); s++; stroff++; pos++; }
+		    {
+		    if  ( ! paragraphUnderlined && ta.taTextIsUnderlined )
+			{
+			sioOutPutCharacter( '_', sos );
+			sioOutPutCharacter( '\b', sos );
+			}
+
+		    if  ( ta.taFontIsBold )
+			{
+			sioOutPutCharacter( *s, sos );
+			sioOutPutCharacter( '\b', sos );
+			}
+
+		    sioOutPutCharacter( *s, sos );
+		    s++; stroff++; pos++;
+		    }
 
 		break;
 
@@ -103,6 +127,7 @@ static int docPlainSaveParaItem(	SimpleOutputStream *		sos,
 		break;
 
 	    case DOCkindPAGEBREAK:
+	    case DOCkindCOLUMNBREAK:
 		sioOutPutCharacter( '\n', sos );
 		sioOutPutCharacter( '\f', sos );
 		pos= 0; paragraphUnderlined= 0;
