@@ -37,10 +37,27 @@ typedef struct RtfBookmarkLevel
     struct RtfBookmarkLevel *	rblPrevious;
     } RtfBookmarkLevel;
 
+typedef struct RtfReadingState
+    {
+    int				rrsBytesPerUnicode;
+    int				rrsUnicodeBytesToSkip;
+
+    TextAttribute		rrsTextAttribute;
+    ParagraphProperties		rrsParagraphProperties;
+    SectionProperties		rrsSectionProperties;
+
+    MemoryBuffer		rrsSavedText;
+
+    struct RtfReadingState *	rrsPrev;
+    } RtfReadingState;
+
 typedef struct RtfReadingContext
     {
+    RtfReadingState *		rrcState;
+
     int				rrcLevel;
     int				rrcSplitLevel;
+    int				rrcInIgnoredGroup;
 				/****************************************/
 				/*  The depth in the document		*/
 				/*  == Type of BufferItem.		*/
@@ -50,6 +67,7 @@ typedef struct RtfReadingContext
 				/*  Complain about unknown control	*/
 				/****************************************/
     int				rrcCharacterAhead;
+    int				rrcAfterNoteref;
 				/****************************************/
 				/*  Character that could not be unread	*/
 				/*  using ungetc().			*/
@@ -60,7 +78,6 @@ typedef struct RtfReadingContext
 				/****************************************/
 				/*  The current position in the input.	*/
 				/****************************************/
-    TextAttribute		rrcCurrentTextAttribute;
     int				rrcInDeletedText;
     BorderProperties		rrcTextBorderProperties;
     int				rrcTextShadingPattern;
@@ -70,15 +87,14 @@ typedef struct RtfReadingContext
 				/****************************************/
 				/*  Text attributes of the current pos.	*/
 				/****************************************/
-    TextFrameProperties		rrcTextFrameProperties;
-    ParagraphProperties		rrcParagraphProperties;
+    FramePosition		rrcParaFramePosition;
+    FramePosition		rrcRowFramePosition;
     CellProperties		rrcCellProperties;
     RowProperties		rrcRowProperties;
-    SectionProperties		rrcSectionProperties;
     BorderProperties		rrcBorderProperties;
     ShapeProperties		rrcShapeProperties;
     DrawingShape		rrcShape;
-    DocumentStyle		rrcStyle;
+    DocumentStyle		rrcDocumentStyle;
     TabStop			rrcTabStop;
     RGB8Color			rrcColor;
     int				rrcGotComponent;
@@ -92,6 +108,10 @@ typedef struct RtfReadingContext
     int				rrcParagraphNumberCount;
 
     DocumentList		rrcDocumentList;
+    DocumentListLevel		rrcDocumentListLevel;
+
+    ListOverride		rrcListOverride;
+    ListOverrideLevel		rrcListOverrideLevel;
 
 				/****************************************/
 				/*  Document properties.		*/
@@ -112,12 +132,12 @@ typedef struct RtfReadingContext
 				/*  For reading 'fields'.		*/
 				/****************************************/
     int				rrcJustAfterPntext;
-    int				rrcPageAfterPntext;
 				/****************************************/
 				/*  For coping with the way word saves	*/
 				/*  {\pntext ... }			*/
 				/****************************************/
-    unsigned char		rrcInputMapping[256];
+    unsigned char		rrcDefaultInputMapping[256];
+    const unsigned char *	rrcTextInputMapping;
     } RtfReadingContext;
 
 
@@ -149,28 +169,29 @@ typedef struct PushedAttribute
 
 typedef struct RtfWritingContext
     {
-    TextAttribute	rwcTextAttribute;
-    ParagraphProperties	rwcParagraphProperties;
-    RowProperties	rwcRowProperties;
+    TextAttribute		rwcTextAttribute;
+    ParagraphProperties		rwcParagraphProperties;
+    RowProperties		rwcRowProperties;
 
-    TextAttribute	rwcOutsideTableTextAttribute;
-    ParagraphProperties	rwcOutsideTableParagraphProperties;
+    TextAttribute		rwcOutsideTableTextAttribute;
+    ParagraphProperties		rwcOutsideTableParagraphProperties;
 
-    PushedAttribute *	rwcOutsideFldrsltAttribute;
-    int			rwcInFldrslt;
-    int			rwcInTable;
+    PushedAttribute *		rwcOutsideFldrsltAttribute;
+    int				rwcInFldrslt;
+    int				rwcInTable;
 
-    int			rwcHasPrecedingTags;
-    int			rwcSaveBookmarks;
+    int				rwcHasPrecedingTags;
+    int				rwcSaveBookmarks;
 
-    int			rwcSaveAsLink;
-    int			rwcSaveAsLinkAsRef;
-    int			rwcSaveAsLinkAsPageref;
-    DocumentField	rwcSaveAsHyperlinkField;
-    DocumentField	rwcSaveAsRefField;
-    DocumentField	rwcSaveAsPagerefField;
+    int				rwcSaveAsLink;
+    int				rwcSaveAsLinkAsRef;
+    int				rwcSaveAsLinkAsPageref;
+    DocumentField		rwcSaveAsHyperlinkField;
+    DocumentField		rwcSaveAsRefField;
+    DocumentField		rwcSaveAsPagerefField;
 
-    unsigned char	rwcOutputMapping[256];
+    unsigned char		rwcDefaultOutputMapping[256];
+    const unsigned char *	rwcTextOutputMapping;
     } RtfWritingContext;
 
 /************************************************************************/
@@ -180,7 +201,7 @@ typedef struct RtfWritingContext
 typedef struct RtfControlWord
     {
     char *		rcwWord;
-    RtfTag		rcwId;
+    RtfTag		rcwID;
     ItemLevel		rcwLevel;
     int			(*rcwApply)( SimpleInputStream *		sis,
 				    const struct RtfControlWord *	rcw,
@@ -200,6 +221,8 @@ typedef int (*RtfAddTextParticule)(	RtfReadingContext *		rrc,
 					const unsigned char *		text,
 					int				len );
 
+typedef int (*RtfCommitGroup)(		RtfReadingContext *		rrc );
+
 /************************************************************************/
 /*  Common data.							*/
 /************************************************************************/
@@ -207,10 +230,12 @@ typedef int (*RtfAddTextParticule)(	RtfReadingContext *		rrc,
 extern RtfControlWord	docRtfEmptyTable[];
 extern RtfControlWord	docRtfDocumentWords[];
 extern RtfControlWord	docRtfDocumentGroups[];
-extern RtfControlWord	docRtfPropertyWords[];
+extern RtfControlWord	docRtfListLevelGroups[];
 
 /************************************************************************/
+/*									*/
 /*  Routine declarations.						*/
+/*									*/
 /************************************************************************/
 
 extern void docRtfInitReadingContext(	RtfReadingContext *		rrc );
@@ -241,7 +266,9 @@ extern int docRtfReadGroup(	SimpleInputStream *	sis,
 
 				const RtfControlWord *	contolWords,
 				const RtfControlWord *	groupWords,
-				RtfAddTextParticule	addParticule	);
+
+				RtfAddTextParticule	addParticule,
+				RtfCommitGroup		commitGroup );
 
 extern int docRtfFindControl(	SimpleInputStream *	sis,
 				RtfReadingContext *	rrc,
@@ -255,17 +282,27 @@ extern int docRtfIgnoreWord(	SimpleInputStream *	sis,
 				int			arg,
 				RtfReadingContext *	rrc	);
 
-extern int docRtfIgnoreParticule(	RtfReadingContext *	rrc,
-					const unsigned char *	text,
-					int			len	);
-
 extern int docRtfIgnoreText(	RtfReadingContext *	rrc,
 				const unsigned char *	text,
-				int			len	);
+				int			len );
 
 extern int docRtfRefuseText(	RtfReadingContext *	rrc,
 				const unsigned char *	text,
-				int			len	);
+				int			len );
+
+extern int docRtfSaveText(	RtfReadingContext *	rrc,
+				const unsigned char *	text,
+				int			len );
+
+extern int docRtfRememberText(	char **			pTarget,
+				int *			pSize,
+				RtfReadingContext *	rrc,
+				int			removeSemicolon );
+
+extern int docRtfRememberUText(	unsigned char **	pTarget,
+				int *			pSize,
+				RtfReadingContext *	rrc,
+				int			removeSemicolon );
 
 extern int docRtfSkipGroup(	SimpleInputStream *	sis,
 				const RtfControlWord *	groupRcw,
@@ -274,12 +311,10 @@ extern int docRtfSkipGroup(	SimpleInputStream *	sis,
 
 extern int docRtfConsumeGroup(	SimpleInputStream *	sis,
 				int			textLevel,
-
-				RtfReadingContext *	internRrc,
-
+				RtfReadingContext *	rrc,
 				const RtfControlWord *	contolWords,
 				const RtfControlWord *	groupWords,
-				RtfAddTextParticule	addParticule	);
+				RtfAddTextParticule	addParticule );
 
 extern int docRtfReadPict(	SimpleInputStream *	sis,
 				const RtfControlWord *	rcw,
@@ -301,9 +336,15 @@ extern int docRtfReadObject(	SimpleInputStream *	sis,
 				int			arg,
 				RtfReadingContext *	rrc );
 
+extern int docRtfReadDrawingObject(
+				SimpleInputStream *	sis,
+				const RtfControlWord *	rcw,
+				int			arg,
+				RtfReadingContext *	rrc );
+
 extern int docRtfAdjustLevel(	RtfReadingContext *	rrc,
 				int			toLevel,
-				int			textLevel	);
+				int			textLevel );
 
 extern int docRtfReadShape(	SimpleInputStream *	sis,
 				const RtfControlWord *	rcw,
@@ -319,7 +360,12 @@ extern int docRtfReadField(	SimpleInputStream *	sis,
 				int			arg,
 				RtfReadingContext *	rrc );
 
-extern int docRtfReadLookupEntry(SimpleInputStream *	sis,
+extern int docRtfReadTextField(	SimpleInputStream *	sis,
+				const RtfControlWord *	rcw,
+				int			arg,
+				RtfReadingContext *	rrc );
+
+extern int docRtfLookupEntry(	SimpleInputStream *	sis,
 				const RtfControlWord *	rcw,
 				int			arg,
 				RtfReadingContext *	rrc );
@@ -334,11 +380,6 @@ extern int docRtfReadFootnote(	SimpleInputStream *	sis,
 				int			arg,
 				RtfReadingContext *	rrc );
 
-extern int docRtfChftn(		SimpleInputStream *	sis,
-				const RtfControlWord *	rcw,
-				int			arg,
-				RtfReadingContext *	rrc );
-
 extern int docRtfBkmkStart(	SimpleInputStream *	sis,
 				const RtfControlWord *	rcw,
 				int			arg,
@@ -349,7 +390,7 @@ extern int docRtfBkmkEnd(	SimpleInputStream *	sis,
 				int			arg,
 				RtfReadingContext *	rrc );
 
-extern int docRtfSpecialToField(	SimpleInputStream *	sis,
+extern int docRtfTextSpecialToField(	SimpleInputStream *	sis,
 					const RtfControlWord *	rcw,
 					int			arg,
 					RtfReadingContext *	rrc );
@@ -383,7 +424,7 @@ extern int docRtfStylesheet(		SimpleInputStream *	sis,
 					int			arg,
 					RtfReadingContext *	rrc );
 
-extern int docRtfListtable(		SimpleInputStream *	sis,
+extern int docRtfListTable(		SimpleInputStream *	sis,
 					const RtfControlWord *	rcw,
 					int			arg,
 					RtfReadingContext *	rrc );
@@ -413,6 +454,14 @@ extern void docRtfEscapeString(		const unsigned char *	s,
 					int			n,
 					SimpleOutputStream *	sos );
 
+extern void docRtfWriteStringDestination(
+					const char *		tag,
+					int *			pCol,
+					const unsigned char *	s,
+					int			n,
+					int			addSemicolon,
+					SimpleOutputStream *	sos );
+
 extern void docRtfWriteNextLine(	int *			pCol,
 					SimpleOutputStream *	sos );
 
@@ -508,6 +557,17 @@ extern int docRtfRememberTextProperty(	SimpleInputStream *	sis,
 					int			arg,
 					RtfReadingContext *	rrc );
 
+extern int docRtfRememberPntextProperty(SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
+extern int docRtfRememberTextShadingPattern(
+					SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
 extern void docRtfWriteArgDestinationBegin( const char *	tag,
 					int *			pCol,
 					int			arg,
@@ -521,13 +581,19 @@ extern int docRtfSaveDocumentProperties( SimpleOutputStream *	sos,
 extern void docRtfReadSetAnsicpg(	RtfReadingContext *	rrc,
 					int			arg );
 
-extern void docRtfSaveTextFrameProperties(
+extern void docRtfSaveParaFrameProperties(
 				SimpleOutputStream *		sos,
 				int *				pCol,
 				const PropertyMask *		updMask,
-				const TextFrameProperties *	tfp );
+				const FramePosition *		fp );
 
-extern int docRtfRememberTextFrameProperty(
+extern int docRtfRememberParaFrameProperty(
+					SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
+extern int docRtfRememberRowFrameProperty(
 					SimpleInputStream *	sis,
 					const RtfControlWord *	rcw,
 					int			arg,
@@ -551,4 +617,156 @@ extern int docRtfSaveDocNotesSeparators(SimpleOutputStream *	sos,
 					const BufferDocument *	bd,
 					RtfWritingContext *	rwc );
 
+extern void docRtfSaveCellProperties(	const CellProperties *	cp,
+					int *			pCol,
+					int			shiftLeft,
+					SimpleOutputStream *	sos );
+
+extern int docRtfRememberCellProperty(	SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
+extern void docRtfSaveRowProperties(	const RowProperties *	rp,
+					int *			pCol,
+					int			col0,
+					int			col1,
+					SimpleOutputStream *	sos );
+
+extern int docRtfRememberRowProperty(	SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
+extern void docRtfSaveAutoSpace(	int *			pCol,
+					SimpleOutputStream *	sos,
+					const char *		unitTag,
+					int			unit,
+					const char *		numberTag,
+					int			number );
+
+extern int docRtfRememberStyleProperty(	SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
+extern int docRtfRememberListLevelProperty(
+					SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
+extern int docRtfRememberListProperty(	SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
+extern void docRtfWriteListTable(
+				SimpleOutputStream *		sos,
+				const unsigned char *		outputMapping,
+				int *				pCol,
+				const DocumentListTable *	dlt );
+
+extern void docRtfPushReadingState(	RtfReadingContext *	rrc,
+					RtfReadingState *	rrs );
+
+extern void docRtfPopReadingState(	RtfReadingContext *	rrc );
+
+extern int docRtfRememberListOverrideLevelProperty(
+					SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
+extern int docRtfRememberListOverrideProperty(
+					SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
+extern int docRtfListOverrideTable(	SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
+extern void docRtfWriteListOverrideTable(
+				SimpleOutputStream *		sos,
+				const unsigned char *		outputMapping,
+				int *				pCol,
+				const ListOverrideTable *	lot );
+
+extern void docRtfWriteListLevel(
+				SimpleOutputStream *		sos,
+				const unsigned char *		outputMapping,
+				int *				pCol,
+				const DocumentListLevel *	dll );
+
+extern int docRtfRememberTabStopProperty(
+				SimpleInputStream *		sis,
+				const RtfControlWord *		rcw,
+				int				arg,
+				RtfReadingContext *		rrc );
+
+extern void docRtfSaveTabStopList(
+				SimpleOutputStream *		sos,
+				int *				pCol,
+				const TabStopList *		tsl );
+
+extern int docRtfFontProperty(	SimpleInputStream *		sis,
+				const RtfControlWord *		rcw,
+				int				arg,
+				RtfReadingContext *		rrc );
+
+extern unsigned long docRtfTagHash(	const unsigned char *	key,
+					int			len );
+
+extern int docRtfTagIndex(		const unsigned char *	key,
+					int			len );
+
+extern const RtfControlWord * docRtfFindPropertyWord(
+					const char *		controlWord );
+
+extern int docRtfObjectProperty(	SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
+extern int docRtfCommitListLevelStyle(	RtfReadingContext *	rrc );
+
+extern int docRtfTextSpecialChar(	SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
+extern int docRtfTextSpecialParticule(	SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
+extern int docRtfDrawingObjectProperty(	SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
+extern int docRtfDrawingObjectCoordinate(
+					SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
+extern int docRtfDocInfoGroup(		SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
+extern int docRtfShpProperty(		SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
+extern int docRtfPnProperty(		SimpleInputStream *	sis,
+					const RtfControlWord *	rcw,
+					int			arg,
+					RtfReadingContext *	rrc );
+
 #    endif	/*  DOC_RTF_H	*/
