@@ -1,6 +1,7 @@
 /************************************************************************/
 /*									*/
 /*  Read an RTF text file into a BufferDocument				*/
+/*  Read fields, bookmarks, footnotes, headers, footers.		*/
 /*									*/
 /************************************************************************/
 
@@ -31,7 +32,7 @@ static int docRtfReadFldrslt(	SimpleInputStream *	sis,
     res= docRtfReadGroup( sis, DOClevPARA,
 				(RtfControlWord *)0, 0, 0, rrc,
 				docRtfDocumentWords, docRtfDocumentGroups,
-				docRtfTextParticule );
+				docRtfTextParticule, (RtfCommitGroup)0 );
 
     if  ( res )
 	{ SLDEB(rcw->rcwWord,res);	}
@@ -67,7 +68,7 @@ static int docRtfReadFldinst(	SimpleInputStream *	sis,
     res= docRtfReadGroup( sis, DOClevPARA,
 				(RtfControlWord *)0, 0, 0, rrc,
 				docRtfDocumentWords, docRtfDocumentGroups,
-				docRtfFldinst );
+				docRtfFldinst, (RtfCommitGroup)0 );
 
     if  ( res )
 	{ SLDEB(rcw->rcwWord,res);	}
@@ -97,11 +98,11 @@ static int docRtfReadStartField(	int *			pPart,
 					int *			pFieldNumber,
 					RtfReadingContext *	rrc	)
     {
+    RtfReadingState *	rrs= rrc->rrcState;
     BufferDocument *	bd= rrc->rrcBd;
     BufferItem *	bi= rrc->rrcBi;
 
     DocumentField *	df;
-    TextParticule *	tp;
 
     int			part= bi->biParaParticuleCount;
 
@@ -113,12 +114,14 @@ static int docRtfReadStartField(	int *			pPart,
 	{ XDEB(df); return -1;	}
     df->dfKind= DOCfkUNKNOWN;
 
-    tp= docInsertTextParticule( bi, -1, bi->biParaStrlen, 0,
-			DOCkindFIELDSTART, rrc->rrcCurrentTextAttribute );
-    if  ( ! tp )
-	{ LDEB(bi->biParaParticuleCount); return -1;	}
+    if  ( docInsertAdminParticule( bd, bi, bi->biParaStrlen,
+					    bi->biParaParticuleCount,
+					    rrc->rrcFieldNumber,
+					    &(rrs->rrsTextAttribute),
+					    DOCkindFIELDSTART ) )
+	{ LDEB(1); return -1;	}
 
-    tp->tpObjectNumber= rrc->rrcFieldNumber;
+    rrc->rrcAfterNoteref= 0;
 
     *pFieldNumber= rrc->rrcFieldNumber;
     *pPart= part;
@@ -130,16 +133,20 @@ static int docRtfTerminateField(	BufferItem *		bi,
 					int			fieldNumber,
 					RtfReadingContext *	rrc )
     {
-    TextParticule *	tp;
+    RtfReadingState *	rrs= rrc->rrcState;
+    BufferDocument *	bd= rrc->rrcBd;
 
     if  ( rrc->rrcBi != bi )
 	{ XXDEB(rrc->rrcBi,bi); }
 
-    tp= docInsertTextParticule( bi, -1, bi->biParaStrlen, 0,
-			DOCkindFIELDEND, rrc->rrcCurrentTextAttribute );
-    if  ( ! tp )
-	{ LDEB(bi->biParaParticuleCount); return -1;	}
-    tp->tpObjectNumber= fieldNumber;
+    if  ( docInsertAdminParticule( bd, bi, bi->biParaStrlen,
+					    bi->biParaParticuleCount,
+					    fieldNumber,
+					    &(rrs->rrsTextAttribute),
+					    DOCkindFIELDEND ) )
+	{ LDEB(1); return -1;	}
+
+    rrc->rrcAfterNoteref= 0;
 
     return 0;
     }
@@ -205,7 +212,7 @@ static int docRtfFinishField(	BufferItem *		bi,
 int docRtfReadField(	SimpleInputStream *	sis,
 			const RtfControlWord *	rcw,
 			int			arg,
-			RtfReadingContext *	rrc	)
+			RtfReadingContext *	rrc )
     {
     int			res;
     BufferItem *	bi= rrc->rrcBi;
@@ -219,7 +226,7 @@ int docRtfReadField(	SimpleInputStream *	sis,
     res= docRtfReadGroup( sis, DOClevPARA,
 				(RtfControlWord *)0, 0, 0, rrc,
 				docRtfFieldWords, docRtfFieldGroups,
-				docRtfIgnoreText );
+				docRtfIgnoreText, (RtfCommitGroup)0 );
     if  ( res )
 	{ SLDEB(rcw->rcwWord,res);	}
 
@@ -249,10 +256,10 @@ static RtfControlWord	docRtfTC_Words[]=
 	{ 0, 0, 0 }
     };
 
-int docRtfReadLookupEntry(	SimpleInputStream *	sis,
+int docRtfLookupEntry(		SimpleInputStream *	sis,
 				const RtfControlWord *	rcw,
 				int			arg,
-				RtfReadingContext *	rrc	)
+				RtfReadingContext *	rrc )
     {
     int			res;
     BufferItem *	bi= rrc->rrcBi;
@@ -267,21 +274,23 @@ int docRtfReadLookupEntry(	SimpleInputStream *	sis,
     RtfControlWord *	contolWords= (RtfControlWord *)0;
     RtfControlWord *	groupWords= (RtfControlWord *)0;
 
-    switch( rcw->rcwId )
+    TextParticule *	tp;
+
+    switch( rcw->rcwID )
 	{
-	case RTFidXE:
+	case DOCfkXE:
 	    particuleKind= DOCkindXE;
 	    fieldKind= DOCfkXE;
 	    contolWords= docRtfXE_Words;
 	    break;
 
-	case RTFidTC:
+	case DOCfkTC:
 	    particuleKind= DOCkindTC;
 	    fieldKind= DOCfkTC;
 	    contolWords= docRtfTC_Words;
 	    break;
 
-	case RTFidTCN:
+	case DOCfkTCN:
 	    particuleKind= DOCkindTC;
 	    fieldKind= DOCfkTCN;
 	    contolWords= docRtfTC_Words;
@@ -294,10 +303,13 @@ int docRtfReadLookupEntry(	SimpleInputStream *	sis,
     if  ( docRtfReadStartField( &startParticule, &fieldNumber, rrc ) )
 	{ LDEB(bi->biParaParticuleCount); return -1;	}
 
+    tp= bi->biParaParticules+ startParticule;
+    tp->tpKind= particuleKind;
+
     res= docRtfReadGroup( sis, DOClevPARA,
 				firstApply, 0, 0, rrc,
 				contolWords, groupWords,
-				docRtfFldinst );
+				docRtfFldinst, (RtfCommitGroup)0 );
 
     if  ( res )
 	{ SLDEB(rcw->rcwWord,res);	}
@@ -313,7 +325,7 @@ int docRtfReadLookupEntry(	SimpleInputStream *	sis,
 
 static int docRtfBookmarkName(	RtfReadingContext *	rrc,
 				const unsigned char *	name,
-				int			len	)
+				int			len )
     {
     if  ( rrc->rrcBookmarkSize+ len > DOCmaxBOOKMARK )
 	{
@@ -361,7 +373,7 @@ int docRtfBkmkStart(	SimpleInputStream *	sis,
     res= docRtfReadGroup( sis, DOClevPARA,
 				(RtfControlWord *)0, 0, 0, rrc,
 				docRtfBkmkStartWords, docRtfEmptyTable,
-				docRtfBookmarkName );
+				docRtfBookmarkName, (RtfCommitGroup)0 );
 
     if  ( res )
 	{ SLDEB(rcw->rcwWord,res); return res;	}
@@ -436,7 +448,7 @@ int docRtfBkmkEnd(	SimpleInputStream *	sis,
     res= docRtfReadGroup( sis, DOClevPARA,
 				(RtfControlWord *)0, 0, 0, rrc,
 				docRtfEmptyTable, docRtfEmptyTable,
-				docRtfBookmarkName );
+				docRtfBookmarkName, (RtfCommitGroup)0 );
 
     if  ( res )
 	{ SLDEB(rcw->rcwWord,res); return res;	}
@@ -488,22 +500,50 @@ int docRtfBkmkEnd(	SimpleInputStream *	sis,
     return res;
     }
 
-int docRtfSpecialToField(	SimpleInputStream *	sis,
+int docRtfTextSpecialToField(	SimpleInputStream *	sis,
 				const RtfControlWord *	rcw,
 				int			arg,
-				RtfReadingContext *	rrc	)
+				RtfReadingContext *	rrc )
     {
-    BufferItem *	bi= rrc->rrcBi;
-    BufferDocument *	bd= rrc->rrcBd;
-    DocumentField *	df;
+    BufferItem *		bi= rrc->rrcBi;
+    BufferDocument *		bd= rrc->rrcBd;
+    DocumentField *		df;
 
-    int			startParticule;
-    int			fieldNumber;
+    int				startParticule;
+    int				fieldNumber;
 
-    switch( rcw->rcwId )
+    const unsigned char *	fieldinst= (const unsigned char *)"?";
+    int				fieldsize= 1;
+    const unsigned char *	fieldRslt= (const unsigned char *)"?";
+
+    int				afterNoteref= 0;
+
+    switch( rcw->rcwID )
 	{
-	case RTFidCHPGN:
+	case DOCfkPAGE:
+	    fieldinst= (const unsigned char *)" PAGE ";
+	    fieldsize= 6;
 	    break;
+	case DOCfkDATE:
+	    fieldinst= (const unsigned char *)" DATE \\* MERGEFORMAT ";
+	    fieldsize= 21;
+	    break;
+	case DOCfkTIME:
+	    fieldinst= (const unsigned char *)" TIME \\* MERGEFORMAT ";
+	    fieldsize= 21;
+	    break;
+	case DOCfkSECTION:
+	    fieldinst= (const unsigned char *)" SECTION ";
+	    fieldsize= 9;
+	    break;
+
+	case DOCfkCHFTN:
+	    fieldinst= (const unsigned char *)" -CHFTN ";
+	    fieldsize= 8;
+	    fieldRslt= (const unsigned char *)"1";
+	    afterNoteref= 1;
+	    break;
+
 	default:
 	    SDEB(rcw->rcwWord); return -1;
 	}
@@ -513,24 +553,37 @@ int docRtfSpecialToField(	SimpleInputStream *	sis,
 
     df= bd->bdFieldList.dflFields+ fieldNumber;
 
-    if  ( docRtfTextParticule( rrc, (unsigned char *)"?", 1 ) )
+    if  ( docRtfTextParticule( rrc, fieldRslt, 1 ) )
 	{ LDEB(2); return -1;	}
 
-    if  ( docSetFieldInst( df, (unsigned char *)" PAGE ", 6 ) )
+    if  ( docSetFieldInst( df, fieldinst, fieldsize ) )
 	{ LDEB(6); return -1;	}
 
     if  ( docRtfFlushBookmarks( -1, bi, rrc ) )
 	{ LDEB(1); return -1;	}
 
-    if  ( docRtfFinishField( bi, fieldNumber, rrc ) )
+    if  ( docRtfTerminateField( bi, fieldNumber, rrc ) )
 	{ LDEB(bi->biParaParticuleCount); return -1;	}
 
+    rrc->rrcAfterNoteref= afterNoteref;
+    df->dfKind= rcw->rcwID;
+
     return 0;
     }
 
 /************************************************************************/
 /*									*/
-/*  Consume Headers and Footers.					*/
+/*  Consume Headers, Footers, Notes &c: Separate item trees that are.	*/
+/*  embedded in the document.						*/
+/*									*/
+/*  1)  Save the current position.					*/
+/*  2)  Make the header/note &c.					*/
+/*  3)  Consume its contents.						*/
+/*  4)  Make sure that no bookmarks protrude beyond the end of the	*/
+/*	external item.							*/
+/*  5)  If the external item happens to be empty, insert an empty	*/
+/*	paragraph to avoid problems later on.				*/
+/*  6)  Restore the current position.					*/
 /*									*/
 /************************************************************************/
 
@@ -540,33 +593,37 @@ static int docRtfReadExternalItem(	BufferItem **		pBi,
 					RtfReadingContext *	rrc,
 					const SelectionScope *	ss )
     {
+    RtfReadingState *	rrs= rrc->rrcState;
     BufferItem *	bi;
 
     BufferItem *	savedBi;
     int			savedLevel;
-    int			savedInField;
+    int			savedInTable;
     int			savedExternalItemKind;
 
     int			res;
 
+    /*  1  */
     savedBi= rrc->rrcBi;
     savedLevel= rrc->rrcLevel;
-    savedInField= rrc->rrcParagraphProperties.ppInTable;
+    savedInTable= rrs->rrsParagraphProperties.ppInTable;
     savedExternalItemKind= rrc->rrcExternalItemKind;
     if  ( ! savedBi )
 	{ XDEB(savedBi); return -1;	}
 
+    /*  2  */
     bi= *pBi;
     if  ( ! bi )
 	{
 	bi= docMakeExternalItem( rrc->rrcBd,
-				    ss, &(rrc->rrcSectionProperties) );
+				    ss, &(rrs->rrsSectionProperties) );
 	if  ( ! bi )
 	    { XDEB(bi); return -1;	}
 
 	*pBi= bi;
 	}
 
+    /*  3  */
     rrc->rrcBi= bi;
     rrc->rrcLevel= DOClevSECT;
     rrc->rrcExternalItemKind= ss->ssInExternalItem;
@@ -574,17 +631,33 @@ static int docRtfReadExternalItem(	BufferItem **		pBi,
     res= docRtfReadGroup( sis, DOClevPARA,
 				(RtfControlWord *)0, 0, 0, rrc,
 				docRtfDocumentWords, docRtfDocumentGroups,
-				docRtfTextParticule );
+				docRtfTextParticule, (RtfCommitGroup)0 );
     if  ( res )
 	{ LDEB(res);	}
 
+    /*  4  */
     if  ( docRtfFlushBookmarks( -1, bi, rrc ) )
 	{ LDEB(1); return -1;	}
 
+    /*  5  */
+    {
+    DocumentPosition	dp;
+
+    if  ( docFirstPosition( &dp, bi ) )
+	{
+	const int	textAttributeNumber= 0;
+
+	if  ( ! docInsertEmptyParagraph( rrc->rrcBd, bi,
+						textAttributeNumber ) )
+	    { LDEB(textAttributeNumber);	}
+	}
+    }
+
     *pExtItKind= rrc->rrcExternalItemKind;
 
+    /*  6  */
     rrc->rrcExternalItemKind= savedExternalItemKind;
-    rrc->rrcParagraphProperties.ppInTable= savedInField;
+    rrs->rrsParagraphProperties.ppInTable= savedInTable;
     rrc->rrcLevel= savedLevel;
     rrc->rrcBi= savedBi;
 
@@ -605,7 +678,7 @@ int docRtfReadExtIt(		SimpleInputStream *	sis,
 
     docInitSelectionScope( &ss );
 
-    switch( rcw->rcwId )
+    switch( rcw->rcwID )
 	{
 	case DOCinSECT_HEADER:
 	case DOCinFIRST_HEADER:
@@ -619,9 +692,9 @@ int docRtfReadExtIt(		SimpleInputStream *	sis,
 	    if  ( ! sectBi || sectBi->biLevel != DOClevSECT )
 		{ XDEB(sectBi); return -1;	}
 
-	    ei= docSectionHeaderFooter( sectBi, rcw->rcwId );
+	    ei= docSectionHeaderFooter( sectBi, rcw->rcwID );
 	    if  ( ! ei )
-		{ LXDEB(rcw->rcwId,ei); return -1;	}
+		{ LXDEB(rcw->rcwID,ei); return -1;	}
 
 	    ss.ssSectNrExternalTo= sectBi->biNumberInParent;
 
@@ -637,9 +710,9 @@ int docRtfReadExtIt(		SimpleInputStream *	sis,
 	    if  ( ! bd )
 		{ XDEB(bd); return -1;	}
 
-	    ei= docDocumentNoteSeparator( bd, rcw->rcwId );
+	    ei= docDocumentNoteSeparator( bd, rcw->rcwID );
 	    if  ( ! ei )
-		{ LXDEB(rcw->rcwId,ei); return -1;	}
+		{ LXDEB(rcw->rcwID,ei); return -1;	}
 
 	    break;
 
@@ -650,7 +723,7 @@ int docRtfReadExtIt(		SimpleInputStream *	sis,
     if  ( ei->eiItem )
 	{ SXDEB(rcw->rcwWord,ei->eiItem);	}
 
-    ss.ssInExternalItem= rcw->rcwId;
+    ss.ssInExternalItem= rcw->rcwID;
     ss.ssNoteArrayIndex= -1;
 
     if  ( docRtfReadExternalItem( &(ei->eiItem), &extItKind, sis, rrc, &ss ) )
@@ -670,11 +743,10 @@ int docRtfReadFootnote(		SimpleInputStream *	sis,
 				int			arg,
 				RtfReadingContext *	rrc )
     {
+    RtfReadingState *	rrs= rrc->rrcState;
     BufferDocument *	bd= rrc->rrcBd;
     BufferItem *	bi= rrc->rrcBi;
 
-    TextParticule *	tp;
-
     DocumentNote *	dn;
     int			noteIndex;
 
@@ -682,6 +754,9 @@ int docRtfReadFootnote(		SimpleInputStream *	sis,
     SelectionScope	ss;
 
     int			externalItemKind;
+    int			autoNumber= 0;
+
+    const int		objectNumber= -1;
 
     docInitSelectionScope( &ss );
 
@@ -691,14 +766,21 @@ int docRtfReadFootnote(		SimpleInputStream *	sis,
     if  ( ! sectBi )
 	{ XDEB(sectBi); return -1;	}
 
-    noteIndex= docInsertNote( &dn, bd, bi, bi->biParaStrlen );
+    if  ( rrc->rrcAfterNoteref )
+	{ autoNumber= 1;	}
+
+    noteIndex= docInsertNote( &dn, bd, bi, bi->biParaStrlen, autoNumber );
     if  ( noteIndex < 0 )
 	{ LDEB(noteIndex); return -1;	}
 
-    tp= docInsertTextParticule( bi, -1, bi->biParaStrlen, 0,
-			    DOCkindNOTE, rrc->rrcCurrentTextAttribute );
-    if  ( ! tp )
-	{ LDEB(bi->biParaParticuleCount); return -1;	}
+    if  ( docInsertAdminParticule( bd, bi, bi->biParaStrlen,
+					    bi->biParaParticuleCount,
+					    objectNumber,
+					    &(rrs->rrsTextAttribute),
+					    DOCkindNOTE ) )
+	{ LDEB(1); return -1;	}
+
+    rrc->rrcAfterNoteref= 0;
 
     ss.ssInExternalItem= DOCinFOOTNOTE;
     ss.ssSectNrExternalTo= sectBi->biNumberInParent;
@@ -708,41 +790,51 @@ int docRtfReadFootnote(		SimpleInputStream *	sis,
 					&externalItemKind, sis, rrc, &ss ) )
 	{ SDEB(rcw->rcwWord); return -1;	}
 
+    dn->dnAutoNumber= autoNumber;
     dn->dnExternalItemKind= externalItemKind;
-    docSetExternalItemKind( dn->dnExternalItem.eiItem,
-						externalItemKind );
+    docSetExternalItemKind( dn->dnExternalItem.eiItem, externalItemKind );
 
     return 0;
     }
 
-int docRtfChftn(		SimpleInputStream *	sis,
+/************************************************************************/
+/*									*/
+/*  Read a group that results in a text field.				*/
+/*									*/
+/************************************************************************/
+
+int docRtfReadTextField(	SimpleInputStream *	sis,
 				const RtfControlWord *	rcw,
 				int			arg,
 				RtfReadingContext *	rrc )
     {
-    BufferDocument *	bd= rrc->rrcBd;
+    int			res;
+    BufferItem *	bi= rrc->rrcBi;
 
     int			startParticule;
     int			fieldNumber= -1;
+
     DocumentField *	df;
 
-    TextAttribute	ta= rrc->rrcCurrentTextAttribute;
-
     if  ( docRtfReadStartField( &startParticule, &fieldNumber, rrc ) )
+	{ LDEB(bi->biParaParticuleCount); return -1;	}
+
+    df= rrc->rrcBd->bdFieldList.dflFields+ fieldNumber;
+    df->dfKind= rcw->rcwID;
+
+    res= docRtfReadGroup( sis, DOClevPARA,
+				(RtfControlWord *)0, 0, 0, rrc,
+				docRtfDocumentWords, docRtfDocumentGroups,
+				docRtfTextParticule, (RtfCommitGroup)0 );
+    if  ( res )
+	{ SLDEB(rcw->rcwWord,res);	}
+
+    if  ( docRtfFlushBookmarks( -1, bi, rrc ) )
 	{ LDEB(1); return -1;	}
 
-    df= bd->bdFieldList.dflFields+ fieldNumber;
-    df->dfKind= DOCfkCHFTN;
+    if  ( docRtfFinishField( bi, fieldNumber, rrc ) )
+	{ LDEB(bi->biParaParticuleCount); return -1;	}
 
-    /*  No: just fake \\up in the formatter temporarily?
-    ta.taSuperSub= DOCfontSUPERSCRIPT;
-    */
-
-    if  ( docSaveParticules( rrc->rrcBi, ta, (const unsigned char *)"1", 1 ) )
-	{ LDEB(1);	}
-
-    if  ( docRtfTerminateField( rrc->rrcBi, fieldNumber, rrc ) )
-	{ LDEB(1); return -1;	}
-
-    return 0;
+    return res;
     }
+
