@@ -230,58 +230,49 @@ static char * docRtfShapeTypeString(	int	shapeType )
 /*									*/
 /************************************************************************/
 
-static int docRtfShpProperty(	SimpleInputStream *	sis,
-				const RtfControlWord *	rcw,
-				int			arg,
-				RtfReadingContext *	rrc	)
+int docRtfShpProperty(	SimpleInputStream *	sis,
+			const RtfControlWord *	rcw,
+			int			arg,
+			RtfReadingContext *	rrc )
     {
-    switch( rcw->rcwId )
+    switch( rcw->rcwID )
 	{
-	case	RTFidSHPLEFT:
+	case	SHPpropLEFT:
 	    rrc->rrcShapeProperties.spTwipsLeftOfAnchor= arg;
 	    break;
-	case	RTFidSHPRIGHT:
+	case	SHPpropRIGHT:
 	    rrc->rrcShapeProperties.spTwipsRightOfAnchor= arg;
 	    break;
-	case	RTFidSHPTOP:
+	case	SHPpropTOP:
 	    rrc->rrcShapeProperties.spTwipsAboveAnchor= arg;
 	    break;
-	case	RTFidSHPBOTTOM:
+	case	SHPpropBOTTOM:
 	    rrc->rrcShapeProperties.spTwipsBelowAnchor= arg;
 	    break;
-	case	RTFidSHPFHDR:
+	case	SHPpropFHDR:
 	    break;
-	case	RTFidSHPBXCOLUMN:
-	    rrc->rrcShapeProperties.spHorizontalAttachment= SHPshaCOLUMN;
+
+	case	SHPpropBX:
+	    rrc->rrcShapeProperties.spHorizontalAttachment= rcw->rcwEnumValue;
 	    break;
-	case	RTFidSHPBXPAGE:
-	    rrc->rrcShapeProperties.spHorizontalAttachment= SHPshaPAGE;
+
+	case	SHPpropBY:
+	    rrc->rrcShapeProperties.spVerticalAttachment= rcw->rcwEnumValue;
 	    break;
-	case	RTFidSHPBXMARGIN:
-	    rrc->rrcShapeProperties.spHorizontalAttachment= SHPshaMARGIN;
-	    break;
-	case	RTFidSHPBYPARA:
-	    rrc->rrcShapeProperties.spVerticalAttachment= SHPsvaPARAGRAPH;
-	    break;
-	case	RTFidSHPBYPAGE:
-	    rrc->rrcShapeProperties.spVerticalAttachment= SHPsvaPAGE;
-	    break;
-	case	RTFidSHPBYMARGIN:
-	    rrc->rrcShapeProperties.spVerticalAttachment= SHPsvaMARGIN;
-	    break;
-	case	RTFidSHPWR:
+
+	case	SHPpropWR:
 	    rrc->rrcShapeProperties.spWrapStyle= arg;
 	    break;
-	case	RTFidSHPWRK:
+	case	SHPpropWRK:
 	    rrc->rrcShapeProperties.spWrapSide= arg;
 	    break;
-	case	RTFidSHPZ:
-	case	RTFidSHPFBLWTXT:
+	case	SHPpropZ:
+	case	SHPpropFBLWTXT:
 	    break;
-	case	RTFidSHPLOCKANCHOR:
+	case	SHPpropLOCKANCHOR:
 	    rrc->rrcShapeProperties.spLockAnchor= ( arg != 0 );
 	    break;
-	case	RTFidSHPLID:
+	case	SHPpropLID:
 	    break;
 	default:
 	    /* SLDEB(rcw->rcwWord,arg); */
@@ -291,34 +282,6 @@ static int docRtfShpProperty(	SimpleInputStream *	sis,
     return 0;
     }
 
-static RtfControlWord	docRtfShapeWords[]=
-    {
-	{ "shpleft",	RTFidSHPLEFT,	DOClevPARA, docRtfShpProperty, },
-	{ "shptop",	RTFidSHPTOP,	DOClevPARA, docRtfShpProperty, },
-	{ "shpbottom",	RTFidSHPBOTTOM,	DOClevPARA, docRtfShpProperty, },
-	{ "shpright",	RTFidSHPRIGHT,	DOClevPARA, docRtfShpProperty, },
-	{ "shplid",	RTFidSHPLID,	DOClevPARA, docRtfShpProperty, },
-	{ "shpz",	RTFidSHPZ,	DOClevPARA, docRtfShpProperty, },
-	{ "shpfhdr",	RTFidSHPFHDR,	DOClevPARA, docRtfShpProperty, },
-	{ "shpbxpage",	RTFidSHPBXPAGE,	DOClevPARA, docRtfShpProperty, },
-	{ "shpbxmargin",RTFidSHPBXMARGIN,DOClevPARA,docRtfShpProperty, },
-	{ "shpbxcolumn",RTFidSHPBXCOLUMN,DOClevPARA,docRtfShpProperty, },
-	{ "shpbxignore",RTFidSHPBXIGNORE,DOClevPARA,docRtfShpProperty, },
-	{ "shpbypage",	RTFidSHPBYPAGE,	DOClevPARA, docRtfShpProperty, },
-	{ "shpbymargin",RTFidSHPBYMARGIN,DOClevPARA,docRtfShpProperty, },
-	{ "shpbycolumn",RTFidSHPBYCOLUMN,DOClevPARA,docRtfShpProperty, },
-	{ "shpbyignore",RTFidSHPBYIGNORE,DOClevPARA,docRtfShpProperty, },
-	{ "shpbypara",	RTFidSHPBYPARA,	DOClevPARA, docRtfShpProperty, },
-	{ "shpwr",	RTFidSHPWR,	DOClevPARA, docRtfShpProperty, },
-	{ "shpwrk",	RTFidSHPWRK,	DOClevPARA, docRtfShpProperty, },
-	{ "shpfblwtxt",	RTFidSHPFBLWTXT, DOClevPARA,docRtfShpProperty, },
-	{ "shplockanchor",
-			RTFidSHPLOCKANCHOR,
-					DOClevPARA, docRtfShpProperty, },
-
-	{ 0, 0, 0, }
-    };
-
 /************************************************************************/
 /*									*/
 /*  Text inside a shape.						*/
@@ -335,7 +298,7 @@ static int docRtfShpText(	SimpleInputStream *	sis,
     res= docRtfReadGroup( sis, DOClevPARA,
 				(RtfControlWord *)0, 0, 0, rrc,
 				docRtfDocumentWords, docRtfDocumentGroups,
-				docRtfTextParticule );
+				docRtfTextParticule, (RtfCommitGroup)0 );
 
     if  ( res )
 	{ SLDEB(rcw->rcwWord,res);	}
@@ -477,7 +440,7 @@ static int docRtfShapeArray(	SimpleInputStream *	sis,
 static int docRtfShapePicture(	SimpleInputStream *	sis,
 				const RtfControlWord *	rcw,
 				int			arg,
-				RtfReadingContext *	rrc	)
+				RtfReadingContext *	rrc )
     {
     int				control;
     char			controlWord[TEDszRTFCONTROL+1];
@@ -489,6 +452,10 @@ static int docRtfShapePicture(	SimpleInputStream *	sis,
     if  ( control != RTFfiCTRLGROUP )
 	{ SLDEB(rcw->rcwWord,control); return -1; }
 
+    /*
+    LSDEB(rrc->rrcCurrentLine,controlWord);
+    */
+
     if  ( strcmp( controlWord, "pict" ) )
 	{ SDEB(controlWord); return -1;	}
 
@@ -552,10 +519,10 @@ static int docRtfShapeNumber(	SimpleInputStream *	sis,
 
     sioInUngetLastRead( sis );
 
-    switch( rcw->rcwId )
+    switch( rcw->rcwID )
 	{
 	case RTFSHP_shapeType:
-	    if  ( rcw->rcwId != RTFSHP_shapeType )
+	    if  ( rcw->rcwID != RTFSHP_shapeType )
 		{ SDEB(docRtfShapeTypeString(val));	}
 
 	    ds->ds_shapeType= val;
@@ -639,6 +606,14 @@ static int docRtfShapeColor(	SimpleInputStream *	sis,
 
 static RtfControlWord	docRtfShapeProperties[]=
 {
+    { "posv",			RTFSHP_posv,
+				DOClevPARA, docRtfShapeNumber, },
+    { "posh",			RTFSHP_posh,
+				DOClevPARA, docRtfShapeNumber, },
+    { "posrelv",		RTFSHP_posrelv,
+				DOClevPARA, docRtfShapeNumber, },
+    { "posrelh",		RTFSHP_posrelh,
+				DOClevPARA, docRtfShapeNumber, },
     { "fFlipV",			RTFSHP_fFlipV,
 				DOClevPARA, docRtfShapeNumber, },
     { "fFlipH",			RTFSHP_fFlipH,
@@ -947,6 +922,10 @@ static RtfControlWord	docRtfShapeProperties[]=
     { "fRelFlipV", 		RTFSHP_fRelFlipV,
 				DOClevPARA, docRtfShapeNumber, }, /* ? */
 
+    { "pictureContrast", 	RTFSHP_pictureContrast,
+				DOClevPARA, docRtfShapeNumber, }, /* ? */
+    { "pictureBrightness", 	RTFSHP_pictureBrightness,
+				DOClevPARA, docRtfShapeNumber, }, /* ? */
     { "pictureDblCrMod", 	RTFSHP_pictureDblCrMod,
 				DOClevPARA, docRtfShapeNumber, }, /* ? */
     { "pictureFillCrMod", 	RTFSHP_pictureFillCrMod,
@@ -1024,6 +1003,14 @@ static RtfControlWord	docRtfShapeProperties[]=
 				DOClevPARA, docRtfShapeNumber, }, /* ? */
     { "cdirFont",	 	RTFSHP_cdirFont,
 				DOClevPARA, docRtfShapeNumber, }, /* ? */
+    { "cropFromLeft",	 	RTFSHP_cropFromLeft,
+				DOClevPARA, docRtfShapeNumber, }, /* ? */
+    { "cropFromRight",	 	RTFSHP_cropFromRight,
+				DOClevPARA, docRtfShapeNumber, }, /* ? */
+    { "cropFromTop",	 	RTFSHP_cropFromTop,
+				DOClevPARA, docRtfShapeNumber, }, /* ? */
+    { "cropFromBottom",	 	RTFSHP_cropFromBottom,
+				DOClevPARA, docRtfShapeNumber, }, /* ? */
 
     { "fillAngle", 		RTFSHP_fillAngle,
 				DOClevPARA, docRtfShapeNumber, },
@@ -1220,14 +1207,36 @@ static int docRtfShpinst(	SimpleInputStream *	sis,
 
     res= docRtfReadGroup( sis, DOClevPARA,
 				(RtfControlWord *)0, 0, 0, rrc,
-				docRtfShapeWords, docRtfShpinstGroups,
-				docRtfIgnoreText );
+				(RtfControlWord *)0, docRtfShpinstGroups,
+				docRtfIgnoreText, (RtfCommitGroup)0 );
     if  ( res )
 	{ SLDEB(rcw->rcwWord,res);	}
 
     return res;
     }
 
+static int docRtfShprslt(	SimpleInputStream *	sis,
+				const RtfControlWord *	rcw,
+				int			arg,
+				RtfReadingContext *	rrc	)
+    {
+    int		res;
+
+    rrc->rrcInIgnoredGroup++;
+
+    res= docRtfReadGroup( sis, DOClevANY,
+				(RtfControlWord *)0, 0, 0, rrc,
+				(const RtfControlWord *)0,
+				(const RtfControlWord *)0,
+				docRtfIgnoreText, (RtfCommitGroup)0 );
+    if  ( res )
+	{ SLDEB(rcw->rcwWord,res);	}
+
+    rrc->rrcInIgnoredGroup--;
+
+    return res;
+    }
+
 /************************************************************************/
 /*									*/
 /*  The actual shape as opposed to the result in simpler RTF.		*/
@@ -1237,7 +1246,7 @@ static int docRtfShpinst(	SimpleInputStream *	sis,
 static RtfControlWord	docRtfShapeGroups[]=
     {
 	{ "shptxt",	RTFidSHPTXT,	DOClevPARA, docRtfShpText, },
-	{ "shprslt",	RTFidSHPRSLT,	DOClevPARA, docRtfSkipGroup, },
+	{ "shprslt",	RTFidSHPRSLT,	DOClevPARA, docRtfShprslt, },
 	{ "shpgrp",	RTFidSHPGRP,	DOClevPARA, docRtfReadShape, },
 	{ "shp",	RTFidSHP,	DOClevPARA, docRtfReadShape, },
 	{ "shpinst",	RTFidSHPINST,	DOClevPARA, docRtfShpinst, },
@@ -1254,8 +1263,8 @@ int docRtfReadShape(	SimpleInputStream *	sis,
 
     res= docRtfReadGroup( sis, DOClevTEXT,
 				(RtfControlWord *)0, 0, 0, rrc,
-				docRtfShapeWords, docRtfShapeGroups,
-				docRtfIgnoreParticule );
+				(RtfControlWord *)0, docRtfShapeGroups,
+				docRtfIgnoreText, (RtfCommitGroup)0 );
 
     if  ( res )
 	{ SLDEB(rcw->rcwWord,res);	}
