@@ -26,7 +26,7 @@
 
 RtfControlWord	docRtfEmptyTable[]=
     {
-	{ 0, }
+	{ 0, 0, 0, }
     };
 
 /************************************************************************/
@@ -41,6 +41,9 @@ void docRtfInitReadingContext(	RtfReadingContext *	rrc	)
 
     rrc->rrcLevel= DOClevOUT;
     rrc->rrcSplitLevel= DOClevOUT;
+    rrc->rrcState= (RtfReadingState *)0;
+    rrc->rrcInIgnoredGroup= 0;
+
 				/****************************************/
 				/*  The depth in the document		*/
 				/*  == Type of BufferItem.		*/
@@ -50,6 +53,7 @@ void docRtfInitReadingContext(	RtfReadingContext *	rrc	)
 				/*  Complain about unknown control.	*/
 				/****************************************/
     rrc->rrcCharacterAhead= EOF;
+    rrc->rrcAfterNoteref= 0;
 
     rrc->rrcCurrentLine= 1;
     rrc->rrcBd= (BufferDocument *)0;
@@ -57,11 +61,8 @@ void docRtfInitReadingContext(	RtfReadingContext *	rrc	)
 				/****************************************/
 				/*  The current position in the input.	*/
 				/****************************************/
-    docInitTextAttribute( &(rrc->rrcCurrentTextAttribute) );
-    docInitBorderProperties( &(rrc->rrcTextBorderProperties) );
     rrc->rrcInDeletedText= 0;
 
-    rrc->rrcCurrentTextAttribute.taFontSizeHalfPoints= 24;
     rrc->rrcTextShadingPattern= DOCspSOLID;
     rrc->rrcTextShadingLevel= 0;
     rrc->rrcTextForeground= -1;
@@ -69,15 +70,17 @@ void docRtfInitReadingContext(	RtfReadingContext *	rrc	)
 				/****************************************/
 				/* Attributes of the current position.	*/
 				/****************************************/
-    docInitTextFrameProperties( &(rrc->rrcTextFrameProperties) );
-    docInitParagraphProperties( &(rrc->rrcParagraphProperties) );
+    docInitTextFrameProperties( &(rrc->rrcParaFramePosition) );
+    docInitTextFrameProperties( &(rrc->rrcRowFramePosition) );
     docInitCellProperties( &(rrc->rrcCellProperties) );
     docInitRowProperties( &(rrc->rrcRowProperties) );
-    docInitSectionProperties( &(rrc->rrcSectionProperties) );
 
     rrc->rrcExternalItemKind= DOCinBODY;
 
     docInitDocumentList( &(rrc->rrcDocumentList) );
+    docInitListOverride( &(rrc->rrcListOverride) );
+    docInitDocumentListLevel( &(rrc->rrcDocumentListLevel) );
+    docInitListOverrideLevel( &(rrc->rrcListOverrideLevel) );
 
     docInitBorderProperties( &(rrc->rrcBorderProperties) );
     /*docInitShapeProperties( &(rrc->rrcShapeProperties) );*/
@@ -97,8 +100,8 @@ void docRtfInitReadingContext(	RtfReadingContext *	rrc	)
 				/****************************************/
 				/*  Document properties.		*/
 				/****************************************/
-    docInitFont( &(rrc->rrcCurrentFont) );
-    docInitStyle( &(rrc->rrcStyle) );
+    docInitDocumentFont( &(rrc->rrcCurrentFont) );
+    docInitDocumentStyle( &(rrc->rrcDocumentStyle) );
 				/****************************************/
 				/*  For reading the font table.		*/
 				/****************************************/
@@ -112,29 +115,36 @@ void docRtfInitReadingContext(	RtfReadingContext *	rrc	)
 				/*  For reading 'fields'.		*/
 				/****************************************/
     rrc->rrcJustAfterPntext= 0;
-    rrc->rrcPageAfterPntext= 0;
 				/****************************************/
 				/*  For coping with the way word saves	*/
 				/*  {\pntext ... }			*/
 				/****************************************/
 
     for ( i= 0; i < 256; i++ )
-	{ rrc->rrcInputMapping[i]= i;	}
+	{ rrc->rrcDefaultInputMapping[i]= i;	}
+    rrc->rrcTextInputMapping= rrc->rrcDefaultInputMapping;
     }
 
-void docRtfCleanReadingContext(	RtfReadingContext *	rrc	)
+void docRtfCleanReadingContext(	RtfReadingContext *	rrc )
     {
-    docCleanParagraphProperties( &(rrc->rrcParagraphProperties) );
+    while( rrc->rrcState )
+	{
+	XDEB(rrc->rrcState);
+	docRtfPopReadingState( rrc );
+	}
+
     /*docCleanCellProperties( &(rrc->rrcRowProperties) );*/
     docCleanRowProperties( &(rrc->rrcRowProperties) );
-    docCleanSectionProperties( &(rrc->rrcSectionProperties) );
     /*docCleanShapeProperties( &(rrc->rrcShapeProperties) );*/
     docCleanShape( &(rrc->rrcShape) );
-    docCleanFont( &(rrc->rrcCurrentFont) );
+    docCleanDocumentFont( &(rrc->rrcCurrentFont) );
 
     docCleanDocumentList( &(rrc->rrcDocumentList) );
+    docCleanListOverride( &(rrc->rrcListOverride) );
+    docCleanDocumentListLevel( &(rrc->rrcDocumentListLevel) );
+    docCleanListOverrideLevel( &(rrc->rrcListOverrideLevel) );
 
-    docCleanStyle( &(rrc->rrcStyle) );
+    docCleanDocumentStyle( &(rrc->rrcDocumentStyle) );
 
     if  ( rrc->rrcParagraphNumbers )
 	{ free( rrc->rrcParagraphNumbers );	}
@@ -155,49 +165,57 @@ void docRtfCleanReadingContext(	RtfReadingContext *	rrc	)
 int docRtfCopyReadingContext(	RtfReadingContext *		to,
 				const RtfReadingContext *	from )
     {
-    ParagraphProperties		ppCopy;
     RowProperties		rpCopy;
-    SectionProperties		spCopy;
     DocumentStyle		dsCopy;
 
     PropertyMask		ppChgMask;
     PropertyMask		ppUpdMask;
 
+    const int * const		fontMap= (const int *)0;
+    const int * const		colorMap= (const int *)0;
+
     PROPmaskCLEAR( &ppChgMask );
 
     PROPmaskCLEAR( &ppUpdMask );
     PROPmaskFILL( &ppUpdMask, PPprop_COUNT );
 
-    docInitParagraphProperties( &ppCopy );
-    if  ( docUpdParaProperties( &ppChgMask, &ppCopy,
-				&ppUpdMask, &(from->rrcParagraphProperties) ) )
-	{ LDEB(1); return -1;	}
-
     docInitRowProperties( &rpCopy );
-    if  ( docCopyRowProperties( &rpCopy, &(from->rrcRowProperties) ) )
+    if  ( docCopyRowProperties( &rpCopy,
+			    &(from->rrcRowProperties), (const int *)0 ) )
 	{ LDEB(1); return -1;	}
 
-    docInitSectionProperties( &spCopy );
-    if  ( docCopySectionProperties( &spCopy, &(from->rrcSectionProperties) ) )
+    docInitDocumentStyle( &dsCopy );
+    if  ( docCopyStyle( &dsCopy, &(from->rrcDocumentStyle) ) )
 	{ LDEB(1); return -1;	}
 
-    docInitStyle( &dsCopy );
-    if  ( docCopyStyle( &dsCopy, &(from->rrcStyle) ) )
-	{ LDEB(1); return -1;	}
+    docCleanDocumentList( &(to->rrcDocumentList) );
+    docCleanListOverride( &(to->rrcListOverride) );
+    docCleanDocumentListLevel( &(to->rrcDocumentListLevel) );
+    docCleanListOverrideLevel( &(to->rrcListOverrideLevel) );
 
     *to= *from;
 
     to->rrcParagraphNumbers= (ParagraphNumber *)0;
     to->rrcParagraphNumberCount= 0;
 
-    to->rrcParagraphProperties= ppCopy;
     to->rrcRowProperties= rpCopy;
-    to->rrcSectionProperties= spCopy;
-    to->rrcStyle= dsCopy;
+    to->rrcDocumentStyle= dsCopy;
 
     to->rrcTopBookmark= from->rrcTopBookmark;
 
     docInitDocumentList( &(to->rrcDocumentList) );
+    docInitListOverride( &(to->rrcListOverride) );
+    docInitDocumentListLevel( &(to->rrcDocumentListLevel) );
+    docInitListOverrideLevel( &(to->rrcListOverrideLevel) );
+
+    docCopyDocumentList( &(to->rrcDocumentList),
+			    &(from->rrcDocumentList), fontMap, colorMap );
+    docCopyListOverride( &(to->rrcListOverride),
+			    &(from->rrcListOverride), fontMap, colorMap );
+    docCopyDocumentListLevel( &(to->rrcDocumentListLevel),
+			    &(from->rrcDocumentListLevel), fontMap, colorMap );
+    docCopyListOverrideLevel( &(to->rrcListOverrideLevel),
+			    &(from->rrcListOverrideLevel), fontMap, colorMap );
 
     return 0;
     }
@@ -206,13 +224,15 @@ int docRtfCopyReadingContext(	RtfReadingContext *		to,
 void docRtfCopyReadingContextBack(	RtfReadingContext *	to,
 					RtfReadingContext *	from )
     {
+    const int * const		fontMap= (const int *)0;
+    const int * const		colorMap= (const int *)0;
+
     to->rrcBi= from->rrcBi;
     to->rrcLevel= from->rrcLevel;
     to->rrcSplitLevel= from->rrcSplitLevel;
     to->rrcCurrentLine= from->rrcCurrentLine;
     to->rrcFieldNumber= from->rrcFieldNumber;
     to->rrcJustAfterPntext= from->rrcJustAfterPntext;
-    to->rrcPageAfterPntext= from->rrcPageAfterPntext;
 
     memcpy( to->rrcBookmarkName, from->rrcBookmarkName, DOCmaxBOOKMARK+1 );
     to->rrcBookmarkSize= from->rrcBookmarkSize;
@@ -220,7 +240,7 @@ void docRtfCopyReadingContextBack(	RtfReadingContext *	to,
     from->rrcTopBookmark= (RtfBookmarkLevel *)0;
 
     to->rrcCurrentFont= from->rrcCurrentFont;
-    docInitFont( &(from->rrcCurrentFont) );
+    docInitDocumentFont( &(from->rrcCurrentFont) );
 
     to->rrcParagraphNumber= from->rrcParagraphNumber;
 
@@ -228,7 +248,83 @@ void docRtfCopyReadingContextBack(	RtfReadingContext *	to,
 
     to->rrcTm= from->rrcTm;
 
-    docInitDocumentList( &(to->rrcDocumentList) );
+    docCopyDocumentList( &(to->rrcDocumentList),
+			    &(from->rrcDocumentList), fontMap, colorMap );
+    docCopyListOverride( &(to->rrcListOverride),
+			    &(from->rrcListOverride), fontMap, colorMap );
+    docCopyDocumentListLevel( &(to->rrcDocumentListLevel),
+			    &(from->rrcDocumentListLevel), fontMap, colorMap );
+    docCopyListOverrideLevel( &(to->rrcListOverrideLevel),
+			    &(from->rrcListOverrideLevel), fontMap, colorMap );
+    }
+
+void docRtfPushReadingState(	RtfReadingContext *	rrc,
+				RtfReadingState *	rrs )
+    {
+    BufferDocument *		bd= rrc->rrcBd;
+    const DocumentProperties *	dp= (const DocumentProperties *)0;
+
+    if  ( bd )
+	{ dp= &(bd->bdProperties); }
+
+    utilInitTextAttribute( &(rrs->rrsTextAttribute) );
+    rrs->rrsTextAttribute.taFontSizeHalfPoints= 24;
+
+    if  ( dp && dp->dpDefaultFont >= 0 )
+	{ rrs->rrsTextAttribute.taFontNumber= dp->dpDefaultFont;	}
+
+    docInitParagraphProperties( &(rrs->rrsParagraphProperties) );
+    docInitSectionProperties( &(rrs->rrsSectionProperties) );
+
+    utilInitMemoryBuffer( &(rrs->rrsSavedText) );
+
+    rrs->rrsBytesPerUnicode= 1;
+    rrs->rrsUnicodeBytesToSkip= 0;
+
+    if  ( rrc->rrcState )
+	{
+	const RtfReadingState *	above= rrc->rrcState;
+
+	rrs->rrsBytesPerUnicode= above->rrsBytesPerUnicode;
+	rrs->rrsUnicodeBytesToSkip= 0;
+
+	/*
+	utilCopyTextAttribute( &(rrs->rrsTextAttribute),
+					&(above->rrsTextAttribute) );
+	*/
+	rrs->rrsTextAttribute= above->rrsTextAttribute;
+
+	docCopyParagraphProperties( &(rrs->rrsParagraphProperties),
+				    &(above->rrsParagraphProperties) );
+	docCopySectionProperties( &(rrs->rrsSectionProperties),
+				    &(above->rrsSectionProperties) );
+	}
+
+    rrs->rrsPrev= rrc->rrcState;
+    rrc->rrcState= rrs;
+
+    return;
+    }
+
+void docRtfPopReadingState(	RtfReadingContext *	rrc )
+    {
+    RtfReadingState *	rrs= rrc->rrcState;
+
+    if  ( ! rrs )
+	{ XDEB(rrs); return;	}
+
+    /*
+    utilCleanTextAttribute( &(rrs->rrsTextAttribute) );
+    */
+    docCleanParagraphProperties( &(rrs->rrsParagraphProperties) );
+    docCleanSectionProperties( &(rrs->rrsSectionProperties) );
+
+    utilCleanMemoryBuffer( &(rrs->rrsSavedText) );
+
+    /**/
+    rrc->rrcState= rrs->rrsPrev;
+
+    return;
     }
 
 /************************************************************************/
@@ -244,10 +340,11 @@ static int docRtfReadControlWord(	SimpleInputStream *	sis,
 					int *			pGotArg,
 					int *			pArg )
     {
-    int		c;
-    int		len= 0;
-    int		arg;
-    int		sign= 1;
+    RtfReadingState *	rrs= rrc->rrcState;
+
+    int			c;
+    int			len= 0;
+    int			sign= 1;
 
     c= sioInGetCharacter( sis );
     while( c == '\n' || c == '\r' )
@@ -343,16 +440,39 @@ static int docRtfReadControlWord(	SimpleInputStream *	sis,
     if  ( c == '-' )
 	{ sign= -1; c= sioInGetCharacter( sis );	}
 
-    *pGotArg= isdigit( c );
+    *pGotArg= ( isdigit( c ) != 0 );
     if  ( *pGotArg )
 	{
-	arg= c- '0'; c= sioInGetCharacter( sis );
+	int arg= c- '0';
+
+	c= sioInGetCharacter( sis );
+
 	while( isdigit( c ) )
 	    { arg= 10* arg+ c- '0'; c= sioInGetCharacter( sis ); }
 
 	*pArg= sign* arg;
 	}
 
+    /* HACK */
+    if  ( *pGotArg && ! strcmp( controlWord, "u" ) )
+	{
+	if  ( *pArg < 0 )
+	    { *pArg= (int)0x1000+ *pArg;	}
+
+	if  ( *pArg >= 0 && *pArg <= 255 )
+	    {
+	    *pC= *pArg;
+
+	    rrs->rrsUnicodeBytesToSkip= rrs->rrsBytesPerUnicode;
+	    }
+	else{ *pC= ISO1_currency;	}
+
+	if  ( c != ' ' )
+	    { sioInUngetLastRead( sis );	}
+
+	return 2;
+	}
+
     if  ( c != ' ' )
 	{ sioInUngetLastRead( sis );	}
 
@@ -369,21 +489,8 @@ const RtfControlWord * docRtfFindWord(	const char *		controlWord,
 					const RtfControlWord *	contolWords,
 					int			listOnly )
     {
-    if  ( ! contolWords )
-	{ return (RtfControlWord *)0;	}
-
-    while( contolWords->rcwWord )
+    if  ( contolWords )
 	{
-	if  ( ! strcmp( contolWords->rcwWord, controlWord ) )
-	    { return contolWords;	}
-
-	contolWords++;
-	}
-
-    if  ( ! listOnly )
-	{
-	contolWords= docRtfPropertyWords;
-
 	while( contolWords->rcwWord )
 	    {
 	    if  ( ! strcmp( contolWords->rcwWord, controlWord ) )
@@ -393,6 +500,13 @@ const RtfControlWord * docRtfFindWord(	const char *		controlWord,
 	    }
 	}
 
+    if  ( ! listOnly )
+	{
+	contolWords= docRtfFindPropertyWord( controlWord );
+
+	return contolWords;
+	}
+
     return (RtfControlWord *)0;
     }
 
@@ -406,29 +520,11 @@ int docRtfApplyControlWord(	SimpleInputStream *	sis,
 	{ arg= -1;	}
 
     if  ( (*rcw->rcwApply) ( sis, rcw, arg, rrc ) )
-	{ SLDEB(rcw->rcwWord,arg); return -1;	}
+	{ LSLDEB(rrc->rrcCurrentLine,rcw->rcwWord,arg); return -1;	}
 
     return 0;
     }
 
-/************************************************************************/
-/*									*/
-/*  Just ignore a control word, a text particule.			*/
-/*									*/
-/************************************************************************/
-
-int docRtfIgnoreWord(	SimpleInputStream *	sis,
-			const RtfControlWord *	rcw,
-			int			arg,
-			RtfReadingContext *	rrc	)
-    { return 0;	}
-
-int docRtfIgnoreParticule(	RtfReadingContext *	rrc,
-				const unsigned char *	text,
-				int			len	)
-    { /* SDEB(text); */ return 0;	}
-
-
 /************************************************************************/
 /*									*/
 /*  Find out what to do.						*/
@@ -448,8 +544,10 @@ int docRtfFindControl(		SimpleInputStream *	sis,
 				int *			pGotArg,
 				int *			pArg		)
     {
-    int				c;
-    int				res;
+    RtfReadingState *	rrs= rrc->rrcState;
+
+    int			c;
+    int			res;
 
     if  ( rrc->rrcCharacterAhead != EOF )
 	{
@@ -487,13 +585,14 @@ int docRtfFindControl(		SimpleInputStream *	sis,
 			case '\'':
 			    if  ( res == 1 )
 				{
-				char	b[3];
+				char		b[3];
+				unsigned int	uc;
 
 				b[0]= sioInGetCharacter( sis );
 				b[1]= sioInGetCharacter( sis );
 				b[2]= '\0';
 
-				sscanf( b, "%x", &c );
+				sscanf( b, "%x", &uc ); c= uc;
 				}
 			    goto defaultCase;
 
@@ -535,18 +634,20 @@ int docRtfFindControl(		SimpleInputStream *	sis,
 
 			if  ( c=='\'' )
 			    {
-			    char	b[3];
+			    char		b[3];
+			    unsigned int	uc;
 
 			    b[0]= sioInGetCharacter( sis );
 			    b[1]= sioInGetCharacter( sis );
 			    b[2]= '\0';
 
-			    sscanf( b, "%x", &c );
+			    sscanf( b, "%x", &uc ); c= uc;
 			    }
 
-			if  ( res != 2 )
-			    { rrc->rrcCharacterAhead= rrc->rrcInputMapping[c]; }
-			else{ rrc->rrcCharacterAhead= c; }
+			if  ( rrs->rrsUnicodeBytesToSkip > 0 )
+			    { rrs->rrsUnicodeBytesToSkip= 0;	}
+
+			rrc->rrcCharacterAhead= c;
 
 			return RTFfiTEXTGROUP;
 			}
@@ -570,7 +671,11 @@ int docRtfFindControl(		SimpleInputStream *	sis,
 		*pC= c; return RTFfiTAB;
 
 	    default: defaultCase:
-		*pC= rrc->rrcInputMapping[c]; return RTFfiCHAR;
+
+		if  ( rrs && rrs->rrsUnicodeBytesToSkip > 0 )
+		    { rrs->rrsUnicodeBytesToSkip--; continue;	}
+
+		*pC= c; return RTFfiCHAR;
 	    }
 	}
     }
@@ -590,9 +695,15 @@ int docRtfFindControl(		SimpleInputStream *	sis,
 
 int docRtfAdjustLevel(	RtfReadingContext *	rrc,
 			int			toLevel,
-			int			textLevel	)
+			int			textLevel )
     {
-    BufferItem *	bi= rrc->rrcBi;
+    RtfReadingState *		rrs= rrc->rrcState;
+    BufferItem *		bi= rrc->rrcBi;
+    BufferDocument *		bd= rrc->rrcBd;
+    const DocumentProperties *	dp= &(bd->bdProperties);
+
+    const int * const		colorMap= (const int *)0;
+    const int * const		listStyleMap= (const int *)0;
 
     if  ( ! bi )
 	{ XDEB(bi); return -1;	}
@@ -602,13 +713,20 @@ int docRtfAdjustLevel(	RtfReadingContext *	rrc,
 	  bi->biLevel == DOClevPARA	&&
 	  bi->biParaParticuleCount == 0	)
 	{
+	const int	part= 0;
+	const int	stroff= 0;
+	const int	objectNumber= -1;
+
 	if  ( docInflateTextString( bi, 0 ) )
 	    { LDEB(bi->biParaStrlen); return -1;	}
 	bi->biParaString[bi->biParaStrlen]= '\0';
 
-	if  ( ! docInsertTextParticule( bi, -1, 0, 0,
-				DOCkindTEXT, rrc->rrcCurrentTextAttribute ) )
-	    { LDEB(bi->biParaParticuleCount); return -1;	}
+	if  ( docInsertAdminParticule( rrc->rrcBd, bi,
+			    stroff, part, objectNumber,
+			    &(rrs->rrsTextAttribute), DOCkindTEXT ) )
+	    { LDEB(1); return -1;	}
+
+	rrc->rrcAfterNoteref= 0;
 	}
 
     while( toLevel < rrc->rrcLevel )
@@ -620,8 +738,16 @@ int docRtfAdjustLevel(	RtfReadingContext *	rrc,
 
 	if  ( bi->biLevel == DOClevPARA )
 	    {
-	    PropertyMask	ppChgMask;
-	    PropertyMask	ppUpdMask;
+	    PropertyMask		ppChgMask;
+	    PropertyMask		ppUpdMask;
+	    int				paraNr;
+	    int				textAttributeNumber;
+
+	    ListNumberTreeNode *	root;
+
+	    textAttributeNumber= bi->biParaParticules->tpTextAttributeNumber;
+
+	    paraNr= docNumberOfParagraph( bi );
 
 	    if  ( docRtfFlushBookmarks( -1, bi, rrc ) )
 		{ LDEB(1); return -1;	}
@@ -630,11 +756,48 @@ int docRtfAdjustLevel(	RtfReadingContext *	rrc,
 
 	    PROPmaskCLEAR( &ppUpdMask );
 	    PROPmaskFILL( &ppUpdMask, PPprop_COUNT );
+	    PROPmaskUNSET( &ppUpdMask, PPpropIN_TABLE );
 
 	    /*  3  */
 	    if  ( docUpdParaProperties( &ppChgMask, &(bi->biParaProperties),
-				&ppUpdMask, &(rrc->rrcParagraphProperties) ) )
+				&ppUpdMask, &(rrs->rrsParagraphProperties),
+				colorMap, listStyleMap ) )
 		{ LDEB(1); return -1;	}
+
+	    if  ( bi->biParaListOverride > 0 )
+		{
+		int				fieldNr= -1;
+		int				partBegin= -1;
+		int				partEnd= -1;
+		int				stroffBegin= -1;
+		int				stroffEnd= -1;
+
+		const ListOverrideTable *	lot;
+
+		lot= &(dp->dpListOverrideTable);
+
+		if  ( bi->biParaListOverride >= lot->lotOverrideCount )
+		    {
+		    LLDEB(bi->biParaListOverride,lot->lotOverrideCount);
+		    return -1;
+		    }
+
+		if  ( docDelimitParaHeadField( &fieldNr, &partBegin, &partEnd,
+					&stroffBegin, &stroffEnd, bi, bd ) )
+		    {
+		    if  ( docInsertParaHeadField( bi, bd,
+					DOCfkLISTTEXT, textAttributeNumber ) )
+			{
+			LDEB(bi->biParaListOverride); return -1;
+			}
+		    }
+
+		root= &(bd->bdListNumberTrees[bi->biParaListOverride]);
+
+		if  ( docListNumberTreeInsertParagraph( root,
+					    bi->biParaListLevel, paraNr ) )
+		    { LDEB(paraNr);	}
+		}
 	    }
 
 	if  ( bi->biLevel == DOClevSECT )
@@ -664,6 +827,8 @@ int docRtfAdjustLevel(	RtfReadingContext *	rrc,
 	    return -1;
 	    }
 
+	rrc->rrcAfterNoteref= 0;
+
 	bi= fresh;
 	
 	rrc->rrcLevel++;
@@ -682,7 +847,8 @@ int docRtfAdjustLevel(	RtfReadingContext *	rrc,
 	    PROPmaskFILL( &ppUpdMask, PPprop_COUNT );
 
 	    if  ( docUpdParaProperties( &ppChgMask, &(bi->biParaProperties),
-				&ppUpdMask, &(rrc->rrcParagraphProperties) ) )
+				&ppUpdMask, &(rrs->rrsParagraphProperties),
+				colorMap, listStyleMap ) )
 		{ LDEB(1); return -1;	}
 
 	    /*  5  */
@@ -699,7 +865,7 @@ int docRtfAdjustLevel(	RtfReadingContext *	rrc,
 		    if  ( ! rowBi->biRowHasTableParagraphs )
 			{
 			if  ( docCopyRowProperties( &(rowBi->biRowProperties),
-						 &(rrc->rrcRowProperties) ) )
+				 &(rrc->rrcRowProperties), (const int *)0 ) )
 			    { LDEB(1); return -1;	}
 
 			rowBi->biRowHasTableParagraphs= 1;
@@ -728,7 +894,7 @@ int docRtfAdjustLevel(	RtfReadingContext *	rrc,
 			    }
 
 			if  ( docCopyRowProperties( &(rowBi->biRowProperties),
-						 &(rrc->rrcRowProperties) ) )
+				 &(rrc->rrcRowProperties), (const int *)0 ) )
 			    { LDEB(1); return -1;	}
 
 			rowBi->biRowHasTableParagraphs= 1;
@@ -741,17 +907,17 @@ int docRtfAdjustLevel(	RtfReadingContext *	rrc,
 		}
 	    }
 
-	if  ( rrc->rrcLevel == DOClevROW )
+	if  ( rrc->rrcLevel == DOClevROW && bi->biRowHasTableParagraphs )
 	    {
 	    if  ( docCopyRowProperties( &(bi->biRowProperties),
-					 &(rrc->rrcRowProperties) ) )
+				 &(rrc->rrcRowProperties), (const int *)0 ) )
 		{ LDEB(1); return -1;	}
 	    }
 
 	if  ( rrc->rrcLevel == DOClevSECT )
 	    {
 	    if  ( docCopySectionProperties( &(bi->biSectProperties),
-					     &(rrc->rrcSectionProperties) ) )
+					     &(rrs->rrsSectionProperties) ) )
 		{ LDEB(1); return -1;	}
 
 	    if  ( rrc->rrcParagraphNumberCount > 0 )
@@ -796,7 +962,7 @@ static int docRtfReadText(	SimpleInputStream *	sis,
 				int *			pArg,
 
 				RtfReadingContext *	rrc,
-				RtfAddTextParticule	addParticule	)
+				RtfAddTextParticule	addParticule )
     {
     int				res;
 
@@ -892,9 +1058,8 @@ static int docRtfReadText(	SimpleInputStream *	sis,
 		    collectedText= fresh;
 		    collectedSize= size;
 		    }
-		if  ( map )
-		    { collectedText[len++]= rrc->rrcInputMapping[c];	}
-		else{ collectedText[len++]= c;				}
+
+		collectedText[len++]= c;
 		collectedText[len  ]= '\0';
 		break;
 
@@ -925,7 +1090,8 @@ int docRtfSkipGroup(	SimpleInputStream *	sis,
 
     if  ( docRtfReadGroup( sis, rcw->rcwLevel,
 		(RtfControlWord *)0, 0, 0, rrc,
-		docRtfEmptyTable, docRtfEmptyTable, docRtfIgnoreParticule ) )
+		docRtfEmptyTable, docRtfEmptyTable,
+		docRtfIgnoreText, (RtfCommitGroup)0 ) )
 	{ SDEB(rcw->rcwWord); return -1;	}
 
     rrc->rrcComplainUnknown= complainUnknown;
@@ -941,41 +1107,39 @@ int docRtfSkipGroup(	SimpleInputStream *	sis,
 
 int docRtfReadGroup(	SimpleInputStream *	sis,
 			int			textLevel,
-
 			const RtfControlWord *	applyFirst,
 			int			gotArg,
 			int			arg,
-
-			RtfReadingContext *	externRrc,
-
+			RtfReadingContext *	rrc,
 			const RtfControlWord *	contolWords,
 			const RtfControlWord *	groupWords,
-			RtfAddTextParticule	addParticule	)
+			RtfAddTextParticule	addParticule,
+			RtfCommitGroup		commitGroup )
     {
     int				rval;
-    RtfReadingContext		internRrc;
+    RtfReadingState		internRrs;
 
-    docRtfInitReadingContext( &internRrc );
-    docRtfCopyReadingContext( &internRrc, externRrc );
+    docRtfPushReadingState( rrc, &internRrs );
 
-    externRrc->rrcCharacterAhead= EOF;
+    rrc->rrcCharacterAhead= EOF;
 
     if  ( applyFirst )
 	{
-	rval= docRtfApplyControlWord( sis, applyFirst,
-						    gotArg, arg, &internRrc );
+	rval= docRtfApplyControlWord( sis, applyFirst, gotArg, arg, rrc );
 	if  ( rval )
 	    { SDEB(applyFirst->rcwWord);	}
 	}
 
-    rval= docRtfConsumeGroup( sis, textLevel, &internRrc,
+    rval= docRtfConsumeGroup( sis, textLevel, rrc,
 				    contolWords, groupWords, addParticule );
 
     if  ( rval )
 	{ LDEB(rval);	}
 
-    docRtfCopyReadingContextBack( externRrc, &internRrc );
-    docRtfCleanReadingContext( &internRrc );
+    if  ( commitGroup && (*commitGroup)( rrc ) )
+	{ LDEB(1); rval= -1;	}
+
+    docRtfPopReadingState( rrc );
 
     return rval;
     }
@@ -996,7 +1160,7 @@ int docRtfReadUnknownGroup(	SimpleInputStream *	sis,
     if  ( docRtfReadGroup( sis, rrc->rrcLevel,
 					(RtfControlWord *)0, 0, 0, rrc,
 					docRtfEmptyTable, docRtfEmptyTable,
-					docRtfIgnoreParticule ) )
+					docRtfIgnoreText, (RtfCommitGroup)0 ) )
 	{ LDEB(1); return -1;	}
 
     rrc->rrcComplainUnknown= complainUnknown;
@@ -1028,15 +1192,15 @@ int docRtfReadWordGroup(	SimpleInputStream *	sis,
 	if  ( docRtfReadGroup( sis, textLevel,
 				rcw, gotArg, arg, rrc,
 				contolWords, groupWords,
-				addParticule ) )
+				addParticule, (RtfCommitGroup)0 ) )
 	    { SDEB(rcw->rcwWord); return -1;	}
 	}
     else{
 	if  ( rrc->rrcComplainUnknown )
-	    { SDEB(controlWord);	}
+	    { LSDEB(rrc->rrcCurrentLine,controlWord);	}
 
 	if  ( docRtfReadUnknownGroup( sis, rrc ) )
-	    { LDEB(1); return -1;	}
+	    { LSDEB(rrc->rrcCurrentLine,controlWord); return -1;	}
 	}
 
     return 0;
@@ -1067,6 +1231,10 @@ int docRtfConsumeGroup(	SimpleInputStream *	sis,
 
     for (;;)
 	{
+	const RtfControlWord *	rcwApplyFirst;
+	int			argApplyFirst;
+	int			gotArgApplyFirst;
+
 	switch( res )
 	    {
 	    case RTFfiCLOSE:
@@ -1082,9 +1250,11 @@ int docRtfConsumeGroup(	SimpleInputStream *	sis,
 		if  ( res < 0 )
 		    { SLDEB(controlWord,res); return -1;	}
 		break;
+
 	    case RTFfiTAB:
 		strcpy( controlWord, "tab" );
 		/*FALLTHROUGH*/
+
 	    case RTFfiWORD:
 		rcw= docRtfFindWord( controlWord, contolWords, 0 );
 		if  ( ! rcw )
@@ -1182,8 +1352,25 @@ int docRtfConsumeGroup(	SimpleInputStream *	sis,
 		if  ( rcw )
 		    { goto groupFound; }
 
+		rcw= docRtfFindWord( controlWord, groupWords, 0 );
+		if  ( rcw )
+		    {
+		    if  ( ! gotArg )
+			{ arg= -1;	}
+
+		    rcwApplyFirst= rcw;
+		    argApplyFirst= arg;
+		    gotArgApplyFirst= gotArg;
+
+		    goto textGroup;
+		    }
+
+		rrc->rrcInIgnoredGroup++;
+
 		if  ( docRtfReadUnknownGroup( sis, rrc ) )
-		    { LDEB(1); return -1;	}
+		    { LDEB(1); rrc->rrcInIgnoredGroup--; return -1;	}
+
+		rrc->rrcInIgnoredGroup--;
 
 		res= docRtfFindControl( sis, rrc, &c,
 					    controlWord, &gotArg, &arg );
@@ -1192,10 +1379,16 @@ int docRtfConsumeGroup(	SimpleInputStream *	sis,
 		break;
 
 	    case RTFfiTEXTGROUP:
+		rcwApplyFirst= (RtfControlWord *)0;
+		argApplyFirst= -1;
+		gotArgApplyFirst= 0;
+
+	      textGroup:
 		if  ( docRtfReadGroup( sis, textLevel,
-					(RtfControlWord *)0, 0, 0, rrc,
-					contolWords, groupWords,
-					addParticule ) )
+			    rcwApplyFirst, gotArgApplyFirst, argApplyFirst,
+			    rrc,
+			    contolWords, groupWords,
+			    addParticule, (RtfCommitGroup)0 ) )
 		    { LDEB(1); return -1;	}
 
 		res= docRtfFindControl( sis, rrc, &c,
@@ -1210,12 +1403,97 @@ int docRtfConsumeGroup(	SimpleInputStream *	sis,
 	}
     }
 
+/************************************************************************/
+/*									*/
+/*  Just ignore a control word.						*/
+/*									*/
+/************************************************************************/
+
+int docRtfIgnoreWord(	SimpleInputStream *	sis,
+			const RtfControlWord *	rcw,
+			int			arg,
+			RtfReadingContext *	rrc )
+    { return 0;	}
+
+/************************************************************************/
+/*									*/
+/*  Handle text..							*/
+/*									*/
+/*  1)  Ignore it.							*/
+/*  2)  Refuse it.							*/
+/*  3)  Save it for later use.						*/
+/*									*/
+/************************************************************************/
+
+/*  1  */
 int docRtfIgnoreText(	RtfReadingContext *	rrc,
 			const unsigned char *	text,
-			int			len	)
+			int			len )
     { return 0; }
 
+/*  2  */
 int docRtfRefuseText(	RtfReadingContext *	rrc,
 			const unsigned char *	text,
-			int			len	)
+			int			len )
     { LDEB(1); return -1; }
+
+/*  3  */
+int docRtfSaveText(	RtfReadingContext *	rrc,
+			const unsigned char *	text,
+			int			len )
+    {
+    RtfReadingState *	rrs= rrc->rrcState;
+
+    if  ( utilAddToMemoryBuffer( &(rrs->rrsSavedText), text, len ) )
+	{ LDEB(len); return -1;	}
+
+    return 0;
+    }
+
+int docRtfRememberText(	char **			pTarget,
+			int *			pSize,
+			RtfReadingContext *	rrc,
+			int			removeSemicolon )
+    {
+    return docRtfRememberUText( (unsigned char **)pTarget, pSize,
+						rrc, removeSemicolon );
+    }
+
+int docRtfRememberUText(	unsigned char **	pTarget,
+				int *			pSize,
+				RtfReadingContext *	rrc,
+				int			removeSemicolon )
+    {
+    RtfReadingState *	rrs= rrc->rrcState;
+
+    unsigned char *	fresh;
+    int			size;
+
+    if  ( rrs->rrsSavedText.mbSize == 0 )
+	{
+	*pSize= 0;
+	return 0;
+	}
+
+    size= rrs->rrsSavedText.mbSize;
+    fresh= realloc( *pTarget, size+ 1 );
+    if  ( ! fresh )
+	{ LXDEB(size,fresh); return -1;	}
+
+    memcpy( fresh, rrs->rrsSavedText.mbBytes, size );
+    fresh[size]= '\0';
+
+    if  ( removeSemicolon		&&
+	  size > 0			&&
+	  fresh[size- 1] == ';'		)
+	{
+	size--;
+	fresh[size]= '\0';
+	}
+
+    utilEmptyMemoryBuffer( &(rrs->rrsSavedText) );
+
+    *pTarget= fresh;
+    *pSize= size;
+    return 0;
+    }
