@@ -3,10 +3,12 @@
 
 #   include	<utilPropMask.h>
 #   include	<utilMemoryBuffer.h>
+#   include	<docExpandedTextAttribute.h>
 
-#   include	<appGeo.h>
+#   include	<geo2DInteger.h>
 #   include	<appDraw.h>
 #   include	"docBuf.h"
+#   include	"docScreenFontList.h"
 
 #   ifndef	TED_APP_H
 #   define	TED_APP_H
@@ -21,89 +23,97 @@
 
 typedef struct TedDocument
     {
-    BufferDocument *	tdDocument;
-    TextAttribute	tdCurrentTextAttribute;
-    int			tdCurrentPhysicalFont;
+    BufferDocument *		tdDocument;
+    TextAttribute		tdCurrentTextAttribute;
+    int				tdCurrentTextAttributeNumber;
+    int				tdCurrentScreenFont;
 
-    DocumentSelection	tdDocumentSelection;
-    SelectionGeometry	tdSelectionGeometry;
+    DocumentSelection		tdDocumentSelection;
+    SelectionGeometry		tdSelectionGeometry;
+    SelectionDescription	tdSelectionDescription;
 
-    APP_WIDGET		tdCopyWidget;
-    APP_WIDGET		tdCutWidget;
-    APP_WIDGET		tdPasteWidget;
+    ScreenFontList		tdScreenFontList;
 
-    APP_WIDGET		tdInsertMenu;
-    APP_WIDGET		tdInsertMenuButton;
-    APP_WIDGET		tdInsPictOption;
-    APP_WIDGET		tdInsFileOption;
-    APP_WIDGET		tdInsSymbolOption;
-    APP_WIDGET		tdInsHyperlinkOption;
-    APP_WIDGET		tdInsBookmarkOption;
-    APP_WIDGET		tdInsInsertFootnoteOption;
-    APP_WIDGET		tdInsInsertChftnsepOption;
-    APP_WIDGET		tdInsInsertTableOption;
-    APP_WIDGET		tdInsInsertPageNumberOption;
-    APP_WIDGET		tdInsInsertLineBreakOption;
-    APP_WIDGET		tdInsInsertPageBreakOption;
-    APP_WIDGET		tdInsInsertSectBreakOption;
+    APP_WIDGET			tdCopyWidget;
+    APP_WIDGET			tdCutWidget;
+    APP_WIDGET			tdPasteWidget;
 
-    APP_WIDGET		tdTableMenu;
-    APP_WIDGET		tdTableMenuButton;
-    APP_WIDGET		tdTabInsertTableOption;
-    APP_WIDGET		tdTabAddRowOption;
-    APP_WIDGET		tdTabAddColumnOption;
+    APP_WIDGET			tdInsertMenu;
+    APP_WIDGET			tdInsertMenuButton;
+    APP_WIDGET			tdInsPictOption;
+    APP_WIDGET			tdInsFileOption;
+    APP_WIDGET			tdInsSymbolOption;
+    APP_WIDGET			tdInsHyperlinkOption;
+    APP_WIDGET			tdInsBookmarkOption;
+    APP_WIDGET			tdInsInsertFootnoteOption;
+    APP_WIDGET			tdInsInsertEndnoteOption;
+    APP_WIDGET			tdInsInsertChftnsepOption;
+    APP_WIDGET			tdInsInsertTableOption;
+    APP_WIDGET			tdInsInsertPageNumberOption;
+    APP_WIDGET			tdInsInsertLineBreakOption;
+    APP_WIDGET			tdInsInsertPageBreakOption;
+    APP_WIDGET			tdInsInsertSectBreakOption;
 
-    APP_WIDGET		tdFormatMenu;
-    APP_WIDGET		tdFormatMenuButton;
-    APP_WIDGET		tdFormatOneParaOption;
+    APP_WIDGET			tdTableMenu;
+    APP_WIDGET			tdTableMenuButton;
+    APP_WIDGET			tdTabInsertTableOption;
+    APP_WIDGET			tdTabAddRowOption;
+    APP_WIDGET			tdTabAddColumnOption;
 
-    APP_WIDGET		tdSelectTableWidget;
-    APP_WIDGET		tdSelectRowWidget;
-    APP_WIDGET		tdSelectColumnOption;
-    APP_WIDGET		tdDrawTableGridOption;
+    APP_WIDGET			tdFormatMenu;
+    APP_WIDGET			tdFormatMenuButton;
+    APP_WIDGET			tdFormatOneParaOption;
 
-    APP_WIDGET		tdFontMenu;
-    APP_WIDGET		tdFontMenuButton;
-    APP_WIDGET		tdFontBoldOption;
-    APP_WIDGET		tdFontItalicOption;
-    APP_WIDGET		tdFontUnderlinedOption;
-    APP_WIDGET		tdFontSuperscriptOption;
-    APP_WIDGET		tdFontSubscriptOption;
+    APP_WIDGET			tdSelectTableWidget;
+    APP_WIDGET			tdSelectRowWidget;
+    APP_WIDGET			tdSelectColumnOption;
+    APP_WIDGET			tdDrawTableGridOption;
 
-    APP_WIDGET		tdToolsMenu;
-    APP_WIDGET		tdToolsMenuButton;
-    APP_WIDGET		tdToolsFormatToolOption;
+    APP_WIDGET			tdFontMenu;
+    APP_WIDGET			tdFontMenuButton;
+    APP_WIDGET			tdFontBoldOption;
+    APP_WIDGET			tdFontItalicOption;
+    APP_WIDGET			tdFontUnderlinedOption;
+    APP_WIDGET			tdFontSuperscriptOption;
+    APP_WIDGET			tdFontSubscriptOption;
 
-    APP_COLOR_RGB	tdSelColor;
-    APP_COLOR_RGB	tdXSelColor;
-    APP_COLOR_RGB	tdFieldColor;
-    APP_COLOR_RGB	tdTableColor;
+    APP_WIDGET			tdToolsMenu;
+    APP_WIDGET			tdToolsMenuButton;
+    APP_WIDGET			tdToolsFormatToolOption;
 
-    int			tdVisibleSelectionCopied;
-    int			tdCanReplaceSelection;
-    int			tdCopiedSelectionClosed;
-    MemoryBuffer	tdCopiedSelection;
-    MemoryBuffer	tdCopiedFont;
-    MemoryBuffer	tdCopiedRuler;
-    AppBitmapImage	tdCopiedImage;
+    APP_COLOR_RGB		tdFieldColor;
+
+    RGB8Color			tdTableColor;
+    RGB8Color			tdSelColor;
+    RGB8Color			tdCopiedSelColor;
+
+    int				tdVisibleSelectionCopied;
+    int				tdCanReplaceSelection;
+    int				tdCopiedSelectionClosed;
+    MemoryBuffer		tdCopiedSelection;
+    MemoryBuffer		tdCopiedFont;
+    MemoryBuffer		tdCopiedRuler;
+    AppBitmapImage		tdCopiedImage;
+
+    void *			tdFindProg;
 
 #   ifdef USE_MOTIF
-    XtIntervalId	tdHideIBarId;
-    XtIntervalId	tdShowIBarId;
+    XtIntervalId		tdHideIBarId;
+    XtIntervalId		tdShowIBarId;
 #   endif
 
 #   ifdef USE_GTK
-    guint		tdHideIBarId;
-    guint		tdShowIBarId;
+    guint			tdHideIBarId;
+    guint			tdShowIBarId;
 #   endif
 
-    APP_WINDOW		tdObjectWindow;
-    APP_WINDOW		tdObjectBottomWindow;
-    APP_WINDOW		tdObjectRightWindow;
-    APP_WINDOW		tdObjectCornerWindow;
-    int			tdObjectSelected;
+    APP_WINDOW			tdObjectWindow;
+    APP_WINDOW			tdObjectBottomWindow;
+    APP_WINDOW			tdObjectRightWindow;
+    APP_WINDOW			tdObjectCornerWindow;
+    int				tdObjectSelected;
 
-    int			tdDrawTableGrid;
+    int				tdDrawTableGrid;
     } TedDocument;
 
 /************************************************************************/
@@ -145,12 +155,22 @@ typedef struct TedAppResources
     char *			tarDefaultAnsicpgString;
     int				tarDefaultAnsicpgInt;
 
+    char *			tarShowTableGridString;
+    int				tarShowTableGridInt;
+
 				/****************************************/
 				/*  Global data.			*/
 				/****************************************/
 
     AppInspector *		tarInspector;
     EditDocument *		tarManualDocument;
+
+				/****************************************/
+				/*  For Ted --Find and Ted --RegFind	*/
+				/****************************************/
+
+    const unsigned char *	tarFindPattern;
+    int				tarFindRegex;
     } TedAppResources;
 
 /************************************************************************/
@@ -212,37 +232,28 @@ extern AppSelectionTargetType TedRulerTargets[];
 extern int tedLayoutItem(	BufferItem *			bi,
 				BufferDocument *		bd,
 				AppDrawingData *		add,
-				DocumentRectangle *		drChanged );
-
-extern int tedLayoutExternalItem(
-				int *				pY1Twips,
-				ExternalItem *			ei,
-				int				page,
-				int				y0Twips,
-				BufferDocument *		bd,
-				AppDrawingData *		add,
+				ScreenFontList *		sfl,
 				DocumentRectangle *		drChanged );
 
 extern int tedFindPosition(	DocumentPosition *		dp,
 				PositionGeometry *		pg,
+				const BufferDocument *		bd,
 				BufferItem *			rootBi,
 				const AppDrawingData *		add,
+				const ScreenFontList *		sfl,
 				int				x,
 				int				y );
 
-extern int tedCalculateX(	const BufferItem *		bi,
-				const TextParticule *		tp,
-				const AppDrawingData *		add,
-				int				stroff );
-
 extern int tedFindParticule(	TextLine *			tl,
 				TextParticule *			tp,
 				int				x,
 				int				y );
 
-extern int tedFindStringOffset(	const TextParticule *		tp,
-				const unsigned char *		paraString,
+extern int tedFindStringOffset(	const BufferDocument *		bd,
+				const BufferItem *		paraBi,
+				int				part,
 				const AppDrawingData *		add,
+				const ScreenFontList *		sfl,
 				int *				pBarX,
 				int				x,
 				int				y );
@@ -250,24 +261,19 @@ extern int tedFindStringOffset(	const TextParticule *		tp,
 extern int tedHasSelection(	const TedDocument *		td );
 extern int tedHasIBarSelection(	const TedDocument *		td );
 
-extern int tedLineDown(		DocumentPosition *		bp,
-				const AppDrawingData *		add );
+extern int tedArrowDown(	DocumentPosition *		bp,
+				const PositionGeometry *	pg,
+				const BufferDocument *		bd,
+				const AppDrawingData *		add,
+				const ScreenFontList *		sfl );
 
-extern int tedLineUp(		DocumentPosition *		bp,
-				const AppDrawingData *		add );
+extern int tedArrowUp(		DocumentPosition *		bp,
+				const PositionGeometry *	pg,
+				const BufferDocument *		bd,
+				const AppDrawingData *		add,
+				const ScreenFontList *		sfl );
 
-extern int tedPageUp(	DocumentPosition *			dp,
-			BufferDocument *			bd,
-			const AppDrawingData *			add,
-			int					pageHeight );
-
-extern int tedPageDown(	DocumentPosition *			dp,
-			BufferDocument *			bd,
-			const AppDrawingData *			add,
-			int					docHeight,
-			int					pageHeight );
-
-extern APP_EVENT_HANDLER( tedObserveFocus, w, voided, event );
+extern APP_EVENT_HANDLER_H( tedObserveFocus, w, voided, event );
 
 extern void tedSplitParagraph(		EditDocument *		ed,
 					int			onNewPage );
@@ -276,8 +282,8 @@ extern void tedEditReplaceSelectionWithTab(	EditDocument *		ed );
 
 extern int tedChangeSelectionProperties(
 				EditDocument *			ed,
-				const PropertyMask *		taUpdMask,
-				TextAttribute			taNew,
+				const PropertyMask *		taSetMask,
+				const TextAttribute *		taSet,
 				const PropertyMask *		ppUpdMask,
 				const ParagraphProperties *	ppNew,
 				const PropertyMask *		spUpdMask,
@@ -285,7 +291,7 @@ extern int tedChangeSelectionProperties(
 
 extern void tedRedrawRectangle(	APP_WIDGET		w,
 				TedDocument *		td,
-				DocumentRectangle *	drClip,
+				const DocumentRectangle *	drClip,
 				AppDrawingData *	add,
 				AppColors *		ac,
 				int			ox,
@@ -298,6 +304,10 @@ extern void tedDocToolFind(		APP_WIDGET	findOption,
 					void *		voided,
 					void *		voidcbs );
 
+extern void tedDocToolFindNext(		APP_WIDGET	findOption,
+					void *		voided,
+					void *		voidcbs );
+
 extern void tedDocToolSpell(		APP_WIDGET	spellOption,
 					void *		voided,
 					void *		voidcbs );
@@ -313,12 +323,10 @@ extern int tedIncludeRtfDocument(	APP_WIDGET		w,
 extern int tedGetDocumentAttributeString(	char *		scratch,
 						TedDocument *	td );
 
-extern int tedGetDocumentAttributes(	TedDocument *		td,
-					const char **		pFamilyName,
-					int *			pPsFamilyNumber,
-					int *			pEncoding,
-					PropertyMask *		pUpdMask,
-					TextAttribute *		pTaNew );
+extern int tedGetDocumentAttributes(
+				TedDocument *			td,
+				PropertyMask *			pUpdMask,
+				TextAttribute *			pTaNew );
 
 extern void tedDocCopy(		EditDocument *	ed );
 extern void tedDocCut(		EditDocument *	ed );
@@ -327,29 +335,35 @@ extern void tedDocSelAll(	EditDocument *	ed );
 
 extern void tedSetSelection(	EditDocument *			ed,
 				const DocumentSelection *	dsSet,
+				int				lastLine,
 				int *				pScrolledX,
 				int *				pScrolledY );
 
-extern void tedAppReplaceSelection(	EditDocument *		ed,
+extern void tedDocEditPaste(	APP_WIDGET			option,
+				void *				voided,
+				void *				voidpbcs );
+
+extern void tedDocReplaceSelection(	EditDocument *		ed,
 					const unsigned char *	word,
-					int			len	);
+					int			len );
+
+extern int tedDeleteTableSliceSelection(	EditDocument *	ed );
 
 extern void tedSetSelectedPosition(
 				EditDocument *			ed,
 				const DocumentPosition *	dp,
+				int				lastLine,
 				int *				pScrolledX,
 				int *				pScrolledY );
 
-extern void tedShowFontsTool(		APP_WIDGET		fontsOption,
-					EditApplication *	ea,
-					void *			voidcbs	 );
+extern int tedSelectItemHome(	EditDocument *			ed,
+				BufferItem *			bi,
+				int *				pScrolledX,
+				int *				pScrolledY );
 
 extern void tedAdaptFontIndicatorsToSelection(	EditApplication *	ea,
 						EditDocument *		ed );
 
-extern void tedAdaptFontsToolToDocument(	EditApplication *	ea,
-						EditDocument *		ed );
-
 extern void tedAdaptToolsToSelection(	EditDocument *		ed );
 
 extern int tedExtendSelectionToXY(
@@ -454,6 +468,10 @@ extern void tedDocInsertFootnote(	APP_WIDGET	option,
 					void *		voided,
 					void *		voidpbcs );
 
+extern void tedDocInsertEndnote(	APP_WIDGET	option,
+					void *		voided,
+					void *		voidpbcs );
+
 extern void tedDocInsertChftnsep(	APP_WIDGET	option,
 					void *		voided,
 					void *		voidpbcs );
@@ -518,6 +536,7 @@ extern InsertedObject * tedObjectMakeBitmapObject(
 					AppBitmapImage *	abi );
 
 extern int tedGetObjectSelection(	TedDocument *   	td,
+					int *			pPart,
 					DocumentPosition *	dpObject,
 					InsertedObject **	pIo );
 
@@ -530,6 +549,7 @@ extern void tedSetObjectWindows(EditDocument *			ed,
 extern void tedMoveObjectWindows(	EditDocument *		ed );
 
 extern int tedResizeObject(		EditDocument *			ed,
+					int				part,
 					const DocumentPosition *	dpObj,
 					const PositionGeometry *	pgObj );
 
@@ -537,7 +557,6 @@ extern int tedReopenObject(	BufferDocument *	bd,
 				BufferItem *		bi,
 				TextParticule *		tp,
 				AppColors *		ac,
-				DocumentFontList *	dfl,
 				AppDrawingData *	add );
 
 extern int tedSaveObjectPicture(	AppBitmapImage *	abiTo,
@@ -550,12 +569,18 @@ extern void tedExposeSelection(	const EditDocument *		ed,
 
 extern void tedPositionGeometry(	PositionGeometry *		pg,
 					const DocumentPosition *	dp,
-					const AppDrawingData *		add );
-
-extern void tedSelectionGeometry(	SelectionGeometry *		sg,
-					const DocumentSelection *	ds,
+					int				lastOne,
 					const BufferDocument *		bd,
-					const AppDrawingData *		add );
+					const AppDrawingData *		add,
+					const ScreenFontList *		sfl );
+
+extern void tedSelectionGeometry(
+			SelectionGeometry *		sg,
+			const DocumentSelection *	ds,
+			int				lastLine,
+			const BufferDocument *		bd,
+			const AppDrawingData *		add,
+			const ScreenFontList *		sfl );
 
 extern void tedDrawRectangle(	APP_WIDGET		w,
 				EditDocument *		ed,
@@ -571,8 +596,8 @@ extern void tedDocVerticalScrollbarCallback(	APP_WIDGET	w,
 						void *		voided,
 						void *		voidscbs );
 
-extern int tedFinishDocumentSetup(	EditApplication *	ea,
-					EditDocument *		ed );
+extern int tedFinishDocumentSetup(	EditDocument *		ed );
+extern void tedDocumentFirstVisible(	EditDocument *		ed );
 
 extern void tedDocFormatCopyRul(	APP_WIDGET	fontsOption,
 					void *		voided,
@@ -590,12 +615,6 @@ extern int tedLayoutDocument(	void *				privateData,
 extern int tedMakeDocumentWidget(	EditApplication *	ea,
 					EditDocument *		ed );
 
-extern int tedChangeParticuleAttribute( AppDrawingData *	add,
-					TextParticule *		tp,
-					DocumentFontList *	dfl,
-					const PropertyMask * 	updMask,
-					TextAttribute		taNew );
-
 extern int tedInsertTable(		EditDocument *		ed,
 					int			rows,
 					int			columns );
@@ -621,14 +640,20 @@ extern void tedDocToolInsertSymbol(	APP_WIDGET	symbolOption,
 					void *		voided,
 					void *		voidcbs );
 
-extern void tedAppSetTableProperties(	EditApplication *	ea,
+extern void tedDocSetTableProperties(	EditDocument *		ed,
 					const TableRectangle *	tr,
+					const PropertyMask *	rpSetMask,
+					const PropertyMask *	cpSetMask,
 					const RowProperties *	rp );
 
-extern int tedGetTableRectangle(	TableRectangle *		tr,
-					const DocumentSelection *	ds );
+extern void tedAppSetTableProperties(	EditApplication *	ea,
+					const TableRectangle *	tr,
+					const PropertyMask *	rpSetMask,
+					const PropertyMask *	cpSetMask,
+					const RowProperties *	rp );
 
-extern void tedAdaptFormatToolToDocument(	EditDocument *	ed );
+extern void tedAdaptFormatToolToDocument(	EditDocument *	ed,
+						int		choosePage );
 
 extern void tedAppSetTableSelection(	EditDocument *		ed,
 					const TableRectangle *	tr );
@@ -638,14 +663,18 @@ extern void tedDocTableSelectTableRectangle(	EditDocument *		ed,
 
 extern void tedChangeTableLayout(	EditDocument *		ed,
 					BufferItem *		sectBi,
-					const RowProperties *	rp,
-					int			rowHeightAlso,
-					int			row0,
-					int			row1 );
+					int			row0Change,
+					int			row1Change,
+					int			col0Change,
+					int			col1Change,
+					int			row1Reformat,
+					const PropertyMask *	rpSetMask,
+					const PropertyMask *	cpSetMask,
+					const RowProperties *	rp );
 
 extern void tedDocTableDrawGrid(	APP_WIDGET	option,
 					void *		voided,
-					void *		voidcbs	 );
+					void *		voidcbs );
 
 extern void tedMergeParagraphsInSelection(	EditDocument *	ed );
 
@@ -664,11 +693,13 @@ extern int tedRunPropertyDialog( EditDocument *			ed,
 				APP_WIDGET			option,
 				const char *			pixmapName );
 
-extern int tedBorderThick(		int *				pWide,
-					const BorderProperties *	bp,
-					const AppDrawingData *		add );
+extern int tedDocSetHyperlink(	EditDocument *		ed,
+				const char *		file,
+				const char *		mark,
+				int			asRef,
+				int			asPageref );
 
-extern int tedSetHyperlink(	EditDocument *		ed,
+extern int tedAppSetHyperlink(	EditApplication *	ea,
 				const char *		file,
 				const char *		mark,
 				int			asRef,
@@ -677,7 +708,8 @@ extern int tedSetHyperlink(	EditDocument *		ed,
 extern int tedSetBookmark(	EditDocument *		ed,
 				const char *		mark );
 
-extern int tedRemoveHyperlink(	EditDocument *		ed );
+extern int tedDocRemoveHyperlink(	EditDocument *		ed );
+extern int tedAppRemoveHyperlink(	EditApplication *	ea );
 
 extern int tedRemoveBookmark(	EditDocument *		ed );
 
@@ -686,14 +718,6 @@ extern int tedCopyBookmarkAsLink(	EditDocument *	ed,
 					int		asPageref,
 					const char *	link );
 
-extern void tedRunLinkDialog(	EditApplication *	ea,
-				EditDocument *		ed,
-				APP_WIDGET		option,
-				const char *		fileName,
-				int			fileSize,
-				const char *		markName,
-				int			markSize );
-
 extern void tedRunBookmarkDialog(	EditApplication *	ea,
 					EditDocument *		ed,
 					APP_WIDGET		option,
@@ -715,14 +739,20 @@ extern void * tedMakePrivateData( void );
 
 extern void tedMakeDocumentReadonly(	EditDocument *	ed );
 
-extern int tedFollowLink(	APP_WIDGET		relative,
-				APP_WIDGET		option,
+extern int tedDocFollowLink(	APP_WIDGET		option,
 				EditDocument *		edFrom,
 				const char *		fileName,
 				int			fileSize,
 				const char *		markName,
 				int			markSize );
 
+extern int tedAppFollowLink(	APP_WIDGET		option,
+				EditApplication *	ea,
+				const char *		fileName,
+				int			fileSize,
+				const char *		markName,
+				int			markSize );
+
 extern int tedSelectWholeParagraph(	EditApplication *	ea,
 					int			direction );
 
@@ -742,26 +772,28 @@ extern int tedGetParaLineHeight(	int *			pLineHeight,
 					EditDocument *		ed );
 
 extern void tedShowFormatTool(	APP_WIDGET		toolOption,
-				EditApplication *	ea,
-				const char *		widgetName,
-				const char *		pixmapName );
+				EditApplication *	ea );
 
 extern void tedFormatToolAdaptToSelection( AppInspector *	ai,
-				BufferDocument *		bd,
+				EditDocument *			ed,
+				int				choosePage,
 				const DocumentSelection *	ds,
 				const SelectionGeometry *	sg,
-				int				fileReadonly );
+				const SelectionDescription *	sd );
 
 extern int tedNewDocument(	EditApplication *	ea,
 				EditDocument *		ed,
 				const char *		filename );
 
-extern int tedPrintDocument(	FILE *				f,
+extern int tedPrintDocument(	SimpleOutputStream *		sos,
 				const PrintJob *		pj,
 				const PrintGeometry *		pg,
 				int				firstPage,
 				int				lastPage );
 
+extern void tedSuggestNup(	PrintGeometry *			pg,
+				void *				privateData );
+
 extern void tedFreeDocument(		void *			voidtd,
 					int			format,
 					AppDrawingData *	add );
@@ -769,23 +801,26 @@ extern void tedFreeDocument(		void *			voidtd,
 extern void tedRedoDocumentLayout(	EditDocument *		ed );
 
 extern void tedFormatShowPagePage( EditApplication *		ea );
+extern void tedFormatShowFontPage( EditApplication *		ea );
+extern void tedFormatShowLinkPage( EditApplication *		ea );
 
 extern void tedSetPageLayout(	EditApplication *		ea,
 				const DocumentGeometry *	dg,
 				const PropertyMask *		setMask,
 				int				whileDocument );
 
-extern int tedSetDocumentProperties(
+extern int tedAppSetDocumentProperties(
 				EditApplication *		ea,
 				const DocumentProperties *	dpNew,
 				const PropertyMask *		updMask );
 
-extern int tedChangeCurrentNote(
-				EditApplication *		ea,
+extern int tedChangeCurrentNote( EditApplication *		ea,
+				int				autoNumber,
+				const unsigned char *		fixedText,
 				int				extItKind );
 
-extern APP_EVENT_HANDLER( tedKeyPressed, w, voided, keyEvent );
-extern APP_EVENT_HANDLER( tedMouseButtonPressed, w, voided, downEvent );
+extern APP_EVENT_HANDLER_H( tedKeyPressed, w, voided, keyEvent );
+extern APP_EVENT_HANDLER_H( tedMouseButtonPressed, w, voided, downEvent );
 
 extern void tedHideObjectWindows(	EditDocument *	ed );
 
@@ -794,11 +829,25 @@ extern int tedAppChangeSectionProperties(
 				const PropertyMask *		spUpdMask,
 				const SectionProperties *	spNew );
 
+extern int tedAppChangeAllSectionProperties(
+				EditApplication *		ea,
+				const PropertyMask *		spUpdMask,
+				const SectionProperties *	spNew );
+
 extern int tedAppChangeParagraphProperties(
 				EditApplication *		ea,
 				const PropertyMask *		ppUpdMask,
 				const ParagraphProperties *	ppNew );
 
+extern int tedAppChangeCurrentList(
+				EditApplication *		ea,
+				const DocumentList *		dlNew );
+
+extern int tedDocChangeParagraphProperties(
+				EditDocument *			ed,
+				const PropertyMask *		ppUpdMask,
+				const ParagraphProperties *	ppNew );
+
 extern void tedAppDeleteHeaderFooter(	EditApplication *	ea,
 					int			which );
 
@@ -809,16 +858,12 @@ extern void tedAppEditHeaderFooter(	EditApplication *	ea,
 
 extern int tedGetSelection(	DocumentSelection *	ds,
 				SelectionGeometry *	sg,
+				SelectionDescription *	sd,
 				TedDocument *		td );
 
 extern void tedFillBookmarkList(	APP_WIDGET		list,
 					DocumentFieldList *	dfl );
 
-extern int tedTextWidth(		const AppPhysicalFont *		apf,
-					const AppDrawingData *		add,
-					const unsigned char *		s,
-					int				len );
-
 extern int tedFindRootForPosition(	ExternalItem **		pEi,
 					BufferItem **		pRootBi,
 					BufferItem **		pBodySectBi,
@@ -828,7 +873,7 @@ extern int tedFindRootForPosition(	ExternalItem **		pEi,
 					int			docX,
 					int			docY );
 
-extern int tedDetermineDefaultCodepage(	TedAppResources *	tar );
+extern void tedDetermineDefaultSettings(	TedAppResources *	tar );
 
 extern int tedReplaceSelectionWithBitmapImage(	EditDocument *		ed,
 						AppBitmapImage *	abi );
@@ -842,4 +887,53 @@ extern int tedSetIBarSelection(		EditDocument *		ed,
 					int *			pScrolledX,
 					int *			pScrolledY );
 
+extern void tedDrawShadedRectangle(
+			AppDrawingData *	add,
+			APP_BITMAP_IMAGE	shadingPixmaps[DOCsp_COUNT],
+			int			pattern,
+			int			x0,
+			int			x1,
+			int			y0,
+			int			y1 );
+
+extern void tedDrawHorizontalBorderLine(AppDrawingData *	add,
+					int			style,
+					int			minThick,
+					int			x0,
+					int			x1,
+					int			y );
+
+extern int tedAppAllocateDocumentColor(	EditApplication *	ea,
+					const RGB8Color *	rgb8 );
+
+extern int tedDocAllocateDocumentColor(	EditDocument *		ed,
+					const RGB8Color *	rgb8 );
+
+extern void tedFontToolSet(	void *				voidea,
+				const ExpandedTextAttribute *	etaSet,
+				const PropertyMask *		taSetMask );
+
+extern void tedListFontToolSet(	void *				voidea,
+				const ExpandedTextAttribute *	etaSet,
+				const PropertyMask *		taSetMask );
+
+extern int tedOpenDocumentFile(	EditApplication *	ea,
+				int *			pFormat,
+				BufferDocument **	pBd,
+				const char *		filename,
+				APP_WIDGET		relative,
+				APP_WIDGET		option );
+
+extern int tedDocFindNext(	EditDocument *		ed );
+extern int tedDocFindPrev(	EditDocument *		ed );
+
+extern int tedFindSetPattern(	EditDocument *		ed,
+				const unsigned char *	pattern,
+				int			useRegex );
+
+extern int tedDrawDrawingPixmap( APP_BITMAP_IMAGE		pixmap,
+				const InsertedObject *		io,
+				AppColors *			ac,
+				AppDrawingData *		parentAdd );
+
 #   endif	/*  TED_APP_H	*/
