@@ -24,7 +24,9 @@ typedef struct BookmarkContext
     AppDialog		bcDialog;
     APP_WIDGET		bcPaned;
 
-    APP_WIDGET		bcMarkTextWidget;
+    APP_WIDGET		bcDestinationLabel;
+    APP_WIDGET		bcDestinationText;
+
     APP_WIDGET		bcMarkListWidget;
 
     APP_WIDGET		bcButtonRow;
@@ -114,30 +116,26 @@ static AppConfigurableResource TED_BookmarkResourceTable[]=
 /*									*/
 /************************************************************************/
 
-static APP_CLOSE_CALLBACK( tedBookmarkDialogClosed, w, voidbc )
+static APP_CLOSE_CALLBACK_H( tedBookmarkDialogClosed, w, voidbc )
     {
     BookmarkContext *		bc= (BookmarkContext *)voidbc;
 
     appGuiBreakDialog( &(bc->bcDialog), AQDrespCANCEL ); return;
     }
 
-static void tedBookmarkCancelPushed(	APP_WIDGET	button,
-					void *		voidbc,
-					void *		call_data )
+static APP_BUTTON_CALLBACK_H( tedBookmarkCancelPushed, button, voidbc )
     {
     BookmarkContext *		bc= (BookmarkContext *)voidbc;
 
     appGuiBreakDialog( &(bc->bcDialog), AQDrespCANCEL ); return;
     }
 
-static void tedBookmarkSetPushed(	APP_WIDGET	button,
-					void *		voidbc,
-					void *		call_data )
+static APP_BUTTON_CALLBACK_H( tedBookmarkSetPushed, button, voidbc )
     {
     BookmarkContext *		bc= (BookmarkContext *)voidbc;
     char *			markName;
 
-    markName= appGetStringFromTextWidget( bc->bcMarkTextWidget );
+    markName= appGetStringFromTextWidget( bc->bcDestinationText );
     if  ( ! markName )
 	{
 	XDEB(markName);
@@ -153,14 +151,12 @@ static void tedBookmarkSetPushed(	APP_WIDGET	button,
     appGuiBreakDialog( &(bc->bcDialog), AQDrespOK ); return;
     }
 
-static void tedBookmarkGotoPushed(	APP_WIDGET	button,
-					void *		voidbc,
-					void *		call_data )
+static APP_BUTTON_CALLBACK_H( tedBookmarkGotoPushed, button, voidbc )
     {
     BookmarkContext *		bc= (BookmarkContext *)voidbc;
     char *			markName;
 
-    markName= appGetStringFromTextWidget( bc->bcMarkTextWidget );
+    markName= appGetStringFromTextWidget( bc->bcDestinationText );
     if  ( ! markName )
 	{
 	XDEB(markName);
@@ -175,9 +171,7 @@ static void tedBookmarkGotoPushed(	APP_WIDGET	button,
     appGuiBreakDialog( &(bc->bcDialog), AQDrespOK ); return;
     }
 
-static void tedBookmarkRemovePushed(	APP_WIDGET	button,
-					void *		voidbc,
-					void *		call_data )
+static APP_BUTTON_CALLBACK_H( tedBookmarkRemovePushed, w, voidbc )
     {
     BookmarkContext *		bc= (BookmarkContext *)voidbc;
 
@@ -187,9 +181,7 @@ static void tedBookmarkRemovePushed(	APP_WIDGET	button,
     appGuiBreakDialog( &(bc->bcDialog), AQDrespOK ); return;
     }
 
-static void tedBookmarkCopyPushed(	APP_WIDGET	button,
-					void *		voidbc,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedBookmarkCopyPushed, w, voidbc )
     {
     BookmarkContext *			bc= (BookmarkContext *)voidbc;
     char *				markName;
@@ -221,7 +213,7 @@ static void tedBookmarkCopyPushed(	APP_WIDGET	button,
 	    break;
 	}
 
-    markName= appGetStringFromTextWidget( bc->bcMarkTextWidget );
+    markName= appGetStringFromTextWidget( bc->bcDestinationText );
     if  ( ! markName )
 	{
 	XDEB(markName);
@@ -244,28 +236,28 @@ static void tedBookmarkCopyPushed(	APP_WIDGET	button,
 /*									*/
 /************************************************************************/
 
-static void tedBookmarkDestinationChanged(	APP_WIDGET	w,
-						void *		voidbc,
-						void *		call_data )
+static APP_TXTYPING_CALLBACK_H( tedBookmarkDestinationChanged, w, voidbc )
     {
     BookmarkContext *		bc= (BookmarkContext *)voidbc;
 
-    char *			text;
-    int				len;
+    char *			markName;
+    int				markSize;
 
     appGuiEnableWidget( bc->bcRemoveBookmarkButton, 0 );
 
-    text= appGetStringFromTextWidget( w );
-    len= strlen( text );
+    markName= appGetStringFromTextWidget( bc->bcDestinationText );
+    markSize= strlen( markName );
 
-    if  ( len > 0 )
+    if  ( markSize > 0 )
 	{
 	EditDocument *		ed= bc->bcDocument;
 	TedDocument *		td= (TedDocument *)ed->edPrivateData;
 	BufferDocument *	bd= td->tdDocument;
 	DocumentFieldList *	dfl= &(bd->bdFieldList);
 
-	if  ( ! docFindBookmarkField( dfl, text, len ) )
+	DocumentField *		df;
+
+	if  ( docGetBookmarkField( &df, markName, markSize, dfl ) < 0 )
 	    {
 	    appGuiEnableWidget( bc->bcGoToBookmarkButton, 0 );
 	    appGuiEnableWidget( bc->bcCopyBookmarkButton, 0 );
@@ -283,7 +275,7 @@ static void tedBookmarkDestinationChanged(	APP_WIDGET	w,
 	appGuiEnableWidget( bc->bcSetBookmarkButton, 0 );
 	}
 
-    appFreeStringFromTextWidget( text );
+    appFreeStringFromTextWidget( markName );
 
     return;
     }
@@ -294,7 +286,7 @@ static void tedBookmarkDestinationChanged(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static APP_LIST_CALLBACK( tedBookmarkChosen, w, voidbc, voidlcs )
+static APP_LIST_CALLBACK_H( tedBookmarkChosen, w, voidbc, voidlcs )
     {
     BookmarkContext *		bc= (BookmarkContext *)voidbc;
 
@@ -304,7 +296,7 @@ static APP_LIST_CALLBACK( tedBookmarkChosen, w, voidbc, voidlcs )
     if  ( ! text )
 	{ XDEB(text); return;	}
 
-    appStringToTextWidget( bc->bcMarkTextWidget, text );
+    appStringToTextWidget( bc->bcDestinationText, text );
 
     appFreeStringFromListCallback( text );
 
@@ -322,23 +314,17 @@ static APP_LIST_CALLBACK( tedBookmarkChosen, w, voidbc, voidlcs )
 /*									*/
 /************************************************************************/
 
-static void tedBookmarkCopyKindKindChosen(	APP_WIDGET	w,
-						void *		voidbc,
-						void *		voidtbcs )
+static APP_OITEM_CALLBACK_H( tedBookmarkCopyKindKindChosen, w, voidbc )
     {
     BookmarkContext *		bc= (BookmarkContext *)voidbc;
 
     int				linkKind;
 
-    for ( linkKind= 0; linkKind < LINKkind_COUNT; linkKind++ )
-	{
-	if  ( bc->bcCopyAsItems[linkKind] == w )
-	    { break;	}
-	}
+    linkKind= appGuiGetOptionmenuItemIndex( &(bc->bcCopyAsOptionmenu), w );
+    if  ( linkKind < 0 || linkKind >= LINKkind_COUNT )
+	{ LLDEB(linkKind,LINKkind_COUNT); return;	}
 
-    if  ( linkKind >= 0 && linkKind < LINKkind_COUNT )
-	{ bc->bcCopyKind= linkKind;		}
-    else{ LLDEB(linkKind,LINKkind_COUNT);	}
+    bc->bcCopyKind= linkKind;
 
     return;
     }
@@ -352,17 +338,12 @@ static void tedBookmarkCopyKindKindChosen(	APP_WIDGET	w,
 static void tedBookmarkMakeDestination(	BookmarkContext *	bc,
 					const char *		labelText )
     {
-    APP_WIDGET		label;
-    APP_WIDGET		text;
+    appMakeLabelInColumn( &(bc->bcDestinationLabel), bc->bcPaned, labelText );
+    appMakeTextInColumn( &(bc->bcDestinationText), bc->bcPaned, 0, 1 );
 
-    appMakeLabelInColumn( &label, bc->bcPaned, labelText );
-    appMakeTextInColumn( &text, bc->bcPaned, 0, 1 );
-
-    appGuiSetTypingCallbackForText( text,
+    appGuiSetTypingCallbackForText( bc->bcDestinationText,
 				tedBookmarkDestinationChanged, (void *)bc );
 
-    bc->bcMarkTextWidget= text;
-    
     return;
     }
 
@@ -419,7 +400,7 @@ static void tedBookmarkMakeList(	BookmarkContext *	bc )
 /*									*/
 /************************************************************************/
 
-static APP_DESTROY_CALLBACK( tedFreeBookmarkContext, w, voidbc )
+static APP_DESTROY_CALLBACK_H( tedFreeBookmarkContext, w, voidbc )
     { free( voidbc ); }
 
 static int tedMakeBookmarkDialog(	BookmarkContext **	pBc,
@@ -436,7 +417,7 @@ static int tedMakeBookmarkDialog(	BookmarkContext **	pBc,
 					sizeof(TED_BookmarkResourceTable)/
 					sizeof(AppConfigurableResource) );
 
-    bc->bcMarkTextWidget= (APP_WIDGET)0;
+    bc->bcDestinationText= (APP_WIDGET)0;
     bc->bcMarkListWidget= (APP_WIDGET)0;
 
     bc->bcSetBookmarkButton= (APP_WIDGET)0;
@@ -519,6 +500,7 @@ void tedRunBookmarkDialog(	EditApplication *	ea,
 
     if  ( markSize > 0 )
 	{
+	DocumentField *		df;
 	char *			saved;
 
 	saved= (char *)malloc( markSize+ 1 );
@@ -527,14 +509,14 @@ void tedRunBookmarkDialog(	EditApplication *	ea,
 	    { XDEB(saved);	}
 	else{
 	    strncpy( saved, markName, markSize )[markSize]= '\0';
-	    appStringToTextWidget( bc->bcMarkTextWidget, saved );
+	    appStringToTextWidget( bc->bcDestinationText, saved );
 	    bc->bcSavedMark= saved;
 	    }
 
 	appGuiEnableWidget( bc->bcRemoveBookmarkButton,
 				    ! ed->edIsReadonly && ! nameProposed );
 
-	if  ( ! docFindBookmarkField( dfl, markName, markSize ) )
+	if  ( docGetBookmarkField( &df, markName, markSize, dfl ) < 0 )
 	    {
 	    appGuiEnableWidget( bc->bcGoToBookmarkButton, 0 );
 	    appGuiEnableWidget( bc->bcCopyBookmarkButton, 0 );
@@ -549,7 +531,7 @@ void tedRunBookmarkDialog(	EditApplication *	ea,
 	appGuiEnableWidget( bc->bcGoToBookmarkButton, 0 );
 	appGuiEnableWidget( bc->bcCopyBookmarkButton, 0 );
 
-	appStringToTextWidget( bc->bcMarkTextWidget, "" );
+	appStringToTextWidget( bc->bcDestinationText, "" );
 	}
 
     if  ( ! nameProposed )
@@ -577,7 +559,7 @@ void tedRunBookmarkDialog(	EditApplication *	ea,
     appOptionmenuRefreshWidth( &(bc->bcCopyAsOptionmenu) );
 
 #   ifdef USE_MOTIF
-    XmProcessTraversal( bc->bcMarkTextWidget, XmTRAVERSE_CURRENT );
+    XmProcessTraversal( bc->bcDestinationText, XmTRAVERSE_CURRENT );
 #   endif
 
     appGuiRunDialog( &(bc->bcDialog), AQDrespNONE, ea );
@@ -591,36 +573,83 @@ void tedRunBookmarkDialog(	EditApplication *	ea,
 /*									*/
 /*  Fill a listbox with the bookmarks in a document.			*/
 /*									*/
+/*  1)  Empty the list widget.						*/
+/*  2)  Collect bookmarks.						*/
+/*  3)  Sort them.							*/
+/*  4)  And insert them in the list.					*/
+/*									*/
 /************************************************************************/
 
+static int tedCompareBookmarkStrings(	const void *	pv1,
+					const void *	pv2 )
+    {
+    const char * const *	ps1= (const char * const *)pv1;
+    const char * const *	ps2= (const char * const *)pv2;
+
+    return strcmp( *ps1, *ps2 );
+    }
+
 void tedFillBookmarkList(	APP_WIDGET		list,
 				DocumentFieldList *	dfl )
     {
     DocumentField *	df;
     int			i;
 
+    int			bookmarkCount= 0;
+    char **		bookmarks= (char **)0;
+
+    /*  1  */
     appGuiEmptyListWidget( list );
 
+    if  ( dfl->dflFieldCount == 0 )
+	{ goto ready;	}
+
+    /*  2  */
+    bookmarks= (char **)malloc( dfl->dflFieldCount* sizeof(char *) );
+    if  ( ! bookmarks )
+	{ LXDEB(dfl->dflFieldCount,bookmarks); goto ready;	}
+
     df= dfl->dflFields;
     for ( i= 0; i < dfl->dflFieldCount; df++, i++ )
 	{
-	const char *		listMarkName;
-	int			listMarkSize;
-	char			scratch[DOCmaxBOOKMARK+1];
+	const char *	name;
+	int		size;
 
-	if  ( df->dfKind != DOCfkBOOKMARK				||
-	      docFieldGetBookmark( df, &listMarkName, &listMarkSize )	)
+	if  ( df->dfKind != DOCfkBOOKMARK		||
+	      docFieldGetBookmark( df, &name, &size )	)
 	    { continue;		}
 
-	if  ( listMarkSize == 0			||
-	      listMarkName[0] == '_'		||
-	      listMarkSize >= DOCmaxBOOKMARK	)
+	if  ( size == 0			||
+	      name[0] == '_'		||
+	      size >= DOCmaxBOOKMARK	)
 	    { continue;	}
 
-	strncpy( scratch, listMarkName, listMarkSize )[listMarkSize]= '\0';
+	bookmarks[bookmarkCount]= malloc( size+ 1 );
+	if  ( ! bookmarks[bookmarkCount] )
+	    { LXDEB(size,bookmarks[bookmarkCount]); goto ready; }
 
-	appGuiAddValueToListWidget( list, scratch );
+	strncpy( bookmarks[bookmarkCount], name, size )[size]= '\0';
+	bookmarkCount++;
 	}
 
+    /*  3  */
+    qsort( bookmarks, bookmarkCount, sizeof(char *),
+					    tedCompareBookmarkStrings );
+
+    /*  4  */
+    for ( i= 0; i < bookmarkCount; i++ )
+	{ appGuiAddValueToListWidget( list, bookmarks[i] );	}
+
+  ready:
+
+    for ( i= 0; i < bookmarkCount; i++ )
+	{
+	if  ( bookmarks[i] )
+	    { free( bookmarks[i] );	}
+	}
+
+    if  ( bookmarks )
+	{ free( bookmarks );	}
+
     return;
     }
