@@ -23,8 +23,6 @@
 /*									*/
 /*  Edit menu actions.							*/
 /*									*/
-/*  2)  If the beginning of the selection is the beginning of a		*/
-/*	kyperlink, try to include the link.				*/
 /*  8)  Also put a copied picture on the clipboard that 'xv' implements	*/
 /*	on the root window.						*/
 /*									*/
@@ -35,13 +33,15 @@ static void tedFinishTextCopy(	TedDocument *			td,
 				const DocumentSelection *	ds )
     {
     BufferDocument *		bd= td->tdDocument;
+    const ScreenFontList *	sfl= &(td->tdScreenFontList);
     SelectionGeometry		sg;
+    const int			lastLine= 1;
 
-    tedSelectionGeometry( &sg, ds, bd, add );
+    tedSelectionGeometry( &sg, ds, lastLine, bd, add, sfl );
 
     td->tdCopiedSelectionClosed= 1;
 
-    if  ( (unsigned)ds->dsEnd.dpStroff < ds->dsEnd.dpBi->biParaStrlen )
+    if  ( ds->dsEnd.dpStroff < ds->dsEnd.dpBi->biParaStrlen )
 	{ td->tdCopiedSelectionClosed= 0;	}
 
     if  ( docIsParaSelection( ds )		&&
@@ -58,75 +58,42 @@ static int tedDocCopySelection(	EditDocument *	ed )
     {
     EditApplication *		ea= ed->edApplication;
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
-    BufferDocument *		bd= td->tdDocument;
     AppDrawingData *		add= &(ed->edDrawingData);
 
-    BufferItem *		bi;
-    TextParticule *		tp;
     DocumentPosition		dpObject;
+    int				partObject;
     InsertedObject *		io;
 
     SimpleOutputStream *	sos;
 
-    DocumentSelection		dsNew;
     const int			saveBookmarks= 0;
 
-    int				startPart;
-    int				endPart;
-
     DocumentSelection		ds;
     SelectionGeometry		sg;
-
-    const char *		fileName= (const char *)0;
-    int				fileSize= 0;
-    const char *		markName= (const char *)0;
-    int				markSize= 0;
+    SelectionDescription	sd;
 
     if  ( ! tedHasSelection( td ) || tedHasIBarSelection( td ) )
 	{ return -1; }
 
-    if  ( tedGetSelection( &ds, &sg, td ) )
+    if  ( tedGetSelection( &ds, &sg, &sd, td ) )
 	{ return -1;	}
 
-    dsNew= ds;
-
-    /*  2  */
-    if  ( ! docGetHyperlinkForPosition( bd, &(dsNew.dsBegin),
-				&startPart, &endPart,
-				&fileName, &fileSize, &markName, &markSize ) )
-	{
-	bi= dsNew.dsBegin.dpBi;
-	tp= bi->biParaParticules+ dsNew.dsBegin.dpParticule;
-
-	if  ( startPart < dsNew.dsBegin.dpParticule			&&
-	      bi->biParaParticules[startPart].tpStroff == tp->tpStroff	)
-	    { dsNew.dsBegin.dpParticule= startPart;	}
-
-	if  ( dsNew.dsEnd.dpBi == bi )
-	    {
-	    tp= bi->biParaParticules+ dsNew.dsEnd.dpParticule;
-
-	    if  ( endPart > dsNew.dsEnd.dpParticule			&&
-		  bi->biParaParticules[endPart].tpStroff == tp->tpStroff )
-		{ dsNew.dsEnd.dpParticule= endPart;	}
-	    }
-	}
-
     sos= sioOutMemoryOpen( &(td->tdCopiedSelection) );
     if  ( ! sos )
 	{ XDEB(sos); return -1;    }
 
-    if  ( docRtfSaveDocument( sos, td->tdDocument, &dsNew, saveBookmarks ) )
+    if  ( docRtfSaveDocument( sos, td->tdDocument, &ds, saveBookmarks ) )
 	{ LDEB(1); sioOutClose( sos ); return -1;	}
 
     if  ( sioOutClose( sos ) )
 	{ LDEB(1); return -1;	}
 
-    tedFinishTextCopy( td, add, &dsNew );
+    tedFinishTextCopy( td, add, &ds );
 
     docInitDocumentPosition( &dpObject );
 
-    if  ( ! tedGetObjectSelection( td, &dpObject, &io )		&&
+    if  ( ea->eaSupportXvCopyPaste					&&
+	  ! tedGetObjectSelection( td, &partObject, &dpObject, &io )	&&
 	  ! tedSaveObjectPicture( &(td->tdCopiedImage), io )	)
 	{
 	/*  8  */
@@ -136,7 +103,7 @@ static int tedDocCopySelection(	EditDocument *	ed )
 	}
 
     if  ( appDocOwnSelection( ed, "PRIMARY",
-		TedPrimaryTextTargets, TedPrimaryTextTargetCount ) )
+			TedPrimaryTextTargets, TedPrimaryTextTargetCount ) )
 	{ LDEB(1); return -1;	}
 
     return 0;
@@ -191,13 +158,30 @@ void tedDocCut(			EditDocument *	ed )
     {
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
 
-    if  ( ! td->tdCanReplaceSelection )
+    int				isTableSlice= 0;
+    int				isRowSlice= 0;
+    int				isColSlice= 0;
+    TableRectangle		tr;
+
+    DocumentSelection		ds;
+    SelectionGeometry		sg;
+    SelectionDescription	sd;
+
+    if  ( tedGetSelection( &ds, &sg, &sd, td ) )
+	{ return;	}
+
+    if  ( ! docGetTableSliceSelection( &isRowSlice, &isColSlice, &tr, &ds ) )
+	{ isTableSlice= 1;	}
+
+    if  ( ! isTableSlice && ! td->tdCanReplaceSelection )
 	{ LDEB(td->tdCanReplaceSelection); return;	}
 
     if  ( tedDocCopySelection( ed ) )
 	{ LDEB(1); return;	}
 
-    tedAppReplaceSelection( ed, (const unsigned char *)0, 0 );
+    if  ( isTableSlice )
+	{ tedDeleteTableSliceSelection( ed );				}
+    else{ tedDocReplaceSelection( ed, (const unsigned char *)0, 0 );	}
 
     return;
     }
@@ -208,11 +192,12 @@ void tedDocCopy(		EditDocument *	ed )
 
     DocumentSelection		ds;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
     const int			scrolledX= 0;
     const int			scrolledY= 0;
 
-    if  ( tedGetSelection( &ds, &sg, td ) )
+    if  ( tedGetSelection( &ds, &sg, &sd, td ) )
 	{ return;	}
 
     if  ( tedDocCopySelection( ed ) )
@@ -240,13 +225,14 @@ void tedPrimaryLost(	APP_WIDGET			w,
 
     DocumentSelection		ds;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
-    if  ( tedGetSelection( &ds, &sg, td ) )
+    if  ( tedGetSelection( &ds, &sg, &sd, td ) )
 	{ return;	}
 
     td->tdVisibleSelectionCopied= 0;
 
-    if  ( tedHasSelection( td ) && ! tedHasIBarSelection( td ) )
+    if  ( tedHasSelection( td ) && ! sd.sdIsIBarSelection )
 	{
 	const int	scrolledX= 0;
 	const int	scrolledY= 0;
@@ -319,7 +305,7 @@ APP_GIVE_COPY( tedCopyPrimaryString, w, event, voided )
     if  ( ! sis )
 	{ XDEB(sis); sioOutClose( sos ); return;	}
 
-    bd= docRtfReadFile( sis, td->tdDocument->bdProperties.dpAnsiCodePage );
+    bd= docRtfReadFile( sis, td->tdDocument->bdProperties.dpAnsiCodepage );
 
     sioInClose( sis );
 
@@ -440,26 +426,30 @@ APP_GIVE_COPY( tedCopyRulerTed, w, event, voided )
 /*									*/
 /************************************************************************/
 
-typedef BufferDocument * (*TED_READ) (	SimpleInputStream *		sis,
-					int				defCpg,
-					const DocumentGeometry *	dg );
+typedef BufferDocument * (*TED_READ) (
+				SimpleInputStream *		sis,
+				int				ansiCodepage,
+				const DocumentGeometry *	dg );
 
 typedef int	(*TED_INCLUDE) (	APP_WIDGET		w,
 					EditDocument *		ed,
 					BufferDocument *	bd );
 
-static BufferDocument * tedPasteRtfReadFile( SimpleInputStream *	sis,
-					int				defCpg,
-					const DocumentGeometry *	dg )
-    { return docRtfReadFile( sis, defCpg );	}
+static BufferDocument * tedPasteRtfReadFile(
+				SimpleInputStream *		sis,
+				int				ansiCodepage,
+				const DocumentGeometry *	dg )
+    { return docRtfReadFile( sis, ansiCodepage );	}
 
-static BufferDocument * tedPastePlainReadFile( SimpleInputStream *	sis,
-					int				defCpg,
-					const DocumentGeometry *	dg )
+static BufferDocument * tedPastePlainReadFile(
+				SimpleInputStream *		sis,
+				int				ansiCodepage,
+				const DocumentGeometry *	dg )
     {
     int		longestPara;
 
-    return docPlainReadFile( sis, &longestPara, dg );
+    return docPlainReadFile( sis, &longestPara,
+					DOCcharsetANSI, ansiCodepage, dg );
     }
 
 static void tedPastePrimaryGeneric(	APP_WIDGET		w,
@@ -476,10 +466,7 @@ static void tedPastePrimaryGeneric(	APP_WIDGET		w,
 
     TedAppResources *		tar= (TedAppResources *)ea->eaResourceData;
 
-    if  ( tar->tarDefaultAnsicpgInt < 0		&&
-	  tar->tarDefaultAnsicpgString		&&
-	  tedDetermineDefaultCodepage( tar )	)
-	{ SDEB(tar->tarDefaultAnsicpgString);	}
+    tedDetermineDefaultSettings( tar );
 
     sis= sioInOpenPaste( w, event );
     if  ( ! sis )
@@ -617,17 +604,17 @@ APP_PASTE_REPLY( tedPasteRulerTed, w, event, voided )
     SimpleInputStream *		sis;
 
     ParagraphProperties		ppNew;
-    TextAttribute		taNew;
+    TextAttribute		taSet;
 
-    PropertyMask		taUpdMask;
+    PropertyMask		taSetMask;
     PropertyMask		ppUpdMask;
     PropertyMask		spUpdMask;
 
-    PROPmaskCLEAR( &taUpdMask );
+    PROPmaskCLEAR( &taSetMask );
     PROPmaskCLEAR( &spUpdMask );
 
     docInitParagraphProperties( &ppNew );
-    docInitTextAttribute( &taNew );
+    utilInitTextAttribute( &taSet );
 
     sis= sioInOpenPaste( w, event );
     if  ( ! sis )
@@ -646,7 +633,7 @@ APP_PASTE_REPLY( tedPasteRulerTed, w, event, voided )
     PROPmaskADD( &ppUpdMask, PPpropTAB_STOPS );
 
     if  ( tedChangeSelectionProperties( ed,
-				    &taUpdMask, taNew,
+				    &taSetMask, &taSet,
 				    &ppUpdMask, &ppNew,
 				    &spUpdMask, (SectionProperties *)0 ) )
 	{ LDEB(1); docCleanParagraphProperties( &ppNew ); return; }
@@ -655,3 +642,62 @@ APP_PASTE_REPLY( tedPasteRulerTed, w, event, voided )
 
     return;
     }
+
+/************************************************************************/
+/*									*/
+/*  Custom 'Paste' functionality.					*/
+/*									*/
+/*  1)  The regular X11 manner.						*/
+/*  2)  Try to get a possible 'paste' from xv. This is just to make it	*/
+/*	possible to cooperate with John Bradley's XV. As XV is a little	*/
+/*	outdated, only support this if the appropriate resource is set.	*/
+/*									*/
+/************************************************************************/
+
+void tedDocEditPaste(		APP_WIDGET	option,
+				void *		voided,
+				void *		voidpbcs )
+    {
+    EditDocument *			ed= (EditDocument *)voided;
+    EditApplication *			ea= ed->edApplication;
+    TedDocument *			td= (TedDocument *)ed->edPrivateData;
+
+    int					res;
+    BitmapDescription			bd;
+    unsigned char *			buffer;
+
+    if  ( ! td->tdCanReplaceSelection )
+	{ LDEB(td->tdCanReplaceSelection); return;	}
+
+    /*  1  */
+    res= appDocAskForPaste( ed, "PRIMARY" );
+    if  ( ! res )
+	{ return;	}
+
+    /*  2  */
+    if  ( ea->eaSupportXvCopyPaste					&&
+	  ! appImgGetXvClipboard( &bd, &buffer,
+					ea->eaToplevel.atTopWidget )	)
+	{
+	AppBitmapImage *	abi;
+
+	abi= (AppBitmapImage *)malloc( sizeof(AppBitmapImage) );
+	if  ( ! abi )
+	    { XDEB(abi); return;	}
+	appInitBitmapImage( abi );
+
+	if  ( bmCopyDescription( &(abi->abiBitmap), &bd ) )
+	    { bmCleanDescription( &bd ); return; }
+
+	bmCleanDescription( &bd );
+	abi->abiBuffer= buffer;
+
+	if  ( tedReplaceSelectionWithBitmapImage( ed, abi ) )
+	    { LDEB(1); return; }
+
+	return;
+	}
+
+    return;
+    }
+
