@@ -165,13 +165,24 @@ static void tedUpdateTableColumns(	EditDocument *		ed,
 
     if  ( changed )
 	{
-	int		rowPropertiesAlso= 0;
+	PropertyMask	rpSetMask;
+	PropertyMask	cpSetMask;
 
-	tedChangeTableLayout( ed, sectBi, rp, rowPropertiesAlso, row0, row1 );
+	const int	col0= 0;
+	const int	col1= csCount- 1;
+
+	PROPmaskCLEAR( &rpSetMask );
+	PROPmaskCLEAR( &cpSetMask );
+
+	PROPmaskADD( &rpSetMask, RPpropLEFT_INDENT );
+	PROPmaskADD( &cpSetMask, CLpropRIGHT_BOUNDARY );
+
+	tedChangeTableLayout( ed, sectBi, row0, row1, col0, col1, row1,
+						&rpSetMask, &cpSetMask, rp );
 
 	tedAdaptToolsToSelection( ed );
 
-	tedAdaptFormatToolToDocument( ed );
+	tedAdaptFormatToolToDocument( ed, 0 );
 
 	appDocumentChanged( ed, 1 );
 	}
@@ -193,21 +204,11 @@ static int tedGetRulerTwips(
 			int				firstIndentPixels,
 			int				leftIndentPixels,
 			int				rightIndentPixels,
-			int				tabCount,
-			const TabStop *			tabStops )
+			const TabStopList *		tslNew )
     {
     ParagraphFrame		pf;
     double			xfac= add->addMagnifiedPixelsPerTwip;
 
-    int				otab;
-    int				ntab;
-    int				tab0;
-    int				otab1;
-    int				ntab1;
-
-    int				pixels;
-    int				twips;
-
     const int			bottom= -1;
     const int			stripHigh= -1;
 
@@ -215,8 +216,9 @@ static int tedGetRulerTwips(
     int				x1GeometryPixels;
 
     BlockFrame			bf;
-    TabStop *			ts;
-    int				tab;
+
+    int				tabCount= tslNew->tslTabStopCount;
+    const TabStop *		tabStops= tslNew->tslTabStops;
 
     docBlockFrameTwips( &bf, bi, bd, bi->biTopPosition.lpPage,
 					    bi->biTopPosition.lpColumn );
@@ -224,12 +226,17 @@ static int tedGetRulerTwips(
     docParagraphFrameTwips( &pf, &bf, bottom, stripHigh, bi );
     tedParagraphFramePixels( &pf, add, bi );
 
-    ts= bi->biParaTabStops;
-    for ( tab= 0; tab < bi->biParaTabCount; ts++, tab++ )
+    {
+    TabStopList *	tsl= &(bi->biParaTabStopList);
+    int			tab;
+    TabStop *		ts= tsl->tslTabStops;
+
+    for ( tab= 0; tab < tsl->tslTabStopCount; ts++, tab++ )
 	{
 	ts->tsPixels= TWIPStoPIXELS( xfac,
 				    pf.pfX0GeometryTwips+ ts->tsTwips );
 	}
+    }
 
     x0GeometryPixels= TWIPStoPIXELS( xfac, pf.pfX0GeometryTwips );
     x1GeometryPixels= TWIPStoPIXELS( xfac, pf.pfX1GeometryTwips );
@@ -244,89 +251,14 @@ static int tedGetRulerTwips(
     rightIndentPixels= x1GeometryPixels- rightIndentPixels;
     pp->ppRightIndentTwips= PIXELStoTWIPS( rightIndentPixels, xfac );
 
-    if  ( tabCount > pp->ppTabCount )
-	{
-	TabStop *	fresh;
-
-	fresh= (TabStop *)realloc( pp->ppTabStops, tabCount* sizeof(TabStop) );
-	if  ( ! fresh )
-	    { LXDEB(tabCount,fresh); return -1;	}
-
-	pp->ppTabStops= fresh;
-	}
-
-    tab0= 0;
-    while( tab0 < pp->ppTabCount && tab0 < tabCount )
-	{
-	twips= pp->ppTabStops[tab0].tsTwips;
-	pixels= x0GeometryPixels+ TWIPStoPIXELS( xfac, twips );
-
-	if  ( pixels != tabStops[tab0].tsPixels )
-	    { break;	}
-
-	pp->ppTabStops[tab0]= tabStops[tab0];
-	pp->ppTabStops[tab0].tsTwips= twips;
-
-	tab0++;
-	}
-
-    otab1= pp->ppTabCount- 1;
-    ntab1= tabCount- 1;
-    while( otab1 >= tab0 && ntab1 >= tab0 )
-	{
-	twips= pp->ppTabStops[otab1].tsTwips;
-	pixels= x0GeometryPixels+ TWIPStoPIXELS( xfac, twips );
-
-	if  ( pixels != tabStops[ntab1].tsPixels )
-	    { break;	}
-
-	/*  No.. Later on
-	pp->ppTabStops[otab1]= tabStops[ntab1];
-	pp->ppTabStops[otab1].tsTwips= twips;
-	*/
-
-	otab1--; ntab1--;
-	}
-
-    if  ( pp->ppTabCount < tabCount )
-	{
-	ntab= tabCount- 1;
-	for ( otab= pp->ppTabCount- 1; otab > otab1; ntab--, otab-- )
-	    {
-	    twips= pp->ppTabStops[otab].tsTwips;
-
-	    pp->ppTabStops[ntab]= tabStops[ntab];
-	    pp->ppTabStops[ntab].tsTwips= twips;
-	    }
-	}
-    else{
-	ntab= ntab1+ 1;
-	for ( otab= otab1+ 1; otab < pp->ppTabCount; ntab++, otab++ )
-	    {
-	    twips= pp->ppTabStops[otab].tsTwips;
-
-	    pp->ppTabStops[ntab]= tabStops[ntab];
-	    pp->ppTabStops[ntab].tsTwips= twips;
-	    }
-	}
-
-    otab= tab0;
-    for ( ntab= tab0; ntab <= ntab1; otab++, ntab++ )
-	{
-	pixels= tabStops[ntab].tsPixels;
-	pixels -= x0GeometryPixels;
-	twips= pixels/ xfac;
-
-	pp->ppTabStops[otab]= tabStops[ntab];
-	pp->ppTabStops[otab].tsTwips= twips;
-	}
-
-    pp->ppTabCount= tabCount;
+    if  ( docRulerMergeTabs( &(pp->ppTabStopList), xfac, x0GeometryPixels,
+						    tabCount, tabStops ) )
+	{ LDEB(tabCount); return -1;	}
 
     return 0;
     }
 
-static APP_EVENT_HANDLER( tedHorizontalRulerButtonDown, w, voided, downEvent )
+static APP_EVENT_HANDLER_H( tedHorizontalRulerButtonDown, w, voided, downEvent )
     {
     EditDocument *		ed= (EditDocument *)voided;
     EditApplication *		ea= ed->edApplication;
@@ -338,8 +270,7 @@ static APP_EVENT_HANDLER( tedHorizontalRulerButtonDown, w, voided, downEvent )
     int				leftIndentNew;
     int				firstIndentNew;
     int				rightIndentNew;
-    int				tabCountNew= 0;
-    TabStop *			tabsNew= (TabStop *)0;
+    TabStopList			tslNew;
 
     int				csCountNew= 0;
     ColumnSeparator *		csNew= (ColumnSeparator *)0;
@@ -349,13 +280,13 @@ static APP_EVENT_HANDLER( tedHorizontalRulerButtonDown, w, voided, downEvent )
 
     int				prop= PPprop_NONE;
 
-    PropertyMask		taUpdMask;
+    PropertyMask		taSetMask;
     PropertyMask		ppChgMask;
     PropertyMask		ppUpdMask;
     PropertyMask		spUpdMask;
 
     ParagraphProperties		ppNew;
-    TextAttribute		taNew;
+    TextAttribute		taSet;
 
     ParagraphFrame		pf;
 
@@ -366,14 +297,20 @@ static APP_EVENT_HANDLER( tedHorizontalRulerButtonDown, w, voided, downEvent )
 
     DocumentSelection		ds;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
-    if  ( tedGetSelection( &ds, &sg, td ) )
+    const int * const	colorMap= (const int *)0;
+    const int * const	listStyleMap= (const int *)0;
+
+    docInitTabStopList( &tslNew );
+
+    if  ( tedGetSelection( &ds, &sg, &sd, td ) )
 	{ LDEB(1); return;	}
     bi= ds.dsBegin.dpBi;
 
-    tabCountNew= bi->biParaProperties.ppTabCount;
+    tslNew.tslTabStopCount= bi->biParaTabStopCount;
 
-    PROPmaskCLEAR( &taUpdMask );
+    PROPmaskCLEAR( &taSetMask );
     PROPmaskCLEAR( &ppUpdMask );
     PROPmaskCLEAR( &spUpdMask );
 
@@ -389,7 +326,7 @@ static APP_EVENT_HANDLER( tedHorizontalRulerButtonDown, w, voided, downEvent )
 
     tedHorizontalRulerTrackMouse(
 			    &firstIndentNew, &leftIndentNew, &rightIndentNew,
-			    &tabCountNew, &tabsNew,
+			    &tslNew,
 			    &csCountNew, &csNew, &prop,
 			    w, ea, downEvent, ed->edTopRuler,
 			    (void *)ed, tedDragVerticalHair );
@@ -419,21 +356,22 @@ static APP_EVENT_HANDLER( tedHorizontalRulerButtonDown, w, voided, downEvent )
 	}
 
     docInitParagraphProperties( &ppNew );
-    docInitTextAttribute( &taNew );
+    utilInitTextAttribute( &taSet );
 
     PROPmaskCLEAR( &ppChgMask );
 
-    if  ( docUpdParaProperties( &ppChgMask, &ppNew,
-				    &ppUpdMask, &(bi->biParaProperties ) ) )
+    if  ( docUpdParaProperties( &ppChgMask, &ppNew, &ppUpdMask,
+						    &(bi->biParaProperties),
+						    colorMap, listStyleMap ) )
 	{ LDEB(1); return;	}
 
     if  ( tedGetRulerTwips( add, bi, &ppNew, bd,
 			    firstIndentNew, leftIndentNew, rightIndentNew,
-			    tabCountNew, tabsNew ) )
+			    &tslNew ) )
 	{ LDEB(1); return;	}
 
     if  ( tedChangeSelectionProperties( ed,
-				    &taUpdMask, taNew,
+				    &taSetMask, &taSet,
 				    &ppUpdMask, &ppNew,
 				    &spUpdMask, (SectionProperties *)0 ) )
 	{ LDEB(1);	}
@@ -473,8 +411,6 @@ void tedDocAdaptHorizontalRuler(	EditDocument *		ed,
 	const int		stripHigh= -1;
 
 	BlockFrame		bf;
-	TabStop *		ts;
-	int			tab;
 
 	docBlockFrameTwips( &bf, bi, bd, bi->biTopPosition.lpPage,
 					    bi->biTopPosition.lpColumn );
@@ -482,12 +418,17 @@ void tedDocAdaptHorizontalRuler(	EditDocument *		ed,
 	docParagraphFrameTwips( &pf, &bf, bottom, stripHigh, bi );
 	tedParagraphFramePixels( &pf, add, bi );
 
-	ts= bi->biParaTabStops;
-	for ( tab= 0; tab < bi->biParaTabCount; ts++, tab++ )
+	{
+	TabStopList *	tsl= &(bi->biParaTabStopList);
+	int		tab;
+	TabStop *	ts= tsl->tslTabStops;
+
+	for ( tab= 0; tab < tsl->tslTabStopCount; ts++, tab++ )
 	    {
 	    ts->tsPixels= TWIPStoPIXELS( xfac,
 					pf.pfX0GeometryTwips+ ts->tsTwips );
 	    }
+	}
 
 	x0GeometryPixels= TWIPStoPIXELS( xfac, pf.pfX0GeometryTwips );
 	x1GeometryPixels= TWIPStoPIXELS( xfac, pf.pfX1GeometryTwips );
@@ -496,11 +437,14 @@ void tedDocAdaptHorizontalRuler(	EditDocument *		ed,
 	firstIndent= pf.pfX0FirstLinePixels;
 	rightIndent= pf.pfX1TextLinesPixels;
 
+	{
+	TabStopList *	tsl= &(bi->biParaTabStopList);
+
 	tedAdaptHorizontalRuler( ed->edTopRuler, ed->edTopRulerWidget,
 		    add->addDocRect.drX0, add->addDocRect.drX1,
 		    firstIndent, leftIndent, rightIndent,
-		    add->addPaperRect.drX1,
-		    bi->biParaTabCount, bi->biParaTabStops );
+		    add->addPaperRect.drX1, tsl );
+	}
 
 	if  ( bi->biParaInTable )
 	    {
@@ -634,8 +578,9 @@ void tedDocFormatCopyRul(	APP_WIDGET	fontsOption,
 
     DocumentSelection		ds;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
-    if  ( tedGetSelection( &ds, &sg, td ) )
+    if  ( tedGetSelection( &ds, &sg, &sd, td ) )
 	{ LDEB(1); return;	}
     bi= ds.dsBegin.dpBi;
 
