@@ -7,6 +7,8 @@
 #   include	<ctype.h>
 
 #   include	<sioStdio.h>
+#   include	<appSystem.h>
+#   include	<utilMatchFont.h>
 
 #   include	<appSpellTool.h>
 
@@ -15,6 +17,7 @@
 
 #   include	"docLayout.h"
 #   include	"docEdit.h"
+#   include	"docFind.h"
 
 #   include	<appDebugon.h>
 
@@ -52,6 +55,7 @@ void tedMoveObjectWindows(	EditDocument *		ed )
     {
     AppDrawingData *		add= &(ed->edDrawingData);
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
+    BufferDocument *		bd= td->tdDocument;
 
     InsertedObject *		io;
 
@@ -60,13 +64,15 @@ void tedMoveObjectWindows(	EditDocument *		ed )
 
     DocumentPosition		dp;
     PositionGeometry		pg;
+    int				part;
+    const int			lastOne= 1;
 
     docInitDocumentPosition( &dp );
 
-    if  ( tedGetObjectSelection( td, &dp, &io ) )
+    if  ( tedGetObjectSelection( td, &part, &dp, &io ) )
 	{ LDEB(1); return;	}
 
-    tedPositionGeometry( &pg, &dp, add );
+    tedPositionGeometry( &pg, &dp, lastOne, bd, add, &(td->tdScreenFontList) );
 
     tedSetObjectWindows( ed, &pg, io, ox, oy );
     }
@@ -106,9 +112,9 @@ void tedDocVerticalScrollbarCallback(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-int tedFinishDocumentSetup(	EditApplication *	ea,
-				EditDocument *		ed )
+int tedFinishDocumentSetup(	EditDocument *		ed )
     {
+    EditApplication *		ea= ed->edApplication;
     AppDrawingData *		add= &(ed->edDrawingData);
     AppColors *			ac= &(ed->edColors);
 
@@ -128,17 +134,53 @@ int tedFinishDocumentSetup(	EditApplication *	ea,
     /*  1  */
     if  ( ! docFirstDocumentPosition( &dp, bd ) )
 	{
+	const int			lastOne= 1;
+	const int			lastLine= 0;
+	int				part;
 	const TextParticule *		tp;
+	ScreenFontList *		sfl;
 
-	docSetIBarSelection( &(td->tdDocumentSelection), &dp );
+	sfl= &(td->tdScreenFontList);
+
+	if  ( tar->tarFindPattern					&&
+	      ! tedFindSetPattern( ed,
+			    tar->tarFindPattern, tar->tarFindRegex )	&&
+	      ! docFindFindNextInDocument( &(td->tdDocumentSelection),
+		    &dp, bd,
+		    docFindParaFindNext, (void *)td->tdFindProg )	)
+	    {
+	    dp= td->tdDocumentSelection.dsBegin;
+	    }
+	else{
+	    docSetIBarSelection( &(td->tdDocumentSelection), &dp );
+	    }
 
 	tedSelectionGeometry( &(td->tdSelectionGeometry),
-				     &(td->tdDocumentSelection), bd, add );
+			     &(td->tdDocumentSelection), lastLine,
+			     bd, add, &(td->tdScreenFontList) );
 
-	tp= dp.dpBi->biParaParticules+ dp.dpParticule;
 
-	td->tdCurrentTextAttribute= tp->tpTextAttribute;
-	td->tdCurrentPhysicalFont= tp->tpPhysicalFont;
+	if  ( docFindParticuleOfPosition( &part, &dp, lastOne ) )
+	    { LDEB(dp.dpStroff); return -1;	}
+
+	tp= dp.dpBi->biParaParticules+ part;
+
+	utilGetTextAttributeByNumber( &(td->tdCurrentTextAttribute),
+						&(bd->bdTextAttributeList),
+						tp->tpTextAttributeNumber );
+	td->tdCurrentTextAttributeNumber= tp->tpTextAttributeNumber;
+
+	td->tdCurrentScreenFont= -1;
+	if  ( td->tdCurrentTextAttributeNumber <
+	      sfl->sflAttributeToScreenCount )
+	    {
+	    td->tdCurrentScreenFont=
+		sfl->sflAttributeToScreen[td->tdCurrentTextAttributeNumber];
+	    }
+
+	docDescribeSelection( &(td->tdSelectionDescription),
+				    &(td->tdDocumentSelection),
+				    bd, ed->edDocumentId, ed->edIsReadonly );
 	}
     else{ docListItem( 0, &(bd->bdItem) );	}
 
@@ -158,30 +200,34 @@ int tedFinishDocumentSetup(	EditApplication *	ea,
 
     if  ( ed->edColors.acAllocator.caDepth < 4 )
 	{
-	if  ( appColorRgb( &(td->tdSelColor), &(ed->edColors), 0, 0, 0 ) )
-	    { LDEB(1); return -1;	}
-
-	if  ( appColorRgb( &(td->tdXSelColor), &(ed->edColors), 0, 0, 0 ) )
-	    { LDEB(1); return -1;	}
-
 	if  ( appColorRgb( &(td->tdFieldColor), &(ed->edColors), 0, 0, 0 ) )
 	    { LDEB(1); return -1;	}
 
-	if  ( appColorRgb( &(td->tdTableColor), &(ed->edColors), 0, 0, 0 ) )
-	    { LDEB(1); return -1;	}
+	td->tdSelColor.rgb8Red= 0;
+	td->tdSelColor.rgb8Green= 0;
+	td->tdSelColor.rgb8Blue= 0;
+
+	td->tdCopiedSelColor.rgb8Red= 0;
+	td->tdCopiedSelColor.rgb8Green= 0;
+	td->tdCopiedSelColor.rgb8Blue= 0;
+
+	td->tdTableColor.rgb8Red= 0;
+	td->tdTableColor.rgb8Green= 0;
+	td->tdTableColor.rgb8Blue= 0;
 	}
     else{
-	if  ( appColorNamed( &td->tdSelColor, &ed->edColors, selColorName ) )
+	APP_COLOR_RGB	acSel;
+	APP_COLOR_RGB	acCopiedSel;
+
+	if  ( appColorNamed( &acSel, &ed->edColors, selColorName ) )
 	    {
-	    if  ( appColorFindRgb( &(td->tdSelColor),
-					   &(ed->edColors), 176, 196, 222 ) )
+	    if  ( appColorFindRgb( &acSel, &(ed->edColors), 176, 196, 222 ) )
 		{ SDEB(selColorName); return -1;	}
 	    }
 
-	if  ( appColorNamed( &(td->tdXSelColor),
-					    &(ed->edColors), xselColorName ) )
+	if  ( appColorNamed( &acCopiedSel, &(ed->edColors), xselColorName ) )
 	    {
-	    if  ( appColorFindRgb( &(td->tdXSelColor),
+	    if  ( appColorFindRgb( &acCopiedSel,
 					    &(ed->edColors), 176, 176, 176 ) )
 		{ SDEB(xselColorName); return -1;	}
 	    }
@@ -193,13 +239,44 @@ int tedFinishDocumentSetup(	EditApplication *	ea,
 		{ LDEB(1); return -1;	}
 	    }
 
-	td->tdTableColor= add->addBackColor;
+#	ifdef USE_MOTIF
+	td->tdSelColor.rgb8Red= acSel.red/ 256;
+	td->tdSelColor.rgb8Green= acSel.green/ 256;
+	td->tdSelColor.rgb8Blue= acSel.blue/ 256;
+
+	td->tdCopiedSelColor.rgb8Red= acCopiedSel.red/ 256;
+	td->tdCopiedSelColor.rgb8Green= acCopiedSel.green/ 256;
+	td->tdCopiedSelColor.rgb8Blue= acCopiedSel.blue/ 256;
+
+	td->tdTableColor.rgb8Red= add->addBackColor.red/ 256;
+	td->tdTableColor.rgb8Green= add->addBackColor.green/ 256;
+	td->tdTableColor.rgb8Blue= add->addBackColor.blue/ 256;
+#	endif
+
+#	ifdef USE_GTK
+	td->tdSelColor.rgb8Red= acSel.red/ 256;
+	td->tdSelColor.rgb8Green= acSel.green/ 256;
+	td->tdSelColor.rgb8Blue= acSel.blue/ 256;
+
+	td->tdCopiedSelColor.rgb8Red= acCopiedSel.red/ 256;
+	td->tdCopiedSelColor.rgb8Green= acCopiedSel.green/ 256;
+	td->tdCopiedSelColor.rgb8Blue= acCopiedSel.blue/ 256;
+
+	td->tdTableColor.rgb8Red= add->addBackColor.red/ 256;
+	td->tdTableColor.rgb8Green= add->addBackColor.green/ 256;
+	td->tdTableColor.rgb8Blue= add->addBackColor.blue/ 256;
+#	endif
 	}
 
     if  ( tedOpenItemObjects( &(bd->bdItem), &(ed->edColors),
 						    &(ed->edDrawingData) ) )
 	{ LDEB(1);	}
 
+    td->tdDrawTableGrid= tar->tarShowTableGridInt;
+
+    appGuiSetToggleItemState( td->tdDrawTableGridOption,
+						td->tdDrawTableGrid >= 0 );
+
     if  ( ed->edIsReadonly )
 	{ tedMakeDocumentReadonly( ed );	}
 
@@ -209,6 +286,41 @@ int tedFinishDocumentSetup(	EditApplication *	ea,
     return 0;
     }
 
+/************************************************************************/
+/*									*/
+/*  Scroll to the selection the first time the document becomes visible	*/
+/*									*/
+/************************************************************************/
+
+void tedDocumentFirstVisible(	EditDocument *	ed )
+    {
+    TedDocument *		td= (TedDocument *)ed->edPrivateData;
+    DocumentRectangle		dr= ed->edVisibleRect;
+
+    int				pScrolledX= 0;
+    int				pScrolledY= 0;
+
+    if  ( td->tdSelectionGeometry.sgRectangle.drX1 > dr.drX1 )
+	{
+	int sh= td->tdSelectionGeometry.sgRectangle.drX1- dr.drX1;
+
+	dr.drX0 += sh;
+	dr.drX1 += sh;
+	}
+    if  ( td->tdSelectionGeometry.sgRectangle.drY1 > dr.drY1 )
+	{
+	int sh= td->tdSelectionGeometry.sgRectangle.drY1- dr.drY1;
+
+	dr.drY0 += sh;
+	dr.drY1 += sh;
+	}
+
+    appScrollToRectangle( ed, dr.drX0, dr.drY0, dr.drX1, dr.drY1,
+						    &pScrolledX, &pScrolledY );
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Make the document widget and the rulers.				*/
@@ -235,21 +347,27 @@ void tedFreeDocument(		void *			voidtd,
 	{
 	BufferDocument *	bd= td->tdDocument;
 	int			noteCount= 0;
+	int			bulletsDeleted= 0;
 	int			paragraphCount= 0;
 
-	docCloseItemObjects( &noteCount, &paragraphCount,
+	docCleanItemObjects( &noteCount, &bulletsDeleted, &paragraphCount,
 					    bd, &(bd->bdItem),
 					    (void *)add, tedCloseObject );
 
 	docFreeDocument( bd );
 	}
 
+    docCleanScreenFontList( &(td->tdScreenFontList) );
+
     utilCleanMemoryBuffer( &(td->tdCopiedSelection) );
     utilCleanMemoryBuffer( &(td->tdCopiedFont) );
     utilCleanMemoryBuffer( &(td->tdCopiedRuler) );
 
     appCleanBitmapImage( &(td->tdCopiedImage) );
 
+    if  ( td->tdFindProg )
+	{ free( td->tdFindProg );	}
+
     free( td );
     }
 
@@ -284,51 +402,35 @@ int tedMakeDocumentWidget(	EditApplication *	ea,
 /*									*/
 /************************************************************************/
 
-static int tedDetermineCodepage(	BufferDocument *	bd,
-					const AppDrawingData *	add )
+static int tedDetermineCodepage(	BufferDocument *	bd )
     {
-    DocumentProperties *	dp= &(bd->bdProperties);
+    const DocumentFontList *	dfl= &(bd->bdProperties.dpFontList);
+    const DocumentFont *	df;
+    int				font;
 
-    PostScriptFaceList		psfl;
-    PostScriptFace *		psf;
-    int				i;
+    int				encoding= -1;
+    int				encodingCount= 0;
 
-    int				charset= -1;
-    int				charsetCount= 0;
-
-    utilInitPostScriptFaceList( &psfl );
-
-    if  ( docPsPrintGetItemFonts( &psfl, &(bd->bdItem),
-			    &(dp->dpFontList), &(add->addPhysicalFontList) ) )
-	{ LDEB(1); return -1;	}
-
-    psf= psfl.psflFaces;
-    for ( i= 0; i < psfl.psflFaceCount; psf++, i++ )
+    df= dfl->dflFonts;
+    for ( font= 0; font < dfl->dflFontCount; df++, font++ )
 	{
-	if  ( charset < 0 )
-	    { charset= psf->psfEncodingUsed; charsetCount= 1;	}
+	if  ( ! df->dfUsed )
+	    { continue;	}
+	if  ( df->dfEncodingSet < 0 )
+	    { LDEB(df->dfEncodingSet); continue;	}
+
+	if  ( encoding < 0 )
+	    { encoding= df->dfEncodingSet; encodingCount= 1;	}
 	else{
-	    if  ( charset != psf->psfEncodingUsed )
-		{ charsetCount++; }
+	    if  ( encoding != df->dfEncodingSet )
+		{ encodingCount++; }
 	    }
 	}
 
-    utilCleanPostScriptFaceList( &psfl );
-
-    if  ( charsetCount != 1 )
+    if  ( encodingCount != 1 )
 	{ return -1;	}
 
-    switch( charset )
-	{
-	case ENCODINGpsISO_8859_1:
-	    return 1252;
-
-	case ENCODINGpsISO_8859_2:
-	    return 1250;
-
-	default:
-	    LDEB(charset); return -1;
-	}
+    return utilWindowsCodepageFromEncoding( encoding );
     }
 
 /************************************************************************/
@@ -343,58 +445,59 @@ int tedNewDocument(	EditApplication *	ea,
     {
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
     BufferDocument *		bd;
-
-    PropertyMask		updMask;
-    TextAttribute		taNew;
-
-    char *			familyName;
-
-    char			scratch[120];
-    time_t			now;
     DocumentProperties *	dp;
+    TextAttribute		ta;
+
+    time_t			now;
 
     TedAppResources *		tar= (TedAppResources *)ea->eaResourceData;
+    int				rval= 0;
 
-    if  ( tar->tarDefaultAnsicpgInt < 0		&&
-	  tar->tarDefaultAnsicpgString		&&
-	  tedDetermineDefaultCodepage( tar )	)
-	{ SDEB(tar->tarDefaultAnsicpgString);	}
+    tedDetermineDefaultSettings( tar );
 
-    PROPmaskCLEAR( &updMask );
-    strcpy( scratch, ea->eaDefaultFont );
-    if  ( appFontGetAttributes( scratch, &familyName, &updMask, &taNew ) )
-	{ SDEB(ea->eaDefaultFont); return -1;	}
+    if  ( appPostScriptFontCatalog( ea ) )
+	{ SDEB(ea->eaAfmDirectory); return -1;	}
 
-    if  ( ! PROPmaskISSET( &updMask, TApropFONTSIZE )	||
-	  taNew.taFontSizeHalfPoints < 6		)
-	{
-	LDEB(taNew.taFontSizeHalfPoints);
-	taNew.taFontSizeHalfPoints= 24;
-	PROPmaskADD( &updMask, TApropFONTSIZE );
-	}
-
-    bd= docNewFile( familyName, 10* taNew.taFontSizeHalfPoints,
-					    &(ea->eaDefaultDocumentGeometry) );
+    bd= docNewFile( &ta, ea->eaDefaultFont,
+				    DOCcharsetANSI, tar->tarDefaultAnsicpgInt,
+				    &(ea->eaPostScriptFontList),
+				    &(ea->eaDefaultDocumentGeometry) );
     if  ( ! bd )
-	{ XDEB(bd); return -1;	}
+	{ XDEB(bd); rval= -1; goto ready;	}
 
     ed->edFormat= TEDdockindRTF; /* rtf */
     td->tdDocument= bd;
     dp= &(bd->bdProperties);
-    dp->dpAnsiCodePage= tar->tarDefaultAnsicpgInt;
+
+    {
+    int		l= 0;
+
+    l += strlen( ea->eaNameAndVersion );
+    l += 2;
+    l += strlen( ea->eaReference );
+    l += 2;
+    l += 1;
+
+    dp->dpGenerator= malloc( l );
+    if  ( ! dp->dpGenerator )
+	{ LXDEB(l,dp->dpGenerator);	}
+    else{
+	sprintf( (char *)dp->dpGenerator, "%s (%s);",
+				ea->eaNameAndVersion, ea->eaReference );
+	}
+    }
+
+    /*  2  */
+    if  ( appPostScriptFontCatalog( ea ) )
+	{ SDEB(ea->eaAfmDirectory); return -1;	}
 
     if  ( filename )
 	{
-	const char *	slash= strrchr( filename, '/' );
-	const char *	dot;
+	const char *	ext= appFileExtensionOfName( filename );
 
-	if  ( slash )
-	    { dot= strrchr( slash+ 1, '.' );	}
-	else{ dot= strrchr( filename, '.' );	}
-
-	if  ( dot && ! strcmp( dot, ".rtf" ) )
+	if  ( ext && ! strcmp( ext, "rtf" ) )
 	    { ed->edFormat= TEDdockindRTF;	}
-	if  ( dot && ! strcmp( dot, ".txt" ) )
+	if  ( ext && ! strcmp( ext, "txt" ) )
 	    { ed->edFormat= TEDdockindTEXT_SAVE_FOLDED;	}
 	}
 
@@ -404,7 +507,9 @@ int tedNewDocument(	EditApplication *	ea,
     now= time( (time_t *)0 );
     dp->dpCreatim= *localtime( &now );
 
-    return 0;
+  ready:
+
+    return rval;
     }
 
 /************************************************************************/
@@ -412,7 +517,7 @@ int tedNewDocument(	EditApplication *	ea,
 /*  Perform the setup of the editing environment for a document.	*/
 /*									*/
 /*  8)  Determine the code page of the document.			*/
-/*  9)  Do not use the 1252 cidepage as an explicit setting: It is	*/
+/*  9)  Do not use the 1252 codepage as an explicit setting: It is	*/
 /*	almost certainly returned by a default 'Ted' installation, and	*/
 /*	is almost impossible to get rid of.				*/
 /*									*/
@@ -463,13 +568,13 @@ int tedLayoutDocument(		void *				privateData,
 	{ LDEB(1); return -1;	}
 
     /*  8  */
-    if  ( bd->bdProperties.dpAnsiCodePage < 0 )
+    if  ( bd->bdProperties.dpAnsiCodepage < 0 )
 	{
-	bd->bdProperties.dpAnsiCodePage= tedDetermineCodepage( bd, add );
+	bd->bdProperties.dpAnsiCodepage= tedDetermineCodepage( bd );
 
 	/*  9  */
-	if  ( bd->bdProperties.dpAnsiCodePage == 1252 )
-	    { bd->bdProperties.dpAnsiCodePage= -1;	}
+	if  ( bd->bdProperties.dpAnsiCodepage == 1252 )
+	    { bd->bdProperties.dpAnsiCodepage= -1;	}
 	}
 
     return 0;
@@ -562,8 +667,11 @@ void * tedMakePrivateData()
 	{ XDEB(td); return (void *)0;	}
 
     td->tdDocument= (BufferDocument *)0;
-    docInitTextAttribute( &(td->tdCurrentTextAttribute) );
-    td->tdCurrentPhysicalFont= -1;
+    utilInitTextAttribute( &(td->tdCurrentTextAttribute) );
+    td->tdCurrentTextAttributeNumber= -1;
+    td->tdCurrentScreenFont= -1;
+
+    docInitScreenFontList( &(td->tdScreenFontList) );
 
     td->tdFormatMenu= (APP_WIDGET)0;
     td->tdFormatMenuButton= (APP_WIDGET)0;
@@ -581,6 +689,7 @@ void * tedMakePrivateData()
     td->tdInsHyperlinkOption= (APP_WIDGET)0;
     td->tdInsBookmarkOption= (APP_WIDGET)0;
     td->tdInsInsertFootnoteOption= (APP_WIDGET)0;
+    td->tdInsInsertEndnoteOption= (APP_WIDGET)0;
     td->tdInsInsertChftnsepOption= (APP_WIDGET)0;
     td->tdInsInsertTableOption= (APP_WIDGET)0;
     td->tdInsInsertPageNumberOption= (APP_WIDGET)0;
@@ -623,6 +732,8 @@ void * tedMakePrivateData()
 
     appInitBitmapImage( &(td->tdCopiedImage) );
 
+    td->tdFindProg= (void *)0;
+
 #   ifdef USE_MOTIF
     td->tdHideIBarId= (XtIntervalId)0;
     td->tdShowIBarId= (XtIntervalId)0;
@@ -632,10 +743,10 @@ void * tedMakePrivateData()
     td->tdShowIBarId= 0;
 #   endif
 
-    td->tdObjectWindow= (APP_ATOM)0;
-    td->tdObjectBottomWindow= (APP_ATOM)0;
-    td->tdObjectRightWindow= (APP_ATOM)0;
-    td->tdObjectCornerWindow= (APP_ATOM)0;
+    td->tdObjectWindow= (APP_WINDOW)0;
+    td->tdObjectBottomWindow= (APP_WINDOW)0;
+    td->tdObjectRightWindow= (APP_WINDOW)0;
+    td->tdObjectCornerWindow= (APP_WINDOW)0;
     td->tdObjectSelected= 0;
 
     td->tdDrawTableGrid= 1;
@@ -649,7 +760,7 @@ void * tedMakePrivateData()
 /*									*/
 /************************************************************************/
 
-APP_EVENT_HANDLER( tedObserveFocus, w, voided, event )
+APP_EVENT_HANDLER_H( tedObserveFocus, w, voided, event )
     {
     EditDocument *	ed= (EditDocument *)voided;
     TedDocument *	td= (TedDocument *)ed->edPrivateData;
@@ -666,17 +777,15 @@ APP_EVENT_HANDLER( tedObserveFocus, w, voided, event )
 	    {
 	    appSetCurrentDocument( ea, ed );
 
-	    if  ( ea->eaFontsTool )
-		{ tedAdaptFontsToolToDocument( ea, ed ); }
-
 	    if  ( ea->eaFindTool && ed->edIsReadonly )
-		{ appFindToolDisableReplace( ea->eaFindTool ); }
+		{ appFindToolEnableReplace( ea->eaFindTool, 0 ); }
 
 	    if  ( ea->eaSpellTool )
 		{ appEnableSpellTool( ea->eaSpellTool, ! ed->edIsReadonly ); }
 
 	    tedAdaptPageToolToDocument( ea, ed );
-	    tedAdaptFormatToolToDocument( ed );
+
+	    tedAdaptFormatToolToDocument( ed, 0 );
 	    }
 
 	if  ( tedHasIBarSelection( td ) )
@@ -693,7 +802,7 @@ APP_EVENT_HANDLER( tedObserveFocus, w, voided, event )
 /*									*/
 /************************************************************************/
 
-int tedPrintDocument(	FILE *				f,
+int tedPrintDocument(	SimpleOutputStream *		sos,
 			const PrintJob *		pj,
 			const PrintGeometry *		pg,
 			int				firstPage,
@@ -726,11 +835,180 @@ int tedPrintDocument(	FILE *				f,
 	  tedLayoutDocumentTree( td, pj->pjDrawingData )	)
 	{ LDEB(1); return -1;	}
 
-    if  ( docPsPrintDocument( f, pj->pjTitle, ea->eaApplicationName,
-				ea->eaReference, pj->pjDrawingData,
-				td->tdDocument, pg, firstPage, lastPage,
-				tedLayoutExternalItem, tedCloseObject ) )
+    if  ( docPsPrintDocument( sos, pj->pjTitle, ea->eaApplicationName,
+				ea->eaReference, ea->eaFontDirectory,
+				pj->pjDrawingData,
+				td->tdDocument, pg,
+				pj->pjUsePostScriptFilters,
+				pj->pjUsePostScriptIndexedImages,
+				firstPage, lastPage,
+				tedCloseObject ) )
 	{ LDEB(1); return -1;	}
 
     return 0;
     }
+
+void tedSuggestNup(	PrintGeometry *	pg,
+			void *		privateData )
+    {
+    TedDocument *		td= (TedDocument *)privateData;
+    BufferDocument *		bd= td->tdDocument;
+    DocumentProperties *	dp= &(bd->bdProperties);
+
+    if  ( dp->dpTwoOnOne )
+	{
+	pg->pgGridRows= 2;
+	pg->pgGridCols= 1;
+	pg->pgGridHorizontal= 1;
+	pg->pgScalePagesToFit= 1;
+	}
+
+    return;
+    }
+
+/************************************************************************/
+/*									*/
+/*  Open a document.							*/
+/*									*/
+/*  1)  Open an input stream.						*/
+/*  2)  Try to read as RTF.						*/
+/*  3)  If this fails, try as plain text.				*/
+/*									*/
+/************************************************************************/
+
+int tedOpenDocumentFile(	EditApplication *	ea,
+				int *			pFormat,
+				BufferDocument **	pBd,
+				const char *		filename,
+				APP_WIDGET		relative,
+				APP_WIDGET		option )
+    {
+    const char *		ext;
+    SimpleInputStream *		sis;
+
+    BufferDocument *		bd;
+    int				triedRtf= 0;
+
+    int				resp;
+    AppFileMessageResources *	afmr= &(ea->eaFileMessageResources);
+    TedAppResources *		tar= (TedAppResources *)ea->eaResourceData;
+
+    int				longestPara;
+
+    tedDetermineDefaultSettings( tar );
+
+    ext= appFileExtensionOfName( filename );
+
+    /*  1  */
+    sis= sioInStdioOpen( filename );
+    if  ( ! sis )
+	{
+	appQuestionRunSubjectErrorDialog( ea, relative, option,
+					filename, afmr->afmrFileNoAccess );
+
+	return -1;
+	}
+
+    if  ( ext && ! strcmp( ext, "rtf" ) )
+	{
+	/*  2  */
+	bd= docRtfReadFile( sis, tar->tarDefaultAnsicpgInt );
+
+	sioInClose( sis );
+	
+	if  ( bd )
+	    {
+	    *pFormat= TEDdockindRTF; *pBd= bd;
+
+	    if  ( docPropertiesSetFilename( &(bd->bdProperties), filename ) )
+		{ LDEB(1);	}
+
+	    return 0;
+	    }
+	else{
+	    resp= appQuestionRunSubjectOkCancelDialog( ea,
+				    relative, option,
+				    filename, tar->tarFileNotRtf,
+				    (char *)0, (char *)0 );
+
+	    if  ( resp != AQDrespOK )
+		{ return -1;	}
+
+	    triedRtf= 1;
+	    }
+	}
+
+    if  ( ext && ! strcmp( ext, "txt" ) )
+	{
+	bd= docPlainReadFile( sis, &longestPara,
+				    DOCcharsetANSI, tar->tarDefaultAnsicpgInt,
+				    &(ea->eaDefaultDocumentGeometry) );
+
+	sioInClose( sis );
+
+	if  ( bd )
+	    {
+	    *pBd= bd;
+
+	    if  ( longestPara > 76 )
+		{ *pFormat= TEDdockindTEXT_SAVE_WIDE;		}
+	    else{ *pFormat= TEDdockindTEXT_SAVE_FOLDED;		}
+
+	    if  ( docPropertiesSetFilename( &(bd->bdProperties), filename ) )
+		{ LDEB(1);	}
+
+	    return 0;
+	    }
+	else{ SXDEB(filename,bd); return -1;	}
+	}
+
+    /*  2  */
+    if  ( ! triedRtf )
+	{
+	bd= docRtfReadFile( sis, tar->tarDefaultAnsicpgInt );
+	if  ( bd )
+	    {
+	    *pFormat= TEDdockindRTF; *pBd= bd;
+	    
+	    if  ( docPropertiesSetFilename( &(bd->bdProperties), filename ) )
+		{ LDEB(1);	}
+
+	    return 0;
+	    }
+
+	resp= appQuestionRunSubjectOkCancelDialog( ea,
+					ea->eaToplevel.atTopWidget, option,
+					filename, tar->tarFileNotRtf,
+					(char *)0, (char *)0 );
+
+	if  ( resp != AQDrespOK )
+	    { return -1;	}
+	}
+
+    sis= sioInStdioOpen( filename );
+    if  ( ! sis )
+	{ SXDEB(filename,sis); return -1;	}
+
+    bd= docPlainReadFile( sis, &longestPara,
+				    DOCcharsetANSI, tar->tarDefaultAnsicpgInt,
+				    &(ea->eaDefaultDocumentGeometry) );
+
+    sioInClose( sis );
+
+    if  ( bd )
+	{
+	*pBd= bd;
+
+	if  ( longestPara > 76 )
+	    { *pFormat= TEDdockindTEXT_SAVE_WIDE;		}
+	else{ *pFormat= TEDdockindTEXT_SAVE_FOLDED;		}
+
+	if  ( docPropertiesSetFilename( &(bd->bdProperties), filename ) )
+	    { LDEB(1);	}
+
+	return 0;
+	}
+
+    SXDEB(filename,bd); return -1;
+    }
+
