@@ -2,6 +2,8 @@
 /*									*/
 /*  Geometry calculations when looking for a position in the document.	*/
 /*									*/
+/*  Find the object in the document for an x,y position.		*/
+/*									*/
 /************************************************************************/
 
 #   include	"tedConfig.h"
@@ -21,6 +23,184 @@
 
 #   include	<appDebugon.h>
 
+/************************************************************************/
+/*									*/
+/*  Find the cell in a table row that contains the given x,y position	*/
+/*									*/
+/*  1)  For all columns..						*/
+/*  2)  Skip cells that just make up a rowspan/colspan.			*/
+/*  3)  For cells with a colspan.. determine the right hand side.	*/
+/*  4)  Skip cell to the left of the mouse click. (But give the last	*/
+/*	one a chance anyway.						*/
+/*  5)  For the top cell of a rowspan, find the bottom of the row where	*/
+/*	it ends.							*/
+/*  6)  Check the bottom of the cell. Normally this is done by the	*/
+/*	caller, but because of rowspan calls, her criterion could have	*/
+/*	been too lenient.						*/
+/*  7)  Found!								*/
+/*									*/
+/************************************************************************/
+
+static BufferItem * tedFindRowCell(	BufferItem *		rowBi,
+					const AppDrawingData *	add,
+					int			x,
+					int			y )
+    {
+    const CellProperties *	cp;
+    int				i;
+
+    int				rowBottom= BI_BELOW_PIXELS( add, rowBi );
+
+    /*  1  */
+    cp= rowBi->biRowProperties.rpCells;
+    for ( i= 0; i < rowBi->biChildCount; cp++, i++ )
+	{
+	BufferItem *	cellBi;
+	int		j;
+
+	int		cellRight= cp->cpRightBoundaryPixels;
+	int		cellBottom= rowBottom;
+
+	/*  2  */
+	if  ( cp->cpMergedWithLeft || cp->cpMergedWithAbove )
+	    { continue; }
+
+	/*  3  */
+	if  ( cp->cpLeftInMergedRow )
+	    {
+	    const CellProperties *	cpp= cp;
+
+	    for ( j= i; j < rowBi->biChildCount- 1; cpp++, j++ )
+		{
+		if  ( ! cpp[1].cpMergedWithLeft )
+		    { break;	}
+
+		cellRight= cpp[1].cpRightBoundaryPixels;
+		}
+	    }
+
+	/*  4  */
+	if  ( x > add->addDocRect.drX0+ cellRight		&&
+	      i < rowBi->biChildCount- 1			)
+	    { continue;	}
+
+	cellBi= rowBi->biChildren[i];
+
+	/*  5  */
+	if  ( cp->cpTopInMergedColumn )
+	    {
+	    int			nextRow= rowBi->biNumberInParent+ 1;
+	    BufferItem *	nextRowBi= (BufferItem *)0;
+
+	    while( nextRow < rowBi->biRowTablePast )
+		{
+		const CellProperties *	nextCp;
+
+		nextRowBi= rowBi->biParent->biChildren[nextRow];
+		nextCp= &(nextRowBi->biRowCells[i]);
+
+		if  ( ! nextCp->cpMergedWithAbove )
+		    { break;	}
+
+		cellBottom= BI_BELOW_PIXELS( add, nextRowBi );
+		nextRow++;
+		}
+	    }
+
+	/*  6  */
+	if  ( y > cellBottom )
+	    { continue;	}
+
+	/*  7  */
+	return cellBi;
+	}
+
+    /*
+    LLDEB(x,y);
+    LDEB(rowBi->biChildCount);
+    */
+    return (BufferItem *)0;
+    }
+
+/************************************************************************/
+/*									*/
+/*  Find a child in a parent that stacks its children in the normal	*/
+/*  page order.								*/
+/*									*/
+/*  1)  For all children..						*/
+/*  2)  A subsequent child?						*/
+/*  3)  Determine the position below the child. Rows with 'rowspan'	*/
+/*	children might protrude below below the actual bottom of the	*/
+/*	child.								*/
+/*  4)  If there is a 'next' on the current page and the y is below the	*/
+/*	bottom of the current child, skip the current child.		*/
+/*  5)  For rows, try to find a cell of the row. (Skips a level!)	*/
+/*  6)  All reasons to reject the child have been covered above so	*/
+/*	return this child.						*/
+/*									*/
+/************************************************************************/
+
+static BufferItem * tedFindGroupChild(	BufferItem *		bi,
+					const AppDrawingData *	add,
+					int			x,
+					int			y,
+					int			page )
+    {
+    int			i;
+    BufferItem *	child;
+
+    /*  1  */
+    for ( i= 0; i < bi->biChildCount; i++ )
+	{
+	BufferItem *	next= (BufferItem *)0;
+	LayoutPosition	lpBelow;
+
+	child= bi->biChildren[i];
+
+	/*  2  */
+	if  ( i+ 1 < bi->biChildCount )
+	    { next= bi->biChildren[i+ 1];	}
+
+	/*  3  */
+	lpBelow= child->biBelowPosition;
+
+	if  ( child->biLevel == DOClevROW	&&
+	      child->biRowHasTableParagraphs	)
+	    { lpBelow= child->biRowBelowAllPosition; }
+
+	/*  4  */
+	if  ( y > LP_YPIXELS( add, &lpBelow )		&&
+	      next					&&
+	      next->biTopPosition.lpPage <= page	)
+	    { continue;	}
+
+	/*  5  */
+	if  ( child->biLevel == DOClevROW	&&
+	      child->biRowHasTableParagraphs	)
+	    {
+	    BufferItem *	cellBi;
+
+	    cellBi= tedFindRowCell( child, add, x, y );
+	    if  ( cellBi )
+		{ return cellBi; }
+
+	    continue;
+	    }
+
+	return child;
+	}
+
+    if  ( bi->biChildCount > 0 )
+	{
+	child= bi->biChildren[bi->biChildCount- 1];
+
+	return child;
+	}
+
+    XDEB(bi->biChildCount);
+    return (BufferItem *)0;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Find the object in the document for an x,y position.		*/
@@ -32,10 +212,8 @@ static BufferItem * tedFindItem(	BufferItem *		bi,
 					int			x,
 					int			y )
     {
-    int				i;
     BufferItem *		child;
     BufferItem *		found;
-    const CellProperties *	cp;
     int				page= y/ add->addPageStepPixels;
 
     switch( bi->biLevel )
@@ -44,30 +222,14 @@ static BufferItem * tedFindItem(	BufferItem *		bi,
 	case DOClevSECT:
 	case DOClevCELL:
 	rowAsGroup:
-	    for ( i= 0; i < bi->biChildCount; i++ )
+
+	    child= tedFindGroupChild( bi, add, x, y, page );
+	    if  ( child )
 		{
-		BufferItem *	next= (BufferItem *)0;
-
-		child= bi->biChildren[i];
-
-		if  ( i+ 1 < bi->biChildCount )
-		    { next= bi->biChildren[i+ 1];	}
-
-		if  ( y < BI_BELOW_PIXELS( add, child )			||
-		      ( next					&&
-		        next->biTopPosition.lpPage > page	)	)
-		    {
-		    found= tedFindItem( child, add, x, y );
-		    return found;
-		    }
-		}
-	    if  ( bi->biChildCount > 0 )
-		{
-		child= bi->biChildren[bi->biChildCount- 1];
-
 		found= tedFindItem( child, add, x, y );
 		return found;
 		}
+
 	    LDEB(bi->biChildCount);
 	    break;
 
@@ -75,44 +237,11 @@ static BufferItem * tedFindItem(	BufferItem *		bi,
 	    if  ( ! bi->biRowHasTableParagraphs )
 		{ goto rowAsGroup;	}
 
-	    cp= bi->biRowProperties.rpCells;
-	    for ( i= 0; i < bi->biChildCount; cp++, i++ )
+	    child= tedFindRowCell( bi, add, x, y );
+	    if  ( child )
 		{
-		BufferItem *	paraBi;
-		int		j;
-		int		right= cp->cpRightBoundaryPixels;
-
-		if  ( cp->cpMergedWithLeft )
-		    { continue; }
-
-		if  ( cp->cpLeftInMergedRow )
-		    {
-		    const CellProperties *	cpp= cp;
-
-		    for ( j= i; j < bi->biChildCount- 1; cpp++, j++ )
-			{
-			if  ( ! cpp[1].cpMergedWithLeft )
-			    { break;	}
-
-			right= cpp[1].cpRightBoundaryPixels;
-			}
-		    }
-
-		if  ( x <= add->addDocRect.drX0+ right		||
-		      i == bi->biChildCount- 1			)
-		    {
-		    child= bi->biChildren[i];
-		    paraBi= tedFindItem( child, add, x, y );
-
-		    if  ( paraBi )
-			{ return paraBi;	}
-
-		    if  ( child->biChildCount > 0 )
-			{
-			found= child->biChildren[child->biChildCount-1];
-			return found;
-			}
-		    }
+		found= tedFindItem( child, add, x, y );
+		return found;
 		}
 
 	    LLDEB(x,y);
@@ -165,7 +294,7 @@ static int tedFindLine(	const BufferItem *	bi,
 int tedFindParticule(	TextLine *	tl,
 			TextParticule *	tp,
 			int		x,
-			int		y	)
+			int		y )
     {
     int				i;
 
@@ -189,22 +318,29 @@ int tedFindParticule(	TextLine *	tl,
 /*									*/
 /************************************************************************/
 
-int tedCalculateX(	const BufferItem *	bi,
-			const TextParticule *	tp,
-			const AppDrawingData *	add,
-			int			stroff	)
+static int tedCalculateX(	const BufferDocument *	bd,
+				const BufferItem *	bi,
+				const TextParticule *	tp,
+				const AppDrawingData *	add,
+				const ScreenFontList *	sfl,
+				int			stroff )
     {
-    const AppPhysicalFontList *	apfl= &(add->addPhysicalFontList);
-    AppPhysicalFont *		apf;
+    const DrawScreenFontList *	apfl= &(add->addScreenFontList);
     const unsigned char *	s;
     int				len;
     int				x;
 
+    TextAttribute		ta;
+    int				textAttr;
+    int				screenFont;
+    DrawScreenFont *		dsf;
+
     switch( tp->tpKind )
 	{
 	case DOCkindOBJECT:
 	case DOCkindTAB:
 	case DOCkindCHFTNSEP:
+	case DOCkindCHFTNSEPC:
 	    len= stroff- tp->tpStroff;
 
 	    if  ( len == 0 )
@@ -223,16 +359,24 @@ int tedCalculateX(	const BufferItem *	bi,
 	    return x;
 
 	case DOCkindTEXT:
-	    if  ( tp->tpPhysicalFont < 0 )
-		{ LDEB(tp->tpPhysicalFont); return tp->tpX0;	}
+	    textAttr= tp->tpTextAttributeNumber;
+
+	    if  ( textAttr >= sfl->sflAttributeToScreenCount	||
+		  sfl->sflAttributeToScreen[textAttr] < 0	)
+		{ LDEB(textAttr); return tp->tpX0;	}
+
+	    screenFont= sfl->sflAttributeToScreen[textAttr];
+	    dsf= apfl->apflFonts+ screenFont;
+
+	    utilGetTextAttributeByNumber( &ta, &(bd->bdTextAttributeList),
+								textAttr );
 
 	    s= bi->biParaString+ tp->tpStroff;
-	    apf= apfl->apflFonts+ tp->tpPhysicalFont;
 	    len= stroff- tp->tpStroff;
 
 	    if  ( len == tp->tpStrlen )
 		{ x= tp->tpX0+ tp->tpPixelsWide;			}
-	    else{ x= tp->tpX0+ tedTextWidth( apf, add, s, len );	}
+	    else{ x= tp->tpX0+ tedTextWidth( dsf, &ta, bd, add, s, len ); }
 	    return x;
 
 	case DOCkindNOTE:
@@ -242,6 +386,7 @@ int tedCalculateX(	const BufferItem *	bi,
 	case DOCkindTC:
 	case DOCkindLINEBREAK:
 	case DOCkindPAGEBREAK:
+	case DOCkindCOLUMNBREAK:
 	    x= tp->tpX0;
 	    return x;
 
@@ -265,29 +410,44 @@ int tedCalculateX(	const BufferItem *	bi,
 /*									*/
 /************************************************************************/
 
-int tedFindStringOffset(	const TextParticule *	tp,
-				const unsigned char *	paraString,
+int tedFindStringOffset(	const BufferDocument *	bd,
+				const BufferItem *	paraBi,
+				int			part,
 				const AppDrawingData *	add,
+				const ScreenFontList *	sfl,
 				int *			pBarX,
 				int			x,
-				int			y	)
+				int			y )
     {
-    const AppPhysicalFontList *	apfl= &(add->addPhysicalFontList);
+    const TextParticule *	tp= paraBi->biParaParticules+ part;
+    const DrawScreenFontList *	apfl= &(add->addScreenFontList);
     int				len;
     int				stroff= tp->tpStroff+ 1;
-    const unsigned char *	s= paraString+ tp->tpStroff;
-    AppPhysicalFont *		apf;
+    const unsigned char *	s= paraBi->biParaString+ tp->tpStroff;
     int				before= tp->tpX0;
     int				past= tp->tpX0;
 
+    int				textAttr;
+    int				screenFont;
+    DrawScreenFont *		dsf;
+
+    TextAttribute		ta;
+
     /*  1  */
     if  ( x < before || tp->tpStrlen == 0 )
 	{ *pBarX= tp->tpX0; return tp->tpStroff; }
 
-    if  ( tp->tpPhysicalFont < 0 )
-	{ LDEB(tp->tpPhysicalFont); return tp->tpStroff;	}
+    textAttr= tp->tpTextAttributeNumber;
 
-    apf= apfl->apflFonts+ tp->tpPhysicalFont;
+    if  ( textAttr >= sfl->sflAttributeToScreenCount	||
+	  sfl->sflAttributeToScreen[textAttr] < 0	)
+	{ LDEB(textAttr); *pBarX= tp->tpX0; return tp->tpStroff;	}
+
+    screenFont= sfl->sflAttributeToScreen[textAttr];
+    dsf= apfl->apflFonts+ screenFont;
+
+    utilGetTextAttributeByNumber( &ta, &(bd->bdTextAttributeList),
+						tp->tpTextAttributeNumber );
 
     /*  2  */
     for ( len= 1; len <= tp->tpStrlen; stroff++, len++ )
@@ -295,8 +455,8 @@ int tedFindStringOffset(	const TextParticule *	tp,
 	int	between;
 
 	if  ( len == tp->tpStrlen )
-	    { past= tp->tpX0+ tp->tpPixelsWide;			}
-	else{ past= tp->tpX0+ tedTextWidth( apf, add, s, len );	}
+	    { past= tp->tpX0+ tp->tpPixelsWide;				}
+	else{ past= tp->tpX0+ tedTextWidth( dsf, &ta, bd, add, s, len ); }
 
 	between= ( before+ past )/ 2;
 
@@ -308,7 +468,8 @@ int tedFindStringOffset(	const TextParticule *	tp,
 	before= past;
 	}
 
-    *pBarX= tp->tpX0+ tp->tpPixelsWide; return tp->tpStroff+ tp->tpStrlen;
+    *pBarX= tp->tpX0+ tp->tpPixelsWide;
+    return tp->tpStroff+ tp->tpStrlen;
     }
 
 /************************************************************************/
@@ -317,12 +478,14 @@ int tedFindStringOffset(	const TextParticule *	tp,
 /*									*/
 /************************************************************************/
 
-int tedFindPosition(	DocumentPosition *	dp,
-			PositionGeometry *	pg,
-			BufferItem *		rootBi,
-			const AppDrawingData *	add,
-			int			x,
-			int			y )
+int tedFindPosition(	DocumentPosition *		dp,
+			PositionGeometry *		pg,
+			const BufferDocument *		bd,
+			BufferItem *			rootBi,
+			const AppDrawingData *		add,
+			const ScreenFontList *		sfl,
+			int				x,
+			int				y )
     {
     const BufferItem *	bi;
 
@@ -338,19 +501,23 @@ int tedFindPosition(	DocumentPosition *	dp,
 	{
 	if  ( y < BI_TOP_PIXELS( add, rootBi ) )
 	    {
+	    const int	lastOne= 0;
+
 	    if  ( docFirstPosition( dp, rootBi ) )
 		{ LDEB(y); return -1; }
 
-	    tedPositionGeometry( pg, dp, add );
+	    tedPositionGeometry( pg, dp, lastOne, bd, add, sfl );
 	    return 0;
 	    }
 
 	if  ( y >= BI_BELOW_PIXELS( add, rootBi ) )
 	    {
+	    const int	lastOne= 0;
+
 	    if  ( docLastPosition( dp, rootBi ) )
 		{ LDEB(y); return -1; }
 
-	    tedPositionGeometry( pg, dp, add );
+	    tedPositionGeometry( pg, dp, lastOne, bd, add, sfl );
 	    return 0;
 	    }
 
@@ -399,8 +566,7 @@ int tedFindPosition(	DocumentPosition *	dp,
 	    if  ( part < 0 )
 		{ LLDEB(x,part); return -1;			}
 
-	    stroff= tedFindStringOffset( bi->biParaParticules+ part,
-					bi->biParaString, add, &barX, x, y );
+	    stroff= tedFindStringOffset( bd, bi, part, add, sfl, &barX, x, y );
 	    if  ( stroff < 0 )
 		{ LDEB(stroff); return -1;	}
 	    }
@@ -420,10 +586,13 @@ int tedFindPosition(	DocumentPosition *	dp,
     tl= bi->biParaLines+ line;
 
     dp->dpBi= (BufferItem *)bi;	/*  Discards const */
-    dp->dpParticule= part;
     dp->dpStroff= stroff;
 
     pg->pgLine= line;
+    pg->pgAtLineHead= 0;
+    if  ( stroff == tl->tlStroff )
+	{ pg->pgAtLineHead= 1;	}
+
     pg->pgXPixels= barX;
     pg->pgTopPosition= tl->tlTopPosition;
     pg->pgY1Pixels= TL_BELOW_PIXELS( add, tl );
@@ -514,8 +683,8 @@ static void tedSelectionRectangle(
 	docParagraphFrameTwips( &pf, &bf, bottom, stripHigh, ds->dsBegin.dpBi );
 	tedParagraphFramePixels( &pf, add, ds->dsBegin.dpBi );
 
-	drSel->drX0= pf.pfX0Pixels;
-	drSel->drX1= pf.pfX1Pixels;
+	dr.drX0= pf.pfX0Pixels;
+	dr.drX1= pf.pfX1Pixels;
 
 	if  ( docSelectionInsideCell( ds ) )
 	    {
@@ -542,8 +711,8 @@ static void tedSelectionRectangle(
 	docParagraphFrameTwips( &pf, &bf, bottom, stripHigh, ds->dsEnd.dpBi );
 	tedParagraphFramePixels( &pf, add, ds->dsEnd.dpBi );
 
-	drSel->drX0= pf.pfX0Pixels;
-	drSel->drX1= pf.pfX1Pixels;
+	dr.drX0= pf.pfX0Pixels;
+	dr.drX1= pf.pfX1Pixels;
 
 	if  ( docSelectionInsideCell( ds ) )
 	    {
@@ -563,23 +732,40 @@ static void tedSelectionRectangle(
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Determine the poition on paper/screen of a certain position in the	*/
+/*  document.								*/
+/*									*/
+/************************************************************************/
+
 void tedPositionGeometry(	PositionGeometry *		pg,
 				const DocumentPosition *	dp,
-				const AppDrawingData *		add )
+				int				lastOne,
+				const BufferDocument *		bd,
+				const AppDrawingData *		add,
+				const ScreenFontList *		sfl )
     {
+    int			part;
     int			line;
 
     TextLine *		tl;
     TextParticule *	tp;
 
-    if  ( docFindLineOfParticule( &line, dp->dpBi, dp->dpParticule ) )
-	{ LDEB(dp->dpParticule); return;	}
+    if  ( docFindParticuleOfPosition( &part, dp, lastOne ) )
+	{ LDEB(dp->dpStroff); return;	}
 
+    if  ( docFindLineOfPosition( &line, dp, lastOne ) )
+	{ LDEB(dp->dpStroff); return;	}
+
+    tp= dp->dpBi->biParaParticules+ part;
     tl= dp->dpBi->biParaLines+ line;
-    tp= dp->dpBi->biParaParticules+ dp->dpParticule;
 
     pg->pgLine= line;
-    pg->pgXPixels= tedCalculateX( dp->dpBi, tp, add, dp->dpStroff );
+    pg->pgAtLineHead= 0;
+    if  ( dp->dpStroff == tl->tlStroff )
+	{ pg->pgAtLineHead= 1;	}
+    pg->pgXPixels= tedCalculateX( bd, dp->dpBi, tp, add, sfl, dp->dpStroff );
     pg->pgTopPosition= tl->tlTopPosition;
     pg->pgY1Pixels= TL_BELOW_PIXELS( add, tl );
     pg->pgBaselinePixels= TL_BASE_PIXELS( add, tl );
@@ -595,12 +781,22 @@ void tedPositionGeometry(	PositionGeometry *		pg,
 
 void tedSelectionGeometry(	SelectionGeometry *		sg,
 				const DocumentSelection *	ds,
+				int				lastLine,
 				const BufferDocument *		bd,
-				const AppDrawingData *		add )
+				const AppDrawingData *		add,
+				const ScreenFontList *		sfl )
     {
-    tedPositionGeometry( &(sg->sgBegin), &(ds->dsBegin), add );
-    tedPositionGeometry( &(sg->sgAnchor), &(ds->dsAnchor), add );
-    tedPositionGeometry( &(sg->sgEnd), &(ds->dsEnd), add );
+    if  ( ds->dsDirection != 0 )
+	{ lastLine= 0;	}
+
+    tedPositionGeometry( &(sg->sgBegin), &(ds->dsBegin),
+					    ds->dsDirection > 0 || lastLine,
+					    bd, add, sfl );
+    tedPositionGeometry( &(sg->sgAnchor), &(ds->dsAnchor),
+					    ds->dsDirection > 0 || lastLine,
+					    bd, add, sfl );
+    tedPositionGeometry( &(sg->sgEnd), &(ds->dsEnd), lastLine,
+					    bd, add, sfl );
 
     tedSelectionRectangle( sg, add, bd, ds );
 
@@ -613,7 +809,11 @@ void tedSelectionGeometry(	SelectionGeometry *		sg,
 /*									*/
 /*  1)  Calculate the page number from the coordinates.			*/
 /*  2)  Find the first section corresponding to this page.		*/
-/*  3)  Oups: not found should have been impossible.			*/
+/*	For finding headers and footers this is sufficient. For finding	*/
+/*	the actual body section, it is not: Below a more precise	*/
+/*	attempt is made.						*/
+/*  3)  If no section can be found, the click might have been in a note	*/
+/*	protruding beyond the the last page with regular text.		*/
 /*  4)  Is the click in the header corresponding to this section?	*/
 /*  5)  Is the click in the footer corresponding to this section?	*/
 /*  6)  Is the click in a note on this page?				*/
@@ -687,7 +887,6 @@ int tedFindRootForPosition(		ExternalItem **		pEi,
     int				page;
     int				sectNr;
 
-    const DocumentGeometry *	dgSect;
     int				pageY;
 
     BufferItem *		bodySectBi= (BufferItem *)0;
@@ -711,57 +910,63 @@ int tedFindRootForPosition(		ExternalItem **		pEi,
 	{
 	bodySectBi= bd->bdItem.biChildren[sectNr];
 
-	if  ( bodySectBi->biTopPosition.lpPage <= page	&&
+	if  ( bodySectBi->biTopPosition.lpPage <= page		&&
 	      bodySectBi->biBelowPosition.lpPage >= page	)
 	    { break;	}
 	}
 
-    /*  3  */
+    /*  3
     if  ( sectNr >= bd->bdItem.biChildCount )
 	{ LLDEB(page,sectNr); return -1;	}
+    */
 
-    dgSect= &(bodySectBi->biSectDocumentGeometry);
-
-    /*  4  */
-    ei= (ExternalItem *)0;
-    inHeaderFooter= docWhatPageHeader( &ei, bodySectBi, page, dp );
-    if  ( ei && ei->eiItem )
+    if  ( sectNr < bd->bdItem.biChildCount )
 	{
-	if  ( docGetExternalItemBox( &drExtern, bodySectBi, ei, justUsed,
-							    page, bd, add ) )
-	    { LDEB(1);	}
-	else{
-	    if  ( docY >= drExtern.drY0 && docY <= drExtern.drY1 )
-		{
-		*pEi= ei;
-		*pRootBi= ei->eiItem;
-		*pBodySectBi= bodySectBi;
-		*pSectNr= sectNr;
-		*pPage= page;
+	const DocumentGeometry *	dgSect;
 
-		return 0;
+	dgSect= &(bodySectBi->biSectDocumentGeometry);
+
+	/*  4  */
+	ei= (ExternalItem *)0;
+	inHeaderFooter= docWhatPageHeader( &ei, bodySectBi, page, dp );
+	if  ( ei && ei->eiItem )
+	    {
+	    if  ( docGetExternalItemBox( &drExtern, bodySectBi, ei, justUsed,
+							    page, bd, add ) )
+		{ LDEB(1);	}
+	    else{
+		if  ( docY >= drExtern.drY0 && docY <= drExtern.drY1 )
+		    {
+		    *pEi= ei;
+		    *pRootBi= ei->eiItem;
+		    *pBodySectBi= bodySectBi;
+		    *pSectNr= sectNr;
+		    *pPage= page;
+
+		    return 0;
+		    }
 		}
 	    }
-	}
 
-    /*  5  */
-    ei= (ExternalItem *)0;
-    inHeaderFooter= docWhatPageFooter( &ei, bodySectBi, page, dp );
-    if  ( ei && ei->eiItem )
-	{
-	if  ( docGetExternalItemBox( &drExtern, bodySectBi, ei, justUsed,
+	/*  5  */
+	ei= (ExternalItem *)0;
+	inHeaderFooter= docWhatPageFooter( &ei, bodySectBi, page, dp );
+	if  ( ei && ei->eiItem )
+	    {
+	    if  ( docGetExternalItemBox( &drExtern, bodySectBi, ei, justUsed,
 							    page, bd, add ) )
-	    { LDEB(1);	}
-	else{
-	    if  ( docY >= drExtern.drY0 && docY <= drExtern.drY1 )
-		{
-		*pEi= ei;
-		*pRootBi= ei->eiItem;
-		*pBodySectBi= bodySectBi;
-		*pSectNr= sectNr;
-		*pPage= page;
+		{ LDEB(1);	}
+	    else{
+		if  ( docY >= drExtern.drY0 && docY <= drExtern.drY1 )
+		    {
+		    *pEi= ei;
+		    *pRootBi= ei->eiItem;
+		    *pBodySectBi= bodySectBi;
+		    *pSectNr= sectNr;
+		    *pPage= page;
 
-		return 0;
+		    return 0;
+		    }
 		}
 	    }
 	}
@@ -770,8 +975,7 @@ int tedFindRootForPosition(		ExternalItem **		pEi,
     dn= bd->bdNotes;
     for ( i= 0; i < bd->bdNoteCount; dn++, i++ )
 	{
-	int		topPixels;
-	int		belowPixels;
+	BufferItem *		noteBodySectBi;
 
 	ei= &(dn->dnExternalItem);
 
@@ -780,26 +984,34 @@ int tedFindRootForPosition(		ExternalItem **		pEi,
 	if  ( ! ei->eiItem )
 	    { XDEB(ei->eiItem); continue;	}
 
-	if  ( ei->eiItem->biTopPosition.lpPage < page )
+	noteBodySectBi= bd->bdItem.biChildren[dn->dnSectNr];
+
+	if  ( ei->eiItem->biBelowPosition.lpPage < page )
 	    { continue;	}
 	if  ( ei->eiItem->biTopPosition.lpPage > page )
 	    { continue;	}
 
-	topPixels= BI_TOP_PIXELS( add, ei->eiItem );
-	belowPixels= BI_BELOW_PIXELS( add, ei->eiItem );
+	if  ( docGetExternalItemBox( &drExtern, noteBodySectBi, ei, justUsed,
+							page, bd, add ) )
+	    { LDEB(1);	}
+	else{
+	    if  ( docY >= drExtern.drY0 && docY <= drExtern.drY1 )
+		{
+		*pEi= ei;
+		*pRootBi= ei->eiItem;
+		*pBodySectBi= noteBodySectBi;
+		*pSectNr= dn->dnSectNr;
+		*pPage= page;
 
-	if  ( docY >= topPixels && docY <= belowPixels )
-	    {
-	    *pEi= ei;
-	    *pRootBi= ei->eiItem;
-	    *pBodySectBi= bodySectBi;
-	    *pSectNr= sectNr;
-	    *pPage= page;
-
-	    return 0;
+		return 0;
+		}
 	    }
 	}
 
+    /*  3  */
+    if  ( sectNr >= bd->bdItem.biChildCount )
+	{ LLDEB(page,sectNr); return -1;	}
+
     /*  7  */
     ret= tedFindTryNoteSeparator( pEi, pRootBi, bd, add,
 				page, docX, docY, DOCinFOOTNOTE, DOCinFTNSEP );
@@ -829,6 +1041,8 @@ int tedFindRootForPosition(		ExternalItem **		pEi,
 	}
 
     /*  9  */
+    bodySectBi= tedFindGroupChild( &(bd->bdItem), add, docX, docY, page );
+
     *pEi= (ExternalItem *)0;
     *pRootBi= bodySectBi;
     *pBodySectBi= bodySectBi;
