@@ -7,7 +7,8 @@
 #   ifndef	TED_FORMAT_TOOL_H
 #   define	TED_FORMAT_TOOL_H
 
-#   include	"tedParaTool.h"
+#   include	"tedParaLayoutTool.h"
+#   include	"tedParaOrnamentsTool.h"
 #   include	"tedRowTool.h"
 #   include	"tedSectionTool.h"
 #   include	"tedPageLayoutTool.h"
@@ -17,6 +18,11 @@
 #   include	"tedColumnTool.h"
 #   include	"tedCellTool.h"
 #   include	"tedTabsTool.h"
+#   include	"tedListTool.h"
+#   include	"tedFontTool.h"
+#   include	"tedLinkTool.h"
+
+#   include	<appRgbChooserPage.h>
 
 /************************************************************************/
 /*									*/
@@ -30,12 +36,19 @@ typedef enum ToolSubjectIndex
     TEDtsiROW,
     TEDtsiCOLUMN,
     TEDtsiCELL,
-    TEDtsiPARA,
+    TEDtsiPARA_LAY,
     TEDtsiTABS,
+    TEDtsiPARA_ORN,
+    TEDtsiLISTS,
     TEDtsiSECT,
     TEDtsiPAGE,
     TEDtsiHEADFOOT,
     TEDtsiNOTES,
+    TEDtsiFONT,
+    TEDtsiLISTFONT,
+    TEDtsiLINK,
+
+    TEDtsiRGB,
 
     TEDtsi_COUNT
 
@@ -56,18 +69,19 @@ typedef struct TedFormatTool
     RowTool			tftRowTool;
     ColumnTool			tftColumnTool;
     CellTool			tftCellTool;
-    ParagraphTool		tftParagraphTool;
+    ParagraphLayoutTool		tftParagraphLayoutTool;
+    TabsTool			tftTabsTool;
+    ParagraphOrnamentsTool	tftParagraphOrnamentsTool;
+    ListTool			tftListsTool;
     SectionTool			tftSectionTool;
     PageLayoutTool		tftPageLayoutTool;
     HeaderFooterTool		tftHeaderFooterTool;
     NotesTool			tftNotesTool;
-    TabsTool			tftTabsTool;
+    AppFontChooser		tftFontTool;
+    AppFontChooser		tftListFontTool;
+    LinkTool			tftLinkTool;
 
-    AppDrawingData		tftPixmapAdd;
-    int				tftPixmapAddAllocated;
-
-    APP_BITMAP_IMAGE		tftShadingPixmaps[DOCsp_COUNT];
-    APP_BITMAP_IMAGE		tftBorderPixmaps[DOCbs_COUNT];
+    RgbChooserPage		tftRgbPage;
     } TedFormatTool;
 
 /************************************************************************/
@@ -76,38 +90,14 @@ typedef struct TedFormatTool
 /*									*/
 /************************************************************************/
 
-extern void tedFormatRefreshHeightChooser(	HeightChooser *	hc,
-						int		height );
-
-extern void tedFormatHeightChosen(	APP_WIDGET		w,
-					HeightChooser *		hc,
-					int			defaultValue );
-
-extern void tedFormatMakeHeightRow(	APP_WIDGET *		pRow,
-					void *			through,
-					APP_WIDGET		parent,
-					HeightChooser *		hc,
-					APP_CALLBACK_FUNC	callback );
-
 extern void tedFormatFinishRowPage( 	RowTool *			rt,
 					TedFormatTool *			tft,
 					const RowPageResources *	rpr );
 
-extern void tedFormatFillHeightChooser(	HeightChooser *		hc,
-					APP_CALLBACK_FUNC	callback,
-					void *			voidtft,
-					const char *		freeText,
-					const char *		atLeastText,
-					const char *		exactlyText );
-
-extern int tedFormatToolGetHeight(	int *		pHeight,
-					HeightChooser *	hc );
-
-extern void tedFormatReflectBorder(	BorderTool *			bt,
-					const BorderProperties *	bp );
-
 extern void tedFormatFillRowPage( RowTool *			rt,
 				const RowPageResources *	rpr,
+				AppInspector *			ai,
+				int				subjectPage,
 				InspectorSubject *		is,
 				APP_WIDGET			pageWidget,
 				const InspectorSubjectResources * isr );
@@ -126,15 +116,12 @@ extern int tedFormatToolGetGapWidth(	const RowProperties *	rp,
 
 extern void tedFormatFillColumnPage(	ColumnTool *		ct,
 				const ColumnPageResources *	cpr,
+				AppInspector *			ai,
+				int				subjectPage,
 				InspectorSubject *		is,
 				APP_WIDGET			pageWidget,
 				const InspectorSubjectResources * isr );
 
-extern void tedFormatToggleBorder(	BorderProperties *	bp,
-					BorderTool *		bt,
-					int			doStyleMenu,
-					int			set );
-
 extern void tedFormatFillSectionPage(	SectionTool *		st,
 				const SectionPageResources *	spr,
 				InspectorSubject *		is,
@@ -174,38 +161,13 @@ extern void tedFormatFinishColumnPage(	ColumnTool *			ct,
 					TedFormatTool *			tft,
 					const ColumnPageResources *	cpr );
 
-extern void tedFormatFillColumnChoosers( ColumnTool *			ct,
+extern void tedColumnToolFillChoosers( ColumnTool *			ct,
 					const ColumnPageResources *	cpr );
 
 extern void tedFormatFinishCellPage(	CellTool *			ct,
 					TedFormatTool *			tft,
 					const CellPageResources *	cpr );
 
-extern void tedFormatMakeBorderTool(	BorderTool *		bt,
-					APP_WIDGET *		pRow,
-					APP_WIDGET		page,
-					const char *		text,
-					APP_TOGGLE_CALLBACK	callback,
-					void *			through );
-
-extern void tedFormatCleanShadingTool(	ShadingTool *		st );
-extern void tedFormatCleanBorderTool(	BorderTool *		bt );
-
-extern void tedFormatMakeShadingTool(	ShadingTool *		st,
-					APP_WIDGET *		pRow,
-					APP_WIDGET		page,
-					const char *		text );
-
-extern void tedFormatFillShadingTool(	ShadingTool *		st,
-					APP_BUTTON_CALLBACK	callBack,
-					void *			target );
-
-extern void tedFormatFinishShadingTool(	ShadingTool *		st,
-					TedFormatTool *		tft );
-
-extern void tedFormatFinishBorderTool(	BorderTool *		bt,
-					TedFormatTool *		tft );
-
 extern void tedFormatFillPageLayoutPage(
 			PageLayoutTool *			plt,
 			const PageLayoutPageResources *		plpr,
@@ -237,8 +199,21 @@ extern void tedFormatToolGetTabsResourceTable(
 					TabsPageResources *		tpr,
 					InspectorSubjectResources *	isr );
 
-extern void tedFormatFinishParagraphPage( ParagraphTool *		pt,
-					TedFormatTool *			tft,
-					const ParagraphPageResources *	ppr );
+extern void tedFormatFinishParaLayoutPage(
+				ParagraphLayoutTool *			plt,
+				TedFormatTool *				tft,
+				const ParagraphLayoutPageResources *	plpr );
+
+extern void tedFormatFinishParaOrnamentsPage(
+				ParagraphOrnamentsTool *		pot,
+				TedFormatTool *				tft,
+				const ParagraphOrnamentsPageResources *	popr );
+
+extern int tedFormatToolGetShading(	PropertyMask *		cpSetMask,
+					RowProperties *		rp,
+					int			col0,
+					int			col1,
+					DocumentProperties *	dp,
+					ShadingTool *		st );
 
 #   endif	/*  TED_FORMAT_TOOL_H */
