@@ -11,11 +11,11 @@
 #   include	<stddef.h>
 #   include	<limits.h>
 
-#   include	<appDebugon.h>
-
 #   include	"tedApp.h"
 #   include	"tedFormatTool.h"
 
+#   include	<appDebugon.h>
+
 /************************************************************************/
 /*									*/
 /*  Set all the controls on the Header/Footer tool, to reflect the	*/
@@ -67,39 +67,57 @@ static void tedFormatToolRefreshHeadFootPage(	HeaderFooterTool *	hft )
 void tedFormatToolRefreshHeaderFooterTool(
 				HeaderFooterTool *		hft,
 				int *				pEnabled,
+				int *				pPref,
 				InspectorSubject *		is,
 				const DocumentSelection *	ds,
+				const SelectionDescription *	sd,
 				const SelectionGeometry *	sg,
-				const DocumentProperties *	dpSet )
+				BufferDocument *		bd )
     {
+    const DocumentProperties *	dpSet= &(bd->bdProperties);
+
     PropertyMask		changed;
     PropertyMask		update;
 
-    BufferItem *		sectBi;
+    BufferItem *		selSectBi;
     const SectionProperties *	spSet;
 
-    int				inHeaderFooter;
+    int				whatHeaderFooter;
 
     PROPmaskCLEAR( &changed );
 
-    sectBi= ds->dsBegin.dpBi;
-    while( sectBi && sectBi->biLevel > DOClevSECT )
-	{ sectBi= sectBi->biParent;	}
+    selSectBi= ds->dsBegin.dpBi;
+    while( selSectBi && selSectBi->biLevel > DOClevSECT )
+	{ selSectBi= selSectBi->biParent;	}
+    if  ( ! selSectBi )
+	{ XDEB(selSectBi); return;	}
 
-    if  ( ! sectBi )
-	{ XDEB(sectBi); return;	}
-
-    inHeaderFooter= sectBi->biInExternalItem;
-    if  ( inHeaderFooter == DOCinBODY )
+    whatHeaderFooter= selSectBi->biInExternalItem;
+    if  ( whatHeaderFooter == DOCinBODY )
 	{
 	ExternalItem *	ei;
 	int		page= sg->sgBegin.pgTopPosition.lpPage;
 
-	inHeaderFooter= docWhatPageHeader( &ei, sectBi, page, dpSet );
-	}
+	whatHeaderFooter= docWhatPageHeader( &ei, selSectBi, page, dpSet );
 
-    hft->hftSectionNumber= sectBi->biNumberInParent;
-    spSet= &(sectBi->biSectProperties);
+	hft->hftSectionNumber= selSectBi->biNumberInParent;
+	spSet= &(selSectBi->biSectProperties);
+	}
+    else{
+	const SelectionScope *	ss= &(selSectBi->biSectSelectionScope);
+	const BufferItem *	docBi= &(bd->bdItem);
+	const BufferItem *	bodySectBi;
+
+	if  ( ! sd->sdInHeaderFooter )
+	    { *pEnabled= 0; return;	}
+
+	bodySectBi= docBi->biChildren[ss->ssSectNrExternalTo];
+
+	hft->hftSectionNumber= bodySectBi->biNumberInParent;
+	spSet= &(bodySectBi->biSectProperties);
+
+	*pPref= 7;
+	}
 
     /**********/
 
@@ -125,20 +143,35 @@ void tedFormatToolRefreshHeaderFooterTool(
 
     /**********/
 
-    hft->hftPagesOptionsEnabled[PAGES_FIRST_PAGE]=
+    if  ( sd->sdInHeaderFooter )
+	{
+	int		i;
+
+	for ( i= 0; i < PAGES__COUNT; i++ )
+	    { hft->hftPagesOptionsEnabled[i]= 0; }
+
+	appGuiEnableWidget( hft->hftSectionFrame, 0 );
+	appGuiEnableWidget( hft->hftDocumentFrame, 0 );
+	}
+    else{
+	hft->hftPagesOptionsEnabled[PAGES_FIRST_PAGE]=
 					    spSet->spHasTitlePage;
-    hft->hftPagesOptionsEnabled[PAGES_SUBSEQUENT_PAGES]=
+	hft->hftPagesOptionsEnabled[PAGES_SUBSEQUENT_PAGES]=
 					    spSet->spHasTitlePage	&&
 					    ! dpSet->dpHasFacingPages;
-    hft->hftPagesOptionsEnabled[PAGES_ALL_PAGES]=
+	hft->hftPagesOptionsEnabled[PAGES_ALL_PAGES]=
 					    ! spSet->spHasTitlePage	&&
 					    ! dpSet->dpHasFacingPages;
-    hft->hftPagesOptionsEnabled[PAGES_ODD_PAGES]=
+	hft->hftPagesOptionsEnabled[PAGES_ODD_PAGES]=
 					    dpSet->dpHasFacingPages;
-    hft->hftPagesOptionsEnabled[PAGES_EVEN_PAGES]=
+	hft->hftPagesOptionsEnabled[PAGES_EVEN_PAGES]=
 					    dpSet->dpHasFacingPages;
 
-    switch( inHeaderFooter )
+	appGuiEnableWidget( hft->hftSectionFrame, 1 );
+	appGuiEnableWidget( hft->hftDocumentFrame, 1 );
+	}
+
+    switch( whatHeaderFooter )
 	{
 	case DOCinSECT_HEADER:
 	case DOCinSECT_FOOTER:
@@ -146,6 +179,7 @@ void tedFormatToolRefreshHeaderFooterTool(
 		{ hft->hftPagesChosen= PAGES_SUBSEQUENT_PAGES;	}
 	    else{ hft->hftPagesChosen= PAGES_ALL_PAGES;		}
 
+	    hft->hftPagesOptionsEnabled[hft->hftPagesChosen]= 1;
 	    appSetOptionmenu( &(hft->hftPagesOptionmenu), hft->hftPagesChosen );
 
 	    break;
@@ -153,18 +187,21 @@ void tedFormatToolRefreshHeaderFooterTool(
 	case DOCinFIRST_HEADER:
 	case DOCinFIRST_FOOTER:
 	    hft->hftPagesChosen= PAGES_FIRST_PAGE;
+	    hft->hftPagesOptionsEnabled[hft->hftPagesChosen]= 1;
 	    appSetOptionmenu( &(hft->hftPagesOptionmenu), hft->hftPagesChosen );
 	    break;
 
 	case DOCinLEFT_HEADER:
 	case DOCinLEFT_FOOTER:
 	    hft->hftPagesChosen= PAGES_EVEN_PAGES;
+	    hft->hftPagesOptionsEnabled[hft->hftPagesChosen]= 1;
 	    appSetOptionmenu( &(hft->hftPagesOptionmenu), hft->hftPagesChosen );
 	    break;
 
 	case DOCinRIGHT_HEADER:
 	case DOCinRIGHT_FOOTER:
 	    hft->hftPagesChosen= PAGES_ODD_PAGES;
+	    hft->hftPagesOptionsEnabled[hft->hftPagesChosen]= 1;
 	    appSetOptionmenu( &(hft->hftPagesOptionmenu), hft->hftPagesChosen );
 	    break;
 
@@ -183,7 +220,7 @@ void tedFormatToolRefreshHeaderFooterTool(
 	    *pEnabled= 0; return;
 
 	default:
-	    LDEB(inHeaderFooter);
+	    LDEB(whatHeaderFooter);
 	    *pEnabled= 0; return;
 	}
 
@@ -201,9 +238,7 @@ void tedFormatToolRefreshHeaderFooterTool(
 /*									*/
 /************************************************************************/
 
-static void tedHeadFootChangeDocPushed(		APP_WIDGET	w,
-						void *		voidhft,
-						void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedHeadFootChangeDocPushed, w, voidhft )
     {
     HeaderFooterTool *		hft= (HeaderFooterTool *)voidhft;
     DocumentProperties *	dp= &(hft->hftDocPropertiesChosen);
@@ -213,14 +248,12 @@ static void tedHeadFootChangeDocPushed(		APP_WIDGET	w,
     PROPmaskCLEAR( &update );
     PROPmaskADD( &update, DPpropFACING_PAGES );
 
-    tedSetDocumentProperties( hft->hftApplication, dp, &update );
+    tedAppSetDocumentProperties( hft->hftApplication, dp, &update );
 
     return;
     }
 
-static void tedHeadFootChangeSectPushed(	APP_WIDGET	w,
-						void *		voidhft,
-						void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedHeadFootChangeSectPushed, w, voidhft )
     {
     HeaderFooterTool *		hft= (HeaderFooterTool *)voidhft;
     SectionProperties *		sp= &(hft->hftSectPropertiesChosen);
@@ -236,9 +269,7 @@ static void tedHeadFootChangeSectPushed(	APP_WIDGET	w,
     return;
     }
 
-static void tedHeadFootRevertDocPushed(		APP_WIDGET	w,
-						void *		voidhft,
-						void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedHeadFootRevertDocPushed, w, voidhft )
     {
     HeaderFooterTool *		hft= (HeaderFooterTool *)voidhft;
     DocumentProperties *	dp= &(hft->hftDocPropertiesChosen);
@@ -260,9 +291,7 @@ static void tedHeadFootRevertDocPushed(		APP_WIDGET	w,
     return;
     }
 
-static void tedHeadFootRevertSectPushed(	APP_WIDGET	w,
-						void *		voidhft,
-						void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedHeadFootRevertSectPushed, w, voidhft )
     {
     HeaderFooterTool *		hft= (HeaderFooterTool *)voidhft;
     SectionProperties *		sp= &(hft->hftSectPropertiesChosen);
@@ -291,9 +320,7 @@ static void tedHeadFootRevertSectPushed(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedHeaderFooterToggleTitlepg(	APP_WIDGET	w,
-						void *		voidhft,
-						void *		voidtbcs )
+static APP_TOGGLE_CALLBACK_H( tedHeaderFooterToggleTitlepg, w, voidhft, voidtbcs )
     {
     HeaderFooterTool *		hft= (HeaderFooterTool *)voidhft;
     SectionProperties *		sp= &(hft->hftSectPropertiesChosen);
@@ -306,9 +333,7 @@ static void tedHeaderFooterToggleTitlepg(	APP_WIDGET	w,
     return;
     }
 
-static void tedHeaderFooterToggleFacingp(	APP_WIDGET	w,
-						void *		voidhft,
-						void *		voidtbcs )
+static APP_TOGGLE_CALLBACK_H( tedHeaderFooterToggleFacingp, w, voidhft, voidtbcs )
     {
     HeaderFooterTool *		hft= (HeaderFooterTool *)voidhft;
     DocumentProperties *	dp= &(hft->hftDocPropertiesChosen);
@@ -327,23 +352,17 @@ static void tedHeaderFooterToggleFacingp(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedHeadFootPagesChosen(	APP_WIDGET	w,
-					void *		voidhft,
-					void *		voidtbcs )
+static APP_OITEM_CALLBACK_H( tedHeadFootPagesChosen, w, voidhft )
     {
     HeaderFooterTool *		hft= (HeaderFooterTool *)voidhft;
 
     int				pages= -1;
 
-    for ( pages= 0; pages < PAGES__COUNT; pages++ )
-	{
-	if  ( hft->hftPagesItems[pages] == w )
-	    { break;	}
-	}
+    pages= appGuiGetOptionmenuItemIndex( &(hft->hftPagesOptionmenu), w );
+    if  ( pages < 0 || pages >= PAGES__COUNT )
+	{ LLDEB(pages,PAGES__COUNT); return;		}
 
-    if  ( pages >= 0 && pages < PAGES__COUNT )
-	{ hft->hftPagesChosen= pages;		}
-    else{ LLDEB(pages,PAGES__COUNT);		}
+    hft->hftPagesChosen= pages;
 
     return;
     }
@@ -355,9 +374,7 @@ static void tedHeadFootPagesChosen(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedHeadFootDeleteHeaderPushed(	APP_WIDGET	w,
-						void *		voidhft,
-						void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedHeadFootDeleteHeaderPushed, w, voidhft )
     {
     HeaderFooterTool *		hft= (HeaderFooterTool *)voidhft;
 
@@ -370,9 +387,7 @@ static void tedHeadFootDeleteHeaderPushed(	APP_WIDGET	w,
     return;
     }
 
-static void tedHeadFootEditHeaderPushed(	APP_WIDGET	button,
-						void *		voidhft,
-						void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedHeadFootEditHeaderPushed, w, voidhft )
     {
     HeaderFooterTool *		hft= (HeaderFooterTool *)voidhft;
     APP_WIDGET			relative;
@@ -383,14 +398,13 @@ static void tedHeadFootEditHeaderPushed(	APP_WIDGET	button,
 
     relative= hft->hftInspector->aiTopWidget;
 
-    tedAppEditHeaderFooter( hft->hftApplication, relative, button,
+    tedAppEditHeaderFooter( hft->hftApplication, relative,
+				    hft->hftEditHeaderButton,
 				    DOC_HeaderScopes[hft->hftPagesChosen] );
     return;
     }
 
-static void tedHeadFootDeleteFooterPushed(	APP_WIDGET	w,
-						void *		voidhft,
-						void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedHeadFootDeleteFooterPushed, w, voidhft )
     {
     HeaderFooterTool *		hft= (HeaderFooterTool *)voidhft;
 
@@ -403,9 +417,7 @@ static void tedHeadFootDeleteFooterPushed(	APP_WIDGET	w,
     return;
     }
 
-static void tedHeadFootEditFooterPushed(	APP_WIDGET	button,
-						void *		voidhft,
-						void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedHeadFootEditFooterPushed, w, voidhft )
     {
     HeaderFooterTool *		hft= (HeaderFooterTool *)voidhft;
     APP_WIDGET			relative;
@@ -416,7 +428,8 @@ static void tedHeadFootEditFooterPushed(	APP_WIDGET	button,
 
     relative= hft->hftInspector->aiTopWidget;
 
-    tedAppEditHeaderFooter( hft->hftApplication, relative, button,
+    tedAppEditHeaderFooter( hft->hftApplication, relative,
+				    hft->hftEditFooterButton,
 				    DOC_FooterScopes[hft->hftPagesChosen] );
     return;
     }
@@ -532,6 +545,22 @@ void tedFormatFinishHeaderFooterPage( HeaderFooterTool *		hft,
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Clean the header footer tool.					*/
+/*									*/
+/************************************************************************/
+
+void tedFormatCleanHeaderFooterTool(	HeaderFooterTool *		hft )
+    {
+    docCleanDocumentProperties( &(hft->hftDocPropertiesChosen) );
+    docCleanDocumentProperties( &(hft->hftDocPropertiesSet) );
+    docCleanSectionProperties( &(hft->hftSectPropertiesChosen) );
+    docCleanSectionProperties( &(hft->hftSectPropertiesSet) );
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Get Header/Footer tool resources.					*/
