@@ -11,7 +11,7 @@
 #   include	<stdio.h>
 #   include	<ctype.h>
 
-#   include	"docLayout.h"
+#   include	"tedLayout.h"
 #   include	"tedApp.h"
 #   include	"tedRuler.h"
 
@@ -167,7 +167,7 @@ static void tedLogEvent(	Widget		w,
 /*									*/
 /************************************************************************/
 
-#   define	TED_DRAG_INTERVAL	(150L)
+#   define	CLICK2_TO_CHANGE_ROOT	0
 
 typedef struct DraggingContext
     {
@@ -178,7 +178,274 @@ typedef struct DraggingContext
     EditDocument *		dcEd;
     } DraggingContext;
 
-static APP_EVENT_HANDLER( tedInputDragMouseMove, w, vdc, event )
+static int tedSelectMousePosition(
+				unsigned int *			pKeyState,
+				APP_WIDGET			w,
+				EditDocument *			ed,
+				DraggingContext *		dc,
+				APP_EVENT *			downEvent )
+    {
+    AppDrawingData *		add= &(ed->edDrawingData);
+    TedDocument *		td= (TedDocument *)ed->edPrivateData;
+    ScreenFontList *		sfl= &(td->tdScreenFontList);
+    BufferDocument *		bd= td->tdDocument;
+
+    int				ox= ed->edVisibleRect.drX0;
+    int				oy= ed->edVisibleRect.drY0;
+
+    int				mouseX;
+    int				mouseY;
+    int				docX;
+    int				docY;
+
+    int				page;
+    int				sectNr;
+
+    int				otherRoot= 0;
+
+    int				scrolledX= 0;
+    int				scrolledY= 0;
+
+    int				button;
+    int				upDown;
+    int				seq;
+    unsigned int		keyState= 0;
+
+    DocumentSelection		dsOld;
+    SelectionGeometry		sg;
+    SelectionDescription	sd;
+
+    PositionGeometry		pgClick;
+
+    ExternalItem *		eiSet= (ExternalItem *)0;
+    BufferItem *		bodySectBiSet;
+    BufferItem *		rootBiSet;
+
+    if  ( tedGetSelection( &dsOld, &sg, &sd, td ) )
+	{ LDEB(1); return 1;	}
+
+    if  ( appGetCoordinatesFromMouseButtonEvent( &mouseX, &mouseY,
+					    &button, &upDown, &seq, &keyState,
+					    w, downEvent ) )
+	{ LDEB(1); return 1;	}
+
+    if  ( mouseX < -ox )
+	{ mouseX=  -ox;	}
+    if  ( mouseY < -oy )
+	{ mouseY=  -oy;	}
+
+    dc->dcMouseX= mouseX;
+    dc->dcMouseY= mouseY;
+
+    docX= mouseX+ ox;
+    docY= mouseY+ oy;
+
+    dc->dcRootItem= &(bd->bdItem);
+
+    if  ( tedFindRootForPosition( &eiSet, &rootBiSet, &bodySectBiSet,
+					    &sectNr, &page, ed, docX, docY ) )
+	{ LLDEB(docX,docY); return 1;	}
+
+    dc->dcRootItem= rootBiSet;
+
+    otherRoot= docSelectionDifferentRoot( &dsOld, rootBiSet );
+
+    if  ( otherRoot )
+	{
+	DocumentPosition		dp;
+	int				inHeadFoot;
+	DocumentRectangle		drChanged;
+
+#	if  CLICK2_TO_CHANGE_ROOT
+	if  ( seq < 2 )
+	    { return 1;	}
+#	endif
+
+	docInitDocumentPosition( &dp );
+
+	inHeadFoot= dc->dcRootItem->biInExternalItem;
+
+	switch( inHeadFoot )
+	    {
+	    case DOCinBODY:
+	    case DOCinFOOTNOTE:
+	    case DOCinENDNOTE:
+		break;
+
+	    case DOCinSECT_HEADER:
+	    case DOCinFIRST_HEADER:
+	    case DOCinLEFT_HEADER:
+	    case DOCinRIGHT_HEADER:
+	    case DOCinSECT_FOOTER:
+	    case DOCinFIRST_FOOTER:
+	    case DOCinLEFT_FOOTER:
+	    case DOCinRIGHT_FOOTER:
+
+		if  ( ! eiSet )
+		    { XDEB(eiSet); return 1;	}
+		if  ( eiSet->eiPageFormattedFor != page )
+		    {
+		    if  ( docLayoutExternalItem( eiSet, &drChanged,
+				page, eiSet->eiY0UsedTwips,
+				bd, bodySectBiSet, add, sfl,
+				tedInitLayoutExternalItem, tedCloseObject ) )
+			{ LDEB(page); return 1; }
+
+		    eiSet->eiPageSelectedUpon= page;
+		    }
+		break;
+
+	    case DOCinFTNSEP:
+	    case DOCinFTNSEPC:
+	    case DOCinFTNCN:
+	    case DOCinAFTNSEP:
+	    case DOCinAFTNSEPC:
+	    case DOCinAFTNCN:
+
+		if  ( ! eiSet )
+		    { XDEB(eiSet); return 1;	}
+		if  ( eiSet->eiPageFormattedFor != page )
+		    {
+		    LLDEB(eiSet->eiPageFormattedFor,page);
+
+		    if  ( docLayoutExternalItem( eiSet, &drChanged,
+				page, eiSet->eiY0UsedTwips,
+				bd, bodySectBiSet, add, sfl,
+				tedInitLayoutExternalItem, tedCloseObject ) )
+			{ LDEB(page); return 1; }
+		    }
+		break;
+
+	    default:
+		LDEB(dc->dcRootItem->biInExternalItem);
+	    }
+
+	if  ( eiSet )
+	    { eiSet->eiPageSelectedUpon= page;	}
+
+#	if CLICK2_TO_CHANGE_ROOT
+	{
+	PositionGeometry	pg;
+
+	if  ( tedFindPosition( &dp, &pg, dc->dcRootItem, add, docX, docY ) )
+	    { LLDEB(docX,docY); return 1;	}
+
+xxx
+	tedSetSelectedPosition( ed, &dp, &scrolledX, &scrolledY );
+
+	tedAdaptToolsToSelection( ed );
+
+	appExposeRectangle( add, 0, 0, 0, 0 );
+
+	dc->dcEd= ed;
+	dc->dcMouseX= mouseX;
+	dc->dcMouseY= mouseY;
+
+	*pKeyState= keyState;
+	return 0;
+	}
+#	endif
+
+	appExposeRectangle( add, 0, 0, 0, 0 );
+	}
+
+    if  ( seq > 1 )
+	{
+	int			wasObject= 0;
+
+	DocumentSelection	dsNew;
+
+	docInitDocumentSelection( &dsNew );
+
+	if  ( tedFindPosition( &(dc->dcAnchorPosition), &pgClick,
+				bd, dc->dcRootItem, add, sfl, docX, docY ) )
+	    { LLDEB(docX,docY); return 1; }
+
+	docSetIBarSelection( &dsNew, &(dc->dcAnchorPosition) );
+
+	if  ( seq > 2 )
+	    {
+	    int		partLineBegin;
+	    int		partLineEnd;
+
+	    docLineSelection( &dsNew, &partLineBegin, &partLineEnd,
+				dc->dcAnchorPosition.dpBi, pgClick.pgLine );
+	    }
+	else{
+	    docWordSelection( &dsNew, &wasObject, &(dc->dcAnchorPosition) );
+	    }
+
+	tedSetSelection( ed, &dsNew, pgClick.pgAtLineHead,
+						    &scrolledX, &scrolledY );
+
+	tedAdaptToolsToSelection( ed );
+
+	if  ( wasObject )
+	    { tedMoveObjectWindows( ed );	}
+
+	dc->dcEd= ed;
+	dc->dcMouseX= mouseX;
+	dc->dcMouseY= mouseY;
+
+	*pKeyState= keyState;
+	return 0;
+	}
+
+    if  ( tedFindPosition( &(dc->dcAnchorPosition), &pgClick,
+				bd, dc->dcRootItem, add, sfl, docX, docY ) )
+	{ LLDEB(docX,docY); return 1;	}
+
+    if  ( ! otherRoot && ( keyState & KEY_SHIFT_MASK ) )
+	{
+	dc->dcAnchorPosition= dsOld.dsAnchor;
+
+	if  ( tedExtendSelectionToXY( ed, dc->dcRootItem,
+			&(dc->dcAnchorPosition), dc->dcMouseX, dc->dcMouseY ) )
+	    { LDEB(1); return 1;	}
+
+	tedAdaptToolsToSelection( ed );
+
+	dc->dcEd= ed;
+	dc->dcMouseX= mouseX;
+	dc->dcMouseY= mouseY;
+
+	*pKeyState= keyState;
+	return 0;
+	}
+
+    if  ( docParaHeadFieldKind( dc->dcAnchorPosition.dpBi, bd ) >= 0 )
+	{
+	int		FieldNr= -1;
+	int		partBegin= -1;
+	int		partEnd= -1;
+	int		stroffBegin= -1;
+	int		stroffEnd= -1;
+
+	if  ( ! docDelimitParaHeadField( &FieldNr, &partBegin, &partEnd,
+					&stroffBegin, &stroffEnd,
+					dc->dcAnchorPosition.dpBi, bd ) )
+	    {
+	    if  ( dc->dcAnchorPosition.dpStroff < stroffEnd )
+		{ dc->dcAnchorPosition.dpStroff=  stroffEnd;	}
+	    }
+	}
+
+    tedSetSelectedPosition( ed, &(dc->dcAnchorPosition), pgClick.pgAtLineHead,
+						    &scrolledX, &scrolledY );
+
+    tedAdaptToolsToSelection( ed );
+
+    dc->dcEd= ed;
+    dc->dcMouseX= mouseX;
+    dc->dcMouseY= mouseY;
+
+    *pKeyState= keyState;
+    return 0;
+    }
+
+#   define	TED_DRAG_INTERVAL	(150L)
+
+static APP_EVENT_HANDLER_H( tedInputDragMouseMove, w, vdc, event )
     {
     DraggingContext *	dc= (DraggingContext *)vdc;
     EditDocument *	ed= dc->dcEd;
@@ -207,10 +474,16 @@ static APP_EVENT_HANDLER( tedInputDragMouseMove, w, vdc, event )
     return;
     }
 
-static APP_EVENT_HANDLER( tedInputDragMouseUp, w, vdc, event )
+static APP_EVENT_HANDLER_H( tedInputDragMouseUp, w, vdc, event )
     {
     DraggingContext *		dc= (DraggingContext *)vdc;
     EditDocument *		ed= dc->dcEd;
+    TedDocument *		td= (TedDocument *)ed->edPrivateData;
+    BufferDocument *		bd= td->tdDocument;
+
+    docDescribeSelection( &(td->tdSelectionDescription),
+				&(td->tdDocumentSelection),
+				bd, ed->edDocumentId, ed->edIsReadonly );
 
     tedAdaptToolsToSelection( ed );
 
@@ -290,249 +563,41 @@ static APP_TIMER_HANDLER( tedTick, voiddc )
 #   endif
     }
 
-#   define	CLICK2_TO_CHANGE_ROOT	0
-
 static void tedButton1Pressed(	APP_WIDGET			w,
 				EditDocument *			ed,
 				APP_EVENT *			downEvent )
     {
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
-    AppDrawingData *		add= &(ed->edDrawingData);
     BufferDocument *		bd= td->tdDocument;
 
     DraggingContext		dc;
+    unsigned int		keyState;
 
-    int				ox= ed->edVisibleRect.drX0;
-    int				oy= ed->edVisibleRect.drY0;
+    if  ( tedSelectMousePosition( &keyState, w, ed, &dc, downEvent ) )
+	{ return;	}
 
-    int				mouseX;
-    int				mouseY;
-    int				docX;
-    int				docY;
-
-    int				page;
-    int				sectNr;
-
-    int				otherRoot= 0;
-
-    int				scrolledX= 0;
-    int				scrolledY= 0;
-
-    int				button;
-    int				upDown;
-    int				seq;
-    unsigned int		keyState= 0;
-
-    DocumentSelection		dsOld;
-    SelectionGeometry		sg;
-
-    PositionGeometry		pgClick;
-
-    ExternalItem *		eiSet= (ExternalItem *)0;
-    BufferItem *		bodySectBiSet;
-    BufferItem *		rootBiSet;
-
-    if  ( tedGetSelection( &dsOld, &sg, td ) )
-	{ LDEB(1); return;	}
-
-    if  ( appGetCoordinatesFromMouseButtonEvent( &mouseX, &mouseY,
-					    &button, &upDown, &seq, &keyState,
-					    w, downEvent ) )
-	{ LDEB(1); return;	}
-
-    if  ( mouseX < -ox )
-	{ mouseX=  -ox;	}
-    if  ( mouseY < -oy )
-	{ mouseY=  -oy;	}
-
-    dc.dcMouseX= mouseX;
-    dc.dcMouseY= mouseY;
-
-    docX= mouseX+ ox;
-    docY= mouseY+ oy;
-
-    dc.dcRootItem= &(bd->bdItem);
-
-    if  ( tedFindRootForPosition( &eiSet, &rootBiSet, &bodySectBiSet,
-					    &sectNr, &page, ed, docX, docY ) )
-	{ LLDEB(docX,docY); return;	}
-
-    dc.dcRootItem= rootBiSet;
-
-    otherRoot= docSelectionDifferentRoot( &dsOld, rootBiSet );
-
-    if  ( otherRoot )
+    if  ( ed->edFileReadOnly || ( keyState & KEY_CONTROL_MASK ) )
 	{
-	DocumentPosition		dp;
-	int				inHeadFoot;
-	DocumentRectangle		drChanged;
+	DocumentSelection	dsHyperlink;
+	int			startPart;
+	int			endPart;
 
-#	if  CLICK2_TO_CHANGE_ROOT
-	if  ( seq < 2 )
-	    { return;	}
-#	endif
-
-	docInitDocumentPosition( &dp );
-
-	inHeadFoot= dc.dcRootItem->biInExternalItem;
-
-	switch( inHeadFoot )
-	    {
-	    case DOCinBODY:
-	    case DOCinFOOTNOTE:
-	    case DOCinENDNOTE:
-		break;
-
-	    case DOCinSECT_HEADER:
-	    case DOCinFIRST_HEADER:
-	    case DOCinLEFT_HEADER:
-	    case DOCinRIGHT_HEADER:
-	    case DOCinSECT_FOOTER:
-	    case DOCinFIRST_FOOTER:
-	    case DOCinLEFT_FOOTER:
-	    case DOCinRIGHT_FOOTER:
-
-		if  ( ! eiSet )
-		    { XDEB(eiSet); return;	}
-		if  ( eiSet->eiPageFormattedFor != page )
-		    {
-		    if  ( docLayoutExternalItem( eiSet, &drChanged,
-				page, eiSet->eiY0UsedTwips,
-				bd, bodySectBiSet, add,
-				tedLayoutExternalItem, tedCloseObject ) )
-			{ LDEB(page); return; }
-
-		    eiSet->eiPageSelectedUpon= page;
-		    }
-		break;
-
-	    case DOCinFTNSEP:
-	    case DOCinFTNSEPC:
-	    case DOCinFTNCN:
-	    case DOCinAFTNSEP:
-	    case DOCinAFTNSEPC:
-	    case DOCinAFTNCN:
-
-		if  ( ! eiSet )
-		    { XDEB(eiSet); return;	}
-		if  ( eiSet->eiPageFormattedFor != page )
-		    {
-		    LLDEB(eiSet->eiPageFormattedFor,page);
-
-		    if  ( docLayoutExternalItem( eiSet, &drChanged,
-				page, eiSet->eiY0UsedTwips,
-				bd, bodySectBiSet, add,
-				tedLayoutExternalItem, tedCloseObject ) )
-			{ LDEB(page); return; }
-		    }
-		break;
-
-	    default:
-		LDEB(dc.dcRootItem->biInExternalItem);
-	    }
-
-	if  ( eiSet )
-	    { eiSet->eiPageSelectedUpon= page;	}
-
-#	if CLICK2_TO_CHANGE_ROOT
-	{
-	PositionGeometry	pg;
-
-	if  ( tedFindPosition( &dp, &pg, dc.dcRootItem, add, docX, docY ) )
-	    { LLDEB(docX,docY); return;	}
-
-	tedSetSelectedPosition( ed, &dp, &scrolledX, &scrolledY );
-
-	tedAdaptToolsToSelection( ed );
-
-	appExposeRectangle( add, 0, 0, 0, 0 );
-
-	return;
-	}
-#	endif
-
-	appExposeRectangle( add, 0, 0, 0, 0 );
-	}
-
-    if  ( seq > 1 )
-	{
-	int			wasObject= 0;
-
-	DocumentSelection	dsNew;
-
-	docInitDocumentSelection( &dsNew );
-
-	if  ( tedFindPosition( &(dc.dcAnchorPosition), &pgClick,
-					    dc.dcRootItem, add, docX, docY ) )
-	    { LLDEB(docX,docY); return; }
-
-	docSetIBarSelection( &dsNew, &(dc.dcAnchorPosition) );
-
-	if  ( seq > 2 )
-	    {
-	    docLineSelection( &dsNew,
-				dc.dcAnchorPosition.dpBi, pgClick.pgLine );
-	    }
-	else{
-	    docWordSelection( &dsNew, &wasObject, &(dc.dcAnchorPosition) );
-	    }
-
-	tedSetSelection( ed, &dsNew, &scrolledX, &scrolledY );
-
-	tedAdaptToolsToSelection( ed );
-
-	if  ( wasObject )
-	    { tedMoveObjectWindows( ed );	}
-
-	return;
-	}
-
-    if  ( tedFindPosition( &(dc.dcAnchorPosition), &pgClick,
-					    dc.dcRootItem, add, docX, docY ) )
-	{ LLDEB(docX,docY); return;	}
-
-    if  ( ! otherRoot && ( keyState & KEY_SHIFT_MASK ) )
-	{
-	dc.dcAnchorPosition= dsOld.dsAnchor;
-
-	if  ( tedExtendSelectionToXY( ed, dc.dcRootItem,
-			&(dc.dcAnchorPosition), dc.dcMouseX, dc.dcMouseY ) )
-	    { LDEB(1); return;	}
-
-	tedAdaptToolsToSelection( ed );
-
-	return;
-	}
-
-    tedSetSelectedPosition( ed, &(dc.dcAnchorPosition),
-						    &scrolledX, &scrolledY );
-
-    tedAdaptToolsToSelection( ed );
-
-    if  ( ed->edFileReadOnly )
-	{
-	int		startPart;
-	int		endPart;
-	const char *	fileName;
-	int		fileSize;
-	const char *	markName;
-	int		markSize;
+	const char *		fileName;
+	int			fileSize;
+	const char *		markName;
+	int			markSize;
 
 	if  ( ! docGetHyperlinkForPosition( bd, &(dc.dcAnchorPosition),
-				&startPart, &endPart,
+				&dsHyperlink, &startPart, &endPart,
 				&fileName, &fileSize, &markName, &markSize ) )
 	    {
-	    tedFollowLink( (APP_WIDGET)0, (APP_WIDGET)0, ed,
+	    tedDocFollowLink( (APP_WIDGET)0, ed,
 				    fileName, fileSize, markName, markSize );
 
 	    return;
 	    }
 	}
 
-    dc.dcEd= ed;
-    dc.dcMouseX= mouseX;
-    dc.dcMouseY= mouseY;
-
     appRunDragLoop( w, ed->edApplication, downEvent,
 				tedInputDragMouseUp,
 				tedInputDragMouseMove,
@@ -551,13 +616,15 @@ static void tedButton1Pressed(	APP_WIDGET			w,
 /*									*/
 /************************************************************************/
 
-static void tedInputSetSelectedPosition( EditDocument *			ed,
-					const DocumentPosition *	dp )
+static void tedInputSetSelectedPosition(
+				EditDocument *			ed,
+				const DocumentPosition *	dp,
+				int				lastLine )
     {
     int			scrolledX= 0;
     int			scrolledY= 0;
 
-    tedSetSelectedPosition( ed, dp, &scrolledX, &scrolledY );
+    tedSetSelectedPosition( ed, dp, lastLine, &scrolledX, &scrolledY );
 
     tedAdaptToolsToSelection( ed );
 
@@ -571,8 +638,9 @@ static void tedInputExtendSelection(	EditDocument *			ed,
 
     DocumentSelection		ds;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
-    if  ( tedGetSelection( &ds, &sg, td ) )
+    if  ( tedGetSelection( &ds, &sg, &sd, td ) )
 	{ LDEB(1); return;	}
 
     tedExtendSelectionToPosition( ed, &(ds.dsAnchor), dpSet );
@@ -585,7 +653,8 @@ static void tedInputExtendSelection(	EditDocument *			ed,
 static void tedInputChangeSelection(
 				EditDocument *			ed,
 				unsigned int			keyState,
-				const DocumentPosition *	dp )
+				const DocumentPosition *	dp,
+				int				lastLine )
     {
     if  ( keyState & KEY_SHIFT_MASK )
 	{
@@ -593,20 +662,43 @@ static void tedInputChangeSelection(
 	return;
 	}
 
-    tedInputSetSelectedPosition( ed, dp );
+    tedInputSetSelectedPosition( ed, dp, lastLine );
+
+    return;
+    }
+
+static void tedInputAvoidHeadField(	DocumentPosition *	dpNew,
+					const BufferDocument *	bd )
+    {
+    int		fieldNr= -1;
+    int		partBegin= -1;
+    int		partEnd= -1;
+    int		stroffBegin= -1;
+    int		stroffEnd= -1;
+
+    int	fieldKind= docParaHeadFieldKind( dpNew->dpBi, bd );
+    if  ( fieldKind < 0 )
+	{ return;	}
+
+    if  ( docDelimitParaHeadField( &fieldNr, &partBegin, &partEnd,
+				&stroffBegin, &stroffEnd, dpNew->dpBi, bd ) )
+	{ LDEB(1);	}
+    else{
+	if  ( dpNew->dpStroff < stroffEnd )
+	    { dpNew->dpStroff=  stroffEnd;	}
+	}
 
     return;
     }
 
 static int tedMoveRightOnKey(	DocumentPosition *	dpNew,
+				const BufferDocument *	bd,
 				int			keyState,
 				const AppDrawingData *	add )
     {
     if  ( keyState & KEY_CONTROL_MASK )
 	{
-	const int	lastOne= 1;
-
-	if  ( docNextWord( dpNew, lastOne ) )
+	if  ( docNextWord( dpNew ) )
 	    { return -1;	}
 	}
     else{
@@ -614,25 +706,51 @@ static int tedMoveRightOnKey(	DocumentPosition *	dpNew,
 	    { return -1;	}
 	}
 
+    if  ( ! ( keyState & KEY_SHIFT_MASK ) )
+	{ tedInputAvoidHeadField( dpNew, bd );	}
+
     return 0;
     }
 
 static int tedMoveLeftOnKey(	DocumentPosition *	dpNew,
+				const BufferDocument *	bd,
 				int			keyState,
 				const AppDrawingData *	add )
     {
-    const int	lastOne= 1;
-
     if  ( keyState & KEY_CONTROL_MASK )
 	{
-	if  ( docPrevWord( dpNew, lastOne ) )
+	if  ( docPrevWord( dpNew ) )
 	    { return -1;	}
 	}
     else{
-	if  ( docPrevPosition( dpNew, lastOne ) )
+	if  ( docPrevPosition( dpNew ) )
 	    { return -1;	}
 	}
 
+    if  ( ! ( keyState & KEY_SHIFT_MASK )	&&
+	  dpNew->dpBi->biParaListOverride > 0	)
+	{
+	int		fieldNr= -1;
+	int		partBegin= -1;
+	int		partEnd= -1;
+	int		stroffBegin= -1;
+	int		stroffEnd= -1;
+
+	if  ( docDelimitParaHeadField( &fieldNr, &partBegin, &partEnd,
+					&stroffBegin, &stroffEnd,
+					dpNew->dpBi, bd ) )
+	    { LDEB(1);	}
+	else{
+	    if  ( dpNew->dpStroff < stroffEnd )
+		{
+		dpNew->dpStroff= 0;
+
+		if  ( docPrevPosition( dpNew ) )
+		    { return -1;	}
+		}
+	    }
+	}
+
     return 0;
     }
 
@@ -645,35 +763,40 @@ static void tedProcessKeyEvent(	EditDocument *		ed,
 				unsigned int		state )
     {
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
+    BufferDocument *		bd= td->tdDocument;
+    const ScreenFontList *	sfl= &(td->tdScreenFontList);
     const AppDrawingData *	add= &(ed->edDrawingData);
 
+    int				res;
+
     DocumentSelection		ds;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
-    if  ( tedGetSelection( &ds, &sg, td ) )
+    if  ( tedGetSelection( &ds, &sg, &sd, td ) )
 	{ LDEB(1); return;	}
 
     if  ( gotString > 0 )
 	{
-	AppPhysicalFont *	apf;
+	DrawScreenFont *	apf;
 
 	if  ( ! td->tdCanReplaceSelection )
 	    { return;	}
 
-	if  ( td->tdCurrentPhysicalFont < 0				||
-	      td->tdCurrentPhysicalFont >=
-				add->addPhysicalFontList.apflCount	)
+	if  ( td->tdCurrentScreenFont < 0				||
+	      td->tdCurrentScreenFont >=
+				add->addScreenFontList.apflFontCount	)
 	    {
-	    LDEB(td->tdCurrentPhysicalFont);
-	    LDEB(add->addPhysicalFontList.apflCount);
+	    LDEB(td->tdCurrentScreenFont);
+	    LDEB(add->addScreenFontList.apflFontCount);
 	    return;
 	    }
 
-	apf= add->addPhysicalFontList.apflFonts+ td->tdCurrentPhysicalFont;
+	apf= add->addScreenFontList.apflFonts+ td->tdCurrentScreenFont;
 
 	if  ( appCharExistsInFont( apf->apfFontStruct, scratch[0] ) )
 	    {
-	    tedAppReplaceSelection( ed, scratch, gotString );
+	    tedDocReplaceSelection( ed, scratch, gotString );
 	    return;
 	    }
 	}
@@ -683,7 +806,8 @@ static void tedProcessKeyEvent(	EditDocument *		ed,
 
     switch( keySym )
 	{
-	DocumentPosition	dpNew;
+	DocumentPosition		dpNew;
+	const PositionGeometry *	pgRef;
 
 #	ifdef KEY_ISO_Left_Tab
 	case  KEY_ISO_Left_Tab:
@@ -708,7 +832,7 @@ static void tedProcessKeyEvent(	EditDocument *		ed,
 		  shiftTab:
 		    if  ( docFirstPosition( &dpNew,
 					ds.dsBegin.dpBi->biParent )	||
-			  docPrevPosition( &dpNew, /*last=*/ 0 )	)
+			  docPrevPosition( &dpNew )			)
 			{ return;	}
 		    }
 		else{
@@ -718,7 +842,11 @@ static void tedProcessKeyEvent(	EditDocument *		ed,
 			{ return;	}
 		    }
 
-		tedInputSetSelectedPosition( ed, &dpNew );
+		{
+		    const int	lastLine= 0;
+
+		    tedInputSetSelectedPosition( ed, &dpNew, lastLine );
+		}
 
 		return;
 		}
@@ -753,89 +881,144 @@ static void tedProcessKeyEvent(	EditDocument *		ed,
 
 	case KEY_KP_Delete:
 	case KEY_Delete:
+
+	    res= tedDeleteTableSliceSelection( ed );
+	    if  ( res < 0 )
+		{ LDEB(res); return;	}
+	    if  ( res == 0 )
+		{ return;		}
+
 	    if  ( ! td->tdCanReplaceSelection )
 		{ return;	}
 
-	    if  ( tedHasIBarSelection( td ) )
+	    if  ( sd.sdIsIBarSelection )
 		{
 		dpNew= ds.dsBegin;
 
-		if  ( tedMoveRightOnKey( &dpNew, state, add ) )
+		if  ( tedMoveRightOnKey( &dpNew, bd, state, add ) )
 		    { return;	}
 
-		if  ( ! docPositionsInsideCell( &(ds.dsBegin), &dpNew ) )
+		if  ( ! docPositionsInsideParagraph( &(ds.dsBegin), &dpNew ) )
 		    { return;	}
 
-		if  ( tedMoveRightOnKey( &(ds.dsEnd), state, add ) )
+		if  ( tedMoveRightOnKey( &(ds.dsEnd), bd, state, add ) )
 		    { return;	}
 
 		    {
 		    int		scrolledX= 0;
 		    int		scrolledY= 0;
 
-		    tedSetSelection( ed, &ds, &scrolledX, &scrolledY );
+		    tedSetSelection( ed, &ds, sg.sgBegin.pgAtLineHead,
+						    &scrolledX, &scrolledY );
 		    }
 		}
 
-	    tedAppReplaceSelection( ed, (const unsigned char *)0, 0 );
+	    tedDocReplaceSelection( ed, (const unsigned char *)0, 0 );
 
 	    return;
 
 	case KEY_BackSpace:
+
+	    res= tedDeleteTableSliceSelection( ed );
+	    if  ( res < 0 )
+		{ LDEB(res); return;	}
+	    if  ( res == 0 )
+		{ return;		}
+
 	    if  ( ! td->tdCanReplaceSelection )
 		{ return;	}
 
-	    if  ( tedHasIBarSelection( td ) )
+	    if  ( sd.sdIsIBarSelection )
 		{
 		dpNew= ds.dsBegin;
 
-		if  ( tedMoveLeftOnKey( &dpNew, state, add ) )
+		if  ( tedMoveLeftOnKey( &dpNew, bd, state, add ) )
 		    { return;	}
 
 		if  ( ! docPositionsInsideCell( &dpNew, &(ds.dsBegin) ) )
 		    {
 		    const int		keyState= 0;
+		    const int		lastLine= 1;
 
-		    tedInputChangeSelection( ed, keyState, &dpNew );
+		    tedInputChangeSelection( ed, keyState, &dpNew, lastLine );
 		    return;
 		    }
 
-		if  ( tedMoveLeftOnKey( &(ds.dsBegin), state, add ) )
+		if  ( tedMoveLeftOnKey( &(ds.dsBegin), bd, state, add ) )
 		    { return;	}
 
 		    {
 		    int		scrolledX= 0;
 		    int		scrolledY= 0;
 
-		    tedSetSelection( ed, &ds, &scrolledX, &scrolledY );
+		    tedSetSelection( ed, &ds, sg.sgBegin.pgAtLineHead,
+						    &scrolledX, &scrolledY );
 		    }
 		}
 
-	    tedAppReplaceSelection( ed, (const unsigned char *)0, 0 );
+	    tedDocReplaceSelection( ed, (const unsigned char *)0, 0 );
 	    return;
 
 	case KEY_KP_Home:
 	case KEY_Home:
-	    if  ( ( state & KEY_SHIFT_MASK ) && ds.dsDirection >= 0 )
-		{ dpNew= ds.dsEnd;	}
-	    else{ dpNew= ds.dsBegin;	}
+	    if  ( ( state & KEY_CONTROL_MASK ) )
+		{
+		if  ( docFirstPosition( &dpNew, &(bd->bdItem) ) )
+		    { return;	}
+		}
+	    else{
+		if  ( ( state & KEY_SHIFT_MASK ) && ds.dsDirection >= 0 )
+		    {
+		    dpNew= ds.dsEnd;
 
-	    if  ( docBeginOfLine( &dpNew ) )
-		{ return;	}
+		    if  ( docBeginOfLine( &dpNew, sg.sgEnd.pgAtLineHead ) )
+			{ return;	}
+		    }
+		else{
+		    dpNew= ds.dsBegin;
 
-	    tedInputChangeSelection( ed, state, &dpNew );
+		    if  ( docBeginOfLine( &dpNew, sg.sgBegin.pgAtLineHead ) )
+			{ return;	}
+		    }
+		}
+
+	    tedInputAvoidHeadField( &dpNew, bd );
+
+	    {
+	    const int	lastLine= 1;
+
+	    tedInputChangeSelection( ed, state, &dpNew, lastLine );
+	    }
 	    return;
 
 	case KEY_KP_End:
 	case KEY_End:
-	    if  ( ! ( state & KEY_SHIFT_MASK ) || ds.dsDirection >= 0 )
-		{ dpNew= ds.dsEnd;	}
-	    else{ dpNew= ds.dsBegin;	}
+	    if  ( ( state & KEY_CONTROL_MASK ) )
+		{
+		if  ( docLastPosition( &dpNew, &(bd->bdItem) ) )
+		    { return;	}
+		}
+	    else{
+		if  ( ! ( state & KEY_SHIFT_MASK ) || ds.dsDirection >= 0 )
+		    {
+		    dpNew= ds.dsEnd;
 
-	    if  ( docEndOfLine( &dpNew ) )
-		{ return;	}
+		    if  ( docEndOfLine( &dpNew, sg.sgEnd.pgAtLineHead ) )
+			{ return;	}
+		    }
+		else{
+		    dpNew= ds.dsBegin;
 
-	    tedInputChangeSelection( ed, state, &dpNew );
+		    if  ( docEndOfLine( &dpNew, sg.sgBegin.pgAtLineHead ) )
+			{ return;	}
+		    }
+		}
+
+	    {
+	    const int	lastLine= 0;
+
+	    tedInputChangeSelection( ed, state, &dpNew, lastLine );
+	    }
 	    return;
 
 	case KEY_KP_Left:
@@ -844,10 +1027,17 @@ static void tedProcessKeyEvent(	EditDocument *		ed,
 		{ dpNew= ds.dsEnd;	}
 	    else{ dpNew= ds.dsBegin;	}
 
-	    if  ( tedMoveLeftOnKey( &dpNew, state, add ) )
-		{ return;	}
+	    if  ( state != 0 || sd.sdIsIBarSelection )
+		{
+		if  ( tedMoveLeftOnKey( &dpNew, bd, state, add ) )
+		    { return;	}
+		}
 
-	    tedInputChangeSelection( ed, state, &dpNew );
+	    {
+	    const int	lastLine= 1;
+
+	    tedInputChangeSelection( ed, state, &dpNew, lastLine );
+	    }
 	    return;
 
 	case KEY_KP_Right:
@@ -856,58 +1046,65 @@ static void tedProcessKeyEvent(	EditDocument *		ed,
 		{ dpNew= ds.dsEnd;	}
 	    else{ dpNew= ds.dsBegin;	}
 
-	    if  ( tedMoveRightOnKey( &dpNew, state, add ) )
-		{ return;	}
+	    if  ( state != 0 || sd.sdIsIBarSelection )
+		{
+		if  ( tedMoveRightOnKey( &dpNew, bd, state, add ) )
+		    { return;	}
+		}
 
-	    tedInputChangeSelection( ed, state, &dpNew );
+	    {
+	    const int	lastLine= 0;
+
+	    tedInputChangeSelection( ed, state, &dpNew, lastLine );
+	    }
 	    return;
 
 	case KEY_KP_Up:
 	case KEY_Up:
-	    if  ( ( state & KEY_SHIFT_MASK ) && ds.dsDirection >= 0 )
-		{ dpNew= ds.dsEnd;	}
-	    else{ dpNew= ds.dsBegin;	}
+	    if  ( ! ( state & KEY_SHIFT_MASK ) || ds.dsDirection >= 0 )
+		{ pgRef= &(sg.sgEnd); dpNew= ds.dsEnd;		}
+	    else{ pgRef= &(sg.sgBegin); dpNew= ds.dsBegin;	}
 
 	    if  ( state & KEY_CONTROL_MASK )
 		{
-		const int	lastOne= 1;
-
 		if  ( dpNew.dpStroff == 0 )
 		    {
 		    dpNew.dpBi= docPrevParagraph( dpNew.dpBi );
 		    if  ( ! dpNew.dpBi )
 			{ return;	}
 
-		    if  ( docParaBegin( &dpNew, lastOne ) )
+		    if  ( docParaBegin( &dpNew ) )
 			{ return;	}
 		    }
 		else{
-		    if  ( docParaBegin( &dpNew, lastOne ) )
+		    if  ( docParaBegin( &dpNew ) )
 			{ return;	}
 		    }
 		}
 	    else{
-		if  ( tedLineUp( &dpNew, add ) )
+		if  ( tedArrowUp( &dpNew, pgRef, bd, add, sfl ) )
 		    { return;	}
 		}
 
-	    tedInputChangeSelection( ed, state, &dpNew );
+	    {
+	    const int	lastLine= sg.sgBegin.pgAtLineHead;
+
+	    tedInputChangeSelection( ed, state, &dpNew, lastLine );
+	    }
 	    return;
 
 	case KEY_KP_Down:
 	case KEY_Down:
 	    if  ( ! ( state & KEY_SHIFT_MASK ) || ds.dsDirection >= 0 )
-		{ dpNew= ds.dsEnd;	}
-	    else{ dpNew= ds.dsBegin;	}
+		{ pgRef= &(sg.sgEnd); dpNew= ds.dsEnd;		}
+	    else{ pgRef= &(sg.sgBegin); dpNew= ds.dsBegin;	}
 
 	    if  ( state & KEY_CONTROL_MASK )
 		{
-		const int	lastOne= 1;
-
 		if  ( dpNew.dpStroff < dpNew.dpBi->biParaStrlen		&&
-		      ( state & KEY_SHIFT_MASK )		)
+		      ( state & KEY_SHIFT_MASK )			)
 		    {
-		    if  ( docParaEnd( &dpNew, lastOne ) )
+		    if  ( docParaEnd( &dpNew ) )
 			{ return;	}
 		    }
 		else{
@@ -915,16 +1112,20 @@ static void tedProcessKeyEvent(	EditDocument *		ed,
 		    if  ( ! dpNew.dpBi )
 			{ return;	}
 
-		    if  ( docParaBegin( &dpNew, lastOne ) )
+		    if  ( docParaBegin( &dpNew ) )
 			{ return;	}
 		    }
 		}
 	    else{
-		if  ( tedLineDown( &dpNew, add ) )
+		if  ( tedArrowDown( &dpNew, pgRef, bd, add, sfl ) )
 		    { return;	}
 		}
 
-	    tedInputChangeSelection( ed, state, &dpNew );
+	    {
+	    const int	lastLine= sg.sgBegin.pgAtLineHead;
+
+	    tedInputChangeSelection( ed, state, &dpNew, lastLine );
+	    }
 	    return;
 
 	case KEY_KP_Prior:
@@ -933,13 +1134,51 @@ static void tedProcessKeyEvent(	EditDocument *		ed,
 		{ dpNew= ds.dsEnd;	}
 	    else{ dpNew= ds.dsBegin;	}
 
-	    if  ( tedPageUp( &dpNew, td->tdDocument, add,
-					ed->edVisibleRect.drY1-
-					ed->edVisibleRect.drY0 )	&&
-		  docFirstDocumentPosition( &dpNew, td->tdDocument )	)
+	    if  ( dpNew.dpBi->biInExternalItem != DOCinBODY )
+		{
+		if  ( ! docPrevSimilarRoot( &dpNew, bd ) )
+		    {
+		    const int	lastLine= 0;
+
+		    tedInputSetSelectedPosition( ed, &dpNew, lastLine );
+		    }
+
+		return;
+		}
+
+	    if  ( docPrevPosition( &dpNew )			&&
+		  docFirstDocumentPosition( &dpNew, bd )	)
 		{ return;	}
 
-	    tedInputChangeSelection( ed, state, &dpNew );
+	    {
+		PositionGeometry	pg;
+		TextLine *		tl;
+		const int		lastLine= 1;
+		int			partNew;
+
+		int			page;
+
+		tedPositionGeometry( &pg, &dpNew, 0, bd, add, sfl );
+
+		tl= dpNew.dpBi->biParaLines+ pg.pgLine;
+
+		page= tl->tlTopPosition.lpPage;
+		while( page >= 0 )
+		    {
+		    int		res;
+
+		    res= docGetTopOfColumn( &dpNew, &partNew, bd, page, 0 );
+		    if  ( res < 0 )
+			{ LDEB(res); return;	}
+		    if  ( res == 0 )
+			{
+			tedInputChangeSelection( ed, state, &dpNew, lastLine );
+			return;
+			}
+
+		    page--;
+		    }
+	    }
 	    return;
 
 	case KEY_KP_Next:
@@ -948,14 +1187,49 @@ static void tedProcessKeyEvent(	EditDocument *		ed,
 		{ dpNew= ds.dsEnd;	}
 	    else{ dpNew= ds.dsBegin;	}
 
-	    if  ( tedPageDown( &dpNew, td->tdDocument, add,
-					add->addDocRect.drY1,
-					ed->edVisibleRect.drY1-
-					ed->edVisibleRect.drY0 )	&&
-		  docLastDocumentPosition( &dpNew, td->tdDocument )	)
-		{ return;	}
+	    if  ( dpNew.dpBi->biInExternalItem != DOCinBODY )
+		{
+		if  ( ! docNextSimilarRoot( &dpNew, bd ) )
+		    {
+		    const int	lastLine= 0;
 
-	    tedInputChangeSelection( ed, state, &dpNew );
+		    tedInputAvoidHeadField( &dpNew, bd );
+
+		    tedInputSetSelectedPosition( ed, &dpNew, lastLine );
+		    }
+
+		return;
+		}
+
+	    {
+		PositionGeometry	pg;
+		TextLine *		tl;
+		const int		lastLine= 1;
+		int			partNew;
+
+		int			page;
+
+		tedPositionGeometry( &pg, &dpNew, lastLine, bd, add, sfl );
+
+		tl= dpNew.dpBi->biParaLines+ pg.pgLine;
+
+		page= tl->tlTopPosition.lpPage+ 1;
+		while( page < bd->bdItem.biBelowPosition.lpPage )
+		    {
+		    int		res;
+
+		    res= docGetTopOfColumn( &dpNew, &partNew, bd, page, 0 );
+		    if  ( res < 0 )
+			{ LDEB(res); return;	}
+		    if  ( res == 0 )
+			{
+			tedInputChangeSelection( ed, state, &dpNew, lastLine );
+			return;
+			}
+
+		    page++;
+		    }
+	    }
 	    return;
 
 #	if 0
@@ -988,7 +1262,7 @@ static void tedProcessKeyEvent(	EditDocument *		ed,
 	    return;
 #	endif
 
-#	if 1
+#	if 0
 	case KEY_y:
 	    docListItem(0,ds.dsBegin.dpBi);
 	    break;
@@ -1005,18 +1279,31 @@ static void tedProcessKeyEvent(	EditDocument *		ed,
 	case KEY_KP_Insert:
 	case KEY_Num_Lock:
 	    return;
+
 	default: unknown:
 #	    ifdef USE_GTK
-	    gtk_accel_group_activate( ed->edToplevel.atAccelGroup,
+#		if GTK_MAJOR_VERSION < 2
+		gtk_accel_group_activate( ed->edToplevel.atAccelGroup,
 							    keySym, state );
+#		else
+		gtk_accel_groups_activate(
+			    G_OBJECT( ed->edToplevel.atAccelGroup ),
+							    keySym, state );
+#		endif
 #	    endif
 
 #	    ifdef APP_DEBUG
 #	    ifdef USE_MOTIF
-	    appDebug( "INPUT \"%s\": %s (%s%s<Key>%s) \"%.*s\"\n",
+	    appDebug( "INPUT \"%s\": %s (%s%s%s%s%s%s%s%s<Key>%s) \"%.*s\"\n",
 			ed->edTitle, APP_X11EventNames[event->type],
 			(state&KEY_SHIFT_MASK)?"Shift":"",
 			(state&KEY_CONTROL_MASK)?"Ctrl":"",
+			(state&LockMask)?"Lock":"",
+			(state&Mod1Mask)?"Mod1":"",
+			(state&Mod2Mask)?"Mod2":"",
+			(state&Mod3Mask)?"Mod3":"",
+			(state&Mod4Mask)?"Mod4":"",
+			(state&Mod5Mask)?"Mod5":"",
 			XKeysymToString( keySym ), gotString, scratch );
 #	    endif
 #	    endif
@@ -1025,7 +1312,7 @@ static void tedProcessKeyEvent(	EditDocument *		ed,
     return;
     }
 
-APP_EVENT_HANDLER( tedKeyPressed, w, voided, keyEvent )
+APP_EVENT_HANDLER_H( tedKeyPressed, w, voided, keyEvent )
     {
     EditDocument *		ed= (EditDocument *)voided;
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
@@ -1053,7 +1340,7 @@ APP_EVENT_HANDLER( tedKeyPressed, w, voided, keyEvent )
 
     appGuiGetStringFromKeyboardEvent( ed->edInputContext, w, keyEvent,
 				&gotString, &gotKey, &state,
-				(char *)scratch, sizeof(scratch), &keySym );
+				scratch, sizeof(scratch), &keySym );
 
     tedStopCursorBlink( ed );
 
@@ -1082,6 +1369,7 @@ typedef struct TedObjectDrag
     InsertedObject *	todIo;
     DocumentPosition	todDp;
     PositionGeometry	todPg;
+    int			todParticule;
 
     int			todMouseX;
     int			todMouseY;
@@ -1109,7 +1397,7 @@ static void tedObjectHandleMove(	TedObjectDrag *		tod,
     int				high= tod->todIo->ioDragHigh;
 
     bi= tod->todDp.dpBi;
-    tp= bi->biParaParticules+ tod->todDp.dpParticule;
+    tp= bi->biParaParticules+ tod->todParticule;
     tl= bi->biParaLines+ tod->todPg.pgLine;
 
     if  ( tod->todRight && tod->todMouseX != mouseX )
@@ -1125,20 +1413,20 @@ static void tedObjectHandleMove(	TedObjectDrag *		tod,
 
     if  ( moved )
 	{
-	int		baseline;
+	int		baseline= TL_BASE_PIXELS( add, tl );
+	int		x= tp->tpX0- ox;
+	int		y= baseline- tod->todIo->ioPixelsHigh- oy;
 
 	baseline= TL_BASE_PIXELS( add, tl );
 
-	appExposeRectangle( add,
-			tp->tpX0- ox,
-			baseline- tod->todIo->ioPixelsHigh- oy, wide, high );
+	appExposeRectangle( add, x, y, wide, high );
 	}
 
     tod->todMouseX= mouseX;
     tod->todMouseY= mouseY;
     }
 
-static APP_EVENT_HANDLER( tedObjectDragMouseMove, w, vtod, event )
+static APP_EVENT_HANDLER_H( tedObjectDragMouseMove, w, vtod, event )
     {
     TedObjectDrag *		tod= (TedObjectDrag *)vtod;
 
@@ -1153,7 +1441,7 @@ static APP_EVENT_HANDLER( tedObjectDragMouseMove, w, vtod, event )
     return;
     }
 
-static APP_EVENT_HANDLER( tedObjectDragMouseUp, w, vtod, event )
+static APP_EVENT_HANDLER_H( tedObjectDragMouseUp, w, vtod, event )
     {
     TedObjectDrag *		tod= (TedObjectDrag *)vtod;
 
@@ -1178,25 +1466,28 @@ static int tedObjectDrag(	APP_WIDGET	w,
 				EditDocument *	ed,
 				APP_EVENT *	downEvent )
     {
-    EditApplication *	ea= ed->edApplication;
-    TedDocument *	td= (TedDocument *)ed->edPrivateData;
-    AppDrawingData *	add= &(ed->edDrawingData);
+    EditApplication *		ea= ed->edApplication;
+    TedDocument *		td= (TedDocument *)ed->edPrivateData;
+    BufferDocument *		bd= td->tdDocument;
+    const ScreenFontList *	sfl= &(td->tdScreenFontList);
+    AppDrawingData *		add= &(ed->edDrawingData);
 
-    int			ox= ed->edVisibleRect.drX0;
-    int			oy= ed->edVisibleRect.drY0;
+    int				ox= ed->edVisibleRect.drX0;
+    int				oy= ed->edVisibleRect.drY0;
 
-    int			wide;
-    int			high;
+    int				wide;
+    int				high;
 
-    int			button;
-    int			upDown;
-    int			seq;
-    unsigned int	keyState;
+    int				button;
+    int				upDown;
+    int				seq;
+    unsigned int		keyState;
 
-    int			x;
-    int			y;
+    int				x;
+    int				y;
 
-    TedObjectDrag	tod;
+    const int			lastOne= 1;
+    TedObjectDrag		tod;
 
     tod.todEd= ed;
 
@@ -1209,10 +1500,11 @@ static int tedObjectDrag(	APP_WIDGET	w,
 	{ LDEB(1); return -1;	}
 
     docInitDocumentPosition( &(tod.todDp) );
-    if  ( tedGetObjectSelection( td, &(tod.todDp), &(tod.todIo) ) )
+    if  ( tedGetObjectSelection( td,
+			&(tod.todParticule), &(tod.todDp), &(tod.todIo) ) )
 	{ return 2;	}
 
-    tedPositionGeometry( &(tod.todPg), &(tod.todDp), add );
+    tedPositionGeometry( &(tod.todPg), &(tod.todDp), lastOne, bd, add, sfl );
 
     wide= tod.todIo->ioPixelsWide;
     high= tod.todIo->ioPixelsHigh;
@@ -1268,7 +1560,7 @@ static int tedObjectDrag(	APP_WIDGET	w,
 	    tod.todIo->ioPixelsWide= TWIPStoPIXELS( xfac,
 		    ( tod.todIo->ioScaleX* tod.todIo->ioTwipsWide )/ 100 );
 
-	    tp= tod.todDp.dpBi->biParaParticules+ tod.todDp.dpParticule;
+	    tp= tod.todDp.dpBi->biParaParticules+ tod.todParticule;
 
 	    tp->tpPixelsWide= tod.todIo->ioPixelsWide;
 	    }
@@ -1282,10 +1574,12 @@ static int tedObjectDrag(	APP_WIDGET	w,
 		    ( tod.todIo->ioScaleY* tod.todIo->ioTwipsHigh )/ 100 );
 	    }
 
-	if  ( tedResizeObject( ed, &(tod.todDp), &(tod.todPg) ) )
+	if  ( tedResizeObject( ed, tod.todParticule,
+					    &(tod.todDp), &(tod.todPg) ) )
 	    { LDEB(1);	}
 
-	tedPositionGeometry( &(tod.todPg), &(tod.todDp), add );
+	tedPositionGeometry( &(tod.todPg), &(tod.todDp), lastOne,
+							    bd, add, sfl );
 
 	appDocumentChanged( ed, 1 );
 	}
@@ -1307,7 +1601,25 @@ static int tedObjectDrag(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-APP_EVENT_HANDLER( tedMouseButtonPressed, w, voided, downEvent )
+static void tedButton3Pressed(	APP_WIDGET			w,
+				EditDocument *			ed,
+				APP_EVENT *			downEvent )
+    {
+    TedDocument *	td= (TedDocument *)ed->edPrivateData;
+    DraggingContext	dc;
+    unsigned int	keyState;
+
+    if  ( tedSelectMousePosition( &keyState, w, ed, &dc, downEvent ) )
+	{ return;	}
+
+    tedShowFormatTool( td->tdToolsFormatToolOption, ed->edApplication );
+
+    tedAdaptFormatToolToDocument( ed, 1 );
+
+    return;
+    }
+
+APP_EVENT_HANDLER_H( tedMouseButtonPressed, w, voided, downEvent )
     {
     EditDocument *	ed= (EditDocument *)voided;
     TedDocument *	td= (TedDocument *)ed->edPrivateData;
@@ -1331,6 +1643,7 @@ APP_EVENT_HANDLER( tedMouseButtonPressed, w, voided, downEvent )
 	    appDocAskForPaste( ed, "PRIMARY" );
 	    break;
 	case Button3:
+	    tedButton3Pressed( w, ed, downEvent );
 	    break;
 
 	case Button4:
@@ -1366,6 +1679,7 @@ APP_EVENT_HANDLER( tedMouseButtonPressed, w, voided, downEvent )
 	    appDocAskForPaste( ed, "PRIMARY" );
 	    break;
 	case 3:
+	    tedButton3Pressed( w, ed, downEvent );
 	    break;
 
 	case 4:
