@@ -11,18 +11,25 @@
 #   include	<stdio.h>
 #   include	<string.h>
 #   include	<ctype.h>
+#   include	<ctype.h>
+#   include	<locale.h>
+
+#   include	<appSystem.h>
+#   include	<utilMatchFont.h>
 
 #   include	"tedApp.h"
 #   include	"tedRuler.h"
 
-#   include	<appFontTool.h>
 #   include	<appSymbolPicker.h>
 #   include	<appSpellTool.h>
+#   include	"docLayout.h"
 
 #   include	<appGeoString.h>
 #   include	<appPaper.h>
 
 #   include	<sioStdio.h>
+#   include	<sioStdin.h>
+#   include	<sioStdout.h>
 #   include	<sioMemory.h>
 #   include	<sioBase64.h>
 
@@ -114,7 +121,12 @@ static AppConfigurableResource TEDApplicationResourceTable[]=
 	/**/
 	APP_RESOURCE( "defaultAnsicpg",
 		    offsetof(TedAppResources,tarDefaultAnsicpgString),
-		    (char *)0 ),
+		    "1252" ),
+
+	/**/
+	APP_RESOURCE( "showTableGrid",
+		    offsetof(TedAppResources,tarShowTableGridString),
+		    "1" ),
     };
 
 
@@ -182,184 +194,13 @@ static AppConfigurableResource TEDFileMessageResourceTable[]=
 
 static TedAppResources			TEDResources;
 
-/************************************************************************/
-/*									*/
-/*  Determine default code page.					*/
-/*									*/
-/************************************************************************/
-
-int tedDetermineDefaultCodepage(	TedAppResources *	tar )
-    {
-    char *		past;
-    int		val;
-
-    val= strtol( tar->tarDefaultAnsicpgString, &past, 10 );
-    if  ( past != tar->tarDefaultAnsicpgString )
-	{
-	while( *past == ' ' )
-	    { past++;	}
-	}
-    if  ( past != tar->tarDefaultAnsicpgString	&&
-	  ! *past					)
-	{ tar->tarDefaultAnsicpgInt= val;			}
-    else{ SDEB(tar->tarDefaultAnsicpgString); return -1;	}
-
-    return 0;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Open a document.							*/
 /*									*/
-/*  1)  Open an input stream.						*/
-/*  2)  Try to read as RTF.						*/
-/*  3)  If this fails, try as plain text.				*/
-/*									*/
-/************************************************************************/
-
-static int tedOpenDocumentFile(	EditApplication *	ea,
-				int *			pFormat,
-				BufferDocument **	pBd,
-				const char *		filename,
-				APP_WIDGET		relative,
-				APP_WIDGET		option )
-    {
-    const char *		slash= strrchr( filename, '/' );
-    const char *		dot;
-    SimpleInputStream *		sis;
-
-    BufferDocument *		bd;
-    int				triedRtf= 0;
-
-    int				resp;
-    AppFileMessageResources *	afmr= &(ea->eaFileMessageResources);
-    TedAppResources *		tar= (TedAppResources *)ea->eaResourceData;
-
-    int				longestPara;
-
-    if  ( tar->tarDefaultAnsicpgInt < 0		&&
-	  tar->tarDefaultAnsicpgString		&&
-	  tedDetermineDefaultCodepage( tar )	)
-	{ SDEB(tar->tarDefaultAnsicpgString);	}
-
-    if  ( slash )
-	{ dot= strrchr( slash+ 1, '.' );	}
-    else{ dot= strrchr( filename, '.' );	}
-
-    /*  1  */
-    sis= sioInStdioOpen( filename );
-    if  ( ! sis )
-	{
-	appQuestionRunSubjectErrorDialog( ea, relative, option,
-					filename, afmr->afmrFileNoAccess );
-
-	return -1;
-	}
-
-    if  ( dot && ! strcmp( dot, ".rtf" ) )
-	{
-	/*  2  */
-	bd= docRtfReadFile( sis, tar->tarDefaultAnsicpgInt );
-
-	sioInClose( sis );
-	
-	if  ( bd )
-	    {
-	    *pFormat= TEDdockindRTF; *pBd= bd;
-
-	    if  ( docPropertiesSetFilename( &(bd->bdProperties), filename ) )
-		{ LDEB(1);	}
-
-	    return 0;
-	    }
-	else{
-	    resp= appQuestionRunSubjectOkCancelDialog( ea,
-				    relative, option,
-				    filename, TEDResources.tarFileNotRtf,
-				    (char *)0, (char *)0 );
-
-	    if  ( resp != AQDrespOK )
-		{ return -1;	}
-
-	    triedRtf= 1;
-	    }
-	}
-
-    if  ( dot && ! strcmp( dot, ".txt" ) )
-	{
-	bd= docPlainReadFile( sis, &longestPara,
-					&(ea->eaDefaultDocumentGeometry) );
-
-	sioInClose( sis );
-
-	if  ( bd )
-	    {
-	    *pBd= bd;
-
-	    if  ( longestPara > 76 )
-		{ *pFormat= TEDdockindTEXT_SAVE_WIDE;		}
-	    else{ *pFormat= TEDdockindTEXT_SAVE_FOLDED;		}
-
-	    if  ( docPropertiesSetFilename( &(bd->bdProperties), filename ) )
-		{ LDEB(1);	}
-
-	    return 0;
-	    }
-	else{ SXDEB(filename,bd); return -1;	}
-	}
-
-    /*  2  */
-    if  ( ! triedRtf )
-	{
-	bd= docRtfReadFile( sis, tar->tarDefaultAnsicpgInt );
-	if  ( bd )
-	    {
-	    *pFormat= TEDdockindRTF; *pBd= bd;
-	    
-	    if  ( docPropertiesSetFilename( &(bd->bdProperties), filename ) )
-		{ LDEB(1);	}
-
-	    return 0;
-	    }
-
-	resp= appQuestionRunSubjectOkCancelDialog( ea,
-					ea->eaToplevel.atTopWidget, option,
-					filename, TEDResources.tarFileNotRtf,
-					(char *)0, (char *)0 );
-
-	if  ( resp != AQDrespOK )
-	    { return -1;	}
-	}
-
-    sis= sioInStdioOpen( filename );
-    if  ( ! sis )
-	{ SXDEB(filename,sis); return -1;	}
-
-    bd= docPlainReadFile( sis, &longestPara,
-				    &(ea->eaDefaultDocumentGeometry) );
-
-    sioInClose( sis );
-
-    if  ( bd )
-	{
-	*pBd= bd;
-
-	if  ( longestPara > 76 )
-	    { *pFormat= TEDdockindTEXT_SAVE_WIDE;		}
-	else{ *pFormat= TEDdockindTEXT_SAVE_FOLDED;		}
-
-	if  ( docPropertiesSetFilename( &(bd->bdProperties), filename ) )
-	    { LDEB(1);	}
-
-	return 0;
-	}
-
-    SXDEB(filename,bd); return -1;
-    }
-
-/************************************************************************/
-/*									*/
-/*  Open a document.							*/
+/*  1)  Read the file.							*/
+/*  2)  Get the list of fonts that are available on the machine.	*/
+/*  3)  Add them to the font list of the document.			*/
 /*									*/
 /************************************************************************/
 
@@ -368,14 +209,33 @@ static int tedOpenDocument(	EditApplication *	ea,
 				int *			pFormat,
 				APP_WIDGET		relative,
 				APP_WIDGET		option,
-				const char *		filename	)
+				const char *		filename )
     {
-    TedDocument *	td= (TedDocument *)voidtd;
+    TedDocument *		td= (TedDocument *)voidtd;
+    BufferDocument *		bd= td->tdDocument;
+    DocumentProperties *	dp= &(bd->bdProperties);
 
+    /*  1  */
     if  ( tedOpenDocumentFile( ea, pFormat, &(td->tdDocument),
 					    filename, relative, option ) )
 	{ /*SDEB(filename);*/ return -1;	}
 
+    bd= td->tdDocument;
+    dp= &(bd->bdProperties);
+
+    /*  2  */
+    if  ( appPostScriptFontCatalog( ea ) )
+	{ SDEB(ea->eaAfmDirectory); return -1;	}
+
+    /*  3  */
+    if  ( utilAddPsFontsToDocList( &(dp->dpFontList), 
+					&(ea->eaPostScriptFontList) ) )
+	{ LDEB(ea->eaPostScriptFontList.psflFamilyCount); return -1;	}
+
+    if  ( utilFindPsFontsForDocFonts( &(dp->dpFontList),
+					&(ea->eaPostScriptFontList) ) )
+	{ LDEB(ea->eaPostScriptFontList.psflFamilyCount); return -1;	}
+
     return 0;
     }
 
@@ -391,12 +251,13 @@ static void tedDocFilePrint(	APP_WIDGET	printOption,
     int			firstPage= -1;
     int			lastPage= -1;
 
-    DocumentSelection	ds;
-    SelectionGeometry	sg;
+    DocumentSelection		ds;
+    SelectionGeometry		sg;
+    SelectionDescription	sd;
 
     const DocumentGeometry *	dgDoc= &(bd->bdProperties.dpGeometry);
 
-    if  ( ! tedGetSelection( &ds, &sg, td )	)
+    if  ( ! tedGetSelection( &ds, &sg, &sd, td )	)
 	{
 	firstPage= sg.sgBegin.pgTopPosition.lpPage;
 	lastPage= sg.sgEnd.pgTopPosition.lpPage;
@@ -414,7 +275,7 @@ static void tedDocFilePrint(	APP_WIDGET	printOption,
 /*									*/
 /************************************************************************/
 
-static APP_DESTROY_CALLBACK( tedManualDestroyed, w, voidtar )
+static APP_DESTROY_CALLBACK_H( tedManualDestroyed, w, voidtar )
     {
     TedAppResources *	tar= (TedAppResources *)voidtar;
 
@@ -466,118 +327,6 @@ static void tedAppManual(	APP_WIDGET	option,
     tedManual( option, ea, voidpbcs, ea->eaToplevel.atTopWidget );
     }
 
-/************************************************************************/
-/*									*/
-/*  Handle a replace from one of the tools.				*/
-/*									*/
-/************************************************************************/
-
-void tedAppReplace(	void *			voidea,
-			const unsigned char *	word	)
-    {
-    EditApplication *	ea= (EditApplication *)voidea;
-    EditDocument *	ed= ea->eaCurrentDocument;
-    TedDocument *	td;
-
-    if  ( ! ed )
-	{ XDEB(ed); return;	}
-
-    td= (TedDocument *)ed->edPrivateData;
-
-    if  ( tedHasSelection( td )		&&
-	  ! tedHasIBarSelection( td )	&&
-	  td->tdCanReplaceSelection	)
-	{ tedAppReplaceSelection( ed, word, strlen( (const char *)word ) ); }
-
-    return;
-    }
-
-/************************************************************************/
-/*									*/
-/*  Callback for the 'Insert File' menu option.				*/
-/*									*/
-/************************************************************************/
-
-static int tedInsertFile(	void *		voided,
-				APP_WIDGET	relative,
-				APP_WIDGET	option,
-				const char *	filename )
-    {
-    EditDocument *		ed= (EditDocument *)voided;
-    EditApplication *		ea= ed->edApplication;
-
-    int				format;
-    BufferDocument *		bd;
-
-    if  ( tedOpenDocumentFile( ea, &format, &bd, filename, relative, option ) )
-	{ SDEB(filename); return -1;	}
-
-    switch( format )
-	{
-	case 0:
-	    if  ( tedIncludeRtfDocument( ed->edDocumentWidget, ed, bd ) )
-		{ LDEB(1); docFreeDocument( bd ); return -1;	}
-	    break;
-	case 1:
-	    if  ( tedIncludePlainDocument( ed->edDocumentWidget, ed, bd ) )
-		{ LDEB(1); docFreeDocument( bd ); return -1;	}
-	    break;
-	default:
-	    LDEB(format);
-	    docFreeDocument( bd ); return -1;
-	}
-	    
-    docFreeDocument( bd );
-
-    appDocumentChanged( ed, 1 );
-
-    return 0;
-    }
-
-void tedDocInsertFile(		APP_WIDGET	option,
-				void *		voided,
-				void *		voidpbcs )
-    {
-    EditDocument *		ed= (EditDocument *)voided;
-    EditApplication *		ea= ed->edApplication;
-    TedDocument *		td= (TedDocument *)ed->edPrivateData;
-
-    if  ( ! td->tdCanReplaceSelection )
-	{ LDEB(td->tdCanReplaceSelection); return;	}
-
-    appRunOpenChooser( option, ed->edToplevel.atTopWidget,
-			    ea->eaFileExtensionCount, ea->eaFileExtensions,
-			    ea->eaDefaultFileFilter,
-			    voided, tedInsertFile, ea );
-
-    return;
-    }
-
-/************************************************************************/
-/*									*/
-/*  Font Menu.								*/
-/*									*/
-/************************************************************************/
-
-static void tedDocFontPaste(	APP_WIDGET	fontsOption,
-				void *		voided,
-				void *		voidpbcbs	 )
-    {
-    EditDocument *			ed= (EditDocument *)voided;
-
-    appDocAskForPaste( ed, "FONT" );
-    }
-
-static void tedDocToolFont(	APP_WIDGET	fontsOption,
-				void *		voided,
-				void *		voidcbs	 )
-    {
-    EditDocument *	ed= (EditDocument *)voided;
-    EditApplication *	ea= ed->edApplication;
-
-    tedShowFontsTool( fontsOption, ea, voidcbs );
-    }
-
 /************************************************************************/
 /*									*/
 /*  Table related menu option callbacks.				*/
@@ -633,9 +382,11 @@ static void tedDocFormatTool(	APP_WIDGET	option,
     EditDocument *	ed= (EditDocument *)voided;
     EditApplication *	ea= ed->edApplication;
 
-    tedShowFormatTool( option, ea, "tedFormatTool", "tedtable" );
+    /* option == td->tdToolsFormatToolOption */
 
-    tedAdaptFormatToolToDocument( ed );
+    tedShowFormatTool( option, ea );
+
+    tedAdaptFormatToolToDocument( ed, 0 );
     }
 
 static void tedDocShowPageTool(		APP_WIDGET	pageOption,
@@ -674,6 +425,37 @@ static void tedDocShowPageTool(		APP_WIDGET	pageOption,
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Font Menu.								*/
+/*									*/
+/************************************************************************/
+
+static void tedDocFontPaste(	APP_WIDGET	fontsOption,
+				void *		voided,
+				void *		voidpbcbs	 )
+    {
+    EditDocument *			ed= (EditDocument *)voided;
+
+    appDocAskForPaste( ed, "FONT" );
+    }
+
+static void tedDocToolFont(	APP_WIDGET	fontsOption,
+				void *		voided,
+				void *		voidcbs	 )
+    {
+    EditDocument *		ed= (EditDocument *)voided;
+    EditApplication *		ea= ed->edApplication;
+    TedDocument *		td= (TedDocument *)ed->edPrivateData;
+
+    /*  To get the correct title  */
+    tedDocFormatTool( td->tdToolsFormatToolOption, voided, voidcbs );
+
+    tedFormatShowFontPage( ea );
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Run document properties dialog.					*/
@@ -695,57 +477,6 @@ static void tedDocFileProps(	APP_WIDGET	option,
     tedRunPropertyDialog( ed, &ds, option, "props" );
     }
 
-/************************************************************************/
-/*									*/
-/*  Custom 'Paste' functionality.					*/
-/*									*/
-/*  1)  The regular X11 manner.						*/
-/*									*/
-/************************************************************************/
-
-static void tedDocEditPaste(	APP_WIDGET	option,
-				void *		voided,
-				void *		voidpbcs )
-    {
-    EditDocument *			ed= (EditDocument *)voided;
-    EditApplication *			ea= ed->edApplication;
-    TedDocument *			td= (TedDocument *)ed->edPrivateData;
-
-    int					res;
-    BitmapDescription			bd;
-    unsigned char *			buffer;
-
-    if  ( ! td->tdCanReplaceSelection )
-	{ LDEB(td->tdCanReplaceSelection); return;	}
-
-    res= appDocAskForPaste( ed, "PRIMARY" );
-    if  ( ! res )
-	{ return;	}
-
-    if  ( ! appImgGetXvClipboard( &bd, &buffer, ea->eaToplevel.atTopWidget ) )
-	{
-	AppBitmapImage *	abi;
-
-	abi= (AppBitmapImage *)malloc( sizeof(AppBitmapImage) );
-	if  ( ! abi )
-	    { XDEB(abi); return;	}
-	appInitBitmapImage( abi );
-
-	if  ( bmCopyDescription( &(abi->abiBitmap), &bd ) )
-	    { bmCleanDescription( &bd ); return; }
-
-	bmCleanDescription( &bd );
-	abi->abiBuffer= buffer;
-
-	if  ( tedReplaceSelectionWithBitmapImage( ed, abi ) )
-	    { LDEB(1); return; }
-
-	return;
-	}
-
-    return;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Mail a document.							*/
@@ -857,53 +588,47 @@ static AppMenuItem TED_DocFileMenuItems[]=
     "docFileMenuSaveAsKey",	(char *)0,
     "docFileMenuSaveAsKeyText",	(char *)0,	ITEMtyOPTION, appDocFileSaveAs,
     },
-#   define	TEDmiDocFileSaveTo		4
-    {
-    "docFileMenuSaveToText",	"Save To ...",
-    "docFileMenuSaveToKey",	(char *)0,
-    "docFileMenuSaveToKeyText",	(char *)0,	ITEMtyOPTION, appDocFileSaveTo,
-    },
-#   define	TEDmiDocFilePrint		5
+#   define	TEDmiDocFilePrint		4
     {
     "docFileMenuPrintText",	"Print ...",
     "docFileMenuPrintKey",	"Ctrl <Key>p",
     "docFileMenuPrintKeyText",	"^P",		ITEMtyOPTION, tedDocFilePrint,
     },
-#   define	TEDmiDocFileMail		6
+#   define	TEDmiDocFileMail		5
     {
     "docFileMenuMailText",	"Mail ...",
     "docFileMenuMailKey",	(char *)0,
     "docFileMenuMailKeyText",	(char *)0,	ITEMtyOPTION, tedDocFileMail,
     },
-#   define	TEDmiDocFileProperties		7
+#   define	TEDmiDocFileProperties		6
     {
     "docFileMenuPropsText",	"Properties ...",
     "docFileMenuPropsKey",	(char *)0,
     "docFileMenuPropsKeyText",	(char *)0,	ITEMtyOPTION, tedDocFileProps,
     },
-#   define	TEDmiDocFileMinimize		8
+#   define	TEDmiDocFileMinimize		7
     {
     "docFileMenuMiniText",	"Minimize",
     "docFileMenuMiniKey",	"Ctrl <Key>m",
     "docFileMenuMiniKeyText",	"^M",		ITEMtyOPTION, appDocFileMini,
     },
-#   define	TEDmiDocFileHide		9
+#   define	TEDmiDocFileHide		8
     {
     "docFileMenuHideText",	"Hide",
     "docFileMenuHideKey",	"Ctrl <Key>h",
     "docFileMenuHideKeyText",	"^H",		ITEMtyOPTION, appDocFileHide,
     },
-#   define	TEDmiDocFileClose		10
+#   define	TEDmiDocFileClose		9
     {
     "docFileMenuCloseText",	"Close",
     "docFileMenuCloseKey",	(char *)0,
     "docFileMenuCloseKeyText",	(char *)0,	ITEMtyOPTION, appDocFileClose,
     },
 
-#   define	TEDmiDocFileQuitSeparator	11
+#   define	TEDmiDocFileQuitSeparator	10
     { "",	"", "", (char *)0, "", (char *)0, ITEMtySEPARATOR, },
 
-#   define	TEDmiDocFileQuit		12
+#   define	TEDmiDocFileQuit		11
     {
     "docFileMenuQuitText",	"Quit",
     "docFileMenuQuitKey",	(char *)0,
@@ -953,6 +678,13 @@ static AppMenuItem TED_DocEditMenuItems[]=
     "docToolMenuFindKeyText",		"^F",
 					ITEMtyOPTION, tedDocToolFind,
     },
+#   define	TEDmiDocEditFindNext	6
+    {
+    "docToolMenuFindNextText",		"Find Next",
+    "docToolMenuFindNextKey",		"<Key>F3",
+    "docToolMenuFindNextKeyText",	"F3",
+					ITEMtyOPTION, tedDocToolFindNext,
+    },
 };
 
 /************************************************************************/
@@ -965,21 +697,21 @@ static AppMenuItem TED_DocInsertMenuItems[]=
 {
 #   define	TEDmiDocInsertInsPict		0
     {
-    "docInsertMenuInsPictText",		"Insert Picture ...",
+    "docInsertMenuInsPictText",		"Picture ...",
     "docInsertMenuInsPictKey",		(char *)0,
     "docInsertMenuInsPictKeyText",	(char *)0,
 					ITEMtyOPTION, tedDocInsertPicture,
     },
 #   define	TEDmiDocInsertInsSymbol		1
     {
-    "docInsertMenuInsertSymbolText",	"Insert Symbol",
+    "docInsertMenuInsertSymbolText",	"Symbol",
     "docInsertMenuInsertSymbolKey",	(char *)0,
     "docInsertMenuInsertSymbolKeyText",	(char *)0,
 					ITEMtyOPTION, tedDocToolInsertSymbol,
     },
 #   define	TEDmiDocInsertInsHyperlink	2
     {
-    "docInsertMenuHyperlinkText",	"Hyperlink ...",
+    "docInsertMenuHyperlinkText",	"Hyperlink",
     "docInsertMenuHyperlinkKey",	"Ctrl <Key>k",
     "docInsertMenuHyperlinkKeyText",	"^K",
 					ITEMtyOPTION, tedDocInsertLink,
@@ -993,60 +725,67 @@ static AppMenuItem TED_DocInsertMenuItems[]=
     },
 #   define	TEDmiDocInsertInsFootnote	4
     {
-    "docInsertMenuFootnoteText",	"Insert Footnote",
+    "docInsertMenuFootnoteText",	"Footnote",
     "docInsertMenuFootnoteKey",		(char *)0,
     "docInsertMenuFootnoteKeyText",	(char *)0,
 					ITEMtyOPTION, tedDocInsertFootnote,
     },
-#   define	TEDmiDocInsertInsChftnsep	5
+#   define	TEDmiDocInsertInsEndnote	5
     {
-    "docInsertMenuChftnsepText",	"Insert Footnote Separator",
+    "docInsertMenuEndnoteText",		"Endnote",
+    "docInsertMenuEndnoteKey",		(char *)0,
+    "docInsertMenuEndnoteKeyText",	(char *)0,
+					ITEMtyOPTION, tedDocInsertEndnote,
+    },
+#   define	TEDmiDocInsertInsChftnsep	6
+    {
+    "docInsertMenuChftnsepText",	"Footnote Separator",
     "docInsertMenuChftnsepKey",		(char *)0,
     "docInsertMenuChftnsepKeyText",	(char *)0,
 					ITEMtyOPTION, tedDocInsertChftnsep,
     },
-#   define	TEDmiDocInsertInsFile		6
+#   define	TEDmiDocInsertInsFile		7
     {
-    "docInsertMenuInsertFileText",	"Insert File ...",
+    "docInsertMenuInsertFileText",	"File ...",
     "docInsertMenuInsertFileKey",	(char *)0,
     "docInsertMenuInsertFileKeyText",	(char *)0,
 					ITEMtyOPTION, tedDocInsertFile,
     },
-#   define	TEDmiDocInsertInsTable		7
+#   define	TEDmiDocInsertInsTable		8
     {
-    "docInsertMenuInsertTableText",	"Insert Table",
+    "docInsertMenuInsertTableText",	"Table",
     "docInsertMenuInsertTableKey",	(char *)0,
     "docInsertMenuInsertTableKeyText",	(char *)0,
 					ITEMtyOPTION, tedDocTableInsertTable,
     },
-#   define	TEDmiDocInsertInsPageNumber	8
+#   define	TEDmiDocInsertInsPageNumber	9
     {
-    "docInsertMenuInsertPageNumberText",	"Insert Page Number",
+    "docInsertMenuInsertPageNumberText",	"Page Number",
     "docInsertMenuInsertPageNumberKey",		(char *)0,
     "docInsertMenuInsertPageNumberKeyText",	(char *)0,
 				    ITEMtyOPTION, tedDocInsertPageNumber,
     },
 
-    /*  9  */
+    /*  10  */
     { "",	"", "", (char *)0, "", (char *)0, ITEMtySEPARATOR, },
 
-#   define	TEDmiDocInsertInsLineBreak	10
+#   define	TEDmiDocInsertInsLineBreak	11
     {
-    "docInsertMenuInsertLineBreakText",	"Insert Line Break",
+    "docInsertMenuInsertLineBreakText",	"Line Break",
     "docInsertMenuInsertLineBreakKey",		(char *)0,
     "docInsertMenuInsertLineBreakKeyText",	(char *)0,
 				    ITEMtyOPTION, tedDocInsertLineBreak,
     },
-#   define	TEDmiDocInsertInsPageBreak	11
+#   define	TEDmiDocInsertInsPageBreak	12
     {
-    "docInsertMenuInsertPageBreakText",	"Insert Page Break",
+    "docInsertMenuInsertPageBreakText",	"Page Break",
     "docInsertMenuInsertPageBreakKey",		(char *)0,
     "docInsertMenuInsertPageBreakKeyText",	(char *)0,
 				    ITEMtyOPTION, tedDocInsertPageBreak,
     },
-#   define	TEDmiDocInsertInsSectBreak	12
+#   define	TEDmiDocInsertInsSectBreak	13
     {
-    "docInsertMenuInsertSectBreakText",	"Insert Section Break",
+    "docInsertMenuInsertSectBreakText",	"Section Break",
     "docInsertMenuInsertSectBreakKey",		(char *)0,
     "docInsertMenuInsertSectBreakKeyText",	(char *)0,
 				    ITEMtyOPTION, tedDocInsertSectBreak,
@@ -1055,8 +794,11 @@ static AppMenuItem TED_DocInsertMenuItems[]=
 };
 
 /************************************************************************/
+/*									*/
 /*  Document windows menu.						*/
+/*									*/
 /************************************************************************/
+
 static AppMenuItem TED_DocWinMenuItems[]=
 {
     {
@@ -1363,7 +1105,6 @@ void tedMakeDocumentReadonly(	EditDocument *	ed )
     XtUnmanageChild( children[TEDmiDocFileOpen] );
     XtUnmanageChild( children[TEDmiDocFileSave] );
     XtUnmanageChild( children[TEDmiDocFileSaveAs] );
-    XtUnmanageChild( children[TEDmiDocFileSaveTo] );
     XtUnmanageChild( children[TEDmiDocFileMail] );
     XtUnmanageChild( children[TEDmiDocFileProperties] );
     XtUnmanageChild( children[TEDmiDocFileQuitSeparator] );
@@ -1381,6 +1122,7 @@ void tedMakeDocumentReadonly(	EditDocument *	ed )
     XtUnmanageChild( td->tdInsSymbolOption );
     XtUnmanageChild( td->tdInsFileOption );
     XtUnmanageChild( td->tdInsInsertFootnoteOption );
+    XtUnmanageChild( td->tdInsInsertEndnoteOption );
     XtUnmanageChild( td->tdInsInsertChftnsepOption );
     XtUnmanageChild( td->tdInsInsertTableOption );
     XtUnmanageChild( td->tdInsInsertPageNumberOption );
@@ -1451,6 +1193,8 @@ static void tedMakePrivateDocumentMenus( EditApplication *	ea,
 	    TED_DocInsertMenuItems[TEDmiDocInsertInsBookmark].amiOptionWidget;
     td->tdInsInsertFootnoteOption=
 	    TED_DocInsertMenuItems[TEDmiDocInsertInsFootnote].amiOptionWidget;
+    td->tdInsInsertEndnoteOption=
+	    TED_DocInsertMenuItems[TEDmiDocInsertInsEndnote].amiOptionWidget;
     td->tdInsInsertChftnsepOption=
 	    TED_DocInsertMenuItems[TEDmiDocInsertInsChftnsep].amiOptionWidget;
     td->tdInsInsertTableOption=
@@ -1549,8 +1293,6 @@ static void tedVisibleDocumentCountChanged(	EditApplication *	ea,
 	    { appEnableSymbolPicker( ea->eaSymbolPicker, 1 ); }
 	if  ( ea->eaSpellTool )
 	    { appEnableSpellTool( ea->eaSpellTool, 1 ); }
-	if  ( ea->eaFontsTool )
-	    { appEnableFontsTool( ea->eaFontsTool, 1 ); }
 	if  ( ea->eaFindTool )
 	    { appEnableFindTool( ea->eaFindTool, 1 ); }
 	if  ( ea->eaPageTool )
@@ -1568,8 +1310,6 @@ static void tedVisibleDocumentCountChanged(	EditApplication *	ea,
 	    { appEnableSymbolPicker( ea->eaSymbolPicker, 0 ); }
 	if  ( ea->eaSpellTool )
 	    { appEnableSpellTool( ea->eaSpellTool, 0 ); }
-	if  ( ea->eaFontsTool )
-	    { appEnableFontsTool( ea->eaFontsTool, 0 ); }
 	if  ( ea->eaFindTool )
 	    { appEnableFindTool( ea->eaFindTool, 0 ); }
 	if  ( ea->eaPageTool )
@@ -1653,11 +1393,35 @@ static int tedWriteRtfMail(	SimpleOutputStream *		sos,
 	sioOutPutCharacter( '\r', sos ); sioOutPutCharacter( '\n', sos );
 	sioOutPutString( "Content-Type: text/plain", sos );
 	sioOutPutCharacter( '\r', sos ); sioOutPutCharacter( '\n', sos );
-	sioOutPutString( "Content-Transfer-Encoding: 7bit", sos );
+	sioOutPutString( "Content-Transfer-Encoding: 8bit", sos );
 	sioOutPutCharacter( '\r', sos ); sioOutPutCharacter( '\n', sos );
 
 	sioOutPutCharacter( '\r', sos ); sioOutPutCharacter( '\n', sos );
 
+	if  ( dp->dpTitle && strcmp( mimeBoundary, dp->dpTitle ) )
+	    {
+	    sioOutPutString( (char *)dp->dpTitle, sos );
+	    sioOutPutCharacter( '\r', sos ); sioOutPutCharacter( '\n', sos );
+	    }
+	else{
+	    DocumentPosition	dp;
+	    DocumentSelection	ds;
+
+	    if  ( docFirstPosition( &dp, &(bd->bdItem) ) )
+		{ sioOutPutString( "application/rtf\r\n", sos );	}
+	    else{
+		const int		fold= 1;
+		const int		closed= 1;
+		const int		direction= 1;
+
+		BufferItem *		bi= dp.dpBi;
+
+		docSetParaSelection( &ds, bi, direction, 0, bi->biParaStrlen );
+
+		return docPlainSaveDocument( sos, bd, &ds, fold, closed );
+		}
+	    }
+
 	sioOutPutCharacter( '-', sos ); sioOutPutCharacter( '-', sos );
 	sioOutPutString( mimeBoundary, sos );
 	sioOutPutCharacter( '\r', sos ); sioOutPutCharacter( '\n', sos );
@@ -1715,10 +1479,13 @@ static int tedWritePlainMail(	SimpleOutputStream *		sos,
 				const char *			mimeBoundary,
 				void *				voidbd )
     {
+    const int	fold= 1;
+    const int	closed= 1;
+
     return docPlainSaveDocument( sos,
 				(BufferDocument *)voidbd,
 				(const DocumentSelection *)0,
-				1, 1 );
+				fold, closed );
     }
 
 static MailContent	TedMailContents[]=
@@ -1744,11 +1511,12 @@ static MailContent	TedMailContents[]=
 
 static AppSelectionTargetType TedPrimaryTargets[]=
     {
-	{ "RTF",	(APP_ATOM)0,	tedPastePrimaryRtf,
+	{ "application/rtf",
+			(APP_ATOM)0,	tedPastePrimaryRtf,
 					tedCopyPrimaryRtf		},
 	{ "STRING",	(APP_ATOM)0,	tedPastePrimaryString,
 					tedCopyPrimaryString		},
-	{ "PNG",	(APP_ATOM)0,	tedPastePrimaryPng,
+	{ "image/png",	(APP_ATOM)0,	tedPastePrimaryPng,
 					tedCopyPrimaryPng		},
 	{ "PIXMAP",	(APP_ATOM)0,	tedPastePrimaryPixmap,
 					tedCopyPrimaryPixmap		},
@@ -1758,7 +1526,8 @@ static AppSelectionTargetType TedPrimaryTargets[]=
 
 AppSelectionTargetType TedPrimaryTextTargets[]=
     {
-	{ "RTF",	(APP_ATOM)0,	tedPastePrimaryRtf,
+	{ "application/rtf",
+			(APP_ATOM)0,	tedPastePrimaryRtf,
 					tedCopyPrimaryRtf		},
 	{ "STRING",	(APP_ATOM)0,	tedPastePrimaryString,
 					tedCopyPrimaryString		},
@@ -1769,9 +1538,10 @@ const int TedPrimaryTextTargetCount=
 
 AppSelectionTargetType TedPrimaryPictureTargets[]=
     {
-	{ "RTF",	(APP_ATOM)0,	tedPastePrimaryRtf,
+	{ "application/rtf",
+			(APP_ATOM)0,	tedPastePrimaryRtf,
 					tedCopyPrimaryRtf		},
-	{ "PNG",	(APP_ATOM)0,	tedPastePrimaryPng,
+	{ "image/png",	(APP_ATOM)0,	tedPastePrimaryPng,
 					tedCopyPrimaryPng		},
 	{ "PIXMAP",	(APP_ATOM)0,	tedPastePrimaryPixmap,
 					tedCopyPrimaryPixmap		},
@@ -1862,7 +1632,7 @@ static EditApplication	TedApplication=
 		    /*  Name of the picture for the application window.	*/
 		    /****************************************************/
     "Ted",
-    "Ted, Version 2.11, March 1, 2002",
+    "Ted, Version 2.16, April 18, 2004",
     "http://www.nllgg.nl/Ted",
 
 
@@ -1967,9 +1737,11 @@ static EditApplication	TedApplication=
     tedNewDocument,
     tedLayoutDocument,
     tedFinishDocumentSetup,
+    tedDocumentFirstVisible,
     NULL,					/*  CanSave		*/
     tedSaveDocument,				/*  Save		*/
     tedFreeDocument,				/*  Free		*/
+    tedSuggestNup,				/*  SuggestNup		*/
     tedPrintDocument,				/*  PrintDocument	*/
     tedDrawRectangle,
 		    /****************************************************/
@@ -2022,17 +1794,188 @@ static EditApplication	TedApplication=
     (AppSelectionType *)0,
     0/ sizeof(AppSelectionType),
 
-    0,0,0,
+    0,0,0,0,
 
     TedMailContents,
     sizeof(TedMailContents)/sizeof(MailContent),
     0,
     };
 
+/************************************************************************/
+/*									*/
+/*  Miscelaneous file conversion calls.					*/
+/*									*/
+/************************************************************************/
+
+static int tedFileConvert(
+		    const char *	nameOut,
+		    const char *	nameIn,
+		    int (*cvf)(	SimpleOutputStream *	_sosOut,
+				SimpleInputStream *	_sisIn ) )
+    {
+    int				rval= 0;
+    SimpleOutputStream *	sosOut= (SimpleOutputStream *)0;
+    SimpleInputStream *		sisIn= (SimpleInputStream *)0;
+
+    sisIn= sioInStdioOpen( nameIn );
+    if  ( ! sisIn )
+	{ SXDEB(nameIn,sisIn); rval= -1; goto ready;	}
+
+    sosOut= sioOutStdioOpen( nameOut );
+    if  ( ! sosOut )
+	{ SXDEB(nameOut,sosOut); rval= -1; goto ready;	}
+
+    if  ( (*cvf)( sosOut, sisIn ) )
+	{ SSDEB(nameOut,nameIn); rval= -1; goto ready;	}
+
+  ready:
+
+    if  ( sosOut )
+	{ sioOutClose( sosOut );	}
+    if  ( sisIn )
+	{ sioInClose( sisIn );	}
+
+    return rval;
+    }
+
+static int tedRtfToPsPaper(	EditApplication *	ea,
+				const char *		paperString )
+    {
+    int				rval= 0;
+
+    SimpleOutputStream *	sosOut= (SimpleOutputStream *)0;
+    SimpleInputStream *		sisIn= (SimpleInputStream *)0;
+
+    TedAppResources *		tar= (TedAppResources *)ea->eaResourceData;
+    BufferDocument *		bd= (BufferDocument *)0;
+
+    AppDrawingData		add;
+    PrintGeometry		pg;
+    LayoutJob			lj;
+    RecalculateFields		rf;
+
+    const int			firstPage= -1;
+    const int			lastPage= -1;
+
+    int				noteNumbersChanged= 0;
+
+    setlocale( LC_NUMERIC, "" );
+
+    appInitDrawingData( &add );
+    utilInitPrintGeometry( &pg );
+    docPsInitLayoutJob( &lj );
+    docInitRecalculateFields( &rf );
+
+    appGetApplicationResourceValues( ea );
+
+    if  ( appPostScriptFontCatalog( ea ) )
+	{ SDEB(ea->eaAfmDirectory); rval= -1; goto ready; }
+
+    if  ( paperString )
+	{
+	DocumentGeometry *	dg= &(pg.pgSheetGeometry);
+	int			paperFormat;
+
+	if  ( appPaperFormatFromString( &paperFormat,
+					    &(dg->dgPageWideTwips),
+					    &(dg->dgPageHighTwips),
+					    ea->eaUnitInt, paperString ) )
+	    { SDEB(paperString); rval= -1; goto ready;	}
+	}
+
+    sisIn= sioInStdinOpen();
+    if  ( ! sisIn )
+	{ XDEB(sisIn); rval= -1; goto ready;	}
+    sosOut= sioOutStdoutOpen();
+    if  ( ! sosOut )
+	{ XDEB(sosOut); rval= -1; goto ready;	}
+
+    bd= docRtfReadFile( sisIn, tar->tarDefaultAnsicpgInt );
+    if  ( ! bd )
+	{ XDEB(bd); rval= -1; goto ready;	}
+
+    if  ( utilFindPsFontsForDocFonts( &(bd->bdProperties.dpFontList),
+					&(ea->eaPostScriptFontList) ) )
+	{
+	LDEB(ea->eaPostScriptFontList.psflFamilyCount);
+	rval= -1; goto ready;
+	}
+
+    add.addPostScriptFontList= &(ea->eaPostScriptFontList);
+
+    rf.rfBd= bd;
+    rf.rfVoidadd= (void *)&add;
+    rf.rfCloseObject= tedCloseObject;
+    rf.rfUpdateFlags= FIELDdoDOC_INFO|FIELDdoDOC_COMPLETE|FIELDdoCHFTN;
+    rf.rfFieldsUpdated= 0;
+
+    if  ( docRecalculateTextLevelFields( &rf, &(bd->bdItem) ) )
+	{ LDEB(1); rval= -1; goto ready;	}
+
+    lj.ljAdd= &add;
+    lj.ljBd= bd;
+    lj.ljChangedItem= &(bd->bdItem);
+
+    if  ( docLayoutItemAndParents( &(bd->bdItem), &lj ) )
+	{ LDEB(1); rval= -1; goto ready;	}
+
+    docRenumberNotes( &noteNumbersChanged, bd );
+
+    rf.rfUpdateFlags= FIELDdoDOC_FORMATTED;
+    rf.rfFieldsUpdated= 0;
+
+    if  ( noteNumbersChanged )
+	{ rf.rfUpdateFlags |= FIELDdoCHFTN;	}
+
+    if  ( docRecalculateTextLevelFields( &rf, &(bd->bdItem) ) )
+	{ LDEB(1); rval= -1; goto ready;	}
+
+    if  ( rf.rfFieldsUpdated > 0 )
+	{
+	docPsCleanLayoutJob( &lj );
+	docPsInitLayoutJob( &lj );
+
+	lj.ljAdd= &add;
+	lj.ljBd= bd;
+	lj.ljChangedItem= &(bd->bdItem);
+
+	if  ( docLayoutItemAndParents( &(bd->bdItem), &lj ) )
+	    { LDEB(1); rval= -1; goto ready;	}
+	}
+
+    setlocale( LC_NUMERIC, "C" );
+
+    if  ( docPsPrintDocument( sosOut, "<stdin>",
+		ea->eaApplicationName, ea->eaReference, ea->eaFontDirectory,
+		&add, bd, &pg,
+		ea->eaUsePostScriptFilters, ea->eaUsePostScriptIndexedImages,
+		firstPage, lastPage, tedCloseObject ) )
+	{ LDEB(1); rval= -1; goto ready;	}
+
+  ready:
+
+    setlocale( LC_NUMERIC, "" );
+
+    docPsCleanLayoutJob( &lj );
+    appCleanDrawingData( &add );
+
+    if  ( bd )
+	{ docFreeDocument( bd );	}
+    if  ( sosOut )
+	{ sioOutClose( sosOut );	}
+    if  ( sisIn )
+	{ sioInClose( sisIn );	}
+
+    return rval;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Main() of the 'Ted' Application.					*/
 /*									*/
+/*  1)  Sanity checks							*/
+/*  2)  Initialisation of the application wide data.			*/
+/*									*/
 /************************************************************************/
 
 int main(	int		argc,
@@ -2040,9 +1983,122 @@ int main(	int		argc,
     {
     EditApplication *	ea= &TedApplication;
 
+    /*  1  */
+    if  ( RPprop_COUNT > PROPmaskMAXPROPS )
+	{ LLDEB(RPprop_COUNT,PROPmaskMAXPROPS); return 1;	}
+    if  ( PPprop_COUNT > PROPmaskMAXPROPS )
+	{ LLDEB(PPprop_COUNT,PROPmaskMAXPROPS); return 1;	}
+    if  ( CLprop_COUNT > PROPmaskMAXPROPS )
+	{ LLDEB(CLprop_COUNT,PROPmaskMAXPROPS); return 1;	}
+    if  ( DPprop_COUNT > PROPmaskMAXPROPS )
+	{ LLDEB(DPprop_COUNT,PROPmaskMAXPROPS); return 1;	}
+
+    if  ( DOCtl_COUNT > (1<<DOCtl_BITS) )
+	{ LLDEB(DOCtl_COUNT,(1<<DOCtl_BITS)); return 1;		}
+    if  ( DOCbs_COUNT > (1<<DOCbs_BITS) )
+	{ LLDEB(DOCbs_COUNT,(1<<DOCbs_BITS)); return 1;		}
+    if  ( DOCkind_COUNT > (1<<DOCkind_BITS) )
+	{ LLDEB(DOCkind_COUNT,(1<<DOCkind_BITS)); return 1;	}
+    if  ( DOCsp_COUNT > (1<<DOCsp_BITS) )
+	{ LLDEB(DOCsp_COUNT,(1<<DOCsp_BITS)); return 1;		}
+    if  ( DOCta_COUNT > (1<<DOCta_BITS) )
+	{ LLDEB(DOCta_COUNT,(1<<DOCta_BITS)); return 1;		}
+    if  ( TRauto_COUNT > (1<<TRauto_BITS) )
+	{ LLDEB(TRauto_COUNT,(1<<TRauto_BITS)); return 1;	}
+
+    if  ( DOxa_COUNT > (1<<DOxa_BITS) )
+	{ LLDEB(DOxa_COUNT,(1<<DOxa_BITS)); return 1;		}
+    if  ( DOya_COUNT > (1<<DOya_BITS) )
+	{ LLDEB(DOya_COUNT,(1<<DOya_BITS)); return 1;		}
+    if  ( DOkind_COUNT > (1<<DOkind_BITS) )
+	{ LLDEB(DOkind_COUNT,(1<<DOkind_BITS)); return 1;	}
+    if  ( DOline_COUNT > (1<<DOline_BITS) )
+	{ LLDEB(DOline_COUNT,(1<<DOline_BITS)); return 1;	}
+    if  ( DOfill_COUNT > (1<<DOfill_BITS) )
+	{ LLDEB(DOfill_COUNT,(1<<DOfill_BITS)); return 1;	}
+
+    /*  2  */
     TEDResources.tarDefaultAnsicpgInt= -1;
+    TEDResources.tarShowTableGridInt= 0;
+
+    TEDResources.tarFindPattern= (const unsigned char *)0;
+    TEDResources.tarFindRegex= 0;
+
+    if  ( argc >= 3			&&
+	  ! strcmp( argv[1], "--Find" )	)
+	{
+	TEDResources.tarFindPattern= (const unsigned char *)argv[2];
+	TEDResources.tarFindRegex= 0;
+	argc -= 2; argv += 2;
+	}
+    else{
+	if  ( argc >= 3				&&
+	      ! strcmp( argv[1], "--RegFind" )	)
+	    {
+	    TEDResources.tarFindPattern= (const unsigned char *)argv[2];
+	    TEDResources.tarFindRegex= 1;
+	    argc -= 2; argv += 2;
+	    }
+	}
+
+    if  ( argc >= 2 && !  strcmp( argv[1], "--TtfToAfm" ) )
+	{
+	if  ( argc != 4 )
+	    { SSLDEB(argv[0],argv[1],argc); return 1;	}
+
+	if  ( tedFileConvert( argv[3], argv[2], psTtfToAfm ) )
+	    { SSSDEB(argv[1],argv[2],argv[3]); return 1;	}
+
+	return 0;
+	}
+
+    if  ( argc >= 2 && !  strcmp( argv[1], "--TtfToPt42" ) )
+	{
+	if  ( argc != 4 )
+	    { SSLDEB(argv[0],argv[1],argc); return 1;	}
+
+	if  ( tedFileConvert( argv[3], argv[2], psTtfToPf42 ) )
+	    { SSSDEB(argv[1],argv[2],argv[3]); return 1;	}
+
+	return 0;
+	}
+
+    if  ( argc >= 2 && !  strcmp( argv[1], "--TtfToPt1" ) )
+	{
+	if  ( argc != 4 )
+	    { SSLDEB(argv[0],argv[1],argc); return 1;	}
+
+	if  ( tedFileConvert( argv[3], argv[2], psTtfToPt1 ) )
+	    { SSSDEB(argv[1],argv[2],argv[3]); return 1;	}
+
+	return 0;
+	}
+
+    if  ( argc >= 2 && !  strcmp( argv[1], "--RtfToPs" ) )
+	{
+	if  ( argc != 2 )
+	    { SSLDEB(argv[0],argv[1],argc); return 1;	}
+
+	if  ( tedRtfToPsPaper( ea, (const char *)0 ) )
+	    { SDEB(argv[1]); return 1;	}
+
+	return 0;
+	}
+
+    if  ( argc >= 2 && !  strcmp( argv[1], "--RtfToPsPaper" ) )
+	{
+	if  ( argc != 3 )
+	    { SSLDEB(argv[0],argv[1],argc); return 1;	}
+
+	if  ( tedRtfToPsPaper( ea, argv[2] ) )
+	    { SSDEB(argv[1],argv[2]); return 1;	}
+
+	return 0;
+	}
+
 
     tedGetNamedPictures( ea );
 
     return appMain( ea, argc, argv );
     }
+
