@@ -11,14 +11,45 @@
 #   include	<stddef.h>
 #   include	<limits.h>
 
-#   include	<appDebugon.h>
-
 #   include	<appGeoString.h>
-#   include	<appUnit.h>
+#   include	"docEvalField.h"
 
 #   include	"tedApp.h"
 #   include	"tedFormatTool.h"
 
+#   include	<appDebugon.h>
+
+/************************************************************************/
+/*									*/
+/*  What property values are relevant for footnotes/endnotes?		*/
+/*									*/
+/************************************************************************/
+
+static const int TED_FootnoteRestarts[]=
+    {
+    FTN_RST_CONTINUOUS,
+    FTN_RST_PER_SECTION,
+    FTN_RST_PER_PAGE,
+    };
+
+static const int TED_FootnotePositions[]=
+    {
+    FTN_POS_BELOW_TEXT,
+    FTN_POS_PAGE_BOTTOM,
+    };
+
+static const int TED_EndnoteRestarts[]=
+    {
+    FTN_RST_CONTINUOUS,
+    FTN_RST_PER_SECTION,
+    };
+
+static const int TED_EndnotePositions[]=
+    {
+    FTN_POS_SECT_END,
+    FTN_POS_DOC_END,
+    };
+
 /************************************************************************/
 /*									*/
 /*  Refresh the 'Notes' page of the format tool.			*/
@@ -46,7 +77,7 @@ static void tedSetNotesMenu(	AppOptionmenu *		aom,
 static void tedNotesRefreshNotesStartNumber(	NotePropertiesTool *	npt,
 						const NotesProperties *	np )
     {
-    if  ( np->npRestart == DPftnRST_CONTINUOUS )
+    if  ( np->npRestart == FTN_RST_CONTINUOUS )
 	{
 	appIntegerToTextWidget( npt->nptStartNumberText, np->npStartNumber );
 	appEnableText( npt->nptStartNumberText, 1 );
@@ -109,12 +140,18 @@ static void tedNotesToolRefreshNoteWidgets(	NotesTool *	nt )
     appGuiSetToggleState( nt->ntEndnoteToggle,
 				nt->ntNoteKindChosen == DOCinENDNOTE );
 
+    appGuiSetToggleState( nt->ntFixedTextToggle, nt->ntFixedTextChosen );
+
+    appEnableText( nt->ntFixedTextText, nt->ntFixedTextChosen );
+    appStringToTextWidget( nt->ntFixedTextText, (char *)nt->ntNoteTextChosen );
+
     return;
     }
 
 void tedFormatToolRefreshNotesTool(
 				NotesTool *			nt,
 				int *				pEnabled,
+				int *				pPref,
 				InspectorSubject *		is,
 				const DocumentSelection *	ds,
 				BufferDocument *		bd )
@@ -126,26 +163,44 @@ void tedFormatToolRefreshNotesTool(
 
     int				enabled= 0;
     int				noteIndex= -1;
+    int				noteFieldNr= -1;
 
     nt->ntInsideNote= 0;
     nt->ntNoteKindSet= DOCinBODY;
+    nt->ntFixedTextSet= 0;
+
+    nt->ntNoteTextSet[0]= '\0';
+    nt->ntNoteTextChosen[0]= '\0';
+
+    nt->ntNoteNumber= 0;
 
     if  ( ds )
 	{
 	DocumentNote *		dn;
 
-	noteIndex= docGetSelectedNote( &dn, bd, ds );
+	noteIndex= docGetSelectedNote( &dn, &noteFieldNr,
+						nt->ntNoteTextSet,
+						sizeof(nt->ntNoteTextSet)- 1,
+						bd, ds );
 	if  ( noteIndex >= 0 )
 	    {
 	    nt->ntNoteKindSet= dn->dnExternalItemKind;
+	    nt->ntNoteNumber= dn->dnNoteNumber;
 
 	    if  ( ds->dsSelectionScope.ssInExternalItem == DOCinFOOTNOTE  ||
 		  ds->dsSelectionScope.ssInExternalItem == DOCinENDNOTE   )
 		{ nt->ntInsideNote= 1;	}
+
+	    if  ( noteFieldNr < 0 )
+		{ nt->ntFixedTextSet= 1;	}
+
+	    *pPref= 9;
 	    }
 	}
 
     nt->ntNoteKindChosen=  nt->ntNoteKindSet;
+    nt->ntFixedTextChosen=  nt->ntFixedTextSet;
+    strcpy( (char *)nt->ntNoteTextChosen, (char *)nt->ntNoteTextSet );
 
     PROPmaskCLEAR( &updMask );
 
@@ -168,6 +223,19 @@ void tedFormatToolRefreshNotesTool(
 							&updMask, dp ) )
 	{ LDEB(1); return;	}
 
+    if  ( ! nt->ntFixedTextSet			&&
+	  nt->ntNoteKindSet != DOCinBODY	)
+	{
+	if  ( docFormatChftnField( nt->ntNoteTextChosen,
+					sizeof(nt->ntNoteTextChosen)- 1,
+					&(nt->ntPropertiesChosen),
+					nt->ntNoteNumber,
+					nt->ntNoteKindChosen ) )
+	    { LDEB(1); nt->ntNoteTextChosen[0]= '\0';	}
+
+	strcpy( (char *)nt->ntNoteTextSet, (char *)nt->ntNoteTextChosen );
+	}
+
     tedNotesToolRefreshDocWidgets( nt );
     tedNotesToolRefreshNoteWidgets( nt );
 
@@ -185,9 +253,7 @@ void tedFormatToolRefreshNotesTool(
 /*									*/
 /************************************************************************/
 
-static void tedNotesToolToNoteRef(	APP_WIDGET	w,
-					void *		voidnt,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedNotesToolToNoteRef, w, voidnt )
     {
     NotesTool *			nt= (NotesTool *)voidnt;
 
@@ -196,9 +262,7 @@ static void tedNotesToolToNoteRef(	APP_WIDGET	w,
     return;
     }
 
-static void tedNotesToolToNote(		APP_WIDGET	w,
-					void *		voidnt,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedNotesToolToNote, w, voidnt )
     {
     NotesTool *			nt= (NotesTool *)voidnt;
 
@@ -216,7 +280,7 @@ static void tedNotesToolToNote(		APP_WIDGET	w,
 static int tedNotesToolGetStartNumber(	NotePropertiesTool *	npt,
 					NotesProperties *	np )
     {
-    if  ( np->npRestart == DPftnRST_CONTINUOUS )
+    if  ( np->npRestart == FTN_RST_CONTINUOUS )
 	{
 	if  ( appGetIntegerFromTextWidget( npt->nptStartNumberText,
 				    &(np->npStartNumber), 1, 0, INT_MAX, 0 ) )
@@ -226,26 +290,107 @@ static int tedNotesToolGetStartNumber(	NotePropertiesTool *	npt,
     return 0;
     }
 
-static void tedChangeNotesPushed(	APP_WIDGET	w,
-					void *		voidnt,
-					void *		voidpbcs )
+static APP_TXACTIVATE_CALLBACK_H( tedNotesFootRestartChanged, w, voidnt )
+    {
+    NotesTool *			nt= (NotesTool *)voidnt;
+    DocumentProperties *	dp= &(nt->ntPropertiesChosen);
+    NotePropertiesTool *	npt= &(nt->ntFootnotePropertiesTool);
+    NotesProperties *		np= &(dp->dpFootnoteProperties);
+
+    if  ( np->npRestart != FTN_RST_CONTINUOUS		||
+	  tedNotesToolGetStartNumber( npt, np )		)
+	{ return;	}
+
+    if  ( ! nt->ntFixedTextChosen		&&
+	  nt->ntNoteKindChosen == DOCinFOOTNOTE	)
+	{
+	if  ( docFormatChftnField( nt->ntNoteTextChosen,
+					sizeof(nt->ntNoteTextChosen)- 1,
+					&(nt->ntPropertiesChosen),
+					nt->ntNoteNumber,
+					nt->ntNoteKindChosen ) )
+	    { LDEB(1); nt->ntNoteTextChosen[0]= '\0';	}
+	else{
+	    appStringToTextWidget( nt->ntFixedTextText,
+					    (char *)nt->ntNoteTextChosen );
+	    }
+	}
+
+    return;
+    }
+
+static APP_TXACTIVATE_CALLBACK_H( tedNotesEndRestartChanged, w, voidnt )
+    {
+    NotesTool *			nt= (NotesTool *)voidnt;
+    DocumentProperties *	dp= &(nt->ntPropertiesChosen);
+    NotePropertiesTool *	npt= &(nt->ntEndnotePropertiesTool);
+    NotesProperties *		np= &(dp->dpEndnoteProperties);
+
+    if  ( np->npRestart != FTN_RST_CONTINUOUS		||
+	  tedNotesToolGetStartNumber( npt, np )		)
+	{ return;	}
+
+    if  ( ! nt->ntFixedTextChosen		&&
+	  nt->ntNoteKindChosen == DOCinENDNOTE	)
+	{
+	if  ( docFormatChftnField( nt->ntNoteTextChosen,
+					sizeof(nt->ntNoteTextChosen)- 1,
+					&(nt->ntPropertiesChosen),
+					nt->ntNoteNumber,
+					nt->ntNoteKindChosen ) )
+	    { LDEB(1); nt->ntNoteTextChosen[0]= '\0';	}
+	else{
+	    appStringToTextWidget( nt->ntFixedTextText,
+					    (char *)nt->ntNoteTextChosen );
+	    }
+	}
+
+    return;
+    }
+
+static APP_BUTTON_CALLBACK_H( tedChangeNotesPushed, w, voidnt )
     {
     NotesTool *			nt= (NotesTool *)voidnt;
     DocumentProperties *	dp= &(nt->ntPropertiesChosen);
 
-    PropertyMask		updMask;
+    PropertyMask		dpUpdMask;
+    PropertyMask		spUpdMask;
 
-    PROPmaskCLEAR( &updMask );
+    SectionProperties		sp;
 
-    PROPmaskADD( &updMask, DPpropFOOTNOTE_POSITION );
-    PROPmaskADD( &updMask, DPpropFOOTNOTE_RESTART );
-    PROPmaskADD( &updMask, DPpropFOOTNOTE_STYLE );
-    PROPmaskADD( &updMask, DPpropFOOTNOTE_STARTNR );
+    docInitSectionProperties( &sp );
 
-    PROPmaskADD( &updMask, DPpropENDNOTE_POSITION );
-    PROPmaskADD( &updMask, DPpropENDNOTE_RESTART );
-    PROPmaskADD( &updMask, DPpropENDNOTE_STYLE );
-    PROPmaskADD( &updMask, DPpropENDNOTE_STARTNR );
+    PROPmaskCLEAR( &dpUpdMask );
+
+    PROPmaskADD( &dpUpdMask, DPpropFOOTNOTE_POSITION );
+    PROPmaskADD( &dpUpdMask, DPpropFOOTNOTE_RESTART );
+    PROPmaskADD( &dpUpdMask, DPpropFOOTNOTE_STYLE );
+    PROPmaskADD( &dpUpdMask, DPpropFOOTNOTE_STARTNR );
+
+    PROPmaskADD( &dpUpdMask, DPpropENDNOTE_POSITION );
+    PROPmaskADD( &dpUpdMask, DPpropENDNOTE_RESTART );
+    PROPmaskADD( &dpUpdMask, DPpropENDNOTE_STYLE );
+    PROPmaskADD( &dpUpdMask, DPpropENDNOTE_STARTNR );
+
+    PROPmaskCLEAR( &spUpdMask );
+
+    PROPmaskADD( &spUpdMask, SPpropFOOTNOTE_POSITION );
+    PROPmaskADD( &spUpdMask, SPpropFOOTNOTE_RESTART );
+    PROPmaskADD( &spUpdMask, SPpropFOOTNOTE_STYLE );
+    PROPmaskADD( &spUpdMask, SPpropFOOTNOTE_STARTNR );
+
+    /* No! PROPmaskADD( &spUpdMask, SPpropENDNOTE_POSITION ); */
+    PROPmaskADD( &spUpdMask, SPpropENDNOTE_RESTART );
+    PROPmaskADD( &spUpdMask, SPpropENDNOTE_STYLE );
+    PROPmaskADD( &spUpdMask, SPpropENDNOTE_STARTNR );
+
+    /**/
+
+    if  ( tedAppChangeAllSectionProperties( nt->ntApplication,
+							&spUpdMask, &sp ) )
+	{ LDEB(1);	}
+
+    /**/
 
     tedNotesToolGetStartNumber( &(nt->ntFootnotePropertiesTool),
 						&(dp->dpFootnoteProperties) );
@@ -253,14 +398,16 @@ static void tedChangeNotesPushed(	APP_WIDGET	w,
     tedNotesToolGetStartNumber( &(nt->ntEndnotePropertiesTool),
 						&(dp->dpEndnoteProperties) );
 
-    tedSetDocumentProperties( nt->ntApplication, dp, &updMask );
+    tedAppSetDocumentProperties( nt->ntApplication, dp, &dpUpdMask );
+
+    /**/
+
+    docCleanSectionProperties( &sp );
 
     return;
     }
 
-static void tedRevertNotesPushed(	APP_WIDGET	w,
-					void *		voidnt,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedRevertNotesPushed, w, voidnt )
     {
     NotesTool *		nt= (NotesTool *)voidnt;
 
@@ -296,24 +443,31 @@ static void tedRevertNotesPushed(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedChangeNotePushed(	APP_WIDGET	w,
-					void *		voidnt,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedChangeNotePushed, w, voidnt )
     {
     NotesTool *			nt= (NotesTool *)voidnt;
 
-    tedChangeCurrentNote( nt->ntApplication, nt->ntNoteKindChosen );
+    char *			fixedText= (char *)0;
+
+    fixedText= appGetStringFromTextWidget( nt->ntFixedTextText );
+
+    tedChangeCurrentNote( nt->ntApplication, ! nt->ntFixedTextChosen,
+						(unsigned char *)fixedText,
+						nt->ntNoteKindChosen );
+
+    appFreeStringFromTextWidget( fixedText );
 
     return;
     }
 
-static void tedRevertNotePushed(	APP_WIDGET	w,
-					void *		voidnt,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedRevertNotePushed, w, voidnt )
     {
     NotesTool *		nt= (NotesTool *)voidnt;
 
     nt->ntNoteKindChosen= nt->ntNoteKindSet;
+    nt->ntFixedTextChosen= nt->ntFixedTextSet;
+
+    strcpy( (char *)nt->ntNoteTextChosen, (char *)nt->ntNoteTextSet );
 
     tedNotesToolRefreshNoteWidgets( nt );
 
@@ -327,9 +481,7 @@ static void tedRevertNotePushed(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedNoteKindToggled(		APP_WIDGET	w,
-					void *		voidnt,
-					void *		voidtbcs )
+static APP_TOGGLE_CALLBACK_H( tedFootnoteToggled, w, voidnt, voidtbcs )
     {
     NotesTool *		nt= (NotesTool *)voidnt;
 
@@ -337,27 +489,80 @@ static void tedNoteKindToggled(		APP_WIDGET	w,
 
     set= appGuiGetToggleStateFromCallback( w, voidtbcs );
 
-    if  ( w == nt->ntFootnoteToggle )
+    if  ( set && nt->ntNoteKindChosen != DOCinFOOTNOTE )
 	{
-	if  ( set )
-	    { nt->ntNoteKindChosen= DOCinFOOTNOTE;	}
+	nt->ntNoteKindChosen= DOCinFOOTNOTE;
+
+	if  ( ! nt->ntFixedTextChosen )
+	    {
+	    if  ( docFormatChftnField( nt->ntNoteTextChosen,
+					    sizeof(nt->ntNoteTextChosen)- 1,
+					    &(nt->ntPropertiesChosen),
+					    nt->ntNoteNumber,
+					    nt->ntNoteKindChosen ) )
+		{ LDEB(1); nt->ntNoteTextChosen[0]= '\0';	}
+	    }
 
 	tedNotesToolRefreshNoteWidgets( nt );
-
-	return;
 	}
 
-    if  ( w == nt->ntEndnoteToggle )
+    return;
+    }
+
+static APP_TOGGLE_CALLBACK_H( tedEndnoteToggled, w, voidnt, voidtbcs )
+    {
+    NotesTool *		nt= (NotesTool *)voidnt;
+
+    int			set;
+
+    set= appGuiGetToggleStateFromCallback( w, voidtbcs );
+
+    if  ( set && nt->ntNoteKindChosen != DOCinENDNOTE )
 	{
-	if  ( set )
-	    { nt->ntNoteKindChosen= DOCinENDNOTE;	}
+	nt->ntNoteKindChosen= DOCinENDNOTE;
+	
+	if  ( ! nt->ntFixedTextChosen )
+	    {
+	    if  ( docFormatChftnField( nt->ntNoteTextChosen,
+					    sizeof(nt->ntNoteTextChosen)- 1,
+					    &(nt->ntPropertiesChosen),
+					    nt->ntNoteNumber,
+					    nt->ntNoteKindChosen ) )
+		{ LDEB(1); nt->ntNoteTextChosen[0]= '\0';	}
+	    }
 
 	tedNotesToolRefreshNoteWidgets( nt );
-
-	return;
 	}
 
-    XDEB(w); return;
+    return;
+    }
+
+static APP_TOGGLE_CALLBACK_H( tedFixedNoteTextToggled, w, voidnt, voidtbcs )
+    {
+    NotesTool *		nt= (NotesTool *)voidnt;
+
+    int			set;
+
+    set= appGuiGetToggleStateFromCallback( w, voidtbcs );
+
+    if  ( nt->ntFixedTextChosen != set )
+	{
+	if  ( ! set )
+	    {
+	    if  ( docFormatChftnField( nt->ntNoteTextChosen,
+					sizeof(nt->ntNoteTextChosen)- 1,
+					&(nt->ntPropertiesChosen),
+					nt->ntNoteNumber,
+					nt->ntNoteKindChosen ) )
+		{ LDEB(1); nt->ntNoteTextChosen[0]= '\0';	}
+	    }
+
+	nt->ntFixedTextChosen= set;
+	}
+
+    tedNotesToolRefreshNoteWidgets( nt );
+
+    return;
     }
 
 /************************************************************************/
@@ -369,8 +574,10 @@ static void tedNoteKindToggled(		APP_WIDGET	w,
 static void tedNotesMakeNotePropertiesTool(
 				APP_WIDGET			pageWidget,
 				const char *			title,
+				NotesTool *			nt,
 				NotePropertiesTool *		npt,
-				const NotesPageResources *	npr )
+				const NotesPageResources *	npr,
+				APP_TXACTIVATE_CALLBACK_T	callBack )
     {
     APP_WIDGET	rowLabel;
     APP_WIDGET	row= (APP_WIDGET )0;
@@ -393,6 +600,12 @@ static void tedNotesMakeNotePropertiesTool(
 			npt->nptPaned, npr->nprFirstNumberText,
 			textColumns, 1 );
 
+    if  ( callBack )
+	{
+	appGuiSetGotValueCallbackForText( npt->nptStartNumberText,
+						    callBack, (void *)nt );
+	}
+
     return;
     }
 
@@ -404,6 +617,8 @@ void tedFormatFillNotesPage(	NotesTool *			nt,
     {
     APP_WIDGET	row= (APP_WIDGET )0;
 
+    const int	textColumns= 6;
+
     /**/
     nt->ntPageResources= npr;
 
@@ -419,8 +634,14 @@ void tedFormatFillNotesPage(	NotesTool *			nt,
     appInspectorMakeToggleRow( &row, nt->ntCurrentNotePaned,
 		    &(nt->ntFootnoteToggle), &(nt->ntEndnoteToggle),
 		    npr->nprFootnoteText, npr->nprEndnoteText,
-		    tedNoteKindToggled,
-		    tedNoteKindToggled, (void *)nt );
+		    tedFootnoteToggled,
+		    tedEndnoteToggled, (void *)nt );
+
+    appMakeToggleAndTextRow( &row, &(nt->ntFixedTextToggle),
+		    &(nt->ntFixedTextText), nt->ntCurrentNotePaned,
+		    npr->nprFixedTextText,
+		    tedFixedNoteTextToggled, (void *)nt,
+		    textColumns, 1 );
 
     appInspectorMakeButtonRow( &row, nt->ntCurrentNotePaned,
 		&(nt->ntToNoteRefButton), &(nt->ntToNoteButton),
@@ -436,10 +657,12 @@ void tedFormatFillNotesPage(	NotesTool *			nt,
 
     /**************/
 
-    tedNotesMakeNotePropertiesTool( pageWidget, npr->nprFootnotesText,
-					&(nt->ntFootnotePropertiesTool), npr );
-    tedNotesMakeNotePropertiesTool( pageWidget, npr->nprEndnotesText,
-					&(nt->ntEndnotePropertiesTool), npr );
+    tedNotesMakeNotePropertiesTool( pageWidget, npr->nprFootnotesText, nt,
+					&(nt->ntFootnotePropertiesTool),
+					npr, tedNotesFootRestartChanged );
+    tedNotesMakeNotePropertiesTool( pageWidget, npr->nprEndnotesText, nt,
+					&(nt->ntEndnotePropertiesTool),
+					npr, tedNotesEndRestartChanged );
 
     /**************/
 
@@ -458,214 +681,236 @@ void tedFormatFillNotesPage(	NotesTool *			nt,
 /*									*/
 /************************************************************************/
 
-static void tedNotesStyleChosen(	APP_WIDGET		w,
-					NotePropertiesTool *	npt,
-					NotesProperties *	np )
+static APP_OITEM_CALLBACK_H( tedFootnoteStyleChosen, w, voidnt )
     {
+    NotesTool *			nt= (NotesTool *)voidnt;
+    DocumentProperties *	dp= &(nt->ntPropertiesChosen);
+
+    NotePropertiesTool *	npt= &(nt->ntFootnotePropertiesTool);
+    NotesProperties *		np= &(dp->dpFootnoteProperties);
+
     int				n;
 
-    for ( n= 0; n < DPftn_NCOUNT; n++ )
-	{
-	if  ( w == npt->nptStyleOptions[n] )
-	    { break;	}
-	}
-
-    if  ( n < 0 || n >= DPftn_NCOUNT )
+    n= appGuiGetOptionmenuItemIndex( &(npt->nptStyleOptionmenu), w );
+    if  ( n < 0 || n >= FTNstyle_COUNT )
 	{ LDEB(n); return;	}
 
     np->npNumberStyle= n;
 
-    return;
-    }
-
-static void tedFootnoteStyleChosen(	APP_WIDGET	w,
-					void *		voidnt,
-					void *		voidpbcs )
-    {
-    NotesTool *			nt= (NotesTool *)voidnt;
-    DocumentProperties *	dp= &(nt->ntPropertiesChosen);
-
-    tedNotesStyleChosen( w, &(nt->ntFootnotePropertiesTool),
-					&(dp->dpFootnoteProperties) );
-    return;
-    }
-
-static void tedEndnoteStyleChosen(	APP_WIDGET	w,
-					void *		voidnt,
-					void *		voidpbcs )
-    {
-    NotesTool *			nt= (NotesTool *)voidnt;
-    DocumentProperties *	dp= &(nt->ntPropertiesChosen);
-
-    tedNotesStyleChosen( w, &(nt->ntEndnotePropertiesTool),
-					&(dp->dpEndnoteProperties) );
-    return;
-    }
-
-static void tedNotesRestartChosen(	APP_WIDGET		w,
-					NotePropertiesTool *	npt,
-					NotesProperties *	np )
-    {
-    int				n;
-
-    for ( n= 0; n < DPftnRST__COUNT; n++ )
+    if  ( ! nt->ntFixedTextChosen		&&
+	  nt->ntNoteKindChosen == DOCinFOOTNOTE	)
 	{
-	if  ( w == npt->nptRestartOptions[n] )
-	    { break;	}
+	if  ( docFormatChftnField( nt->ntNoteTextChosen,
+					sizeof(nt->ntNoteTextChosen)- 1,
+					&(nt->ntPropertiesChosen),
+					nt->ntNoteNumber,
+					nt->ntNoteKindChosen ) )
+	    { LDEB(1); nt->ntNoteTextChosen[0]= '\0';	}
+	else{
+	    appStringToTextWidget( nt->ntFixedTextText,
+					    (char *)nt->ntNoteTextChosen );
+	    }
 	}
 
-    if  ( n < 0 || n >= DPftnRST__COUNT )
+    return;
+    }
+
+static APP_OITEM_CALLBACK_H( tedEndnoteStyleChosen, w, voidnt )
+    {
+    NotesTool *			nt= (NotesTool *)voidnt;
+    DocumentProperties *	dp= &(nt->ntPropertiesChosen);
+
+    NotePropertiesTool *	npt= &(nt->ntEndnotePropertiesTool);
+    NotesProperties *		np= &(dp->dpEndnoteProperties);
+
+    int				n;
+
+    n= appGuiGetOptionmenuItemIndex( &(npt->nptStyleOptionmenu), w );
+    if  ( n < 0 || n >= FTNstyle_COUNT )
 	{ LDEB(n); return;	}
 
-    np->npRestart= n;
+    np->npNumberStyle= n;
+
+    if  ( ! nt->ntFixedTextChosen		&&
+	  nt->ntNoteKindChosen == DOCinENDNOTE	)
+	{
+	if  ( docFormatChftnField( nt->ntNoteTextChosen,
+					sizeof(nt->ntNoteTextChosen)- 1,
+					&(nt->ntPropertiesChosen),
+					nt->ntNoteNumber,
+					nt->ntNoteKindChosen ) )
+	    { LDEB(1); nt->ntNoteTextChosen[0]= '\0';	}
+	else{
+	    appStringToTextWidget( nt->ntFixedTextText,
+					    (char *)nt->ntNoteTextChosen );
+	    }
+	}
+
+    return;
+    }
+
+static APP_OITEM_CALLBACK_H( tedFootnoteRestartChosen, w, voidnt )
+    {
+    NotesTool *			nt= (NotesTool *)voidnt;
+    DocumentProperties *	dp= &(nt->ntPropertiesChosen);
+
+    NotePropertiesTool *	npt= &(nt->ntFootnotePropertiesTool);
+    NotesProperties *		np= &(dp->dpFootnoteProperties);
+
+    int				n;
+
+    n= appGuiGetOptionmenuItemIndex( &(npt->nptRestartOptionmenu), w );
+    if  ( n < 0 || n >= sizeof(TED_FootnoteRestarts)/sizeof(int) )
+	{ LLDEB(n,sizeof(TED_FootnoteRestarts)/sizeof(int)); return;	}
+
+    np->npRestart= TED_FootnoteRestarts[n];
 
     tedNotesRefreshNotesStartNumber( npt, np );
 
+    if  ( ! nt->ntFixedTextChosen		&&
+	  nt->ntNoteKindChosen == DOCinFOOTNOTE	)
+	{
+	if  ( docFormatChftnField( nt->ntNoteTextChosen,
+					sizeof(nt->ntNoteTextChosen)- 1,
+					&(nt->ntPropertiesChosen),
+					nt->ntNoteNumber,
+					nt->ntNoteKindChosen ) )
+	    { LDEB(1); nt->ntNoteTextChosen[0]= '\0';	}
+	else{
+	    appStringToTextWidget( nt->ntFixedTextText,
+					    (char *)nt->ntNoteTextChosen );
+	    }
+	}
     return;
     }
 
-static void tedFootnoteRestartChosen(	APP_WIDGET	w,
-					void *		voidnt,
-					void *		voidpbcs )
+static APP_OITEM_CALLBACK_H( tedEndnoteRestartChosen, w, voidnt )
     {
     NotesTool *			nt= (NotesTool *)voidnt;
     DocumentProperties *	dp= &(nt->ntPropertiesChosen);
 
-    tedNotesRestartChosen( w, &(nt->ntFootnotePropertiesTool),
-						&(dp->dpFootnoteProperties) );
-    return;
-    }
+    NotePropertiesTool *	npt= &(nt->ntEndnotePropertiesTool);
+    NotesProperties *		np= &(dp->dpEndnoteProperties);
 
-static void tedEndnoteRestartChosen(	APP_WIDGET	w,
-					void *		voidnt,
-					void *		voidpbcs )
-    {
-    NotesTool *			nt= (NotesTool *)voidnt;
-    DocumentProperties *	dp= &(nt->ntPropertiesChosen);
-
-    tedNotesRestartChosen( w, &(nt->ntEndnotePropertiesTool),
-						&(dp->dpEndnoteProperties) );
-    return;
-    }
-
-static void tedNotesPositionChosen(	APP_WIDGET		w,
-					NotePropertiesTool *	npt,
-					NotesProperties *	np )
-    {
     int				n;
 
-    for ( n= 0; n < DPftnPOS__COUNT; n++ )
+    n= appGuiGetOptionmenuItemIndex( &(npt->nptRestartOptionmenu), w );
+    if  ( n < 0 || n >= sizeof(TED_EndnoteRestarts)/sizeof(int) )
+	{ LLDEB(n,sizeof(TED_EndnoteRestarts)/sizeof(int)); return;	}
+
+    np->npRestart= TED_EndnoteRestarts[n];
+
+    tedNotesRefreshNotesStartNumber( npt, np );
+
+    if  ( ! nt->ntFixedTextChosen		&&
+	  nt->ntNoteKindChosen == DOCinENDNOTE	)
 	{
-	if  ( w == npt->nptPositionOptions[n] )
-	    { break;	}
+	if  ( docFormatChftnField( nt->ntNoteTextChosen,
+					sizeof(nt->ntNoteTextChosen)- 1,
+					&(nt->ntPropertiesChosen),
+					nt->ntNoteNumber,
+					nt->ntNoteKindChosen ) )
+	    { LDEB(1); nt->ntNoteTextChosen[0]= '\0';	}
+	else{
+	    appStringToTextWidget( nt->ntFixedTextText,
+					    (char *)nt->ntNoteTextChosen );
+	    }
 	}
 
-    if  ( n < 0 || n >= DPftnPOS__COUNT )
+    return;
+    }
+
+static APP_OITEM_CALLBACK_H( tedFootnotePositionChosen, w, voidnt )
+    {
+    NotesTool *			nt= (NotesTool *)voidnt;
+    DocumentProperties *	dp= &(nt->ntPropertiesChosen);
+
+    NotePropertiesTool *	npt= &(nt->ntFootnotePropertiesTool);
+    NotesProperties *		np= &(dp->dpFootnoteProperties);
+
+    int				n;
+
+    n= appGuiGetOptionmenuItemIndex( &(npt->nptPositionOptionmenu), w );
+    if  ( n < 0 || n >= sizeof(TED_FootnotePositions)/sizeof(int) )
+	{ LLDEB(n,sizeof(TED_FootnotePositions)/sizeof(int)); return;	}
+
+    np->npPosition= TED_FootnotePositions[n];
+
+    return;
+    }
+
+static APP_OITEM_CALLBACK_H( tedEndnotePositionChosen, w, voidnt )
+    {
+    NotesTool *			nt= (NotesTool *)voidnt;
+    DocumentProperties *	dp= &(nt->ntPropertiesChosen);
+
+    NotePropertiesTool *	npt= &(nt->ntEndnotePropertiesTool);
+    NotesProperties *		np= &(dp->dpEndnoteProperties);
+
+    int				n;
+
+    n= appGuiGetOptionmenuItemIndex( &(npt->nptPositionOptionmenu), w );
+    if  ( n < 0 || n >= sizeof(TED_EndnotePositions)/sizeof(int) )
+	{ LLDEB(n,sizeof(TED_EndnotePositions)/sizeof(int)); return;	}
+
+    np->npPosition= TED_EndnotePositions[n];
+    if  ( n < 0 || n >= FTN_POS__COUNT )
 	{ LDEB(n); return;	}
 
-    np->npPosition= n;
-
-    return;
-    }
-
-static void tedFootnotePositionChosen(	APP_WIDGET	w,
-					void *		voidnt,
-					void *		voidpbcs )
-    {
-    NotesTool *			nt= (NotesTool *)voidnt;
-    DocumentProperties *	dp= &(nt->ntPropertiesChosen);
-
-    tedNotesPositionChosen( w, &(nt->ntFootnotePropertiesTool),
-						&(dp->dpFootnoteProperties) );
-    return;
-    }
-
-static void tedEndnotePositionChosen(	APP_WIDGET	w,
-					void *		voidnt,
-					void *		voidpbcs )
-    {
-    NotesTool *			nt= (NotesTool *)voidnt;
-    DocumentProperties *	dp= &(nt->ntPropertiesChosen);
-
-    tedNotesPositionChosen( w, &(nt->ntEndnotePropertiesTool),
-						&(dp->dpEndnoteProperties) );
     return;
     }
 
 static void tedNotePropertiesToolFillStyleChooser(
 				NotesTool *			nt,
-				APP_BUTTON_CALLBACK		callBack,
+				APP_OITEM_CALLBACK_T		callBack,
 				NotePropertiesTool *		npt,
 				const NotesPageResources *	npr )
     {
     int		i;
 
-    appFillInspectorMenu( DPftn_NCOUNT, DPftnNAR,
+    appFillInspectorMenu( FTNstyle_COUNT, FTNstyleNAR,
 		    npt->nptStyleOptions, npr->nprNumberStyleMenuTexts,
 		    &(npt->nptStyleOptionmenu),
 		    callBack, (void *)nt );
 
-    for ( i= 0; i < DPftnPOS__COUNT; i++ )
+    for ( i= 0; i < FTN_POS__COUNT; i++ )
 	{ npt->nptPositionOptions[i]= (APP_WIDGET)0;	}
 
-    for ( i= 0; i < DPftnRST__COUNT; i++ )
+    for ( i= 0; i < FTN_RST__COUNT; i++ )
 	{ npt->nptRestartOptions[i]= (APP_WIDGET)0;	}
 
     return;
     }
 
-static void tedFootnotePropertiesToolFillChoosers(
-				NotesTool *			nt,
-				NotePropertiesTool *		npt,
-				const NotesPageResources *	npr )
+static void tedNotePropertiesToolFillChoosers(
+			NotesTool *			nt,
+			NotePropertiesTool *		npt,
+			const NotesPageResources *	npr,
+			const int *			restarts,
+			int				restartCount,
+			APP_OITEM_CALLBACK_T		restartChosen,
+			const int *			positions,
+			int				positionCount,
+			APP_OITEM_CALLBACK_T		positionChosen )
     {
-    npt->nptRestartOptions[DPftnRST_CONTINUOUS]=
-		appAddItemToOptionmenu( &(npt->nptRestartOptionmenu),
-				npr->nprRestartMenuTexts[DPftnRST_CONTINUOUS],
-				tedFootnoteRestartChosen, (void *)nt );
-    npt->nptRestartOptions[DPftnRST_PER_SECTION]=
-		appAddItemToOptionmenu( &(npt->nptRestartOptionmenu),
-				npr->nprRestartMenuTexts[DPftnRST_PER_SECTION],
-				tedFootnoteRestartChosen, (void *)nt );
-    npt->nptRestartOptions[DPftnRST_PER_PAGE]=
-		appAddItemToOptionmenu( &(npt->nptRestartOptionmenu),
-				npr->nprRestartMenuTexts[DPftnRST_PER_PAGE],
-				tedFootnoteRestartChosen, (void *)nt );
+    int		i;
 
-    npt->nptPositionOptions[DPftnPOS_BELOW_TEXT]=
-		appAddItemToOptionmenu( &(npt->nptPositionOptionmenu),
-				npr->nprPositionMenuTexts[DPftnPOS_BELOW_TEXT],
-				tedFootnotePositionChosen, (void *)nt );
-    npt->nptPositionOptions[DPftnPOS_PAGE_BOTTOM]=
-		appAddItemToOptionmenu( &(npt->nptPositionOptionmenu),
-				npr->nprPositionMenuTexts[DPftnPOS_PAGE_BOTTOM],
-				tedFootnotePositionChosen, (void *)nt );
-
-    return;
-    }
-
-static void tedEndnotePropertiesToolFillChoosers(
-				NotesTool *			nt,
-				NotePropertiesTool *		npt,
-				const NotesPageResources *	npr )
-    {
-    npt->nptRestartOptions[DPftnRST_CONTINUOUS]=
+    for ( i= 0; i < restartCount; i++ )
+	{
+	npt->nptRestartOptions[i]=
 		appAddItemToOptionmenu( &(npt->nptRestartOptionmenu),
-				npr->nprRestartMenuTexts[DPftnRST_CONTINUOUS],
-				tedEndnoteRestartChosen, (void *)nt );
-    npt->nptRestartOptions[DPftnRST_PER_SECTION]=
-		appAddItemToOptionmenu( &(npt->nptRestartOptionmenu),
-				npr->nprRestartMenuTexts[DPftnRST_PER_SECTION],
-				tedEndnoteRestartChosen, (void *)nt );
+			npr->nprRestartMenuTexts[restarts[i]],
+			restartChosen, (void *)nt );
+	}
 
-    npt->nptPositionOptions[DPftnPOS_SECT_END]=
+    for ( i= 0; i < positionCount; i++ )
+	{
+	npt->nptPositionOptions[i]=
 		appAddItemToOptionmenu( &(npt->nptPositionOptionmenu),
-				npr->nprPositionMenuTexts[DPftnPOS_SECT_END],
-				tedEndnotePositionChosen, (void *)nt );
-    npt->nptPositionOptions[DPftnPOS_DOC_END]=
-		appAddItemToOptionmenu( &(npt->nptPositionOptionmenu),
-				npr->nprPositionMenuTexts[DPftnPOS_DOC_END],
-				tedEndnotePositionChosen, (void *)nt );
+			npr->nprPositionMenuTexts[positions[i]],
+			positionChosen, (void *)nt );
+	}
+
     return;
     }
 
@@ -678,11 +923,27 @@ void tedFormatFillNotesChoosers(	NotesTool *			nt,
     tedNotePropertiesToolFillStyleChooser( nt, tedEndnoteStyleChosen,
 				    &(nt->ntEndnotePropertiesTool), npr );
 
-    tedFootnotePropertiesToolFillChoosers( nt,
-				    &(nt->ntFootnotePropertiesTool), npr );
+    tedNotePropertiesToolFillChoosers( nt,
+				&(nt->ntFootnotePropertiesTool), npr,
 
-    tedEndnotePropertiesToolFillChoosers( nt,
-				    &(nt->ntEndnotePropertiesTool), npr );
+				TED_FootnoteRestarts,
+				sizeof(TED_FootnoteRestarts)/sizeof(int),
+				tedFootnoteRestartChosen,
+
+				TED_FootnotePositions,
+				sizeof(TED_FootnotePositions)/sizeof(int),
+				tedFootnotePositionChosen );
+
+    tedNotePropertiesToolFillChoosers( nt,
+				&(nt->ntEndnotePropertiesTool), npr,
+
+				TED_EndnoteRestarts,
+				sizeof(TED_EndnoteRestarts)/sizeof(int),
+				tedEndnoteRestartChosen,
+
+				TED_EndnotePositions,
+				sizeof(TED_EndnotePositions)/sizeof(int),
+				tedEndnotePositionChosen );
 
     }
 
@@ -704,6 +965,20 @@ void tedFormatFinishNotesPage(		NotesTool *			nt,
     tedFinishNotePropertiesTool( &(nt->ntEndnotePropertiesTool) );
     }
 
+/************************************************************************/
+/*									*/
+/*  Clean the notes tool.						*/
+/*									*/
+/************************************************************************/
+
+void tedFormatCleanNotesTool(	NotesTool *	nt )
+    {
+    docCleanDocumentProperties( &(nt->ntPropertiesSet) );
+    docCleanDocumentProperties( &(nt->ntPropertiesChosen) );
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Get notes tool resources.						*/
@@ -739,10 +1014,14 @@ static AppConfigurableResource TED_TedNotesToolResourceTable[]=
     APP_RESOURCE( "formatToolNoteIsFootnote",
 		offsetof(NotesPageResources,nprFootnoteText),
 		"Footnote" ),
-    APP_RESOURCE( "formatToolNoteIsFootnote",
+    APP_RESOURCE( "formatToolNoteIsEndnote",
 		offsetof(NotesPageResources,nprEndnoteText),
 		"Endnote" ),
 
+    APP_RESOURCE( "formatToolNoteHasFixedText",
+		offsetof(NotesPageResources,nprFixedTextText),
+		"Fixed Text" ),
+
     APP_RESOURCE( "formatToolToNoteRef",
 		offsetof(NotesPageResources,nprToNoteRefText),
 		"Find Note" ),
@@ -772,45 +1051,45 @@ static AppConfigurableResource TED_TedNotesToolResourceTable[]=
 		"Number Format" ),
     /**/
     APP_RESOURCE( "formatToolNoteNumberStyleNar",
-	    offsetof(NotesPageResources,nprNumberStyleMenuTexts[DPftnNAR]),
+	    offsetof(NotesPageResources,nprNumberStyleMenuTexts[FTNstyleNAR]),
 	    "1, 2, 3" ),
     APP_RESOURCE( "formatToolNoteNumberStyleNalc",
-	    offsetof(NotesPageResources,nprNumberStyleMenuTexts[DPftnNALC]),
+	    offsetof(NotesPageResources,nprNumberStyleMenuTexts[FTNstyleNALC]),
 	    "a, b, c" ),
     APP_RESOURCE( "formatToolNoteNumberStyleNauc",
-	    offsetof(NotesPageResources,nprNumberStyleMenuTexts[DPftnNAUC]),
+	    offsetof(NotesPageResources,nprNumberStyleMenuTexts[FTNstyleNAUC]),
 	    "A, B, C" ),
     APP_RESOURCE( "formatToolNoteNumberStyleNrlc",
-	    offsetof(NotesPageResources,nprNumberStyleMenuTexts[DPftnNRLC]),
+	    offsetof(NotesPageResources,nprNumberStyleMenuTexts[FTNstyleNRLC]),
 	    "i, ii, iii" ),
     APP_RESOURCE( "formatToolNoteNumberStyleNruc",
-	    offsetof(NotesPageResources,nprNumberStyleMenuTexts[DPftnNRUC]),
+	    offsetof(NotesPageResources,nprNumberStyleMenuTexts[FTNstyleNRUC]),
 	    "I, II, III" ),
     APP_RESOURCE( "formatToolNoteNumberStyleNchi",
-	    offsetof(NotesPageResources,nprNumberStyleMenuTexts[DPftnNCHI]),
+	    offsetof(NotesPageResources,nprNumberStyleMenuTexts[FTNstyleNCHI]),
 	    "*, +, #, $" ),
     /**/
     APP_RESOURCE( "formatToolNoteNumberingContinuous",
-	offsetof(NotesPageResources,nprRestartMenuTexts[DPftnRST_CONTINUOUS]),
+	offsetof(NotesPageResources,nprRestartMenuTexts[FTN_RST_CONTINUOUS]),
 	"Continuous" ),
     APP_RESOURCE( "formatToolNoteNumberingPerSection",
-	offsetof(NotesPageResources,nprRestartMenuTexts[DPftnRST_PER_SECTION]),
+	offsetof(NotesPageResources,nprRestartMenuTexts[FTN_RST_PER_SECTION]),
 	"Per Section" ),
     APP_RESOURCE( "formatToolNoteNumberingPerPage",
-	offsetof(NotesPageResources,nprRestartMenuTexts[DPftnRST_PER_PAGE]),
+	offsetof(NotesPageResources,nprRestartMenuTexts[FTN_RST_PER_PAGE]),
 	"Per Page" ),
     /**/
     APP_RESOURCE( "formatToolNotePositionSectEnd",
-	offsetof(NotesPageResources,nprPositionMenuTexts[DPftnPOS_SECT_END]),
+	offsetof(NotesPageResources,nprPositionMenuTexts[FTN_POS_SECT_END]),
 	"End of Section" ),
     APP_RESOURCE( "formatToolNotePositionDocEnd",
-	offsetof(NotesPageResources,nprPositionMenuTexts[DPftnPOS_DOC_END]),
+	offsetof(NotesPageResources,nprPositionMenuTexts[FTN_POS_DOC_END]),
 	"End of Document" ),
     APP_RESOURCE( "formatToolNotePositionBelowText",
-	offsetof(NotesPageResources,nprPositionMenuTexts[DPftnPOS_BELOW_TEXT]),
+	offsetof(NotesPageResources,nprPositionMenuTexts[FTN_POS_BELOW_TEXT]),
 	"Below Text" ),
     APP_RESOURCE( "formatToolNotePositionPageBottom",
-	offsetof(NotesPageResources,nprPositionMenuTexts[DPftnPOS_PAGE_BOTTOM]),
+	offsetof(NotesPageResources,nprPositionMenuTexts[FTN_POS_PAGE_BOTTOM]),
 	"Page Bottom" ),
     };
 
