@@ -1,6 +1,6 @@
 /************************************************************************/
 /*									*/
-/*  Convert a windows metafile to a Pixmap.				*/
+/*  Open/Close objects inserted in the document.			*/
 /*									*/
 /************************************************************************/
 
@@ -15,6 +15,7 @@
 #   include	<sioEndian.h>
 #   include	<sioStdio.h>
 #   include	<sioHex.h>
+#   include	<appSystem.h>
 
 #   include	<appDraw.h>
 #   include	<appImage.h>
@@ -35,14 +36,12 @@ int tedDrawObject(		const BufferItem *	bi,
     {
     InsertedObject *	io= bi->biParaObjects+ tp->tpObjectNumber;
 
-    if  ( tp->tpPhysicalFont < 0 )
-	{ LDEB(tp->tpPhysicalFont); return -1;	}
-
     if  ( ( io->ioKind == DOCokPICTWMETAFILE	||
 	    io->ioKind == DOCokMACPICT		||
 	    io->ioKind == DOCokPICTPNGBLIP	||
 	    io->ioKind == DOCokPICTJPEGBLIP	||
-	    io->ioKind == DOCokINCLUDEPICTURE	)	&&
+	    io->ioKind == DOCokINCLUDEPICTURE	||
+	    io->ioKind == DOCokDRAWING_OBJECT	)	&&
 	  io->ioPixmap					)
 	{
 	appDrawDrawPixmap( add, io->ioPixmap,
@@ -203,11 +202,25 @@ static int tedOpenEpsfObject(		InsertedObject *	io,
     return 0;
     }
 
+static int tedOpenDrawingObject(	InsertedObject *	io,
+					AppColors *		ac,
+					AppDrawingData *	add )
+    {
+    io->ioPixmap= appMakePixmap( add, io->ioPixelsWide, io->ioPixelsHigh );
+    if  ( ! io->ioPixmap )
+	{ XDEB(io->ioPixmap); return -1;	}
+
+    if  ( tedDrawDrawingPixmap( io->ioPixmap, io, ac, add ) )
+	{ LDEB(1); return -1;	}
+
+    return 0;
+    }
+
 static int tedOpenObject(	InsertedObject *	io,
 				AppColors *		ac,
 				AppDrawingData *	add )
     {
-    if  ( io->ioTwipsWide < 10 || io->ioTwipsHigh < 10 )
+    if  ( io->ioTwipsWide < 1 || io->ioTwipsHigh < 1 )
 	{ LLDEB(io->ioTwipsWide,io->ioTwipsHigh); return -1; }
 
     switch( io->ioKind )
@@ -267,7 +280,7 @@ static int tedOpenObject(	InsertedObject *	io,
 
 	case DOCokINCLUDEPICTURE:
 
-	    if  ( io->ioResultKind == DOCokEPSF )
+	    if  ( io->ioResultKind == DOCokEPS_FILE )
 		{
 		if  ( ! io->ioPixmap					&&
 		      tedOpenEpsfObject( io,
@@ -276,7 +289,28 @@ static int tedOpenObject(	InsertedObject *	io,
 
 		return 0;
 		}
-	    else{ LLDEB(io->ioKind,io->ioResultKind); return 0;	}
+
+	    if  ( io->ioResultKind == DOCokBITMAP_FILE	&&
+		  io->ioPrivate				)
+		{
+		APP_BITMAP_IMAGE	pixmap;
+		AppBitmapImage *	abi= (AppBitmapImage *)io->ioPrivate;
+
+		if  ( appImgMakePixmap( add, &pixmap,
+			    io->ioPixelsWide, io->ioPixelsHigh, ac, abi ) )
+		    { LLDEB(io->ioKind,io->ioResultKind); return 0;	}
+
+		io->ioPixmap= pixmap;
+		return 0;
+		}
+
+	    LLDEB(io->ioKind,io->ioResultKind); return 0;
+
+	case DOCokDRAWING_OBJECT:
+	    if  ( ! io->ioPixmap			&&
+		  tedOpenDrawingObject( io, ac, add )	)
+		{ LDEB(1); return 0;	}
+	    return 0;
 
 	default:
 	    LDEB(io->ioKind); return 0;
@@ -300,13 +334,9 @@ static int tedOpenParaObjects(	BufferItem *		bi,
 	if  ( tp->tpKind != DOCkindOBJECT )
 	    { continue;	}
 
-	if  ( tp->tpPhysicalFont < 0 )
-	    { LLDEB(part,tp->tpPhysicalFont); continue;	}
-
 	io= bi->biParaObjects+ tp->tpObjectNumber;
 
-	if  ( ! io->ioPrivate			&&
-	      tedOpenObject( io, ac, add )	)
+	if  ( tedOpenObject( io, ac, add )	)
 	    { LDEB(part); continue;	}
 	}
 
@@ -394,8 +424,7 @@ void tedCloseObject(		BufferDocument *	bd,
     {
     InsertedObject *	io;
 
-    if  ( tp->tpKind != DOCkindOBJECT	||
-	  tp->tpPhysicalFont < 0	)
+    if  ( tp->tpKind != DOCkindOBJECT )
 	{ return;	}
 
     io= bi->biParaObjects+ tp->tpObjectNumber;
@@ -419,7 +448,8 @@ void tedCloseObject(		BufferDocument *	bd,
 		    io->ioPixmap= (APP_WINDOW)0;
 		    }
 
-		appCleanBitmapImage( abi ); free( abi );
+		if  ( abi )
+		    { appCleanBitmapImage( abi ); free( abi );	}
 
 		io->ioPrivate= (void *)0;
 		}
@@ -433,6 +463,29 @@ void tedCloseObject(		BufferDocument *	bd,
 		appDrawFreePixmap( add, io->ioPixmap );
 		io->ioPixmap= (APP_WINDOW)0;
 		}
+
+	    if  ( io->ioResultKind == DOCokBITMAP_FILE )
+		{
+		AppBitmapImage *	abi= (AppBitmapImage *)io->ioPrivate;
+
+		if  ( abi )
+		    { appCleanBitmapImage( abi ); free( abi );	}
+
+		io->ioPrivate= (void *)0;
+		}
+
+	    break;
+
+	case DOCokDRAWING_OBJECT:
+
+	    if  ( io->ioPixmap )
+		{
+		AppDrawingData *	add= (AppDrawingData *)voidadd;
+
+		appDrawFreePixmap( add, io->ioPixmap );
+		io->ioPixmap= (APP_WINDOW)0;
+		}
+
 	    break;
 
 	default:
@@ -452,7 +505,6 @@ int tedReopenObject(	BufferDocument *	bd,
 			BufferItem *		bi,
 			TextParticule *		tp,
 			AppColors *		ac,
-			DocumentFontList *	dfl,
 			AppDrawingData *	add )
     {
     InsertedObject *		io= bi->biParaObjects+ tp->tpObjectNumber;
@@ -471,18 +523,17 @@ static void tedScaleObjectToParagraph(	EditDocument *		ed,
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
     BufferDocument *		bd= td->tdDocument;
     ParagraphFrame		pf;
-    double			scaleX;
-    double			scaleY;
 
     DocumentSelection		ds;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
     const int			bottom= -1;
     const int			stripHigh= -1;
 
     BlockFrame			bf;
 
-    if  ( tedGetSelection( &ds, &sg, td ) )
+    if  ( tedGetSelection( &ds, &sg, &sd, td ) )
 	{ LDEB(1); return;	}
 
     docBlockFrameTwips( &bf, ds.dsBegin.dpBi, bd,
@@ -491,19 +542,7 @@ static void tedScaleObjectToParagraph(	EditDocument *		ed,
 
     docParagraphFrameTwips( &pf, &bf, bottom, stripHigh, ds.dsBegin.dpBi );
 
-    scaleX= (double)( pf.pfX1TextLinesTwips- pf.pfX0TextLinesTwips )/
-						    (double)io->ioTwipsWide;
-
-    scaleY= (double)pf.pfPageHigh/ (double)io->ioTwipsHigh;
-
-    if  ( scaleY > scaleX )
-	{ scaleY=  scaleX;	}
-
-    if  ( scaleY < 0.99 )
-	{
-	io->ioScaleX= (int)( 99* scaleY );
-	io->ioScaleY= (int)( 99* scaleY );
-	}
+    docLayoutScaleObjectToFitParagraphFrame( io, &pf );
 
     return;
     }
@@ -599,10 +638,12 @@ InsertedObject * tedObjectMakeBitmapObject(
 	io->ioKind= DOCokPICTPNGBLIP;
 	}
     else{
-	if  ( appMetaSaveBitmapMetafile( &(abi->abiBitmap), abi->abiBuffer,
-								    sosHex ) )
+	if  ( bmWmfWriteWmf( &(abi->abiBitmap), abi->abiBuffer, sosHex ) )
 	    { LDEB(1); return (InsertedObject *)0;	}
 
+	io->ioMetafileIsBitmap= 1;
+	io->ioMetafileBitmapBpp= abi->abiBitmap.bdBitsPerPixel;
+
 	io->ioKind= DOCokPICTWMETAFILE;
 	io->ioMapMode= 8;
 	}
@@ -621,6 +662,11 @@ InsertedObject * tedObjectMakeBitmapObject(
 				( io->ioScaleX* io->ioTwipsWide )/ 100 );
     io->ioPixelsHigh= TWIPStoPIXELS( xfac,
 				( io->ioScaleY* io->ioTwipsHigh )/ 100 );
+    if  ( io->ioPixelsWide < 1 )
+	{ io->ioPixelsWide=  1;	}
+    if  ( io->ioPixelsHigh < 1 )
+	{ io->ioPixelsHigh=  1;	}
+
     io->ioPrivate= abi;
 
     io->io_xWinExt= (int) ( ( 100000.0* io->ioTwipsWide )/
@@ -721,7 +767,7 @@ static InsertedObject * tedMakeMetafileObject(
 	io->io_yWinExt= top- bottom;
 	}
     else{
-	if  ( 0 && key == 0x90001 )
+	if  ( key == 0x90001 )
 	    {
 	    io->ioTwipsWide= 20* 72* 4;		/*  10 cm	*/
 	    io->ioTwipsHigh= 20* 72* 4;		/*  10 cm	*/
@@ -766,6 +812,10 @@ static InsertedObject * tedMakeMetafileObject(
 				( io->ioScaleX* io->ioTwipsWide )/ 100 );
     io->ioPixelsHigh= TWIPStoPIXELS( xfac,
 				( io->ioScaleY* io->ioTwipsHigh )/ 100 );
+    if  ( io->ioPixelsWide < 1 )
+	{ io->ioPixelsWide=  1;	}
+    if  ( io->ioPixelsHigh < 1 )
+	{ io->ioPixelsHigh=  1;	}
 
     io->io_xWinExt= (int) ( ( 100000.0* io->ioTwipsWide )/
 						    ( 20* POINTS_PER_M ) );
@@ -789,20 +839,18 @@ static int tedObjectOpenPicture(	void *		voided,
     {
     EditDocument *		ed= (EditDocument *)voided;
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
-    const char *		slash= strrchr( filename, '/' );
-    const char *		dot;
+    const char *		ext;
 
     DocumentSelection		ds;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
-    if  ( tedGetSelection( &ds, &sg, td ) )
+    if  ( tedGetSelection( &ds, &sg, &sd, td ) )
 	{ LDEB(1); return -1;	}
 
-    if  ( slash )
-	{ dot= strrchr( slash+ 1, '.' );	}
-    else{ dot= strrchr( filename, '.' );	}
+    ext= appFileExtensionOfName( filename );
 
-    if  ( ! dot || strcmp( dot, ".wmf" ) )
+    if  ( ! ext || strcmp( ext, "wmf" ) )
 	{
 	double			factor;
 	AppBitmapImage *	abi;
