@@ -60,8 +60,8 @@ extern void * tedMakeLeftRuler(	int			width,
 extern void tedFreeHorizontalRuler(	void *		voidttr );
 extern void tedFreeVerticalRuler(	void *		voidtlr );
 
-extern APP_EVENT_HANDLER( tedRedrawHorizontalRuler, w, voidttr, event );
-extern APP_EVENT_HANDLER( tedRedrawVerticalRuler, w, voidtlr, event );
+extern APP_EVENT_HANDLER_H( tedRedrawHorizontalRuler, w, voidttr, event );
+extern APP_EVENT_HANDLER_H( tedRedrawVerticalRuler, w, voidtlr, event );
 
 extern void tedScrollHorizontalRuler(	void *			voidttr,
 					APP_WIDGET		w,
@@ -73,22 +73,20 @@ extern void tedSetHorizontalRulerRange(	void *		voidttr,
 					int		docVisX1,
 					int		docBackX1 );
 
-extern void tedAdaptHorizontalRuler(	void *		voidttr,
-					APP_WIDGET	w,
-					int		docX0,
-					int		docX1,
-					int		firstIndent,
-					int		leftIndent,
-					int		rightIndent,
-					int		rulerC1,
-					int		tabCount,
-					TabStop *	tabStops );
+extern void tedAdaptHorizontalRuler(	void *			voidttr,
+					APP_WIDGET		w,
+					int			docX0,
+					int			docX1,
+					int			firstIndent,
+					int			leftIndent,
+					int			rightIndent,
+					int			rulerC1,
+					const TabStopList *	tslSet );
 
 extern void tedHorizontalRulerTrackMouse( int *			pFirstIndent,
 					int *			pLeftIndent,
 					int *			pRightIndent,
-					int *			pTabCount,
-					TabStop **		pTabStops,
+					TabStopList *		tsl,
 					int *			pCsCount,
 					ColumnSeparator **	pCs,
 					int *			pProperty,
@@ -99,8 +97,8 @@ extern void tedHorizontalRulerTrackMouse( int *			pFirstIndent,
 					void *			voided,
 					DragHair		dragHair );
 
-extern APP_EVENT_HANDLER( tedHorizontalRulerConfigure, w, voidttr, event );
-extern APP_EVENT_HANDLER( tedVerticalRulerConfigure, w, voidtlr, event );
+extern APP_EVENT_HANDLER_H( tedHorizontalRulerConfigure, w, voidttr, event );
+extern APP_EVENT_HANDLER_H( tedVerticalRulerConfigure, w, voidtlr, event );
 
 extern int tedSetRulerColumns(	APP_WIDGET		w,
 				void *			voidttr,
