@@ -10,7 +10,6 @@
 #   include	<stdio.h>
 #   include	<stddef.h>
 #   include	<limits.h>
-#   include	<appDebugon.h>
 
 #   include	<appGeoString.h>
 #   include	<appUnit.h>
@@ -18,7 +17,7 @@
 #   include	"tedApp.h"
 #   include	"tedFormatTool.h"
 
-#   define	DRH_CM		4.5
+#   include	<appDebugon.h>
 
 /************************************************************************/
 /*									*/
@@ -62,11 +61,18 @@ static void tedFormatToolRefreshSectionPage(	SectionTool *	st )
 
     appIntegerToTextWidget( st->stColumnsText, sp->spColumnCount );
 
-    appGeoLengthToString( scratch, sp->spColumnSpacingTwips, UNITtyPOINTS );
-    appStringToTextWidget( st->stColumnSpacingText, scratch );
+    if  ( sp->spColumnCount > 1 )
+	{
+	appGeoLengthToString( scratch, sp->spColumnSpacingTwips, UNITtyPOINTS );
+	appStringToTextWidget( st->stColumnSpacingText, scratch );
 
-    appGeoLengthToString( scratch, sp->spColumnWidthTwips, UNITtyPOINTS );
-    appStringToTextWidget( st->stColumnWidthText, scratch );
+	appGeoLengthToString( scratch, sp->spColumnWidthTwips, UNITtyPOINTS );
+	appStringToTextWidget( st->stColumnWidthText, scratch );
+	}
+    else{
+	appStringToTextWidget( st->stColumnSpacingText, "" );
+	appStringToTextWidget( st->stColumnWidthText, "" );
+	}
 
     return;
     }
@@ -74,6 +80,7 @@ static void tedFormatToolRefreshSectionPage(	SectionTool *	st )
 void tedFormatToolRefreshSectionTool(
 				SectionTool *			st,
 				int *				pEnabled,
+				int *				pPref,
 				InspectorSubject *		is,
 				const DocumentSelection *	ds )
     {
@@ -112,9 +119,7 @@ void tedFormatToolRefreshSectionTool(
 /*									*/
 /************************************************************************/
 
-static void tedFormatChangeSectPushed(	APP_WIDGET	w,
-					void *		voidst,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatChangeSectPushed, w, voidst )
     {
     SectionTool *		st= (SectionTool *)voidst;
     SectionProperties *		spNew= &(st->stPropertiesChosen);
@@ -147,15 +152,18 @@ static void tedFormatChangeSectPushed(	APP_WIDGET	w,
 						    1, 0, INT_MAX, 0 ) )
 	{ return;	}
 
-    if  ( appGetLengthFromTextWidget( st->stColumnSpacingText,
+    if  ( spNew->spColumnCount > 1 )
+	{
+	if  ( appGetLengthFromTextWidget( st->stColumnSpacingText,
 		    &(spNew->spColumnSpacingTwips), &changed, UNITtyPOINTS,
 		    minValue, adaptToMin, maxValue, adaptToMax ) )
-	{ return;	}
+	    { return;	}
 
-    if  ( appGetLengthFromTextWidget( st->stColumnWidthText,
+	if  ( appGetLengthFromTextWidget( st->stColumnWidthText,
 		    &(spNew->spColumnWidthTwips), &changed, UNITtyPOINTS,
 		    minValue, adaptToMin, maxValue, adaptToMax ) )
-	{ return;	}
+	    { return;	}
+	}
 
     PROPmaskCLEAR( &updMask );
     PROPmaskFILL( &updMask, SPprop_COUNT );
@@ -166,53 +174,39 @@ static void tedFormatChangeSectPushed(	APP_WIDGET	w,
     return;
     }
 
-static void tedSectNumberStyleChosen(	APP_WIDGET	w,
-					void *		voidst,
-					void *		voidtbcs )
+static APP_OITEM_CALLBACK_H( tedSectNumberStyleChosen, w, voidst )
     {
     SectionTool *			st= (SectionTool *)voidst;
     SectionProperties *			sp= &(st->stPropertiesChosen);
 
     int					style;
 
-    for ( style= 0; style < DOCpgn_COUNT; style++ )
-	{
-	if  ( st->stNumberStyleItems[style] == w )
-	    { break;	}
-	}
+    style= appGuiGetOptionmenuItemIndex( &(st->stNumberStyleOptionmenu), w );
+    if  ( style < 0 || style >= DOCpgn_COUNT )
+	{ LLDEB(style,DOCpgn_COUNT); return;	}
 
-    if  ( style >= 0 && style < DOCpgn_COUNT )
-	{ sp->spPageNumberStyle= style;	}
-    else{ LLDEB(style,DOCpgn_COUNT);	}
+    sp->spPageNumberStyle= style;
 
     return;
     }
 
-static void tedSectBreakKindChosen(	APP_WIDGET	w,
-					void *		voidst,
-					void *		voidtbcs )
+static APP_OITEM_CALLBACK_H( tedSectBreakKindChosen, w, voidst )
     {
     SectionTool *			st= (SectionTool *)voidst;
     SectionProperties *			sp= &(st->stPropertiesChosen);
 
     int					breakKind;
 
-    for ( breakKind= 0; breakKind < DOCsbk_COUNT; breakKind++ )
-	{
-	if  ( st->stBreakKindItems[breakKind] == w )
-	    { break;	}
-	}
+    breakKind= appGuiGetOptionmenuItemIndex( &(st->stBreakKindOptionmenu), w );
+    if  ( breakKind < 0 || breakKind >= DOCsbk_COUNT )
+	{ LLDEB(breakKind,DOCsbk_COUNT); return;	}
 
-    if  ( breakKind >= 0 && breakKind < DOCsbk_COUNT )
-	{ sp->spBreakKind= breakKind;		}
-    else{ LLDEB(breakKind,DOCsbk_COUNT);	}
+    sp->spBreakKind= breakKind;
 
     return;
     }
 
-static void tedFormatPageRestartToggled(	APP_WIDGET	w,
-						void *		voidst,
-						void *		voidtbcs )
+static APP_TOGGLE_CALLBACK_H( tedFormatPageRestartToggled, w, voidst, voidtbcs )
     {
     SectionTool *		st= (SectionTool *)voidst;
     SectionProperties *		sp= &(st->stPropertiesChosen);
@@ -233,9 +227,7 @@ static void tedFormatPageRestartToggled(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedSectColSpacingChanged(	APP_WIDGET	w,
-					void *		voidst,
-					void *		voidpbcs )
+static APP_TXACTIVATE_CALLBACK_H( tedSectColSpacingChanged, w, voidst )
     {
     SectionTool *		st= (SectionTool *)voidst;
     SectionProperties *		sp= &(st->stPropertiesChosen);
@@ -245,14 +237,13 @@ static void tedSectColSpacingChanged(	APP_WIDGET	w,
 
     value= sp->spColumnSpacingTwips;
 
-    tedFormatValidateDimension( &value, &changed, w, value );
+    tedFormatValidateDimension( &value, &changed,
+					    st->stColumnSpacingText, value );
 
     return;
     }
 
-static void tedSectColWidthChanged(	APP_WIDGET	w,
-					void *		voidst,
-					void *		voidpbcs )
+static APP_TXACTIVATE_CALLBACK_H( tedSectColWidthChanged, w, voidst )
     {
     SectionTool *		st= (SectionTool *)voidst;
     SectionProperties *		sp= &(st->stPropertiesChosen);
@@ -262,7 +253,8 @@ static void tedSectColWidthChanged(	APP_WIDGET	w,
 
     value= sp->spColumnWidthTwips;
 
-    tedFormatValidateDimension( &value, &changed, w, value );
+    tedFormatValidateDimension( &value, &changed,
+					    st->stColumnWidthText, value );
 
     return;
     }
@@ -273,9 +265,7 @@ static void tedSectColWidthChanged(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedFormatRevertSectPushed(	APP_WIDGET	w,
-					void *		voidst,
-					void *		voidpbcs	 )
+static APP_BUTTON_CALLBACK_H( tedFormatRevertSectPushed, w, voidst )
     {
     SectionTool *	st= (SectionTool *)voidst;
 
@@ -293,9 +283,7 @@ static void tedFormatRevertSectPushed(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedFormatPrevSect(		APP_WIDGET	w,
-					void *		voidst,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatPrevSect, w, voidst )
     {
     SectionTool *		st= (SectionTool *)voidst;
     EditApplication *		ea= st->stApplication;
@@ -305,9 +293,7 @@ static void tedFormatPrevSect(		APP_WIDGET	w,
     return;
     }
 
-static void tedFormatNextSect(		APP_WIDGET	w,
-					void *		voidst,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatNextSect, w, voidst )
     {
     SectionTool *		st= (SectionTool *)voidst;
     EditApplication *		ea= st->stApplication;
@@ -317,9 +303,7 @@ static void tedFormatNextSect(		APP_WIDGET	w,
     return;
     }
 
-static void tedFormatSelectSect(	APP_WIDGET	w,
-					void *		voidst,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatSelectSect, w, voidst )
     {
     SectionTool *		st= (SectionTool *)voidst;
     EditApplication *		ea= st->stApplication;
@@ -329,9 +313,7 @@ static void tedFormatSelectSect(	APP_WIDGET	w,
     return;
     }
 
-static void tedFormatDeleteSect(	APP_WIDGET	w,
-					void *		voidst,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatDeleteSect, w, voidst )
     {
     SectionTool *		st= (SectionTool *)voidst;
     EditApplication *		ea= st->stApplication;
@@ -339,9 +321,7 @@ static void tedFormatDeleteSect(	APP_WIDGET	w,
     tedDeleteCurrentSection( ea );
     }
 
-static void tedFormatInsertSect(	APP_WIDGET	w,
-					void *		voidst,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatInsertSect, w, voidst )
     {
     SectionTool *		st= (SectionTool *)voidst;
     EditApplication *		ea= st->stApplication;
@@ -351,9 +331,7 @@ static void tedFormatInsertSect(	APP_WIDGET	w,
     return;
     }
 
-static void tedFormatAppendSect(	APP_WIDGET	w,
-					void *		voidst,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatAppendSect, w, voidst )
     {
     SectionTool *		st= (SectionTool *)voidst;
     EditApplication *		ea= st->stApplication;
@@ -488,6 +466,20 @@ void tedFormatFinishSectionPage(	SectionTool *			st,
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Clean a section tool						*/
+/*									*/
+/************************************************************************/
+
+void tedFormatCleanSectionTool(		SectionTool *		st )
+    {
+    docCleanSectionProperties( &(st->stPropertiesSet) );
+    docCleanSectionProperties( &(st->stPropertiesChosen) );
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Get section tool resources.						*/
