@@ -33,63 +33,10 @@
 /*									*/
 /************************************************************************/
 
-static int tedInsertRows(	BufferItem *		sectBi,
-				BufferItem **		pNewBi,
-				const BufferItem *	refRowBi,
-				const RowProperties *	rp,
-				TextAttribute		ta,
-				AppDrawingData *	add,
-				int			n,
-				int			paraNr,
-				int			rows,
-				EditOperation *		eo )
-    {
-    DocumentPosition	dp;
-
-    int			row;
-    int			firstPos= n;
-
-    for ( row= 0; row < rows; n++, row++ )
-	{
-	BufferItem *		rowBi= (BufferItem *)0;
-
-	const int		stroffFrom= 0;
-	const int		sectShift= 0;
-	const int		stroffShift= 0;
-	const int		paraShift= rp->rpCellCount;
-
-	rowBi= docInsertRowItem( eo->eoBd, sectBi, n, rp, ta );
-	if  ( ! rowBi )
-	    { XDEB(rowBi); return -1;	}
-
-	if  ( sectBi->biInExternalItem == DOCinBODY )
-	    {
-	    docShiftNoteReferences( eo->eoBd, paraNr, stroffFrom,
-				    sectShift, paraShift, stroffShift );
-	    }
-
-	docEditShiftReformatRangeParaNr( eo, paraNr+ 1, paraShift );
-
-	paraNr += rp->rpCellCount;
-
-	if  ( refRowBi						&&
-	      docCopyRowColumnAttributes( rowBi, refRowBi )	)
-	    { LDEB(1); return -1;	}
-
-	docEditIncludeItemInReformatRange( eo, rowBi );
-	}
-
-    if  ( docFirstPosition( &dp, sectBi->biChildren[firstPos] ) )
-	{ LDEB(firstPos);	}
-
-    *pNewBi= dp.dpBi; return 0;
-    }
-
 int tedInsertTable(		EditDocument *		ed,
 				int			rows,
 				int			columns )
     {
-    AppDrawingData *		add= &(ed->edDrawingData);
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
     BufferDocument *		bd= td->tdDocument;
     DocumentProperties *	dp= &(bd->bdProperties);
@@ -100,7 +47,7 @@ int tedInsertTable(		EditDocument *		ed,
     int				col;
     int				paraNr;
 
-    TextAttribute		ta;
+    int				textAttributeNumber;
 
     int				wide;
 
@@ -109,22 +56,24 @@ int tedInsertTable(		EditDocument *		ed,
     EditOperation		eo;
     DocumentSelection		dsOld;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
     BufferItem *		bi;
 
     DocumentSelection		dsRep;
 
-    tedStartEditOperation( &eo, &dsOld, &sg, ed, 1 );
+    tedStartEditOperation( &eo, &dsOld, &sg, &sd, ed, 1 );
     bi= dsOld.dsBegin.dpBi;
 
     /*  0  */
-    eo.eoChangedRectangle.drY0--;
+    eo.eoChangedRect.drY0--;
 
     /*  1  */
     if  ( tedEditReplaceSelection( &eo, &dsRep, ed,
 					    (const unsigned char *)0, 0 ) )
 	{ LDEB(1); return -1;	}
 
+
     /*  2  */
     bi= dsRep.dsBegin.dpBi;
     if  ( dsRep.dsBegin.dpStroff == bi->biParaStrlen )
@@ -163,27 +112,38 @@ int tedInsertTable(		EditDocument *		ed,
 	    }
 	}
 
-    ta= bi->biParaParticules[0].tpTextAttribute;
+    textAttributeNumber= bi->biParaParticules[0].tpTextAttributeNumber;
 
     wide=   dp->dpGeometry.dgPageWideTwips;
     wide -= dp->dpGeometry.dgLeftMarginTwips;
     wide -= dp->dpGeometry.dgRightMarginTwips;
 
     docInitRowProperties( &rp );
+
+    {
+    TextAttribute	ta;
+
+    utilGetTextAttributeByNumber( &ta, &(bd->bdTextAttributeList),
+							textAttributeNumber );
+
     rp.rpHalfGapWidthTwips= 5* ta.taFontSizeHalfPoints;
     rp.rpLeftIndentTwips=  -5* ta.taFontSizeHalfPoints;
+    }
 
     for ( col= 0; col < columns; col++ )
 	{
-	CellProperties	cp;
+	CellProperties		cp;
+	const int * const	colorMap= (const int *)0;
 
 	docInitCellProperties( &cp );
 	cp.cpRightBoundaryTwips= ( ( col+ 1 )* wide )/ columns;
 
-	if  ( docInsertRowColumn( &rp, col, &cp ) )
+	if  ( docInsertRowColumn( &rp, col, &cp, colorMap ) )
 	    { LDEB(col); return -1;	}
 
 	rp.rpHasTableParagraphs= 1;
+
+	docCleanCellProperties( &cp );
 	}
 
     newBi= (BufferItem *)0;
@@ -191,72 +151,96 @@ int tedInsertTable(		EditDocument *		ed,
     sectBi= bi->biParent->biParent->biParent;
     paraNr= docNumberOfParagraph( bi );
 
-    if  ( tedInsertRows( sectBi, &newBi, refRowBi,
-			    &rp, ta, add,
+    if  ( docInsertTableRows( sectBi, &newBi, refRowBi,
+			    &rp, textAttributeNumber,
 			    bi->biParent->biParent->biNumberInParent, paraNr,
 			    rows, &eo ) )
 	{ LDEB(rows); return -1;	}
 
     docCleanRowProperties( &rp );
 
+    docSectDelimitTables( sectBi );
+
     tedEditFinishIBarSelection( ed, &eo, newBi, 0 );
 
     return 0;
     }
 
+/************************************************************************/
+/*									*/
+/*  Insert a number of rows in a table.					*/
+/*									*/
+/*  1)  Check that the section is not empty. Ted avoids empty items at	*/
+/*	any price. Every Item is supposed to contain at least one child	*/
+/*	or at least one particule.					*/
+/*  2)  Sanity check.							*/
+/*  4)  Section related administration.					*/
+/*  5)  Set an I-Bar selection of the first position in the first	*/
+/*	(empty) cell of the first fresh row.				*/
+/*									*/
+/************************************************************************/
+
 static int tedInsertRowsInTable(
-				BufferItem *			sectBi,
-				EditOperation *			eo,
-				const DocumentSelection *	ds,
-				const RowProperties *		rp,
-				EditDocument *			ed,
-				int				row,
-				int				rows )
-	    {
-    AppDrawingData *		add= &(ed->edDrawingData);
-    BufferItem *		refParaBi;
-
+			BufferItem *			sectBi,
+			EditOperation *			eo,
+			int				paraNr,
+			int				textAttributeNumber,
+			BufferItem *			refRowBi,
+			EditDocument *			ed,
+			int				row,
+			int				rows )
+    {
     BufferItem *		newBi;
-    BufferItem *		refRowBi;
-    int				paraNr;
-
-    TextAttribute		ta;
-
-    DocumentPosition		dpRef;
+    const RowProperties *	rp;
 
+    /*  1  */
     if  ( sectBi->biChildCount == 0 )
 	{ LDEB(sectBi->biChildCount); return -1;	}
 
-    if  ( row < sectBi->biChildCount )
-	{ refRowBi= sectBi->biChildren[row];				}
-    else{ refRowBi= sectBi->biChildren[sectBi->biChildCount- 1];	}
+    /*  2  */
+    if  ( refRowBi->biLevel != DOClevROW	||
+	  ! refRowBi->biRowHasTableParagraphs	)
+	{
+	SDEB(docLevelStr(refRowBi->biLevel));
+	LDEB(refRowBi->biRowHasTableParagraphs);
+	return -1;
+	}
 
-    if  ( docLastPosition(  &dpRef, refRowBi ) )
-	{ LDEB(1); return -1;	}
-
-    refParaBi= ds->dsBegin.dpBi;
-    ta= refParaBi->biParaParticules[0].tpTextAttribute;
     newBi= (BufferItem *)0;
-    refRowBi= (BufferItem *)0;
-    paraNr= docNumberOfParagraph( dpRef.dpBi )+ 1;
 
-    if  ( refParaBi->biParaInTable )
-	{ refRowBi= refParaBi->biParent->biParent;	}
-
-    if  ( tedInsertRows( sectBi, &newBi, refRowBi, rp, ta, add,
-						    row, paraNr, rows, eo ) )
+    rp= &(refRowBi->biRowProperties);
+    if  ( docInsertTableRows( sectBi, &newBi, refRowBi, rp,
+						textAttributeNumber,
+						row, paraNr, rows, eo ) )
 	{ LDEB(rows); return -1;	}
 
+    /*  4  */
+    docSectDelimitTables( sectBi );
+
+    /*  5  */
     tedEditFinishIBarSelection( ed, eo, newBi, 0 );
 
     return 0;
     }
 
+/************************************************************************/
+/*									*/
+/*  Insert a row before the current row in a table.			*/
+/*									*/
+/*  1)  Start edit operation.						*/
+/*  2)  Find the position of the beginning of the selection in the	*/
+/*	table.								*/
+/*  3)  Get the row that serves as a template.				*/
+/*  4)  Paragraph number of the paragraph that comes directly below	*/
+/*	the fresh rows.							*/
+/*  5)  Finish..							*/
+/*									*/
+/************************************************************************/
+
 int tedInsertRowInTable(	EditDocument *		ed )
     {
-    const RowProperties *	rp;
-
     BufferItem *		sectBi;
+    BufferItem *		refRowBi;
 
     int				col;
     int				row;
@@ -268,26 +252,59 @@ int tedInsertRowInTable(	EditDocument *		ed )
     EditOperation		eo;
     DocumentSelection		ds;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
-    tedStartEditOperation( &eo, &ds, &sg, ed, 1 );
+    DocumentPosition		dpRef;
+    int				part;
+    const int			lastOne= 1;
+    int				paraNr;
+    int				textAttributeNumber;
 
+    /*  1  */
+    tedStartEditOperation( &eo, &ds, &sg, &sd, ed, 1 );
+
+    /*  2  */
     if  ( docDelimitTable( ds.dsBegin.dpBi, &sectBi,
 						&col, &row0, &row, &row1 ) )
 	{ LDEB(1); return -1;	}
 
-    rp= &(sectBi->biChildren[row]->biRowProperties);
+    /*  3  */
+    refRowBi= sectBi->biChildren[row];
+    if  ( docFirstPosition( &dpRef, refRowBi ) )
+	{ LDEB(row); return -1;	}
+    if  ( docFindParticuleOfPosition( &part, &dpRef, lastOne ) )
+	{ LDEB(dpRef.dpStroff); return -1;	}
 
-    if  ( tedInsertRowsInTable( sectBi, &eo, &ds, rp, ed, row, rows ) )
+    /*  4  */
+    paraNr= docNumberOfParagraph( dpRef.dpBi );
+    textAttributeNumber= dpRef.dpBi->biParaParticules[part].
+						    tpTextAttributeNumber;
+
+    /*  5  */
+    if  ( tedInsertRowsInTable( sectBi, &eo,
+		    paraNr, textAttributeNumber, refRowBi, ed, row, rows ) )
 	{ LDEB(row); return -1;	}
 
     return 0;
     }
 
+/************************************************************************/
+/*									*/
+/*  Insert a row after the current row in a table.			*/
+/*									*/
+/*  1)  Start edit operation.						*/
+/*  2)  Find the position of the end of the selection in the table	*/
+/*  3)  Get the row that serves as a template.				*/
+/*  4)  Paragraph number of the paragraph that comes directly below	*/
+/*	the fresh rows.							*/
+/*  5)  Finish..							*/
+/*									*/
+/************************************************************************/
+
 int tedAppendRowToTable(	EditDocument *		ed )
     {
-    const RowProperties *	rp;
-
     BufferItem *		sectBi;
+    BufferItem *		refRowBi;
 
     int				col;
     int				row;
@@ -299,16 +316,37 @@ int tedAppendRowToTable(	EditDocument *		ed )
     EditOperation		eo;
     DocumentSelection		ds;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
-    tedStartEditOperation( &eo, &ds, &sg, ed, 1 );
+    DocumentPosition		dpRef;
+    int				part;
+    const int			lastOne= 0;
+    int				paraNr;
+    int				textAttributeNumber;
 
-    if  ( docDelimitTable( ds.dsBegin.dpBi, &sectBi,
+    /*  1  */
+    tedStartEditOperation( &eo, &ds, &sg, &sd, ed, 1 );
+
+    /*  2  */
+    if  ( docDelimitTable( ds.dsEnd.dpBi, &sectBi,
 						&col, &row0, &row, &row1 ) )
 	{ LDEB(1); return -1;	}
 
-    rp= &(sectBi->biChildren[row]->biRowProperties);
+    /*  3  */
+    refRowBi= sectBi->biChildren[row];
+    if  ( docLastPosition( &dpRef, refRowBi ) )
+	{ LDEB(row); return -1;	}
+    if  ( docFindParticuleOfPosition( &part, &dpRef, lastOne ) )
+	{ LDEB(dpRef.dpStroff); return -1;	}
 
-    if  ( tedInsertRowsInTable( sectBi, &eo, &ds, rp, ed, row+ 1, rows ) )
+    /*  4  */
+    paraNr= docNumberOfParagraph( dpRef.dpBi )+ 1;
+    textAttributeNumber= dpRef.dpBi->biParaParticules[part].
+							tpTextAttributeNumber;
+
+    /*  5  */
+    if  ( tedInsertRowsInTable( sectBi, &eo,
+		    paraNr, textAttributeNumber, refRowBi, ed, row+ 1, rows ) )
 	{ LDEB(row); return -1;	}
     
     return 0;
@@ -322,9 +360,12 @@ int tedAppendRowToTable(	EditDocument *		ed )
 /*									*/
 /************************************************************************/
 
-int tedDeleteRowsFromTable(	EditDocument *		ed,
-				int			delRow0,
-				int			delRow1 )
+static int tedDeleteRowsFromTableOperation(
+					EditDocument *		ed,
+					EditOperation *		eo,
+					DocumentSelection *	ds,
+					int			delRow0,
+					int			delRow1 )
     {
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
     BufferDocument *		bd= td->tdDocument;
@@ -338,20 +379,15 @@ int tedDeleteRowsFromTable(	EditDocument *		ed,
 
     DocumentPosition		dpNew;
 
-    EditOperation		eo;
-
     int				paraNr;
+    int				sectionsDeleted= 0;
+    int				firstParaDeleted= -1;
+    int				paragraphsDeleted= 0;
 
     const int			stroffFrom= 0;
-    const int			sectShift= 0;
     const int			stroffShift= 0;
 
-    DocumentSelection		ds;
-    SelectionGeometry		sg;
-
-    tedStartEditOperation( &eo, &ds, &sg, ed, 1 );
-
-    if  ( docDelimitTable( ds.dsBegin.dpBi, &selSectBi,
+    if  ( docDelimitTable( ds->dsBegin.dpBi, &selSectBi,
 						&col, &row0, &row, &row1 ) )
 	{ LDEB(1); return -1;	}
 
@@ -375,41 +411,66 @@ int tedDeleteRowsFromTable(	EditDocument *		ed,
 	docInitDocumentPosition( &dpNew );
 	if  ( docFirstPosition( &dpNew,
 				selSectBi->biChildren[delRow0] )	||
-	      docPrevPosition( &dpNew, /*lastOne=*/ 0 )			)
+	      docPrevPosition( &dpNew )					)
 	    {
 	    docInitDocumentPosition( &dpNew );
 	    }
 	}
 
-    tedEditIncludeRowsInRedraw( &eo, ed, selSectBi, delRow0, delRow1 );
+    tedEditIncludeRowsInRedraw( eo, ed, selSectBi, delRow0, delRow1 );
 
-    docEditDeleteItems( &eo, selSectBi, delRow0, delRow1- delRow0+ 1 );
+    docEditIncludeItemInReformatRange( eo, selSectBi );
+
+    docEditDeleteItems( eo, &sectionsDeleted,
+				&firstParaDeleted, &paragraphsDeleted,
+				selSectBi, delRow0, delRow1- delRow0+ 1 );
 
     /* NO! done by docEditDeleteItems()
-    docEditShiftReformatRangeParaNr( eo, paraNr+ 1, -eo.eoParagraphsDeleted );
+    docEditShiftReformatRangeParaNr( eo, paraNr+ 1, -paragraphsDeleted );
     */
 
     if  ( selSectBi->biInExternalItem == DOCinBODY )
 	{
-	docShiftNoteReferences( eo.eoBd, paraNr, stroffFrom,
-				    sectShift, -eo.eoParagraphsDeleted,
+	const int	isSplit= 0;
+
+	docEditShiftReferences( eo->eoBd, paraNr, isSplit, stroffFrom,
+				    -sectionsDeleted, -paragraphsDeleted,
 				    stroffShift );
 	}
 
-    docEditIncludeItemInReformatRange( &eo, selSectBi );
-
     if  ( ! dpNew.dpBi )
 	{
 	dpNew.dpBi= docInsertEmptyParagraph( bd, selSectBi,
-						td->tdCurrentTextAttribute );
+					td->tdCurrentTextAttributeNumber );
 
-	docEditIncludeItemInReformatRange( &eo, selSectBi );
+	docEditIncludeItemInReformatRange( eo, selSectBi );
 
 	if  ( docFirstPosition( &dpNew, dpNew.dpBi ) )
 	    { LDEB(1);	}
 	}
 
-    tedEditFinishIBarSelection( ed, &eo, dpNew.dpBi, dpNew.dpStroff );
+    docSectDelimitTables( selSectBi );
+
+    tedEditFinishIBarSelection( ed, eo, dpNew.dpBi, dpNew.dpStroff );
+
+    return 0;
+    }
+
+int tedDeleteRowsFromTable(	EditDocument *		ed,
+				int			delRow0,
+				int			delRow1 )
+    {
+    EditOperation		eo;
+    DocumentSelection		ds;
+    SelectionGeometry		sg;
+    SelectionDescription	sd;
+
+    tedStartEditOperation( &eo, &ds, &sg, &sd, ed, 1 );
+
+    if  ( tedDeleteRowsFromTableOperation( ed, &eo, &ds, delRow0, delRow1 ) )
+	{ LDEB(1); return -1;	}
+
+    appDocumentChanged( ed, 1 );
 
     return 0;
     }
@@ -420,35 +481,6 @@ int tedDeleteRowsFromTable(	EditDocument *		ed,
 /*									*/
 /************************************************************************/
 
-static BufferItem * tedInsertCellInRow(	BufferDocument *	bd,
-					BufferItem *		rowBi,
-					int			col,
-					const CellProperties *	cp,
-					TextAttribute		ta )
-    {
-    RowProperties *	rp;
-
-    BufferItem *	cellBi;
-    BufferItem *	paraBi;
-
-    rp= &(rowBi->biRowProperties);
-
-    if  ( docInsertRowColumn( rp, col, cp ) )
-	{ LDEB(col); return (BufferItem *)0;	}
-
-    cellBi= docInsertItem( bd, rowBi, col, DOClevCELL );
-    if  ( ! cellBi )
-	{ LXDEB(col,cellBi); return (BufferItem *)0;	}
-
-    paraBi= docInsertEmptyParagraph( bd, cellBi, ta );
-    if  ( ! paraBi )
-	{ XDEB(paraBi); return (BufferItem *)0;	}
-
-    paraBi->biParaInTable= 1;
-
-    return paraBi;
-    }
-
 static int tedSplitColumnInRows(	EditDocument *	ed,
 					EditOperation *	eo,
 					BufferItem *	sectBi,
@@ -458,63 +490,17 @@ static int tedSplitColumnInRows(	EditDocument *	ed,
 					int		col,
 					int		after )
     {
-    TedDocument *		td= (TedDocument *)ed->edPrivateData;
-    BufferDocument *		bd= td->tdDocument;
-
-    BufferItem *		newBi= (BufferItem *)0;
-
-    BufferItem *		rowBi;
-
-    TextAttribute		ta;
+    BufferItem *		newParaBi= (BufferItem *)0;
 
     tedEditIncludeRowsInRedraw( eo, ed, sectBi, row0, row1 );
 
-    if  ( after )
-	{ after= 1;	}
+    if  ( docSplitColumnInRows( &newParaBi, eo,
+				    sectBi, row0, row, row1, col, after ) )
+	{ LDEB(1); return -1;	}
 
-    while( row0 <= row1 )
-	{
-	RowProperties *		rp;
+    docSectDelimitTables( sectBi );
 
-	CellProperties *	oldCp;
-	CellProperties		newCp;
-
-	int			left;
-	int			middle;
-
-	BufferItem *		cellBi;
-	BufferItem *		paraBi;
-
-	rowBi= sectBi->biChildren[row0];
-	rp= &(rowBi->biRowProperties);
-
-	oldCp= rp->rpCells+ col;
-
-	if  ( col == 0 )
-	    { left= rp->rpLeftIndentTwips;			}
-	else{ left= rp->rpCells[col-1].cpRightBoundaryTwips;	}
-
-	middle= ( rp->rpCells[col].cpRightBoundaryTwips+ left )/ 2;
-
-	newCp= *oldCp;
-	if  ( after )
-	    { oldCp->cpRightBoundaryTwips= middle;	}
-	else{ newCp. cpRightBoundaryTwips= middle;	}
-
-	cellBi= rowBi->biChildren[col];
-	paraBi= cellBi->biChildren[0];
-	ta= paraBi->biParaParticules[0].tpTextAttribute;
-
-	paraBi= tedInsertCellInRow( bd, rowBi, col+ after, &newCp, ta );
-	if  ( row0 == row )
-	    { newBi= paraBi;	}
-
-	docEditIncludeItemInReformatRange( eo, rowBi );
-
-	row0++;
-	}
-
-    tedEditFinishIBarSelection( ed, eo, newBi, 0 );
+    tedEditFinishIBarSelection( ed, eo, newParaBi, 0 );
 
     return 0;
     }
@@ -533,8 +519,9 @@ int tedInsertColumnInTable(	EditDocument *		ed )
     EditOperation		eo;
     DocumentSelection		ds;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
-    tedStartEditOperation( &eo, &ds, &sg, ed, 1 );
+    tedStartEditOperation( &eo, &ds, &sg, &sd, ed, 1 );
 
     if  ( docDelimitTable( ds.dsBegin.dpBi, &sectBi,
 					    &col, &row0, &row, &row1 ) )
@@ -557,8 +544,9 @@ int tedAppendColumnToTable(	EditDocument *		ed )
     EditOperation		eo;
     DocumentSelection		ds;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
-    tedStartEditOperation( &eo, &ds, &sg, ed, 1 );
+    tedStartEditOperation( &eo, &ds, &sg, &sd, ed, 1 );
 
     if  ( docDelimitTable( ds.dsBegin.dpBi, &sectBi,
 					    &col, &row0, &row, &row1 ) )
@@ -567,11 +555,14 @@ int tedAppendColumnToTable(	EditDocument *		ed )
     return tedSplitColumnInRows( ed, &eo, sectBi, row0, row, row1, col, after );
     }
 
-int tedDeleteColumnsFromRows(	EditDocument *	ed,
-				int		delRow0,
-				int		delRow1,
-				int		delCol0,
-				int		delCol1 )
+static int tedDeleteColumnsFromRowsOperation(
+					EditDocument *		ed,
+					EditOperation *		eo,
+					DocumentSelection *	ds,
+					int			delRow0,
+					int			delRow1,
+					int			delCol0,
+					int			delCol1 )
     {
     BufferItem *		sectBi;
     BufferItem *		rowBi;
@@ -583,18 +574,11 @@ int tedDeleteColumnsFromRows(	EditDocument *	ed,
 
     DocumentPosition		dpNew;
 
-    int				count= delCol1- delCol0+ 1;
-
-    EditOperation		eo;
-    DocumentSelection		ds;
-    SelectionGeometry		sg;
-
-    tedStartEditOperation( &eo, &ds, &sg, ed, 1 );
-
-    if  ( docDelimitTable( ds.dsBegin.dpBi, &sectBi,
+    if  ( docDelimitTable( ds->dsBegin.dpBi, &sectBi,
 					    &col, &row0, &row, &row1 ) )
+	{ LDEB(1); return -1;	}
 
-    tedEditIncludeRowsInRedraw( &eo, ed, sectBi, row0, row1 );
+    tedEditIncludeRowsInRedraw( eo, ed, sectBi, delRow0, delRow1 );
 
     docInitDocumentPosition( &dpNew );
     rowBi= sectBi->biChildren[delRow1];
@@ -606,131 +590,146 @@ int tedDeleteColumnsFromRows(	EditDocument *	ed,
 	rowBi= sectBi->biChildren[delRow0];
 	if  ( delCol0 <= 0						||
 	      docFirstPosition( &dpNew, rowBi->biChildren[delCol0] )	||
-	      docPrevPosition( &dpNew, /*lastOne=*/ 0 )			)
+	      docPrevPosition( &dpNew )					)
 	    { docInitDocumentPosition( &dpNew ); }
 	}
 
-    while( delRow0 <= delRow1 )
-	{
-	int		wide;
-	RowProperties *	rp;
+    if  ( docDeleteColumnsFromRows( eo, sectBi,
+				    delRow0, delRow1, delCol0, delCol1 ) )
+	{ LDEB(1); return -1;	}
 
-	rowBi= sectBi->biChildren[delRow0];
-	rp= &(rowBi->biRowProperties);
-
-	if  ( delCol0 == 0 )
-	    { wide= rp->rpLeftIndentTwips;				}
-	else{ wide= rp->rpCells[delCol0-1].cpRightBoundaryTwips;	}
-	wide= rp->rpCells[delCol1].cpRightBoundaryTwips- wide;
-
-	if  ( rp->rpCellCount <= count )
-	    { LLDEB(rp->rpCellCount,count);	}
-	else{
-	    rp->rpCellCount -= count;
-
-	    for ( col= delCol0; col < rp->rpCellCount; col++ )
-		{
-		docCleanCellProperties( &(rp->rpCells[col]) );
-		rp->rpCells[col]= rp->rpCells[col+ count];
-		rp->rpCells[col].cpRightBoundaryTwips -= wide;
-		}
-
-	    docEditDeleteItems( &eo, rowBi, delCol0, count );
-
-	    docEditIncludeItemInReformatRange( &eo, rowBi );
-	    }
-
-	delRow0++;
-	}
-
-    tedEditFinishIBarSelection( ed, &eo, dpNew.dpBi, dpNew.dpStroff );
+    tedEditFinishIBarSelection( ed, eo, dpNew.dpBi, dpNew.dpStroff );
 
     return 0;
     }
 
+int tedDeleteColumnsFromRows(	EditDocument *	ed,
+				int		delRow0,
+				int		delRow1,
+				int		delCol0,
+				int		delCol1 )
+    {
+    EditOperation		eo;
+    DocumentSelection		ds;
+    SelectionGeometry		sg;
+    SelectionDescription	sd;
+
+    tedStartEditOperation( &eo, &ds, &sg, &sd, ed, 1 );
+
+    return tedDeleteColumnsFromRowsOperation( ed, &eo, &ds,
+					delRow0, delRow1, delCol0, delCol1 );
+    }
+
 /************************************************************************/
 /*									*/
-/*  'Set' event handler of the table tool.				*/
+/*  Implementation of the 'Change' functionality of the table related	*/
+/*  pages of the format tool.						*/
+/*									*/
+/*  1)  Retrieve the current selection of the document.			*/
+/*  2)  Obtain a selection for the table rectangle.			*/
+/*  3)  Were the colspan/rowspan of a cell to be changed?		*/
+/*  4)  Remove spans from mask.						*/
+/*  5)  Update other properties.					*/
 /*									*/
 /************************************************************************/
 
-void tedAppSetTableProperties(	EditApplication *	ea,
+void tedDocSetTableProperties(	EditDocument *		ed,
 				const TableRectangle *	trChange,
+				const PropertyMask *	rpSetMask,
+				const PropertyMask *	cpSetMask,
 				const RowProperties *	rpFrom )
     {
-    EditDocument *		ed= ea->eaCurrentDocument;
     TedDocument *		td;
     BufferDocument *		bd;
 
     BufferItem *		selSectBi;
-
-    RowProperties *		rpTo;
-    const CellProperties *	cpFrom;
-    CellProperties *		cpTo;
-
-    int				rowPropertiesAlso= 0;
+    int				row1Reformat;
 
     DocumentSelection		dsNew;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
-    if  ( ! ed )
-	{ XDEB(ed); return;	}
+    PropertyMask		cellSpanMask;
+    PropertyMask		cpSetMaskStripped;
 
     td= (TedDocument *)ed->edPrivateData;
     bd= td->tdDocument;
 
+    /*  1  */
     docInitDocumentSelection( &dsNew );
-    if  ( tedGetSelection( &dsNew, &sg, td ) )
+    if  ( tedGetSelection( &dsNew, &sg, &sd, td ) )
 	{ LDEB(1); return;	}
 
+    /*  2  */
     if  ( docTableRectangleSelection( &dsNew, &selSectBi, bd, trChange ) )
 	{ LDEB(1); return;	}
 
-    if  ( trChange->trCol0 >= 0 && trChange->trCol0 == trChange->trCol1 )
+    row1Reformat= trChange->trRow1;
+
+    /*  3  */
+    PROPmaskCLEAR( &cellSpanMask );
+    PROPmaskADD( &cellSpanMask, CLprop_ROWSPAN );
+    PROPmaskADD( &cellSpanMask, CLprop_COLSPAN );
+
+    utilPropMaskAnd( &cellSpanMask, &cellSpanMask, cpSetMask );
+
+    if  ( ! utilPropMaskIsEmpty( &cellSpanMask ) )
 	{
-	int			row;
+	int		row1SpanChanged= trChange->trRow0- 1;
 
-	cpFrom= rpFrom->rpCells+ trChange->trCol0;
+	if  ( docChangeCellSpans( &row1SpanChanged, selSectBi,
+					    trChange->trRow0, trChange->trCol0,
+					    trChange->trCellRowspan,
+					    trChange->trCellColspan ) )
+	    { LDEB(1); return;	}
 
-	for ( row= trChange->trRow0; row <= trChange->trRow1; row++ )
-	    {
-	    rpTo= &(selSectBi->biChildren[row]->biRowProperties);
-	    cpTo= rpTo->rpCells+ trChange->trCol0;
-
-	    cpTo->cpLeftBorder= cpFrom->cpLeftBorder;
-	    cpTo->cpRightBorder= cpFrom->cpRightBorder;
-	    }
+	if  ( row1Reformat < row1SpanChanged )
+	    { row1Reformat=  row1SpanChanged;	}
 	}
 
-    if  ( trChange->trRow0 >= 0 && trChange->trRow0 == trChange->trRow1 )
-	{
-	int			col;
-
-	rpTo= &(selSectBi->biChildren[trChange->trRow0]->biRowProperties);
-	cpFrom= rpFrom->rpCells;
-	cpTo= rpTo->rpCells;
-
-	for ( col= trChange->trCol0;
-	      col <= trChange->trCol1;
-	      cpFrom++, cpTo++, col++ )
-	    {
-	    cpTo->cpTopBorder= cpFrom->cpTopBorder;
-	    cpTo->cpBottomBorder= cpFrom->cpBottomBorder;
-	    }
-
-	rowPropertiesAlso= 1;
-	}
+    /*  4  */
+    cpSetMaskStripped= *cpSetMask;
+    PROPmaskUNSET( &cpSetMaskStripped, CLprop_ROWSPAN );
+    PROPmaskUNSET( &cpSetMaskStripped, CLprop_COLSPAN );
 
+    /*  5  */
     tedChangeTableLayout( ed, selSectBi,
-	    rpFrom, rowPropertiesAlso, trChange->trRow0, trChange->trRow1 );
+				    trChange->trRow0, trChange->trRow1,
+				    trChange->trCol0, trChange->trCol1,
+				    row1Reformat,
+				    rpSetMask, &cpSetMaskStripped, rpFrom );
 
     tedAdaptToolsToSelection( ed );
 
-    appDocumentChanged( ed, 1 );
+    if  ( ! utilPropMaskIsEmpty( rpSetMask )	||
+	  ! utilPropMaskIsEmpty( cpSetMask )	)
+	{ appDocumentChanged( ed, 1 );	}
 
     return;
     }
 
+void tedAppSetTableProperties(	EditApplication *	ea,
+				const TableRectangle *	trChange,
+				const PropertyMask *	rpSetMask,
+				const PropertyMask *	cpSetMask,
+				const RowProperties *	rpFrom )
+    {
+    EditDocument *		ed= ea->eaCurrentDocument;
+
+    if  ( ! ed )
+	{ XDEB(ed); return;	}
+
+    tedDocSetTableProperties( ed, trChange, rpSetMask, cpSetMask, rpFrom );
+
+    return;
+    }
+
+/************************************************************************/
+/*									*/
+/*  Select a rectangle in a table.					*/
+/*									*/
+/************************************************************************/
+
 void tedAppSetTableSelection(	EditDocument *		ed,
 				const TableRectangle *	trSet )
     {
@@ -742,15 +741,16 @@ void tedAppSetTableSelection(	EditDocument *		ed,
 
     DocumentSelection		ds;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
     if  ( ! ed )
 	{ XDEB(ed); return;	}
 
     td= (TedDocument *)ed->edPrivateData;
 
-    tedGetSelection( &ds, &sg, td );
+    tedGetSelection( &ds, &sg, &sd, td );
 
-    if  ( ! tedGetTableRectangle( &tr, &ds ) )
+    if  ( ! docGetTableRectangle( &tr, &ds ) )
 	{ tableRectangle= 1;	}
 
     if  ( ! tableRectangle )
@@ -768,59 +768,69 @@ void tedAppSetTableSelection(	EditDocument *		ed,
 
 /************************************************************************/
 /*									*/
-/*  Update the of a table in response to a user action.			*/
+/*  Change table layout from various table related tools.		*/
+/*									*/
+/*  1)  Reformat at least all rows that are touched.			*/
+/*  2)  Start edit operation.						*/
+/*  3)  Include all rows touched by the operation in the reformat.	*/
+/*  4)  If the 'IsTableHeader' property is changed, the whole table	*/
+/*	must be reformatted.						*/
+/*  5)  Apply changes.							*/
+/*  6)  Finish edit operation. I.E: Redo layout where needed and expose	*/
+/*	the affected area to be redrawn.				*/
 /*									*/
 /************************************************************************/
 
 void tedChangeTableLayout(	EditDocument *		ed,
 				BufferItem *		sectBi,
-				const RowProperties *	rp,
-				int			rowPropertiesAlso,
-				int			row0,
-				int			row1 )
+				int			row0Change,
+				int			row1Change,
+				int			col0Change,
+				int			col1Change,
+				int			row1Reformat,
+				const PropertyMask *	rpSetMask,
+				const PropertyMask *	cpSetMask,
+				const RowProperties *	rpSet )
     {
-    BufferItem *		rowBi= sectBi->biChildren[row0];
-
     EditOperation		eo;
     DocumentSelection		ds;
     SelectionGeometry		sg;
+    SelectionDescription	sd;
 
-    tedStartEditOperation( &eo, &ds, &sg, ed, 1 );
+    /*  1  */
+    if  ( row1Reformat < row1Change )
+	{ row1Reformat=  row1Change;	}
 
-    tedEditIncludeRowsInRedraw( &eo, ed, sectBi, row0, row1 );
+    /*  2  */
+    tedStartEditOperation( &eo, &ds, &sg, &sd, ed, 1 );
 
-    while( row0 <= row1 )
+    /*  3  */
+    docEditIncludeRowsInReformatRange( &eo,
+				    sectBi, row0Change, row1Reformat );
+    tedEditIncludeRowsInRedraw( &eo, ed,
+				    sectBi, row0Change, row1Reformat );
+
+    /*  4  */
+    if  ( PROPmaskISSET( rpSetMask, RPpropIS_TABLE_HEADER ) )
 	{
-	CellProperties *	cp;
-	int			col;
+	const BufferItem *	rowBi= sectBi->biChildren[row0Change];
 
-	rowBi= sectBi->biChildren[row0];
+	if  ( row0Change != row1Change )
+	    { LLDEB(row0Change,row1Change);	}
 
-	rowBi->biRowLeftIndentTwips= rp->rpLeftIndentTwips;
-
-	if  ( rowPropertiesAlso )
-	    {
-	    rowBi->biRowHeightTwips= rp->rpHeightTwips;
-	    rowBi->biRowKeepOnOnePage= rp->rpKeepOnOnePage;
-	    rowBi->biRowIsTableHeader= rp->rpIsTableHeader;
-	    }
-
-	rowBi->biRowHalfGapWidthTwips= rp->rpHalfGapWidthTwips;
-	rowBi->biRowLeftIndentTwips= rp->rpLeftIndentTwips;
-
-	cp= rp->rpCells;
-	for ( col= 0; col < rp->rpCellCount; cp++, col++ )
-	    {
-	    rowBi->biRowCells[col].cpRightBoundaryTwips=
-						cp->cpRightBoundaryTwips;
-	    }
-
-	docEditIncludeItemInReformatRange( &eo, rowBi );
-
-	row0++;
+	docEditIncludeRowsInReformatRange( &eo,
+				sectBi, row0Change, rowBi->biRowTablePast- 1 );
+	tedEditIncludeRowsInRedraw( &eo, ed,
+				sectBi, row0Change, rowBi->biRowTablePast- 1 );
 	}
 
-    if  ( tedEditFinishRangeSelection( ed, &eo, &ds ) )
+    /*  5  */
+    docChangeTableLayout( &eo, sectBi,
+			    row0Change, row1Change, col0Change, col1Change,
+			    rpSetMask, cpSetMask, rpSet );
+
+    /*  6  */
+    if  ( tedEditFinishOldSelection( ed, &eo ) )
 	{ LDEB(1);	}
 
     return;
@@ -840,12 +850,77 @@ void tedDocTableDrawGrid(	APP_WIDGET	option,
     TedDocument *		td= (TedDocument *)ed->edPrivateData;
     int				set;
 
-    set= appGuiGetToggleStateFromCallback( option, voidcbs );
+    set= appGuiGetMenuToggleStateFromCallback( option, voidcbs );
 
-    td->tdDrawTableGrid= set;
+    if  ( set )
+	{ td->tdDrawTableGrid=  1;	}
+    else{ td->tdDrawTableGrid= -1;	}
 
     appDocExposeRectangle( ed, (const DocumentRectangle *)0, 0,0 );
 
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Delete a slice from a table. Either horizontal or vertical or both:	*/
+/*  the table as a whole.						*/
+/*									*/
+/*  1)  See whether the ccurrent selection is a table slice.		*/
+/*  2)  If it is a row slice (Whole rows) delete the rows. This covers	*/
+/*	deleteing the whole table as well. [ Handling it in the column	*/
+/*	branch would leave empty rows.]					*/
+/*  3)  If it is a column slice (Whole columns) delete the columns.	*/
+/*  4)  Impossible!							*/
+/*									*/
+/************************************************************************/
+
+int tedDeleteTableSliceSelection(	EditDocument *		ed )
+    {
+    int				isRowSlice= 0;
+    int				isColSlice= 0;
+    TableRectangle		tr;
+
+    EditOperation		eo;
+    DocumentSelection		ds;
+    SelectionGeometry		sg;
+    SelectionDescription	sd;
+
+    int				ret;
+
+    if  ( ed->edIsReadonly )
+	{ return 1;	}
+
+    tedStartEditOperation( &eo, &ds, &sg, &sd, ed, 1 );
+
+    /*  1  */
+    if  ( docGetTableSliceSelection( &isRowSlice, &isColSlice, &tr, &ds ) )
+	{ return 1;	}
+
+    /*  2  */
+    if  ( isRowSlice )
+	{
+	ret= tedDeleteRowsFromTableOperation( ed, &eo, &ds,
+							tr.trRow0, tr.trRow1 );
+	if  ( ret )
+	    { LDEB(ret); return -1;	}
+
+	appDocumentChanged( ed, 1 );
+	return 0;
+	}
+
+    /*  3  */
+    if  ( isColSlice )
+	{
+	ret= tedDeleteColumnsFromRowsOperation( ed, &eo, &ds,
+				tr.trRow0, tr.trRow1, tr.trCol0, tr.trCol1 );
+	if  ( ret )
+	    { LDEB(ret); return -1;	}
+
+	appDocumentChanged( ed, 1 );
+	return 0;
+	}
+
+    /*  4  */
+    LLDEB(isRowSlice,isColSlice); return -1;
+    }
