@@ -11,14 +11,14 @@
 #   include	<stddef.h>
 #   include	<limits.h>
 
-#   include	<appDebugon.h>
-
 #   include	<appGeoString.h>
 #   include	<appUnit.h>
 
 #   include	"tedApp.h"
 #   include	"tedFormatTool.h"
 
+#   include	<appDebugon.h>
+
 /************************************************************************/
 /*									*/
 /*  Refresh the controls with the current tab stop.			*/
@@ -57,13 +57,14 @@ static void tedTabsToolSelectTab(	TabsTool *	tt,
 					int		n )
     {
     ParagraphProperties *	pp= &(tt->ttParaPropertiesChosen);
+    const TabStopList *		tsl= &(pp->ppTabStopList);
 
-    if  ( n >= 0 && n >= pp->ppTabCount )
-	{ LLDEB(n,pp->ppTabCount); n= -1;	}
+    if  ( n >= 0 && n >= tsl->tslTabStopCount )
+	{ LLDEB(n,tsl->tslTabStopCount); n= -1;	}
 
     if  ( n >= 0 )
 	{
-	tt->ttTabStopValue= pp->ppTabStops[n];
+	tt->ttTabStopValue= tsl->tslTabStops[n];
 	tt->ttTabStopNumber= n;
 
 	appGuiSelectPositionInListWidget( tt->ttTabPositionList, n );
@@ -90,6 +91,8 @@ static void tedTabsToolSelectTab(	TabsTool *	tt,
 static void tedFormatToolRefreshTabsPage(	TabsTool *	tt )
     {
     ParagraphProperties *	pp= &(tt->ttParaPropertiesChosen);
+    const TabStopList *		tsl= &(pp->ppTabStopList);
+
     int				below= -1;
     int				above= -1;
     int				da= INT_MAX;
@@ -102,8 +105,8 @@ static void tedFormatToolRefreshTabsPage(	TabsTool *	tt )
 
     appGuiEmptyListWidget( tt->ttTabPositionList );
 
-    ts= pp->ppTabStops;
-    for ( i= 0; i < pp->ppTabCount; ts++, i++ )
+    ts= tsl->tslTabStops;
+    for ( i= 0; i < tsl->tslTabStopCount; ts++, i++ )
 	{
 	appGeoLengthToString( scratch, ts->tsTwips, UNITtyPOINTS );
 	appGuiAddValueToListWidget( tt->ttTabPositionList, scratch );
@@ -120,7 +123,7 @@ static void tedFormatToolRefreshTabsPage(	TabsTool *	tt )
 	    }
 	}
 
-    if  ( pp->ppTabCount == 0 )
+    if  ( tsl->tslTabStopCount == 0 )
 	{ tedTabsToolSelectTab( tt, -1 );		}
     else{
 	if  ( da < db )
@@ -140,6 +143,7 @@ static void tedFormatToolRefreshTabsPage(	TabsTool *	tt )
 void tedFormatToolRefreshTabsTool(
 				TabsTool *			tt,
 				int *				pEnabled,
+				int *				pPref,
 				InspectorSubject *		is,
 				const DocumentSelection *	ds,
 				const DocumentProperties *	dp )
@@ -149,6 +153,9 @@ void tedFormatToolRefreshTabsTool(
     PropertyMask			chgMask;
     PropertyMask			updMask;
 
+    const int * const			colorMap= (const int *)0;
+    const int * const			listStyleMap= (const int *)0;
+
     pp= &(ds->dsBegin.dpBi->biParaProperties);
 
     PROPmaskCLEAR( &updMask );
@@ -157,13 +164,13 @@ void tedFormatToolRefreshTabsTool(
     PROPmaskCLEAR( &chgMask );
 
     if  ( docUpdParaProperties( &chgMask, &(tt->ttParaPropertiesChosen),
-							    &updMask, pp ))
+				    &updMask, pp, colorMap, listStyleMap ) )
 	{ LDEB(1); return ;	}
 
     PROPmaskCLEAR( &chgMask );
 
     if  ( docUpdParaProperties( &chgMask, &(tt->ttParaPropertiesSet),
-							    &updMask, pp ))
+				    &updMask, pp, colorMap, listStyleMap ) )
 	{ LDEB(1); return ;	}
 
     /**/
@@ -206,14 +213,15 @@ void tedFormatToolRefreshTabsTool(
 /*									*/
 /************************************************************************/
 
-static void tedFormatTabsRevertParaPushed(	APP_WIDGET	w,
-						void *		voidtt,
-						void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatTabsRevertParaPushed, w, voidtt )
     {
-    TabsTool *			tt= (TabsTool *)voidtt;
+    TabsTool *		tt= (TabsTool *)voidtt;
 
-    PropertyMask		chgMask;
-    PropertyMask		updMask;
+    PropertyMask	chgMask;
+    PropertyMask	updMask;
+
+    const int * const	colorMap= (const int *)0;
+    const int * const	listStyleMap= (const int *)0;
 
     PROPmaskCLEAR( &chgMask );
 
@@ -221,16 +229,15 @@ static void tedFormatTabsRevertParaPushed(	APP_WIDGET	w,
     PROPmaskADD( &updMask, PPpropTAB_STOPS );
 
     docUpdParaProperties( &chgMask, &(tt->ttParaPropertiesChosen),
-				    &updMask, &(tt->ttParaPropertiesSet) );
+			&updMask, &(tt->ttParaPropertiesSet),
+			colorMap, listStyleMap );
 
     tedFormatToolRefreshTabsPage( tt );
 
     return;
     }
 
-static void tedFormatTabsRevertDocPushed(	APP_WIDGET	w,
-						void *		voidtt,
-						void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatTabsRevertDocPushed, w, voidtt )
     {
     TabsTool *			tt= (TabsTool *)voidtt;
 
@@ -264,15 +271,15 @@ static void tedFormatTabsRevertDocPushed(	APP_WIDGET	w,
 /************************************************************************/
 
 static int tedTabToolValueTooCloseToExisting(
-				    const ParagraphProperties *	pp,
-				    int				current,
-				    int				value )
+				    const TabStopList *	tsl,
+				    int			current,
+				    int			value )
     {
     int				i;
     TabStop *			ts;
 
-    ts= pp->ppTabStops;
-    for ( i= 0; i < pp->ppTabCount; ts++, i++ )
+    ts= tsl->tslTabStops;
+    for ( i= 0; i < tsl->tslTabStopCount; ts++, i++ )
 	{
 	if  ( i == current )
 	    { continue;	}
@@ -291,35 +298,33 @@ static int tedTabToolValueTooCloseToExisting(
 /*									*/
 /************************************************************************/
 
-static void tedTabPositionChanged(	APP_WIDGET	w,
-					void *		voidtt,
-					void *		voidpbcs )
+static APP_TXACTIVATE_CALLBACK_H( tedTabPositionChanged, w, voidtt )
     {
     TabsTool *			tt= (TabsTool *)voidtt;
     ParagraphProperties *	pp= &(tt->ttParaPropertiesChosen);
+    const TabStopList *		tsl= &(pp->ppTabStopList);
 
     int				value;
     int				changed;
 
     if  ( tt->ttTabStopNumber < 0		||
-	  tt->ttTabStopNumber >= pp->ppTabCount	)
+	  tt->ttTabStopNumber >= tsl->tslTabStopCount	)
 	{ value= 0;	}
     else{
-	TabStop *	ts= pp->ppTabStops+ tt->ttTabStopNumber;
+	TabStop *	ts= tsl->tslTabStops+ tt->ttTabStopNumber;
 
 	value= ts->tsTwips;
 	}
 
     changed= 0;
 
-    tedFormatValidateDimension( &value, &changed, w, value );
+    tedFormatValidateDimension( &value, &changed,
+					    tt->ttTabPositionText, value );
 
     return;
     }
 
-static void tedTabDeftabChanged(	APP_WIDGET	w,
-					void *		voidtt,
-					void *		voidpbcs )
+static APP_TXACTIVATE_CALLBACK_H( tedTabDeftabChanged, w, voidtt )
     {
     TabsTool *			tt= (TabsTool *)voidtt;
     DocumentProperties *	dp= &(tt->ttDocPropertiesChosen);
@@ -327,7 +332,7 @@ static void tedTabDeftabChanged(	APP_WIDGET	w,
     int				value= dp->dpTabIntervalTwips;
     int				changed= 0;
 
-    tedFormatValidateDimension( &value, &changed, w, value );
+    tedFormatValidateDimension( &value, &changed, tt->ttTabDefaultText, value );
 
     return;
     }
@@ -338,10 +343,11 @@ static void tedTabDeftabChanged(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static APP_LIST_CALLBACK( tedTabsToolTabChosen, w, voidtt, voidlcs )
+static APP_LIST_CALLBACK_H( tedTabsToolTabChosen, w, voidtt, voidlcs )
     {
     TabsTool *			tt= (TabsTool *)voidtt;
     ParagraphProperties *	pp= &(tt->ttParaPropertiesChosen);
+    const TabStopList *		tsl= &(pp->ppTabStopList);
 
     int				position;
 
@@ -352,10 +358,10 @@ static APP_LIST_CALLBACK( tedTabsToolTabChosen, w, voidtt, voidlcs )
 	{ return;	}
     */
 
-    if  ( position < 0 || position >= pp->ppTabCount )
-	{ LLDEB(position,pp->ppTabCount);	}
+    if  ( position < 0 || position >= tsl->tslTabStopCount )
+	{ LLDEB(position,tsl->tslTabStopCount);	}
 
-    tt->ttTabStopValue= pp->ppTabStops[position];
+    tt->ttTabStopValue= tsl->tslTabStops[position];
     tt->ttTabStopNumber= position;
 
     tedTabToolRefreshCurrentTab( tt );
@@ -369,9 +375,7 @@ static APP_LIST_CALLBACK( tedTabsToolTabChosen, w, voidtt, voidlcs )
 /*									*/
 /************************************************************************/
 
-static void tedFormatPrevPara(		APP_WIDGET	w,
-					void *		voidtt,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatPrevPara, w, voidtt )
     {
     TabsTool *			tt= (TabsTool *)voidtt;
     EditApplication *		ea= tt->ttApplication;
@@ -381,9 +385,7 @@ static void tedFormatPrevPara(		APP_WIDGET	w,
     return;
     }
 
-static void tedFormatNextPara(		APP_WIDGET	w,
-					void *		voidtt,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatNextPara, w, voidtt )
     {
     TabsTool *			tt= (TabsTool *)voidtt;
     EditApplication *		ea= tt->ttApplication;
@@ -403,13 +405,12 @@ static void tedFormatNextPara(		APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedTabToolAddTab(		APP_WIDGET	w,
-					void *		voidtt,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedTabToolAddTab, w, voidtt )
     {
     TabsTool *			tt= (TabsTool *)voidtt;
     EditApplication *		ea= tt->ttApplication;
     ParagraphProperties *	pp= &(tt->ttParaPropertiesChosen);
+    const TabStopList *		tsl= &(pp->ppTabStopList);
 
     const TabsPageResources *	tpr= tt->ttPageResources;
 
@@ -430,10 +431,11 @@ static void tedTabToolAddTab(		APP_WIDGET	w,
 	{ return;	}
 
     /*  2  */
-    if  ( tedTabToolValueTooCloseToExisting( pp, -1, value ) )
+    if  ( tedTabToolValueTooCloseToExisting( tsl, -1, value ) )
 	{
 	appQuestionRunErrorDialog( tt->ttApplication,
-					tt->ttInspector->aiTopWidget, w,
+					tt->ttInspector->aiTopWidget,
+					(APP_WIDGET)w,
 					tpr->tprTooCloseMessage );
 	return;
 	}
@@ -467,13 +469,12 @@ static void tedTabToolAddTab(		APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedFormatChangeParaTabs(	APP_WIDGET	w,
-					void *		voidtt,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatChangeParaTabs, w, voidtt )
     {
     TabsTool *			tt= (TabsTool *)voidtt;
     EditApplication *		ea= tt->ttApplication;
     ParagraphProperties *	pp= &(tt->ttParaPropertiesChosen);
+    const TabStopList *		tsl= &(pp->ppTabStopList);
 
     const TabsPageResources *	tpr= tt->ttPageResources;
 
@@ -494,10 +495,11 @@ static void tedFormatChangeParaTabs(	APP_WIDGET	w,
 	{ return;	}
 
     /*  3  */
-    if  ( tedTabToolValueTooCloseToExisting( pp, tt->ttTabStopNumber, value ) )
+    if  ( tedTabToolValueTooCloseToExisting( tsl, tt->ttTabStopNumber, value ) )
 	{
 	appQuestionRunErrorDialog( tt->ttApplication,
-					tt->ttInspector->aiTopWidget, w,
+					tt->ttInspector->aiTopWidget,
+					(APP_WIDGET)w,
 					tpr->tprTooCloseMessage );
 	return;
 	}
@@ -506,8 +508,8 @@ static void tedFormatChangeParaTabs(	APP_WIDGET	w,
 	{ tt->ttTabStopValue.tsTwips= value;	}
 
     /*  4  */
-    if  ( tt->ttTabStopNumber >= 0		&&
-	  tt->ttTabStopNumber < pp->ppTabCount	)
+    if  ( tt->ttTabStopNumber >= 0			&&
+	  tt->ttTabStopNumber < tsl->tslTabStopCount	)
 	{ docParaDeleteTab( pp, tt->ttTabStopNumber );	}
 
     /*  5  */
@@ -536,26 +538,25 @@ static void tedFormatChangeParaTabs(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedTabToolDeleteTab(	APP_WIDGET	w,
-					void *		voidtt,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedTabToolDeleteTab, w, voidtt )
     {
     TabsTool *			tt= (TabsTool *)voidtt;
     EditApplication *		ea= tt->ttApplication;
     ParagraphProperties *	pp= &(tt->ttParaPropertiesChosen);
+    const TabStopList *		tsl= &(pp->ppTabStopList);
 
     PropertyMask		updMask;
 
     /*  1  */
     if  ( tt->ttTabStopNumber < 0		||
-	  tt->ttTabStopNumber >= pp->ppTabCount	)
-	{ LLDEB(tt->ttTabStopNumber,pp->ppTabCount); return;	}
+	  tt->ttTabStopNumber >= tsl->tslTabStopCount	)
+	{ LLDEB(tt->ttTabStopNumber,tsl->tslTabStopCount); return;	}
 
     /*  2  */
     docParaDeleteTab( pp, tt->ttTabStopNumber );
 
-    if  ( tt->ttTabStopNumber < pp->ppTabCount )
-	{ tt->ttTabStopValue= pp->ppTabStops[tt->ttTabStopNumber];	}
+    if  ( tt->ttTabStopNumber < tsl->tslTabStopCount )
+	{ tt->ttTabStopValue= tsl->tslTabStops[tt->ttTabStopNumber];	}
 
     PROPmaskCLEAR( &updMask );
     PROPmaskADD( &updMask, PPpropTAB_STOPS );
@@ -574,9 +575,7 @@ static void tedTabToolDeleteTab(	APP_WIDGET	w,
 /************************************************************************/
 
 /*  2  */
-static void tedFormatTabsApplyDocPushed(	APP_WIDGET	w,
-						void *		voidtt,
-						void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( tedFormatTabsApplyDocPushed, w, voidtt )
     {
     TabsTool *			tt= (TabsTool *)voidtt;
     EditApplication *		ea= tt->ttApplication;
@@ -603,7 +602,7 @@ static void tedFormatTabsApplyDocPushed(	APP_WIDGET	w,
     if  ( changed )
 	{ dp->dpTabIntervalTwips= value;	}
 
-    if  ( tedSetDocumentProperties( ea, dp, &updMask ) )
+    if  ( tedAppSetDocumentProperties( ea, dp, &updMask ) )
 	{ LDEB(1);	}
 
     return;
@@ -616,24 +615,18 @@ static void tedFormatTabsApplyDocPushed(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedTabAlignmentChosen(	APP_WIDGET	w,
-					void *		voidtt,
-					void *		voidtbcs )
+static APP_OITEM_CALLBACK_H( tedTabAlignmentChosen, w, voidtt )
     {
     TabsTool *			tt= (TabsTool *)voidtt;
     TabStop *			ts= &(tt->ttTabStopValue);
 
-    int				alignment= -1;
+    int				alignment;
 
-    for ( alignment= 0; alignment < DOCta_COUNT; alignment++ )
-	{
-	if  ( tt->ttAlignmentItems[alignment] == w )
-	    { break;	}
-	}
+    alignment= appGuiGetOptionmenuItemIndex( &(tt->ttAlignmentOptionmenu), w );
+    if  ( alignment < 0 || alignment >= DOCta_COUNT )
+	{ LLDEB(alignment,DOCta_COUNT); return;	}
 
-    if  ( alignment >= 0 && alignment < DOCta_COUNT )
-	{ ts->tsAlignment= alignment;		}
-    else{ LLDEB(alignment,DOCta_COUNT);	}
+    ts->tsAlignment= alignment;
 
     return;
     }
@@ -645,24 +638,18 @@ static void tedTabAlignmentChosen(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void tedTabLeaderChosen(		APP_WIDGET	w,
-					void *		voidtt,
-					void *		voidtbcs )
+static APP_OITEM_CALLBACK_H( tedTabLeaderChosen, w, voidtt )
     {
     TabsTool *			tt= (TabsTool *)voidtt;
     TabStop *			ts= &(tt->ttTabStopValue);
 
-    int				leader= -1;
+    int				leader;
 
-    for ( leader= 0; leader < DOCtl_COUNT; leader++ )
-	{
-	if  ( tt->ttLeaderItems[leader] == w )
-	    { break;	}
-	}
+    leader= appGuiGetOptionmenuItemIndex( &(tt->ttLeaderOptionmenu), w );
+    if  ( leader < 0 || leader >= DOCtl_COUNT )
+	{ LLDEB(leader,DOCtl_COUNT); return;	}
 
-    if  ( leader >= 0 && leader < DOCtl_COUNT )
-	{ ts->tsLeader= leader;		}
-    else{ LLDEB(leader,DOCtl_COUNT);	}
+    ts->tsLeader= leader;
 
     return;
     }
@@ -770,11 +757,11 @@ void tedFormatFillTabsPage(	TabsTool *			tt,
 
 /************************************************************************/
 /*									*/
-/*  Finish the paragraph page.						*/
+/*  Finish the tabs page.						*/
 /*									*/
 /************************************************************************/
 
-void tedFormatFillTabsChoosers(		TabsTool *			tt )
+void tedTabsToolFillChoosers(		TabsTool *			tt )
     {
     const TabsPageResources *	tpr= tt->ttPageResources;
 
@@ -804,6 +791,22 @@ void tedFormatFinishTabsPage(		TabsTool *			tt )
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Clean allocated resources in a tabs tool.				*/
+/*									*/
+/************************************************************************/
+
+void tedFormatCleanParaTabsTool(	TabsTool *			tt )
+    {
+    docCleanDocumentProperties( &(tt->ttDocPropertiesSet) );
+    docCleanDocumentProperties( &(tt->ttDocPropertiesChosen) );
+    docCleanParagraphProperties( &(tt->ttParaPropertiesSet) );
+    docCleanParagraphProperties( &(tt->ttParaPropertiesChosen) );
+    
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Get tabs tool resources.						*/
