@@ -12,6 +12,8 @@
 #   include	<string.h>
 
 #   include	<appSystem.h>
+#   include	<utilFontmap.h>
+#   include	<utilT1Entries.h>
 #   include	"appFrame.h"
 
 #   include	<appDebugon.h>
@@ -95,9 +97,11 @@ void appDocFileClose(	APP_WIDGET	option,
 	{ appRunReallyCloseDialog( option, ed ); return; }
 
     appCloseDocument( ea, ed );
+
+    return;
     }
 
-APP_CLOSE_CALLBACK( appDocFileCloseCallback, w, voided )
+APP_CLOSE_CALLBACK_H( appDocFileCloseCallback, w, voided )
     {
     EditDocument *	ed= (EditDocument *)voided;
     EditApplication *	ea= ed->edApplication;
@@ -292,12 +296,25 @@ void appDocEditSelAll(	APP_WIDGET	option,
 /************************************************************************/
 
 void appCloseDocument(	EditApplication *	ea,
-			EditDocument *		ed	)
+			EditDocument *		ed )
     {
     appRemoveDocument( ea, ed );
 
+    /* No! is done by the widget destroy callback
+    if  ( ed->edPrivateData )
+	{
+	(*ea->eaFreeDocument)( ed->edPrivateData, ed->edFormat,
+						    &(ed->edDrawingData) );
+	ed->edPrivateData= (void *)0;
+	}
+    */
+
     appDestroyShellWidget( ed->edToplevel.atTopWidget );
 
+    /* No! is done by the widget destroy callback
+    appFreeDocument( ea, ed );
+    */
+
     return;
     }
 
@@ -484,6 +501,7 @@ void appInitEditDocument(	EditApplication *	ea,
     appInitDrawingData( &(ed->edDrawingData) );
 
     ed->edMapped= 0;
+    ed->edNotYetDrawn= 1;
 
     return;
     }
@@ -515,6 +533,9 @@ int appMakeDocumentWindow(	EditDocument **		pEd,
 
     appInitEditDocument( ea, ed );
 
+    ea->eaNextDocumentId++; /* Never 0 */
+    ed->edDocumentId= ea->eaNextDocumentId;
+
     if  ( filename )
 	{ ed->edFilename= strdup( filename );	}
     else{ ed->edFilename= (char *)0;		}
@@ -552,9 +573,7 @@ int appSetupDocument(	EditApplication *	ea,
 
     appSetDrawingEnvironment( add, ea->eaMagnification, xfac,
 			screenPixelsPerMM,
-			ea->eaAfmDirectory,
-			ea->eaGhostscriptFontmap,
-			ea->eaGhostscriptFontToXmapping,
+			&(ea->eaPostScriptFontList),
 			ea->eaToplevel.atTopWidget );
 
     add->addPageGapPixels= (int)( ea->eaPageGapMM* verPixPerMM );
@@ -630,7 +649,7 @@ int appSetupDocument(	EditApplication *	ea,
     gtk_widget_realize( ed->edToplevel.atTopWidget );
 #   endif
 
-    if  ( ea->eaFinishDocumentSetup && (*ea->eaFinishDocumentSetup)( ea, ed ) )
+    if  ( ea->eaFinishDocumentSetup && (*ea->eaFinishDocumentSetup)( ed ) )
 	{ SDEB(ed->edFilename); return -1; }
 
     appShowShellWidget( ed->edToplevel.atTopWidget );
@@ -765,6 +784,69 @@ int appNewDocument(	EditApplication *	ea,
     return 0;
     }
 
+/************************************************************************/
+/*									*/
+/*  Find out whether a document can be saved in a certain format.		*/
+/*									*/
+/************************************************************************/
+
+int appDocumentTestCanSave(	EditApplication *		ea,
+				const AppFileExtension *	afe,
+				void *				privateData,
+				unsigned int			useFlags,
+				int				format )
+    {
+    if  ( ( afe->afeUseFlags & useFlags ) != useFlags )
+	{ return -1;	}
+
+    if  ( ea->eaCanSaveDocument					&&
+	  (*ea->eaCanSaveDocument)( privateData, format )	)
+	{ return -1;	}
+
+    return 0;
+    }
+
+int appDocumentGetSaveFormat(	EditApplication *		ea,
+				const char *			filename,
+				void *				privateData,
+				unsigned int			useFlags,
+				int				format )
+    {
+    const char *	ext;
+    AppFileExtension *	afe;
+    int			i;
+
+    ext= appFileExtensionOfName( filename );
+
+    if  ( format >= 0 && format < ea->eaFileExtensionCount )
+	{
+	afe= ea->eaFileExtensions+ format;
+
+	if  ( ! appDocumentTestCanSave( ea, afe, privateData,
+							useFlags, format ) )
+	    {
+	    if  ( ! ext					||
+		  ! afe->afeExtension			||
+		  ! strcmp( ext, afe->afeExtension )	)
+		{ return format;	}
+	    }
+	}
+
+    afe= ea->eaFileExtensions;
+    for ( i= 0; i < ea->eaFileExtensionCount; afe++, i++ )
+	{
+	if  ( appDocumentTestCanSave( ea, afe, privateData, useFlags, i ) )
+	    { continue;	}
+
+	if  ( ! ext					||
+	      ! afe->afeExtension			||
+	      ! strcmp( ext, afe->afeExtension )	)
+	    { return i;	}
+	}
+
+    return -1;
+    }
+
 /************************************************************************/
 /*									*/
 /*  File Conversion entry point. ea is supposed to be initialized.	*/
@@ -779,14 +861,9 @@ int appFileConvert(	EditApplication *	ea,
     {
     int			interactive= ( ea->eaToplevel.atTopWidget != NULL );
 
-    const char *	slash;
-    const char *	dot;
-    AppFileExtension *	afe;
-
     void *		privateData= (void *)0;
     int			fromFormat= -1;
-
-    int			i;
+    int			toFormat;
 
     if  ( ! strcmp( fromName, toName ) )
 	{ SSDEB(fromName,toName); return -1;	}
@@ -803,33 +880,18 @@ int appFileConvert(	EditApplication *	ea,
 						(APP_WIDGET)0, fromName ) )
 	{ SDEB(fromName); return -1; }
 
-    slash= strrchr( toName, '/' );
-    if  ( slash )
-	{ dot= strrchr( slash+ 1, '.' );	}
-    else{ dot= strrchr( toName, '.' );	}
+    toFormat= appDocumentGetSaveFormat( ea, toName, privateData,
+						APPFILE_CAN_SAVE, fromFormat );
 
-    afe= ea->eaFileExtensions;
-    for ( i= 0; i < ea->eaFileExtensionCount; afe++, i++ )
+    if  ( toFormat >= 0 )
 	{
-	if  ( ! ( afe->afeUseFlags & APPFILE_CAN_SAVE ) )
-	    { continue;	}
-
-	if  ( ea->eaCanSaveDocument				&&
-	      (*ea->eaCanSaveDocument)( privateData, i )	)
-	    { continue;	}
-
-	if  ( dot					&&
-	      afe->afeExtension			&&
-	      strcmp( dot+ 1, afe->afeExtension )	)
-	    { continue;	}
-
-	if  ( (*ea->eaSaveDocument)( privateData, i,
+	if  ( (*ea->eaSaveDocument)( privateData, toFormat,
 				ea->eaNameAndVersion, toName, toName ) )
 	    {
 	    if  ( interactive )
 		{
 		appReportSaveFailure( ea, (APP_WIDGET)0,
-					    ea->eaToplevel.atTopWidget, toName );
+					ea->eaToplevel.atTopWidget, toName );
 		}
 
 	    (*ea->eaFreeDocument)( privateData, fromFormat,
@@ -842,7 +904,7 @@ int appFileConvert(	EditApplication *	ea,
 	return 0;
 	}
 
-    SSDEB(fromName,toName);
+    SSLDEB(fromName,toName,toFormat);
     (*ea->eaFreeDocument)( privateData, fromFormat, (AppDrawingData *)0 );
     return -1;
     }
@@ -854,9 +916,7 @@ int appFileConvert(	EditApplication *	ea,
 /*									*/
 /************************************************************************/
 
-void appDestroyEditDocument(	APP_WIDGET		w,
-				void *			voided,
-				void *			callData )
+APP_DESTROY_CALLBACK_H( appDestroyEditDocument, w, voided )
     {
     EditDocument *		ed= (EditDocument *)voided;
     EditApplication *		ea= ed->edApplication;
@@ -865,6 +925,7 @@ void appDestroyEditDocument(	APP_WIDGET		w,
 	{
 	(*ea->eaFreeDocument)( ed->edPrivateData, ed->edFormat,
 						    &(ed->edDrawingData) );
+	ed->edPrivateData= (void *)0;
 	}
 
     appFreeDocument( ea, ed );
@@ -878,7 +939,7 @@ void appDestroyEditDocument(	APP_WIDGET		w,
 /*									*/
 /************************************************************************/
 
-APP_EVENT_HANDLER( appDocExposeHandler, w, voided, exposeEvent )
+APP_EVENT_HANDLER_H( appDocExposeHandler, w, voided, exposeEvent )
     {
     EditDocument *		ed= (EditDocument *)voided;
     AppDrawingData *		add= &(ed->edDrawingData);
@@ -889,6 +950,12 @@ APP_EVENT_HANDLER( appDocExposeHandler, w, voided, exposeEvent )
 
     DocumentRectangle		drClip;
 
+    if  ( ed->edNotYetDrawn				&&
+	  ea->eaDocumentFirstVisible			)
+	{ (*ea->eaDocumentFirstVisible)( ed );	}
+
+    ed->edNotYetDrawn= 0;
+
     /*  1  */
     appCollectExposures( &drClip, add, ox, oy, exposeEvent );
 
@@ -965,3 +1032,28 @@ void appDocExposeRectangle(	const EditDocument *		ed,
 	}
     }
 
+int appPostScriptFontCatalog(		EditApplication *	ea )
+    {
+    const int		mapNames= 1;
+
+    if  ( ea->eaPostScriptFontList.psflFamilyCount > 0 )
+	{ return 0;	}
+
+    if  ( psFontCatalog( &(ea->eaPostScriptFontList), ea->eaAfmDirectory ) )
+	{ SDEB(ea->eaAfmDirectory); return -1;	}
+
+    if  ( ea->eaGhostscriptFontmap			&&
+	  ! ea->eaGhostscriptMappingsRead		)
+	{
+	if  ( utilFontmapReadMap( ea->eaGhostscriptFontmap ) )
+	    { SDEB(ea->eaGhostscriptFontmap); return -1;	}
+
+	if  ( utilSetT1EntriesForFonts( ea->eaGhostscriptFontToXmapping,
+				    mapNames, &(ea->eaPostScriptFontList) ) )
+	    { SDEB(ea->eaGhostscriptFontToXmapping); return -1; }
+
+	ea->eaGhostscriptMappingsRead= 1;
+	}
+
+    return 0;
+    }
