@@ -5,8 +5,8 @@
 
 #   include	<utilPropMask.h>
 
-#   include	<appGeo.h>
-#   include	<docFont.h>
+#   include	<geo2DInteger.h>
+#   include	<utilDocFont.h>
 #   include	<psFont.h>
 #   include	<bmcolor.h>
 
@@ -43,32 +43,30 @@
 /*									*/
 /************************************************************************/
 
-typedef struct AppPhysicalFont
+typedef struct DrawScreenFont
     {
-    TextAttribute	apfAttribute;
-    int			apfDocFamilyNumber;
-    int			apfFontEncoding;
-
-    int			apfPsFamilyNumber;
-    int			apfPsFaceNumber;
+			/*  KEY  */
+    short int		apfPsFamilyNumber;
+    short int		apfFontEncoding;
+    short int		apfFaceIndex;
+    short int		apfFullSizePixels;
 
+			/*  DEP  */
+    short int		apfUnderLinePositionPixels;
+    short int		apfUnderlineThicknessPixels;
+    short int		apfAscentPixels;
+    short int		apfScapsSizePixels;
+    short int		apfDescentPixels;
+    short int		apfXHeightPixels;
     APP_FONT *		apfFontStruct;
-    unsigned long	apfUnderLinePositionPixels;
-    unsigned long	apfUnderlineThicknessPixels;
-    int			apfAscentPixels;
-    int			apfDescentPixels;
-    int			apfFullSizePixels;
-    unsigned long	apfXHeightPixels;
-    } AppPhysicalFont;
+    APP_FONT *		apfScapsFontStruct;
+    } DrawScreenFont;
 
-typedef struct AppPhysicalFontList
+typedef struct DrawScreenFontList
     {
-    const char *	apflAfmDirectory;
-    const char *	apflGhostscriptFontmap;
-    const char *	apflGhostscriptFontToXmapping;
-    int			apflCount;
-    AppPhysicalFont *	apflFonts;
-    } AppPhysicalFontList;
+    int			apflFontCount;
+    DrawScreenFont *	apflFonts;
+    } DrawScreenFontList;
 
 typedef struct AppDrawingData
     {
@@ -86,7 +84,8 @@ typedef struct AppDrawingData
     double			addMagnifiedPixelsPerTwip;
     double			addScreenPixelsPerMM;
 
-    AppPhysicalFontList		addPhysicalFontList;
+    const PostScriptFontList *	addPostScriptFontList;
+    DrawScreenFontList		addScreenFontList;
 
     APP_COLOR_RGB		addForeColor;
     APP_COLOR_RGB		addBackColor;
@@ -147,6 +146,12 @@ typedef struct AppColors
 #   endif
     } AppColors;
 
+#   ifdef USE_MOTIF
+
+extern const char * const APP_VisualClasses[];
+
+#   endif
+
 /************************************************************************/
 /*									*/
 /*  Procedure declarations.						*/
@@ -170,44 +175,28 @@ extern void appInitDrawingData(		AppDrawingData *	add );
 extern void appCleanDrawingData(	AppDrawingData *	add );
 
 
-extern void appInitFontList(	AppPhysicalFontList *	apfl );
+extern void appInitFontList(	DrawScreenFontList *	apfl );
 extern void appCleanFontList(	AppDrawingData *	add,
-				AppPhysicalFontList *	apfl );
+				DrawScreenFontList *	apfl );
 
 extern int appFontCatalog(	AppDrawingData *	add,
 				AppFontFamily **	pFamilies,
 				int *			pCount		);
 
-extern void appFontFormatCurrent(	char *			target,
-					AppFontFamily *		aff,
-					AppFontTypeface *	aft,
-					int			size	);
-
-extern int appOpenDocumentFont(	AppDrawingData *		add,
+extern int appOpenScreenFont(	AppDrawingData *		add,
 				const DocumentFontList *	dfl,
-				TextAttribute			ta );
-
-extern void appFontAttributeString(	char *			target,
-					const char *		familyName,
-					const PropertyMask *	updMask,
-					TextAttribute		taNew );
+				const TextAttribute *		ta );
 
 extern int appFontXFont(	char *			target,
 				AppDrawingData *	add,
 				const AppFontFamily *	aff,
 				int			encoding,
-				AppFontTypeface *	aft,
-				int			twipsSize,
-				int			variant );
+				const AppFontTypeface *	aft,
+				int			pixelSize );
 
 extern int appCharExistsInFont(	const APP_FONT *	xfs,
 				int			ch );
 
-extern int appFontGetAttributes(	char *		attributeString,
-					char **		pFamilyName,
-					PropertyMask *	pUpdMask,
-					TextAttribute *	pTaNew );
-
 extern void appInitVerticalXFont(	VerticalXFont *	vxf );
 
 extern void appCleanVerticalXFont(	VerticalXFont *		vxf,
@@ -227,14 +216,12 @@ extern int appDrawVerticalString(	VerticalXFont *		vxf,
 					int			length );
 
 extern void appSetDrawingEnvironment(
-			AppDrawingData *	add,
-			double			magnification,
-			double			xfac,
-			double			screenPixPerMM,
-			const char *		afmDirectory,
-			const char *		apflGhostscriptFontmap,
-			const char *		ghostscriptFontToXmapping,
-			APP_WIDGET		appWidget );
+			AppDrawingData *		add,
+			double				magnification,
+			double				xfac,
+			double				screenPixPerMM,
+			const PostScriptFontList *	psfl,
+			APP_WIDGET			appWidget );
 
 extern void appExposeRectangle(		const AppDrawingData *	add,
 					int			x,
@@ -350,14 +337,15 @@ extern void appDrawDrawPixmap(		AppDrawingData *	add,
 					int			xDest,
 					int			yDest );
 
-extern int appGetCoordinatesFromMouseButtonEvent( int *		pX,
-						int *		pY,
-						int *		pButton,
-						int *		pUpDown,
-						int *		pSeq,
-						unsigned int *	pKeyState,
-						APP_WIDGET	w,
-						APP_EVENT *	event );
+extern int appGetCoordinatesFromMouseButtonEvent(
+					int *			pX,
+					int *			pY,
+					int *			pButton,
+					int *			pUpDown,
+					int *			pSeq,
+					unsigned int *		pKeyState,
+					const APP_WIDGET	w,
+					const APP_EVENT *	event );
 
 extern int appGetCoordinatesFromMouseMoveEvent( int *		pX,
 						int *		pY,
@@ -411,19 +399,19 @@ extern void appDrawFillArc(	AppDrawingData *	add,
 				int			angle2 );
 
 extern void appDrawSetRedrawHandler(	APP_WIDGET		w,
-					APP_EVENT_HANDLER	handler,
+					APP_EVENT_HANDLER_T	handler,
 					void *			through );
 
 extern void appDrawSetConfigureHandler(	APP_WIDGET		w,
-					APP_EVENT_HANDLER	handler,
+					APP_EVENT_HANDLER_T	handler,
 					void *			through );
 
 extern void appDrawSetButtonPressHandler( APP_WIDGET		w,
-					APP_EVENT_HANDLER	handler,
+					APP_EVENT_HANDLER_T	handler,
 					void *			through );
 
 extern void appDrawSetKeyboardHandler( APP_WIDGET		w,
-					APP_EVENT_HANDLER	handler,
+					APP_EVENT_HANDLER_T	handler,
 					void *			through );
 
 extern int appDrawGetInoutFromFocusEvent(	int *		pInOut,
