@@ -17,8 +17,7 @@ typedef struct AppEncodingMenu
     AppOptionmenu	aemEncodingOptionmenu;
     APP_WIDGET		aemFontEncodingOptions[ENCODINGps_COUNT];
 
-    int			aemFontEncodingSet;
-    int			aemFontEncodingChosen;
+    int			aemFontEncoding;
     } AppEncodingMenu;
 
 /************************************************************************/
@@ -27,19 +26,22 @@ typedef struct AppEncodingMenu
 /*									*/
 /************************************************************************/
 
-extern void appEncodingMenuSetOptionmenu(
+extern void appEncodingMenuUnsetEncoding(
+				AppEncodingMenu *		aem );
+
+extern void appEncodingMenuSetEncoding(
 				AppEncodingMenu *		aem,
-				const SupportedCharset *	sc,
+				const DocumentFontFamily *	dff,
 				int				enc );
 
-extern int appEncodingMenuEncodingFromWidget(
-				int *				pEnc,
-				const AppEncodingMenu *		aem,
-				APP_WIDGET			w );
+extern void appEncodingMenuSetDocEncoding(
+				AppEncodingMenu *	aem,
+				const int *		fontForEncoding,
+				int			enc );
 
 extern void appEncodingMenuFillOptionmenu(
-				char **				opts,
-				APP_BUTTON_CALLBACK		callBack,
+				char * const *			opts,
+				APP_OITEM_CALLBACK_T		callBack,
 				void *				through,
 				AppEncodingMenu *		aem );
 
@@ -49,5 +51,5 @@ extern void appEncodingMenuGetOptionTexts(
 
 extern void appEncodingMenuAdaptToFamilyEncodings(
 				AppEncodingMenu *		aem,
-				const AppFontFamily *		aff );
+				const DocumentFontFamily *	dff );
 
