@@ -9,9 +9,6 @@
 #   include	<stdlib.h>
 #   include	<stdio.h>
 #   include	<stddef.h>
-#   include	<appDebugon.h>
-
-#   include	<reg.h>
 
 #   include	"appFrame.h"
 
@@ -19,6 +16,8 @@
 #   include	<X11/keysym.h>
 #   endif
 
+#   include	<appDebugon.h>
+
 /************************************************************************/
 /*									*/
 /*  Represents a find tool.						*/
@@ -61,6 +60,7 @@ typedef struct AppFindTool
     FindToolFind		aftFindNext;
     FindToolFind		aftFindPrev;
     FindToolReplace		aftReplace;
+    FindToolSetPattern		aftSetPattern;
 
     void *			aftTarget;
 
@@ -136,9 +136,7 @@ static void appFindToolSetFindResult(	AppFindTool *	aft,
     return;
     }
 
-static void appFindToolPatternChanged(	APP_WIDGET	w,
-					void *		voidaft,
-					void *		call_data )
+static APP_TXTYPING_CALLBACK_H( appFindToolPatternChanged, w, voidaft )
     {
     AppFindTool *	aft= (AppFindTool *)voidaft;
 
@@ -147,9 +145,7 @@ static void appFindToolPatternChanged(	APP_WIDGET	w,
     return;
     }
 
-static void appFindToolReplacementChanged(	APP_WIDGET	w,
-						void *		voidaft,
-						void *		call_data )
+static APP_TXTYPING_CALLBACK_H( appFindToolReplacementChanged, w, voidaft )
     {
     AppFindTool *	aft= (AppFindTool *)voidaft;
 
@@ -164,43 +160,26 @@ static void appFindToolReplacementChanged(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static int appFindToolGetProgram(	regProg **	pProg,
-					AppFindTool *	aft )
+static int appFindToolSetProgram(	AppFindTool *	aft )
     {
     char *		pattern;
-    regProg *		prog;
+    int			res;
+
+    if  ( ! aft->aftSetPattern )
+	{ XDEB(aft->aftSetPattern); return -1;	}
 
     pattern= appGetStringFromTextWidget( aft->aftPatternText );
 
-    if  ( aft->aftUseRegex )
-	{
-	prog= regCompile( (const unsigned char *)pattern );
-	if  ( ! prog )
-	    {
-	    SXDEB(pattern,prog);
-	    appFreeStringFromTextWidget( pattern ); return -1;
-	    }
-	}
-    else{
-	prog= regCompileEscaped( (const unsigned char *)pattern );
-	if  ( ! prog )
-	    {
-	    SXDEB(pattern,prog);
-	    appFreeStringFromTextWidget( pattern ); return -1;
-	    }
-	}
+    res= (*aft->aftSetPattern)( aft->aftTarget,
+				(unsigned char *)pattern, aft->aftUseRegex );
 
     appFreeStringFromTextWidget( pattern );
 
-    *pProg= prog; return 0;
+    return res;
     }
 
-static void appFindToolFindNext(	APP_WIDGET	w,
-					void *		voidaft,
-					void *		voidpbcs )
+static void appFindToolFindNext(	AppFindTool *	aft )
     {
-    AppFindTool *	aft= (AppFindTool *)voidaft;
-    regProg *		prog;
     int			result;
 
     if  ( ! aft->aftFindNext )
@@ -209,23 +188,27 @@ static void appFindToolFindNext(	APP_WIDGET	w,
 	appFindToolSetFindResult( aft, -1 ); return;
 	}
 
-    if  ( appFindToolGetProgram( &prog, aft ) )
+    if  ( appFindToolSetProgram( aft ) )
 	{ appFindToolSetFindResult( aft, -1 ); return; }
 
-    result= (*aft->aftFindNext)( aft->aftTarget, prog );
+    result= (*aft->aftFindNext)( aft->aftTarget );
     appFindToolSetFindResult( aft, result );
 
-    free( prog );
-
     return;
     }
 
-static void appFindToolFindPrev(	APP_WIDGET	w,
-					void *		voidaft,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( appFindToolFindNextPushed, w, voidaft )
+    {
+    AppFindTool *	aft= (AppFindTool *)voidaft;
+
+    appFindToolFindNext( aft );
+
+    return;
+    }
+
+static APP_BUTTON_CALLBACK_H( appFindToolFindPrev, w, voidaft )
     {
     AppFindTool *	aft= (AppFindTool *)voidaft;
-    regProg *		prog;
     int			result;
 
     if  ( ! aft->aftFindPrev )
@@ -234,14 +217,12 @@ static void appFindToolFindPrev(	APP_WIDGET	w,
 	appFindToolSetFindResult( aft, -1 ); return;
 	}
 
-    if  ( appFindToolGetProgram( &prog, aft ) )
+    if  ( appFindToolSetProgram( aft ) )
 	{ appFindToolSetFindResult( aft, -1 ); return; }
 
-    result= (*aft->aftFindPrev)( aft->aftTarget, prog );
+    result= (*aft->aftFindPrev)( aft->aftTarget );
     appFindToolSetFindResult( aft, result );
 
-    free( prog );
-
     return;
     }
 
@@ -251,11 +232,8 @@ static void appFindToolFindPrev(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void appFindToolReplaceSelection(	APP_WIDGET	w,
-						void *		voidaft,
-						void *		voidpbcs )
+static void appFindToolReplaceSelection( AppFindTool *	aft )
     {
-    AppFindTool *	aft= (AppFindTool *)voidaft;
     char *		replacement;
 
     if  ( ! aft->aftReplace )
@@ -272,20 +250,25 @@ static void appFindToolReplaceSelection(	APP_WIDGET	w,
     return;
     }
 
-static void appFindToolReplaceNext(	APP_WIDGET	w,
-					void *		voidaft,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( appFindToolReplaceSelectionPushed, w, voidaft )
     {
-    appFindToolReplaceSelection( w, voidaft, voidpbcs );
+    AppFindTool *	aft= (AppFindTool *)voidaft;
 
-    appFindToolFindNext( w, voidaft, voidpbcs );
+    appFindToolReplaceSelection( aft );
+    }
+
+static APP_BUTTON_CALLBACK_H( appFindToolReplaceNext, w, voidaft )
+    {
+    AppFindTool *	aft= (AppFindTool *)voidaft;
+
+    appFindToolReplaceSelection( aft );
+
+    appFindToolFindNext( aft );
 
     return;
     }
 
-static void appFindRegexToggled(	APP_WIDGET	w,
-					void *		voidaft,
-					void *		voidtbcs )
+static APP_TOGGLE_CALLBACK_H( appFindRegexToggled, w, voidaft, voidtbcs )
     {
     AppFindTool *	aft= (AppFindTool *)voidaft;
     int			set;
@@ -307,8 +290,8 @@ static void appFindToolMakeButtonRow(	APP_WIDGET *		pLeftButton,
 					APP_WIDGET *		pRightButton,
 					char *			leftLabel,
 					char *			rightLabel,
-					APP_BUTTON_CALLBACK	leftCallback,
-					APP_BUTTON_CALLBACK	rightCallback,
+					APP_BUTTON_CALLBACK_T	leftCallback,
+					APP_BUTTON_CALLBACK_T	rightCallback,
 					AppFindTool *		aft,
 					APP_WIDGET		parent )
 
@@ -380,7 +363,7 @@ static APP_WIDGET appFindMakeFindFrame(	APP_WIDGET		parent,
     appFindToolMakeButtonRow(
 		&(aft->aftFindNextButton), &(aft->aftFindPrevButton),
 		aftr->aftrFindNext, aftr->aftrFindPrevious,
-		appFindToolFindNext, appFindToolFindPrev,
+		appFindToolFindNextPushed, appFindToolFindPrev,
 		aft, paned );
 
     /***************/
@@ -423,7 +406,7 @@ static APP_WIDGET appFindMakeReplaceFrame( APP_WIDGET		parent,
     appFindToolMakeButtonRow(
 		&(aft->aftReplaceButton), &(aft->aftReplaceNextButton),
 		aftr->aftrReplaceFound, aftr->aftrReplaceNext,
-		appFindToolReplaceSelection, appFindToolReplaceNext,
+		appFindToolReplaceSelectionPushed, appFindToolReplaceNext,
 		aft, paned );
 
     /***************/
@@ -445,7 +428,7 @@ static void appDestroyFindTool(	AppFindTool *	aft )
     free( aft );
     }
 
-static APP_CLOSE_CALLBACK( appCloseFindTool, w, voidaft )
+static APP_CLOSE_CALLBACK_H( appCloseFindTool, w, voidaft )
     {
     AppFindTool *	aft= (AppFindTool *)voidaft;
 
@@ -455,7 +438,7 @@ static APP_CLOSE_CALLBACK( appCloseFindTool, w, voidaft )
     }
 
 #   ifdef USE_MOTIF
-static void appFindToolCancel(		APP_WIDGET		w,
+static void appFindToolKey(		APP_WIDGET		w,
 					void *			voidaft,
 					XEvent *		event,
 					Boolean *		pRefused )
@@ -467,6 +450,9 @@ static void appFindToolCancel(		APP_WIDGET		w,
 
 	if  ( XKeysymToKeycode( XtDisplay(w), XK_Escape ) == kevent->keycode )
 	    { appDestroyFindTool( aft ); *pRefused= 0; return; }
+
+	if  ( XKeysymToKeycode( XtDisplay(w), XK_F3 ) == kevent->keycode )
+	    { appFindToolFindNext( aft ); *pRefused= 0; return; }
 	}
 
     *pRefused= 1; return;
@@ -564,6 +550,7 @@ void * appMakeFindTool(		APP_WIDGET		findOption,
 				FindToolFind		findNext,
 				FindToolFind		findPrev,
 				FindToolReplace		replace,
+				FindToolSetPattern	setPattern,
 				void *			target )
     {
     AppFindTool *	aft;
@@ -592,6 +579,7 @@ void * appMakeFindTool(		APP_WIDGET		findOption,
     aft->aftDestroy= destroy;
     aft->aftFindNext= findNext;
     aft->aftFindPrev= findPrev;
+    aft->aftSetPattern= setPattern;
     aft->aftReplace= replace;
     aft->aftTarget= target;
     aft->aftVisible= 0;
@@ -620,19 +608,19 @@ void * appMakeFindTool(		APP_WIDGET		findOption,
 
 #   ifdef USE_MOTIF
     XtInsertEventHandler( aft->aftPatternText, KeyReleaseMask, False,
-				appFindToolCancel, (void *)aft, XtListHead );
+				appFindToolKey, (void *)aft, XtListHead );
     XtInsertEventHandler( aft->aftReplaceText, KeyReleaseMask, False,
-				appFindToolCancel, (void *)aft, XtListHead );
+				appFindToolKey, (void *)aft, XtListHead );
 
     XtInsertEventHandler( aft->aftReplaceButton, KeyReleaseMask, False,
-				appFindToolCancel, (void *)aft, XtListHead );
+				appFindToolKey, (void *)aft, XtListHead );
     XtInsertEventHandler( aft->aftReplaceNextButton, KeyReleaseMask, False,
-				appFindToolCancel, (void *)aft, XtListHead );
+				appFindToolKey, (void *)aft, XtListHead );
 
     XtInsertEventHandler( aft->aftFindNextButton, KeyReleaseMask, False,
-				appFindToolCancel, (void *)aft, XtListHead );
+				appFindToolKey, (void *)aft, XtListHead );
     XtInsertEventHandler( aft->aftFindPrevButton, KeyReleaseMask, False,
-				appFindToolCancel, (void *)aft, XtListHead );
+				appFindToolKey, (void *)aft, XtListHead );
 #   endif
 
     return (void *)aft;
@@ -705,11 +693,29 @@ void appEnableFindTool(		void *	voidaft,
     return;
     }
 
-void appFindToolDisableReplace(		void *	voidaft )
+void appFindToolEnableReplace(	void *	voidaft,
+				int	enabled )
     {
     AppFindTool *	aft= (AppFindTool *)voidaft;
 
-    appGuiEnableWidget( aft->aftReplaceFrame, 0 );
+    appGuiEnableWidget( aft->aftReplaceFrame, enabled );
+
+    return;
+    }
+
+void appFindToolSetPattern(	void *			voidaft,
+				const unsigned char *	pattern,
+				int			useRegex )
+    {
+    AppFindTool *	aft= (AppFindTool *)voidaft;
+
+    aft->aftUseRegex= useRegex;
+
+    appStringToTextWidget( aft->aftPatternText, (const char *)pattern );
+
+    appGuiSetToggleState( aft->aftRegexToggle, aft->aftUseRegex );
+
+    appFindToolReflectPatternText( aft );
 
     return;
     }
