@@ -2,7 +2,6 @@
 
 #   include	<stdlib.h>
 #   include	<stdio.h>
-#   include	<appDebugon.h>
 
 #   include	"appFrame.h"
 #   include	"appSystem.h"
@@ -29,6 +28,8 @@
 #   include	<Xm/SashP.h>
 #   include	<Xm/DrawingA.h>
 
+#   include	<appDebugon.h>
+
 /************************************************************************/
 /*									*/
 /*  Tame the sashes in an XmPanedWindow.				*/
@@ -86,6 +87,9 @@ void appEmptyParentWidget(	Widget		parent )
     if  ( childCount == 0 )
 	{ return;	}
 
+    /*  1  */
+    XtUnmanageChildren( children, childCount );
+
     save= (WidgetList)malloc( childCount* sizeof(Widget) );
     if  ( save )
 	{
@@ -95,11 +99,7 @@ void appEmptyParentWidget(	Widget		parent )
 	    { save[i]= children[i];	}
 
 	for ( i= childCount- 1; i >= 0; i-- )
-	    {
-	    /*  1  */
-	    XtUnmanageChild( save[i] );
-	    XtDestroyWidget( save[i] );
-	    }
+	    { XtDestroyWidget( save[i] );	}
 
 	free( save );
 	}
@@ -230,7 +230,7 @@ char * APP_X11EventNames[]=
 void appMakeButtonInRow(	APP_WIDGET *		pButton,
 				APP_WIDGET		row,
 				const char *		text,
-				APP_BUTTON_CALLBACK	callback,
+				APP_BUTTON_CALLBACK_T	callback,
 				void *			through,
 				int			position,
 				int			showAsDefault )
@@ -287,7 +287,7 @@ void appMakeButtonInRow(	APP_WIDGET *		pButton,
 
 APP_WIDGET appMakeToggleInRow(	APP_WIDGET		row,
 				const char *		text,
-				APP_TOGGLE_CALLBACK	callback,
+				APP_TOGGLE_CALLBACK_T	callback,
 				void *			through,
 				int			position )
     {
@@ -315,7 +315,7 @@ APP_WIDGET appMakeToggleInRow(	APP_WIDGET		row,
 
     XtSetArg( al[ac], XmNrecomputeSize,		True ); ac++;
 
-    XtSetArg( al[ac], XmNmarginTop,		6 ); ac++;
+    XtSetArg( al[ac], XmNmarginTop,		TBrmargT ); ac++;
 
     XtSetArg( al[ac], XmNalignment,		XmALIGNMENT_BEGINNING ); ac++;
 
@@ -355,56 +355,6 @@ APP_WIDGET appMakeRowInColumn(	APP_WIDGET	column,
     return row;
     }
 
-/************************************************************************/
-/*									*/
-/*  Make a row with a label and a text widget.				*/
-/*									*/
-/************************************************************************/
-
-void appMakeLabelInRow(		APP_WIDGET *		pLabel,
-				APP_WIDGET		row,
-				int			column,
-				int			colspan,
-				const char *		labelText )
-    {
-    Widget		label;
-
-    XmString		labelString;
-
-    Arg			al[20];
-    int			ac= 0;
-
-    labelString= XmStringCreateLocalized( (char *)labelText );
-
-    ac= 0;
-    XtSetArg( al[ac], XmNtopAttachment,		XmATTACH_FORM ); ac++;
-    XtSetArg( al[ac], XmNtopOffset,		0 ); ac++;
-
-    XtSetArg( al[ac], XmNleftAttachment,	XmATTACH_POSITION ); ac++;
-    XtSetArg( al[ac], XmNleftPosition,		column ); ac++;
-
-    XtSetArg( al[ac], XmNrightAttachment,	XmATTACH_POSITION ); ac++;
-    XtSetArg( al[ac], XmNrightPosition,		column+ colspan ); ac++;
-
-    XtSetArg( al[ac], XmNrecomputeSize,		True ); ac++;
-    XtSetArg( al[ac], XmNalignment,		XmALIGNMENT_BEGINNING ); ac++;
-    XtSetArg( al[ac], XmNlabelString,		labelString ); ac++;
-
-#   if 1
-    XtSetArg( al[ac], XmNmarginTop,		6 ); ac++;
-#   else
-    XtSetArg( al[ac], XmNmarginHeight,		8 ); ac++;
-#   endif
-
-    label= XmCreateLabel( row, WIDGET_NAME, al, ac );
-
-    XmStringFree( labelString );
-
-    XtManageChild( label );
-
-    *pLabel= label;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Make a frame, possibly with a title.				*/
@@ -471,22 +421,22 @@ void appMakeColumnFrameInColumn(	APP_WIDGET *		pFrame,
     appMakeFrame( &frame, parent, title );
 
     ac= 0;
-    XtSetArg( al[ac],	XmNmarginWidth,		0 ); ac++;
-    XtSetArg( al[ac],	XmNmarginHeight,	0 ); ac++;
+    XtSetArg( al[ac], XmNmarginWidth,		0 ); ac++;
+    XtSetArg( al[ac], XmNmarginHeight,	0 ); ac++;
     bboard= XmCreateForm( frame, WIDGET_NAME, al, ac );
 
     ac= 0;
-    XtSetArg( al[ac],	XmNleftAttachment,	XmATTACH_FORM ); ac++;
-    XtSetArg( al[ac],	XmNrightAttachment,	XmATTACH_FORM ); ac++;
-    XtSetArg( al[ac],	XmNtopAttachment,	XmATTACH_FORM ); ac++;
-    XtSetArg( al[ac],	XmNbottomAttachment,	XmATTACH_FORM ); ac++;
+    XtSetArg( al[ac], XmNleftAttachment,	XmATTACH_FORM ); ac++;
+    XtSetArg( al[ac], XmNrightAttachment,	XmATTACH_FORM ); ac++;
+    XtSetArg( al[ac], XmNtopAttachment,		XmATTACH_FORM ); ac++;
+    XtSetArg( al[ac], XmNbottomAttachment,	XmATTACH_FORM ); ac++;
 
-    XtSetArg( al[ac],	XmNsashWidth,		1 ); ac++;
-    XtSetArg( al[ac],	XmNsashHeight,		1 ); ac++;
-    XtSetArg( al[ac],	XmNseparatorOn,		False ); ac++;
-    XtSetArg( al[ac],	XmNmarginWidth,		5 ); ac++;
-    XtSetArg( al[ac],	XmNmarginHeight,	5 ); ac++;
-    XtSetArg( al[ac],	XmNspacing,		5 ); ac++;
+    XtSetArg( al[ac], XmNsashWidth,		1 ); ac++;
+    XtSetArg( al[ac], XmNsashHeight,		1 ); ac++;
+    XtSetArg( al[ac], XmNseparatorOn,		False ); ac++;
+    XtSetArg( al[ac], XmNmarginWidth,		PWmargW ); ac++;
+    XtSetArg( al[ac], XmNmarginHeight,		PWmargH ); ac++;
+    XtSetArg( al[ac], XmNspacing,		PWspacing ); ac++;
     paned= XmCreatePanedWindow( bboard, WIDGET_NAME, al, ac );
 
     XtManageChild( paned );
@@ -537,7 +487,8 @@ void appMakeRowFrameInColumn(	APP_WIDGET *		pFrame,
 
 void appMakeColumnInRow(	APP_WIDGET *	pColumn,
 				APP_WIDGET	row,
-				int		position )
+				int		position,
+				int		colspan )
     {
     Arg			al[20];
     int			ac= 0;
@@ -545,20 +496,20 @@ void appMakeColumnInRow(	APP_WIDGET *	pColumn,
     Widget		column;
 
     ac= 0;
-    XtSetArg( al[ac],	XmNtopAttachment,	XmATTACH_FORM ); ac++;
-    XtSetArg( al[ac],	XmNbottomAttachment,	XmATTACH_FORM ); ac++;
-    XtSetArg( al[ac],	XmNleftAttachment,	XmATTACH_POSITION ); ac++;
-    XtSetArg( al[ac],	XmNrightAttachment,	XmATTACH_POSITION ); ac++;
-    XtSetArg( al[ac],	XmNleftPosition,	position ); ac++;
-    XtSetArg( al[ac],	XmNrightPosition,	position+ 1 ); ac++;
+    XtSetArg( al[ac], XmNtopAttachment,		XmATTACH_FORM ); ac++;
+    XtSetArg( al[ac], XmNbottomAttachment,	XmATTACH_FORM ); ac++;
+    XtSetArg( al[ac], XmNleftAttachment,	XmATTACH_POSITION ); ac++;
+    XtSetArg( al[ac], XmNrightAttachment,	XmATTACH_POSITION ); ac++;
+    XtSetArg( al[ac], XmNleftPosition,		position ); ac++;
+    XtSetArg( al[ac], XmNrightPosition,		position+ colspan ); ac++;
 
-    XtSetArg( al[ac],	XmNsashWidth,		1 ); ac++;
-    XtSetArg( al[ac],	XmNsashHeight,		1 ); ac++;
-    XtSetArg( al[ac],	XmNseparatorOn,		False ); ac++;
-    XtSetArg( al[ac],	XmNmarginWidth,		5 ); ac++;
-    XtSetArg( al[ac],	XmNmarginHeight,	5 ); ac++;
-    XtSetArg( al[ac],	XmNspacing,		5 ); ac++;
-    XtSetArg( al[ac],	XmNallowResize,		True ); ac++;
+    XtSetArg( al[ac], XmNsashWidth,		1 ); ac++;
+    XtSetArg( al[ac], XmNsashHeight,		1 ); ac++;
+    XtSetArg( al[ac], XmNseparatorOn,		False ); ac++;
+    XtSetArg( al[ac], XmNmarginWidth,		PWmargW ); ac++;
+    XtSetArg( al[ac], XmNmarginHeight,		PWmargH ); ac++;
+    XtSetArg( al[ac], XmNspacing,		PWspacing ); ac++;
+    XtSetArg( al[ac], XmNallowResize,		True ); ac++;
 
     column= XmCreatePanedWindow( row, WIDGET_NAME, al, ac );
 
@@ -567,38 +518,6 @@ void appMakeColumnInRow(	APP_WIDGET *	pColumn,
     *pColumn= column; return;
     }
 
-void appMakeLabelInColumn(	APP_WIDGET *		pLabel,
-				APP_WIDGET		column,
-				const char *		labelText )
-    {
-    Widget		label;
-
-    XmString		labelString;
-
-    Arg			al[20];
-    int			ac= 0;
-
-    labelString= XmStringCreateLocalized( (char *)labelText );
-
-    ac= 0;
-    XtSetArg( al[ac], XmNskipAdjust,		True ); ac++;
-    XtSetArg( al[ac], XmNrecomputeSize,		True ); ac++;
-    XtSetArg( al[ac], XmNalignment,		XmALIGNMENT_BEGINNING ); ac++;
-    XtSetArg( al[ac], XmNlabelString,		labelString ); ac++;
-
-    XtSetArg( al[ac], XmNallowResize,		True ); ac++;
-
-    label= XmCreateLabel( column, WIDGET_NAME, al, ac );
-
-    XmStringFree( labelString );
-
-    XtManageChild( label );
-
-    appMotifTurnOfSashTraversal( column );
-
-    *pLabel= label; return;
-    }
-
 void appMakeImageInColumn(	APP_WIDGET *		pLabel,
 				APP_WIDGET		column,
 				APP_BITMAP_IMAGE	pixmap,
@@ -650,8 +569,8 @@ void appMakeColumnToggle(	Widget *		pToggle,
     XtSetArg( al[ac], XmNalignment,		XmALIGNMENT_BEGINNING ); ac++;
     XtSetArg( al[ac], XmNset,			set ); ac++;
 
-    XtSetArg( al[ac], XmNmarginTop,		1 ); ac++;
-    XtSetArg( al[ac], XmNmarginHeight,		5 ); ac++;
+    XtSetArg( al[ac], XmNmarginTop,		TBcmargT ); ac++;
+    XtSetArg( al[ac], XmNmarginHeight,		TBcmargH ); ac++;
 
     if  ( labelString )
 	{ XtSetArg( al[ac], XmNlabelString,	labelString ); ac++; }
@@ -726,7 +645,7 @@ void appMakeVerticalTool(	APP_WIDGET *		pShell,
 				const char *		widgetName,
 				int			userResizable,
 				APP_WIDGET		option,
-				APP_CLOSE_CALLBACK	closeCallback,
+				APP_CLOSE_CALLBACK_T	closeCallback,
 				void *			through )
     {
     Widget		shell;
@@ -793,12 +712,12 @@ void appMakeVerticalTool(	APP_WIDGET *		pShell,
     appSetCloseCallback( shell, ea, closeCallback, through );
 
     ac= 0;
-    XtSetArg( al[ac],	XmNsashWidth,		1 ); ac++;
-    XtSetArg( al[ac],	XmNsashHeight,		1 ); ac++;
-    XtSetArg( al[ac],	XmNseparatorOn,		False ); ac++;
-    XtSetArg( al[ac],	XmNmarginWidth,		5 ); ac++;
-    XtSetArg( al[ac],	XmNmarginHeight,	5 ); ac++;
-    XtSetArg( al[ac],	XmNspacing,		5 ); ac++;
+    XtSetArg( al[ac], XmNsashWidth,		1 ); ac++;
+    XtSetArg( al[ac], XmNsashHeight,		1 ); ac++;
+    XtSetArg( al[ac], XmNseparatorOn,		False ); ac++;
+    XtSetArg( al[ac], XmNmarginWidth,		PWmargW ); ac++;
+    XtSetArg( al[ac], XmNmarginHeight,		PWmargH ); ac++;
+    XtSetArg( al[ac], XmNspacing,		PWspacing ); ac++;
     paned= XmCreatePanedWindow( shell, WIDGET_NAME, al, ac );
 
     XtManageChild( paned );
@@ -1568,8 +1487,7 @@ int appGuiGetToggleState(	APP_WIDGET		toggle )
     return XmToggleButtonGetState( toggle ) != 0;
     }
 
-int appGuiGetToggleStateFromCallback(	APP_WIDGET		toggle,
-					void *			voidcbs )
+int appGuiGetToggleStateFromCallbackMotif( void *	voidcbs )
     {
     XmToggleButtonCallbackStruct *	cbs;
 
@@ -1624,23 +1542,6 @@ void appIconifyShellWidget(		APP_WIDGET		shell )
     XIconifyWindow( display, win, DefaultScreen( display ) );
     }
 
-void appGuiChangeLabelText(	APP_WIDGET	labelWidget,
-				const char *	label )
-    {
-    XmString			labelString;
-
-    labelString= XmStringCreateLocalized( (char *)label );
-
-    XtVaSetValues( labelWidget,
-			XmNlabelString,		labelString,
-			NULL );
-
-    XmStringFree( labelString );
-    
-    return;
-    }
-
-
 void appGuiChangeButtonText(	APP_WIDGET	button,
 				const char *	label )
     {
@@ -1719,7 +1620,7 @@ void appGuiMakeDrawingAreaInColumn( APP_WIDGET *		pDrawing,
 				    int				wide,
 				    int				high,
 				    int				heightResizable,
-				    APP_EVENT_HANDLER		redraw,
+				    APP_EVENT_HANDLER_T		redraw,
 				    void *			through )
     {
     APP_WIDGET	drawing;
@@ -1738,8 +1639,8 @@ void appGuiMakeDrawingAreaInColumn( APP_WIDGET *		pDrawing,
 	}
 
     if  ( high <= 0 || heightResizable )
-	{ XtSetArg( al[ac], XmNskipAdjust,	True ); ac++;	}
-    else{ XtSetArg( al[ac], XmNskipAdjust,	False ); ac++;	}
+	{ XtSetArg( al[ac], XmNskipAdjust,	False ); ac++;	}
+    else{ XtSetArg( al[ac], XmNskipAdjust,	True ); ac++;	}
 
     if  ( wide > 0 )
 	{
@@ -1763,76 +1664,6 @@ void appGuiMakeDrawingAreaInColumn( APP_WIDGET *		pDrawing,
     *pDrawing= drawing;
     }
 
-void appGuiGetStringFromKeyboardEvent(	APP_INPUT_CONTEXT	ic,
-					APP_WIDGET		w,
-					APP_EVENT *		event,
-					int *			pGotString,
-					int *			pGotKey,
-					unsigned int *		pState,
-					char *			buf,
-					int			capacity,
-					APP_KEY_VALUE *		pKey )
-    {
-    int			gotString;
-    XKeyPressedEvent *	keyEvent= &(event->xkey);
-
-    if  ( event->type != KeyPress )
-	{ LLDEB(event->type,KeyPress); *pGotString= 0; return;	}
-
-    if  ( ic )
-	{
-	Status	status;
-
-	gotString= XmbLookupString( ic, keyEvent, buf, capacity- 1,
-							    pKey, &status );
-	switch( status )
-	    {
-	    case XBufferOverflow:
-		LDEB(status);
-		*pGotString= 0;
-		*pGotKey= 0;
-		return;
-
-	    case XLookupNone:
-		*pGotString= 0;
-		*pGotKey= 0;
-		return;
-
-	    case XLookupBoth:
-		buf[gotString]= '\0';
-		*pGotString= gotString;
-		*pGotKey= 1;
-		*pState= keyEvent->state;
-		return;
-
-	    case XLookupChars:
-		buf[gotString]= '\0';
-		*pGotString= gotString;
-		*pGotKey= 0;
-		*pState= keyEvent->state;
-		return;
-
-	    case XLookupKeySym:
-		*pGotString= 0;
-		*pGotKey= 1;
-		*pState= keyEvent->state;
-		return;
-	    }
-	}
-    else{
-	gotString= XLookupString( keyEvent, buf, capacity- 1,
-					    pKey, (XComposeStatus *)0 );
-
-	if  ( gotString > 0 )
-	    { buf[gotString]= '\0';	}
-	*pGotString= gotString;
-	*pGotKey= 1;
-	*pState= keyEvent->state;
-	return;
-	}
-
-    }
-
 /************************************************************************/
 /*									*/
 /*  Calculate magnification factors.					*/
@@ -1932,8 +1763,8 @@ static void appDragMouseMove(		APP_WIDGET		w,
 void appRunDragLoop(	APP_WIDGET		w,
 			EditApplication *	ea,
 			APP_EVENT *		downEvent,
-			APP_EVENT_HANDLER	upHandler,
-			APP_EVENT_HANDLER	moveHandler,
+			APP_EVENT_HANDLER_T	upHandler,
+			APP_EVENT_HANDLER_T	moveHandler,
 			int			timerInterval,
 			APP_TIMER_CALLBACK	timerHandler,
 			void *			through )
@@ -1976,14 +1807,14 @@ void appRunDragLoop(	APP_WIDGET		w,
     }
 
 void appGuiSetFocusChangeHandler(	APP_WIDGET		w,
-					APP_EVENT_HANDLER	handler,
+					APP_EVENT_HANDLER_T	handler,
 					void *			through )
     {
     XtAddEventHandler( w, FocusChangeMask, False, handler, through );
     }
 
 void appSetDestroyCallback(	APP_WIDGET		shell,
-				APP_DESTROY_CALLBACK	destroyCallback,
+				APP_DESTROY_CALLBACK_T	destroyCallback,
 				void *			through )
     {
     if  ( destroyCallback )
@@ -1995,7 +1826,7 @@ void appSetDestroyCallback(	APP_WIDGET		shell,
 
 void appSetCloseCallback(	APP_WIDGET		shell,
 				EditApplication *	ea,
-				APP_CLOSE_CALLBACK	closeCallback,
+				APP_CLOSE_CALLBACK_T	closeCallback,
 				void *			through )
     {
     if  ( closeCallback && ea->eaCloseAtom > 0 )
