@@ -210,22 +210,17 @@ static void appMailDialogShowContent(	AppMailDialog *	amd )
     }
 
 /************************************************************************/
-/*  A content was chosen.						*/
+/*									*/
+/*  A content type was chosen.						*/
+/*									*/
 /************************************************************************/
 
-static void appContentChosen(	APP_WIDGET	w,
-				void *		voidamd,
-				void *		voidpbcs )
+static APP_OITEM_CALLBACK_H( appContentChosen, w, voidamd )
     {
     int			i;
     AppMailDialog *	amd= (AppMailDialog *)voidamd;
 
-    for ( i= 0; i < amd->amdContentOptionCount; i++ )
-	{
-	if  ( amd->amdContentOptions[i] == w )
-	    { break;	}
-	}
-
+    i= appGuiGetOptionmenuItemIndex(  &(amd->amdContentOptionmenu), w );
     if  ( i < 0 || i >= amd->amdContentOptionCount )
 	{ LLDEB(i,amd->amdContentOptionCount); return;	}
 
@@ -263,9 +258,7 @@ static void appMailCheckTo(	AppMailDialog *		amd )
     return;
     }
 
-static void appMailToChanged(	APP_WIDGET	w,
-				void *		voidamd,
-				void *		call_data )
+static APP_TXTYPING_CALLBACK_H( appMailToChanged, w, voidamd )
     {
     AppMailDialog *	amd= (AppMailDialog *)voidamd;
 
@@ -280,9 +273,7 @@ static void appMailToChanged(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void appMailDialogCancelPushed(	APP_WIDGET	w,
-					void *		voidamd,
-					void *		voidpbcs	 )
+static APP_BUTTON_CALLBACK_H( appMailDialogCancelPushed, w, voidamd )
     {
     AppMailDialog *	amd= (AppMailDialog *)voidamd;
 
@@ -291,9 +282,7 @@ static void appMailDialogCancelPushed(	APP_WIDGET	w,
     return;
     }
 
-static void appMailDialogSendPushed(	APP_WIDGET	w,
-					void *		voidamd,
-					void *		voidpbcs	 )
+static APP_BUTTON_CALLBACK_H( appMailDialogSendPushed, w, voidamd )
     {
     AppMailDialog *	amd= (AppMailDialog *)voidamd;
 
@@ -380,7 +369,7 @@ static void appMailDialogFillContentMenu(	AppMailDialog *		amd,
 /*  A find tool must be destroyed.					*/
 /************************************************************************/
 
-static APP_CLOSE_CALLBACK( appCloseMailDialog, w, voidamd )
+static APP_CLOSE_CALLBACK_H( appCloseMailDialog, w, voidamd )
     {
     AppMailDialog *	amd= (AppMailDialog *)voidamd;
 
@@ -547,7 +536,7 @@ static AppMailDialog * appMakeMailDialog( EditApplication *	ea,
 
     appMakeVerticalDialog( &(amd->amdDialog), &paned, ea,
 						appCloseMailDialog,
-						(APP_CALLBACK_FUNC)0,
+						(APP_DESTROY_CALLBACK_T)0,
 						(void *)amd,
 						ea->eaMailDialogName );
 
