@@ -65,7 +65,7 @@ void appAppFileMini(	APP_WIDGET	option,
 /*									*/
 /************************************************************************/
 
-APP_CLOSE_CALLBACK( appAppWmClose, w, voidea )
+APP_CLOSE_CALLBACK_H( appAppWmClose, w, voidea )
     {
     EditApplication *		ea= (EditApplication *)voidea;
 
@@ -368,7 +368,7 @@ static void appAppSetWindowsOption(	APP_WIDGET		menu,
     ami.amiItemText= label;
     ami.amiKey= (char *)0;
     ami.amiKeyText= (char *)0;
-    ami.amiCallback= appDocToFront;
+    ami.amiCallback= (APP_MENU_CALLBACK)appDocToFront;
 
     windowsOption= appSetToggleMenuItem( menu, &(ed->edToplevel),
 							&ami, (void *)ed );
@@ -417,66 +417,91 @@ void appSetDocument(	EditApplication *	ea,
 /************************************************************************/
 
 static AppConfigurableResource	APP_ApplicationResourceTable[]=
-    {
-	APP_RESOURCE( "unit",
-		    offsetof(EditApplication,eaUnitString),
-		    "inch" ),
-	APP_RESOURCE( "paper",
-		    offsetof(EditApplication,eaPaperString),
-		    "letter" ),
-	APP_RESOURCE( "leftMargin",
-		    offsetof(EditApplication,eaLeftMarginString),
-		    "90pt" ),
-	APP_RESOURCE( "rightMargin",
-		    offsetof(EditApplication,eaRightMarginString),
-		    "90pt" ),
-	APP_RESOURCE( "topMargin",
-		    offsetof(EditApplication,eaTopMarginString),
-		    "72pt" ),
-	APP_RESOURCE( "bottomMargin",
-		    offsetof(EditApplication,eaBottomMarginString),
-		    "72pt" ),
-	APP_RESOURCE( "magnification",
-		    offsetof(EditApplication,eaMagnificationString),
-		    "1.2" ),
+{
+    APP_RESOURCE( "unit",
+		offsetof(EditApplication,eaUnitString),
+		"inch" ),
+    APP_RESOURCE( "paper",
+		offsetof(EditApplication,eaPaperString),
+		"letter" ),
+    APP_RESOURCE( "leftMargin",
+		offsetof(EditApplication,eaLeftMarginString),
+		"90pt" ),
+    APP_RESOURCE( "rightMargin",
+		offsetof(EditApplication,eaRightMarginString),
+		"90pt" ),
+    APP_RESOURCE( "topMargin",
+		offsetof(EditApplication,eaTopMarginString),
+		"72pt" ),
+    APP_RESOURCE( "bottomMargin",
+		offsetof(EditApplication,eaBottomMarginString),
+		"72pt" ),
+    APP_RESOURCE( "magnification",
+		offsetof(EditApplication,eaMagnificationString),
+		"1.2" ),
 
-	APP_RESOURCE( "author",
-		    offsetof(EditApplication,eaAuthor),
-		    (char *)0 ),
+    APP_RESOURCE( "supportXvCopyPaste",
+		offsetof(EditApplication,eaSupportXvCopyPasteString),
+		(char *)0 ),
 
-	APP_RESOURCE( "pageNumberFormat",
-		    offsetof(EditApplication,eaPageNumberFormat),
-		    "Page %d" ),
+    APP_RESOURCE( "hideSaveToOption",
+		offsetof(EditApplication,eaHideSaveToOptionString),
+		(char *)0 ),
 
-	APP_RESOURCE( "afmDirectory",
-		    offsetof(EditApplication,eaAfmDirectory),
-		    AFMDIR ),
-	APP_RESOURCE( "ghostscriptFontmap",
-		    offsetof(EditApplication,eaGhostscriptFontmap),
-		    (char *)0 ),
-	APP_RESOURCE( "ghostscriptFontToXmapping",
-		    offsetof(EditApplication,eaGhostscriptFontToXmapping),
-		    (char *)0 ),
-	APP_RESOURCE( "defaultFont",
-		    offsetof(EditApplication,eaDefaultFont),
-		    "Helvetica,,,10" ),
-	APP_RESOURCE( "faxCommand",
-		    offsetof(EditApplication,eaFaxCommand),
-		    (char *)0 ),
-	APP_RESOURCE( "customPrintCommand",
-		    offsetof(EditApplication,eaCustomPrintCommand),
-		    (char *)0 ),
-	APP_RESOURCE( "customPrinterName",
-		    offsetof(EditApplication,eaCustomPrinterName),
-		    (char *)0 ),
+    APP_RESOURCE( "usePostScriptFilters",
+		offsetof(EditApplication,eaUsePostScriptFiltersString),
+		(char *)0 ),
 
-	APP_RESOURCE( "rulerFont",
-		    offsetof(EditApplication,eaRulerFont),
-		    "-*-helvetica-medium-r-*-*-%d-*-*-*-*-*-iso8859-1" ),
-	APP_RESOURCE( "printDialogFont",
-		    offsetof(EditApplication,eaPrintDialogFont),
-		    "-*-lucidatypewriter-bold-r-*-*-%d-*-*-*-m-*-iso8859-1" ),
-    };
+    APP_RESOURCE( "usePostScriptIndexedImages",
+		offsetof(EditApplication,eaUsePostScriptIndexedImagesString),
+		(char *)0 ),
+
+    APP_RESOURCE( "author",
+		offsetof(EditApplication,eaAuthor),
+		(char *)0 ),
+
+    APP_RESOURCE( "pageNumberFormat",
+		offsetof(EditApplication,eaPageNumberFormat),
+		"Page %d" ),
+
+    APP_RESOURCE( "afmDirectory",
+		offsetof(EditApplication,eaAfmDirectory),
+		AFMDIR ),
+    APP_RESOURCE( "fontDirectory",
+		offsetof(EditApplication,eaFontDirectory),
+		(char *)0 ),
+    APP_RESOURCE( "ghostscriptFontmap",
+		offsetof(EditApplication,eaGhostscriptFontmap),
+		(char *)0 ),
+    APP_RESOURCE( "ghostscriptFontToXmapping",
+		offsetof(EditApplication,eaGhostscriptFontToXmapping),
+		(char *)0 ),
+    APP_RESOURCE( "defaultFont",
+		offsetof(EditApplication,eaDefaultFont),
+		"Helvetica,,,10" ),
+    APP_RESOURCE( "faxCommand",
+		offsetof(EditApplication,eaFaxCommand),
+		(char *)0 ),
+    APP_RESOURCE( "customPrintCommand",
+		offsetof(EditApplication,eaCustomPrintCommand),
+		(char *)0 ),
+    APP_RESOURCE( "customPrinterName",
+		offsetof(EditApplication,eaCustomPrinterName),
+		(char *)0 ),
+    APP_RESOURCE( "customPrintCommand2",
+		offsetof(EditApplication,eaCustomPrintCommand2),
+		(char *)0 ),
+    APP_RESOURCE( "customPrinterName2",
+		offsetof(EditApplication,eaCustomPrinterName2),
+		(char *)0 ),
+
+    APP_RESOURCE( "rulerFont",
+		offsetof(EditApplication,eaRulerFont),
+		"-*-helvetica-medium-r-*-*-%d-*-*-*-*-*-iso8859-1" ),
+    APP_RESOURCE( "printDialogFont",
+		offsetof(EditApplication,eaPrintDialogFont),
+		"-*-lucidatypewriter-bold-r-*-*-%d-*-*-*-m-*-iso8859-1" ),
+};
 
 /************************************************************************/
 /*									*/
@@ -537,6 +562,26 @@ static void appInitializeGeometrySettings(	EditApplication *	ea )
 	    { SDEB(ea->eaBottomMarginString);	}
 	}
 
+    ea->eaSupportXvCopyPaste= 0;
+    if  ( ea->eaSupportXvCopyPasteString			&&
+	  ! strcmp( ea->eaSupportXvCopyPasteString, "1" )	)
+	{ ea->eaSupportXvCopyPaste= 1;	}
+
+    ea->eaHideSaveToOption= 0;
+    if  ( ea->eaHideSaveToOptionString			&&
+	  ! strcmp( ea->eaHideSaveToOptionString, "1" )	)
+	{ ea->eaHideSaveToOption= 1;	}
+
+    ea->eaUsePostScriptFilters= 0;
+    if  ( ea->eaUsePostScriptFiltersString			&&
+	  ! strcmp( ea->eaUsePostScriptFiltersString, "1" )	)
+	{ ea->eaUsePostScriptFilters= 1;	}
+
+    ea->eaUsePostScriptIndexedImages= 0;
+    if  ( ea->eaUsePostScriptIndexedImagesString			&&
+	  ! strcmp( ea->eaUsePostScriptIndexedImagesString, "1" )	)
+	{ ea->eaUsePostScriptIndexedImages= 1;	}
+
     if  ( ea->eaToplevel.atTopWidget )
 	{
 	double			horPixPerMM;
@@ -687,6 +732,31 @@ void appGetApplicationResourceValues(	EditApplication *	ea )
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Collect a list of printers.						*/
+/*									*/
+/************************************************************************/
+
+int appGetPrintDestinations(		EditApplication *	ea )
+    {
+    if  ( ea->eaPrintDestinationsCollected )
+	{ return 0;	}
+
+    ea->eaPrintDestinationsCollected= 1;
+
+    if  ( utilPrinterGetPrinters( &(ea->eaPrintDestinationCount),
+				    &(ea->eaDefaultPrintDestination),
+				    &(ea->eaPrintDestinations),
+				    ea->eaCustomPrintCommand,
+				    ea->eaCustomPrinterName,
+				    ea->eaCustomPrintCommand2,
+				    ea->eaCustomPrinterName2 ) )
+	{ LDEB(1); return -1; 	}
+
+    return 0;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Handle Special Calls.						*/
@@ -730,6 +800,8 @@ static int appMainHandleSpecialCall(	EditApplication *	ea,
 	return 4;
 	}
 
+    /**  print to file **/
+
     if  ( argc == 4				&&
 	  ! strcmp( command, "printToFile" )	)
 	{
@@ -754,6 +826,8 @@ static int appMainHandleSpecialCall(	EditApplication *	ea,
 	return 5;
 	}
 
+    /**  print to printer **/
+
     if  ( argc == 3			&&
 	  ! strcmp( command, "print" )	)
 	{
@@ -803,6 +877,8 @@ static int appMainHandleSpecialCall(	EditApplication *	ea,
 	return 5;
 	}
 
+    /**  ----- **/
+
     return 0;
     }
 
@@ -841,14 +917,6 @@ int appMain(	EditApplication *	ea,
     /*  b  */
     appGetApplicationResourceValues( ea );
 
-    if  ( ea->eaPrintDestinationCount == 0				&&
-	  utilPrinterGetPrinters( &(ea->eaPrintDestinationCount),
-				    &(ea->eaDefaultPrintDestination),
-				    &(ea->eaPrintDestinations),
-				    ea->eaCustomPrintCommand,
-				    ea->eaCustomPrinterName )		)
-	{ LDEB(1); 	}
-
     if  ( appFinishApplicationWindow( ea ) )
 	{ LDEB(1); return -1;	}
 
@@ -962,7 +1030,11 @@ int appMain(	EditApplication *	ea,
 		}
 	    }
 	else{
-	    if  ( appAbsoluteName( absolute, 1000, argv[arg] ) < 0 )
+	    const char * const	fileRelativeTo= (const char *)0;
+	    int			relativeIsFile= 1;
+
+	    if  ( appAbsoluteName( absolute, 1000,
+			    argv[arg], relativeIsFile, fileRelativeTo ) < 0 )
 		{
 		SDEB(argv[arg]);
 		ed= appOpenDocument( ea, ea->eaToplevel.atTopWidget,
