@@ -456,7 +456,7 @@ static void appDrawVerticalRuler(	APP_WIDGET		w,
     return;
     }
 
-APP_EVENT_HANDLER( appRedrawHorizontalMetricRuler, w, voidmr, event )
+APP_EVENT_HANDLER_H( appRedrawHorizontalMetricRuler, w, voidmr, event )
     {
     MetricRuler *	mr= (MetricRuler *)voidmr;
     RulerData *		rd= &(mr->mrRulerData);
@@ -480,7 +480,7 @@ APP_EVENT_HANDLER( appRedrawHorizontalMetricRuler, w, voidmr, event )
     appDrawHorizontalRuler( w, mr, &drClip, ox );
     }
 
-APP_EVENT_HANDLER( appRedrawVerticalMetricRuler, w, voidmr, event )
+APP_EVENT_HANDLER_H( appRedrawVerticalMetricRuler, w, voidmr, event )
     {
     MetricRuler *	mr= (MetricRuler *)voidmr;
     RulerData *		rd= &(mr->mrRulerData);
@@ -504,7 +504,7 @@ APP_EVENT_HANDLER( appRedrawVerticalMetricRuler, w, voidmr, event )
     appDrawVerticalRuler( w, mr, &drClip, oy );
     }
 
-APP_EVENT_HANDLER( appHorizontalRulerConfigure, w, voidmr, event )
+APP_EVENT_HANDLER_H( appHorizontalRulerConfigure, w, voidmr, event )
     {
     MetricRuler *	mr= (MetricRuler *)voidmr;
     RulerData *		rd= &(mr->mrRulerData);
@@ -520,7 +520,7 @@ APP_EVENT_HANDLER( appHorizontalRulerConfigure, w, voidmr, event )
     return;
     }
 
-APP_EVENT_HANDLER( appVerticalRulerConfigure, w, voidmr, event )
+APP_EVENT_HANDLER_H( appVerticalRulerConfigure, w, voidmr, event )
     {
     MetricRuler *	mr= (MetricRuler *)voidmr;
     RulerData *		rd= &(mr->mrRulerData);
@@ -664,7 +664,7 @@ static void appHorizontalRulerSetValue(	MetricRulerDrag *	mrd,
     return;
     }
 
-static APP_EVENT_HANDLER( appHorizontalRulerMouseMove, w, vmrd, event )
+static APP_EVENT_HANDLER_H( appHorizontalRulerMouseMove, w, vmrd, event )
     {
     MetricRulerDrag *		mrd= (MetricRulerDrag *)vmrd;
 
@@ -679,7 +679,7 @@ static APP_EVENT_HANDLER( appHorizontalRulerMouseMove, w, vmrd, event )
     return;
     }
 
-static APP_EVENT_HANDLER( appHorizontalRulerMouseUp, w, vmrd, event )
+static APP_EVENT_HANDLER_H( appHorizontalRulerMouseUp, w, vmrd, event )
     {
     MetricRulerDrag *		mrd= (MetricRulerDrag *)vmrd;
 
@@ -980,7 +980,7 @@ static void appVerticalRulerSetValue(	MetricRulerDrag *	mrd,
     return;
     }
 
-static APP_EVENT_HANDLER( appVerticalRulerMouseMove, w, vmrd, event )
+static APP_EVENT_HANDLER_H( appVerticalRulerMouseMove, w, vmrd, event )
     {
     MetricRulerDrag *		mrd= (MetricRulerDrag *)vmrd;
 
@@ -995,7 +995,7 @@ static APP_EVENT_HANDLER( appVerticalRulerMouseMove, w, vmrd, event )
     return;
     }
 
-static APP_EVENT_HANDLER( appVerticalRulerMouseUp, w, vmrd, event )
+static APP_EVENT_HANDLER_H( appVerticalRulerMouseUp, w, vmrd, event )
     {
     MetricRulerDrag *	mrd= (MetricRulerDrag *)vmrd;
 
