@@ -11,15 +11,15 @@
 #   include	<stddef.h>
 #   include	<limits.h>
 
-#   include	<appDebugon.h>
-
 #   include	<appGeoString.h>
 #   include	<appUnit.h>
 
 #   include	"appFrame.h"
 #   include	"appPageLayoutTool.h"
 
-#   define	DRH_MM		45
+#   include	<appDebugon.h>
+
+#   define	DRH_MM		40
 
 /************************************************************************/
 /*									*/
@@ -32,12 +32,10 @@ static void appPageLayoutRefreshPageLayoutPage(	PageLayoutTool *	plt )
     EditApplication *		ea= plt->pltApplication;
     AppDrawingData *		add= &(plt->pltDrawingData);
     DocumentGeometry *		dg= &(plt->pltGeometryChosen);
-    const int			set= 1;
 
     char			scratch[50];
 
-    appPaperChooserAdaptToGeometry( &(plt->pltPaperChooser),
-						    plt->pltUnitType, dg, set );
+    appPaperChooserAdaptToGeometry( &(plt->pltPaperChooser), dg );
 
     appMarginToolShowMargins( &(plt->pltMarginTool), plt->pltUnitType, dg );
 
@@ -91,11 +89,9 @@ void appPageLayoutPageRefresh(	PageLayoutTool *		plt,
 /*									*/
 /************************************************************************/
 
-static void appPageLayoutChangeLayoutPushed(	APP_WIDGET	w,
-						void *		voidplt,
-						void *		voidpbcs )
+static void appPageLayoutChangeLayout(	PageLayoutTool *	plt,
+					int			wholeDocument )
     {
-    PageLayoutTool *		plt= (PageLayoutTool *)voidplt;
     EditApplication *		ea= plt->pltApplication;
 
     int				changed;
@@ -123,7 +119,7 @@ static void appPageLayoutChangeLayoutPushed(	APP_WIDGET	w,
     dgNew= plt->pltGeometryChosen;
 
     if  ( appPaperChooserGetSize( &sizeUpdMask, &(plt->pltPaperChooser),
-						plt->pltUnitType, &dgNew ) )
+								    &dgNew ) )
 	{ LDEB(1); return;	}
 
     PROPmaskCLEAR( &margChgMask );
@@ -154,19 +150,30 @@ static void appPageLayoutChangeLayoutPushed(	APP_WIDGET	w,
 	PROPmaskUNSET( &setMask, DGpropFOOTER_POSITION );
 	}
 
-    if  ( w == plt->pltChangeSelectionWidget )
-	{ (*ea->eaSetPageLayout)( ea, &dgNew, &setMask, 0 );	}
-    else{ (*ea->eaSetPageLayout)( ea, &dgNew, &setMask, 1 );	}
-
+    (*ea->eaSetPageLayout)( ea, &dgNew, &setMask, wholeDocument );
 
     return;
     }
 
-static void appSectHeaderFooterPosChanged(	APP_WIDGET	w,
-						void *		voidplt,
-						void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( appPageLayoutChangeSelectionPushed, w, voidplt )
+    {
+    PageLayoutTool *	plt= (PageLayoutTool *)voidplt;
+    const int		wholeDocument= 0;
+
+    appPageLayoutChangeLayout( plt, wholeDocument );
+    }
+
+static APP_BUTTON_CALLBACK_H( appPageLayoutChangeDocumentPushed, w, voidplt )
+    {
+    PageLayoutTool *	plt= (PageLayoutTool *)voidplt;
+    const int		wholeDocument= 1;
+
+    appPageLayoutChangeLayout( plt, wholeDocument );
+    }
+
+static void appSectHeaderFooterPosChanged(	PageLayoutTool *	plt,
+						APP_WIDGET		w )
     {
-    PageLayoutTool *		plt= (PageLayoutTool *)voidplt;
     EditApplication *		ea= plt->pltApplication;
     DocumentGeometry *		dgSect= &(plt->pltGeometryChosen);
 
@@ -193,15 +200,31 @@ static void appSectHeaderFooterPosChanged(	APP_WIDGET	w,
 	}
     }
 
+static APP_TXTYPING_CALLBACK_H( appSectHeaderPositionChanged, w, voidplt )
+    {
+    PageLayoutTool *		plt= (PageLayoutTool *)voidplt;
+
+    appSectHeaderFooterPosChanged( plt, plt->pltHeaderPositionText );
+
+    return;
+    }
+
+static APP_TXTYPING_CALLBACK_H( appSectFooterPositionChanged, w, voidplt )
+    {
+    PageLayoutTool *		plt= (PageLayoutTool *)voidplt;
+
+    appSectHeaderFooterPosChanged( plt, plt->pltFooterPositionText );
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  One of the margins has been changed by the user.			*/
 /*									*/
 /************************************************************************/
 
-static void appPaperMarginChanged(	APP_WIDGET	w,
-					void *		voidplt,
-					void *		voidcbs	 )
+static APP_TXACTIVATE_CALLBACK_H( appPaperMarginChanged, w, voidplt )
     {
     PageLayoutTool *		plt= (PageLayoutTool *)voidplt;
     AppDrawingData *		add= &(plt->pltDrawingData);
@@ -245,15 +268,22 @@ static void appPaperMarginChanged(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void appPageLayoutRevertLayoutPushed(	APP_WIDGET	w,
-						void *		voidplt,
-						void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( appPageLayoutRevertSelectionPushed, w, voidplt )
     {
     PageLayoutTool *	plt= (PageLayoutTool *)voidplt;
 
-    if  ( w == plt->pltRevertSelectionWidget )
-	{ plt->pltGeometryChosen= plt->pltGeometrySetSelection;	}
-    else{ plt->pltGeometryChosen= plt->pltGeometrySetDocument;	}
+    plt->pltGeometryChosen= plt->pltGeometrySetSelection;
+
+    appPageLayoutRefreshPageLayoutPage( plt );
+
+    return;
+    }
+
+static APP_BUTTON_CALLBACK_H( appPageLayoutRevertDocumentPushed, w, voidplt )
+    {
+    PageLayoutTool *	plt= (PageLayoutTool *)voidplt;
+
+    plt->pltGeometryChosen= plt->pltGeometrySetDocument;
 
     appPageLayoutRefreshPageLayoutPage( plt );
 
@@ -266,7 +296,7 @@ static void appPageLayoutRevertLayoutPushed(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static APP_EVENT_HANDLER( appPageLayoutToolDrawPage, w, voidplt, exposeEvent )
+static APP_EVENT_HANDLER_H( appPageLayoutToolDrawPage, w, voidplt, exposeEvent )
     {
     PageLayoutTool *		plt= (PageLayoutTool *)voidplt;
     AppDrawingData *		add= &(plt->pltDrawingData);
@@ -281,67 +311,22 @@ static APP_EVENT_HANDLER( appPageLayoutToolDrawPage, w, voidplt, exposeEvent )
 /*									*/
 /************************************************************************/
 
-static void appPageLayoutToolPaperSizeChosen(	APP_WIDGET	w,
-						void *		voidplt,
-						void *		voidpbcs )
+static void appPageLayoutToolPaperRectChanged(
+				PaperChooser *			pc,
+				void *				voidplt,
+				const DocumentGeometry *	dg )
     {
     PageLayoutTool *		plt= (PageLayoutTool *)voidplt;
     AppDrawingData *		add= &(plt->pltDrawingData);
-    DocumentGeometry *		dg= &(plt->pltGeometryChosen);
 
-    appPaperSizeChosen( w, &(plt->pltPaperChooser), dg, plt->pltUnitType );
+    plt->pltGeometryChosen.dgPageWideTwips= dg->dgPageWideTwips;
+    plt->pltGeometryChosen.dgPageHighTwips= dg->dgPageHighTwips;
 
     appExposeRectangle( add, 0, 0, 0, 0 );
 
     return;
     }
 
-static void appPageLayoutToolPaperSizeChanged(	APP_WIDGET	w,
-						void *		voidplt,
-						void *		voidcbs )
-    {
-    PageLayoutTool *		plt= (PageLayoutTool *)voidplt;
-    AppDrawingData *		add= &(plt->pltDrawingData);
-    DocumentGeometry *		dg= &(plt->pltGeometryChosen);
-
-    const int			set= 0;
-
-    appPaperSizeChanged( &(plt->pltPaperChooser), dg, plt->pltUnitType );
-
-    appPaperChooserAdaptToGeometry( &(plt->pltPaperChooser),
-						plt->pltUnitType, dg, set );
-
-    appExposeRectangle( add, 0, 0, 0, 0 );
-
-    return;
-    }
-
-/************************************************************************/
-/*									*/
-/*  One of the orientation toggle buttons has been pushed.		*/
-/*									*/
-/************************************************************************/
-
-static void appPageLayoutToolOrientationChosen(	APP_WIDGET	w,
-						void *		voidplt,
-						void *		voidtbcs  )
-    {
-    PageLayoutTool *		plt= (PageLayoutTool *)voidplt;
-    AppDrawingData *		add= &(plt->pltDrawingData);
-    DocumentGeometry *		dg= &(plt->pltGeometryChosen);
-
-    int					changed;
-
-    appPaperOrientationChosen( w, voidtbcs, &changed,
-						&(plt->pltPaperChooser),
-						plt->pltUnitType, dg );
-
-    if  ( changed )
-	{ appExposeRectangle( add, 0, 0, 0, 0 );	}
-
-    return;
-    }
-
 /************************************************************************/
 /*									*/
 /*  Make the frame for the header and footer positions.			*/
@@ -356,15 +341,17 @@ static void appPageLayoutMakeHeaderFooterWidgets(
     APP_WIDGET	label;
     APP_WIDGET	row;
 
+    const int	colspan= 1;
+
     appMakeRowFrameInColumn( &(plt->pltHeaderFooterFrame),
 					&(plt->pltHeaderFooterInside),
 					parent, 2, plpr->plprPositionsText );
 
     appMakeColumnInRow( &(plt->pltHeaderFooterLeftColumn),
-					plt->pltHeaderFooterInside, 0 );
+				    plt->pltHeaderFooterInside, 0, colspan );
 
     appMakeColumnInRow( &(plt->pltHeaderFooterRightColumn),
-					plt->pltHeaderFooterInside, 1 );
+				    plt->pltHeaderFooterInside, 1, colspan );
 
     /**********************/
 #   define	TW	7
@@ -378,10 +365,10 @@ static void appPageLayoutMakeHeaderFooterWidgets(
 				    plpr->plprFooterPositionText, TW, 1 );
 
     appGuiSetGotValueCallbackForText( plt->pltHeaderPositionText,
-				appSectHeaderFooterPosChanged, (void *)plt );
+				appSectHeaderPositionChanged, (void *)plt );
 
     appGuiSetGotValueCallbackForText( plt->pltFooterPositionText,
-				appSectHeaderFooterPosChanged, (void *)plt );
+				appSectFooterPositionChanged, (void *)plt );
 
     return;
     }
@@ -435,15 +422,14 @@ void appPageLayoutPageFillPage(	PageLayoutTool *		plt,
 				    appPageLayoutToolDrawPage, (void *)plt );
 
     appMakePaperChooserWidgets( pageWidget, plpr->plprPage,
+					    plt->pltUnitType,
 					    &(plt->pltPaperChooser),
-					    appPageLayoutToolPaperSizeChanged,
+					    appPageLayoutToolPaperRectChanged,
 					    (void *)plt );
 
     appPaperChooserAddOrientationToggles( &(plt->pltPaperChooser),
 					    plpr->plprPortraitText,
-					    plpr->plprLandscapeText,
-					    appPageLayoutToolOrientationChosen,
-						(void *)plt );
+					    plpr->plprLandscapeText );
 
     /**************/
 
@@ -463,17 +449,21 @@ void appPageLayoutPageFillPage(	PageLayoutTool *		plt,
     if  ( plt->pltManageSelection )
 	{
 	appInspectorMakeButtonRow( &row, pageWidget,
-	    &(plt->pltRevertSelectionWidget), &(plt->pltChangeSelectionWidget),
-	    plpr->plprRevertSelectionText, plpr->plprChangeSelectionText,
-	    appPageLayoutRevertLayoutPushed, appPageLayoutChangeLayoutPushed,
-	    (void *)plt );
+				    &(plt->pltRevertSelectionWidget),
+				    &(plt->pltChangeSelectionWidget),
+				    plpr->plprRevertSelectionText,
+				    plpr->plprChangeSelectionText,
+				    appPageLayoutRevertSelectionPushed,
+				    appPageLayoutChangeSelectionPushed,
+				    (void *)plt );
 	}
 
     appInspectorMakeButtonRow( &row, pageWidget,
-	    &(is->isRevertButton), &(is->isApplyButton),
-	    isr->isrRevert, isr->isrApplyToSubject,
-	    appPageLayoutRevertLayoutPushed, appPageLayoutChangeLayoutPushed,
-	    (void *)plt );
+				&(is->isRevertButton), &(is->isApplyButton),
+				isr->isrRevert, isr->isrApplyToSubject,
+				appPageLayoutRevertDocumentPushed,
+				appPageLayoutChangeDocumentPushed,
+				(void *)plt );
 
     plt->pltChangeDocumentWidget= is->isApplyButton;
 
@@ -490,8 +480,7 @@ void appPageLayoutPageFillChoosers(	PageLayoutTool *		plt,
 					const PageLayoutPageResources *	plpr )
     {
     appPaperChooserFillMenu( &(plt->pltPaperChooser),
-			plpr->plprCustomPageSizeText,
-			appPageLayoutToolPaperSizeChosen, (void *)plt );
+					    plpr->plprCustomPageSizeText );
 
     return;
     }
