@@ -10,10 +10,10 @@
 #   include	<stdio.h>
 #   include	<stddef.h>
 
-#   include	<appDebugon.h>
-
 #   include	<appFrame.h>
 
+#   include	<appDebugon.h>
+
 typedef struct QuestionContext
     {
     AppDialog	qcDialog;
@@ -57,26 +57,35 @@ static AppConfigurableResource APP_QuestionResourceTable[]=
 /*									*/
 /************************************************************************/
 
-static void appQuestionButtonPushed(	APP_WIDGET	button,
-					void *		voidqc,
-					void *		call_data )
+static APP_BUTTON_CALLBACK_H( appQuestionOkButtonPushed, button, voidqc )
     {
     QuestionContext *		qc= (QuestionContext *)voidqc;
 
-    if  ( button == qc->qcOkButton )
-	{ appGuiBreakDialog( &(qc->qcDialog), AQDrespOK ); return;	}
+    appGuiBreakDialog( &(qc->qcDialog), AQDrespOK );
+    return;
+    }
 
-    if  ( button == qc->qcYesButton )
-	{ appGuiBreakDialog( &(qc->qcDialog), AQDrespYES ); return;	}
+static APP_BUTTON_CALLBACK_H( appQuestionYesButtonPushed, button, voidqc )
+    {
+    QuestionContext *		qc= (QuestionContext *)voidqc;
 
-    if  ( button == qc->qcNoButton )
-	{ appGuiBreakDialog( &(qc->qcDialog), AQDrespNO ); return;	}
+    appGuiBreakDialog( &(qc->qcDialog), AQDrespYES );
+    return;
+    }
 
-    if  ( button == qc->qcCancelButton )
-	{ appGuiBreakDialog( &(qc->qcDialog), AQDrespCANCEL ); return;	}
+static APP_BUTTON_CALLBACK_H( appQuestionNoButtonPushed, button, voidqc )
+    {
+    QuestionContext *		qc= (QuestionContext *)voidqc;
 
-    LDEB(1);
-    appGuiBreakDialog( &(qc->qcDialog), AQDrespFAILURE );
+    appGuiBreakDialog( &(qc->qcDialog), AQDrespNO );
+    return;
+    }
+
+static APP_BUTTON_CALLBACK_H( appQuestionCancelButtonPushed, button, voidqc )
+    {
+    QuestionContext *		qc= (QuestionContext *)voidqc;
+
+    appGuiBreakDialog( &(qc->qcDialog), AQDrespCANCEL );
     return;
     }
 
@@ -98,7 +107,7 @@ static APP_WIDGET appQuestionMakeOkRow(	APP_WIDGET		parent,
     buttonRow= appMakeRowInColumn( parent, 1, heightResizable );
 
     appMakeButtonInRow( &(qc->qcOkButton), buttonRow, okText,
-		    appQuestionButtonPushed, (void *)qc, 0, showAsDefault );
+		    appQuestionOkButtonPushed, (void *)qc, 0, showAsDefault );
 
     return buttonRow;
     }
@@ -117,13 +126,16 @@ static APP_WIDGET appQuestionMakeYesNoCancelRow( APP_WIDGET	parent,
     buttonRow= appMakeRowInColumn( parent, 3, heightResizable );
 
     appMakeButtonInRow( &(qc->qcYesButton), buttonRow, yesText,
-		appQuestionButtonPushed, (void *)qc, 0, showYesAsDefault );
+				appQuestionYesButtonPushed,
+				(void *)qc, 0, showYesAsDefault );
 
     appMakeButtonInRow( &(qc->qcNoButton), buttonRow, noText,
-		appQuestionButtonPushed, (void *)qc, 1, showOtherAsDefault );
+				appQuestionNoButtonPushed,
+				(void *)qc, 1, showOtherAsDefault );
 
     appMakeButtonInRow( &(qc->qcCancelButton), buttonRow, cancelText,
-		appQuestionButtonPushed, (void *)qc, 2, showOtherAsDefault );
+				appQuestionCancelButtonPushed,
+				(void *)qc, 2, showOtherAsDefault );
 
     appGuiSetDefaultButtonForDialog( &(qc->qcDialog), qc->qcYesButton );
     appGuiSetCancelButtonForDialog( &(qc->qcDialog), qc->qcCancelButton );
@@ -144,10 +156,12 @@ static APP_WIDGET appQuestionMakeOkCancelRow( APP_WIDGET	parent,
     buttonRow= appMakeRowInColumn( parent, 2, heightResizable );
 
     appMakeButtonInRow( &(qc->qcOkButton), buttonRow, okText,
-		    appQuestionButtonPushed, (void *)qc, 0, showAsDefault );
+				appQuestionOkButtonPushed,
+				(void *)qc, 0, showAsDefault );
 
     appMakeButtonInRow( &(qc->qcCancelButton), buttonRow, cancelText,
-		    appQuestionButtonPushed, (void *)qc, 1, showAsDefault );
+				appQuestionCancelButtonPushed,
+				(void *)qc, 1, showAsDefault );
 
     appGuiSetDefaultButtonForDialog( &(qc->qcDialog), qc->qcOkButton );
     appGuiSetCancelButtonForDialog( &(qc->qcDialog), qc->qcCancelButton );
@@ -169,10 +183,12 @@ static APP_WIDGET appQuestionMakeYesNoRow( APP_WIDGET		parent,
     buttonRow= appMakeRowInColumn( parent, 2, heightResizable );
 
     appMakeButtonInRow( &(qc->qcYesButton), buttonRow, yesText,
-		    appQuestionButtonPushed, (void *)qc, 0, showAsDefault );
+					    appQuestionYesButtonPushed,
+					    (void *)qc, 0, showAsDefault );
 
     appMakeButtonInRow( &(qc->qcNoButton), buttonRow, noText,
-		    appQuestionButtonPushed, (void *)qc, 1, showAsDefault );
+					    appQuestionNoButtonPushed,
+					    (void *)qc, 1, showAsDefault );
 
     appGuiSetDefaultButtonForDialog( &(qc->qcDialog), qc->qcYesButton );
     appGuiSetCancelButtonForDialog( &(qc->qcDialog), qc->qcNoButton );
@@ -186,7 +202,7 @@ static APP_WIDGET appQuestionMakeYesNoRow( APP_WIDGET		parent,
 /*									*/
 /************************************************************************/
 
-static APP_DESTROY_CALLBACK( appFreeQuestionContext, w, voidqc )
+static APP_DESTROY_CALLBACK_H( appFreeQuestionContext, w, voidqc )
     { free( voidqc ); return;	}
 
 static int appMakeQuestionDialog( 	EditApplication *	ea,
@@ -214,7 +230,7 @@ static int appMakeQuestionDialog( 	EditApplication *	ea,
 				    sizeof(AppConfigurableResource) );
 
     appMakeVerticalDialog( &(qc->qcDialog), &paned, ea,
-						(APP_CALLBACK_FUNC)0,
+						(APP_CLOSE_CALLBACK_T)0,
 						appFreeQuestionContext,
 						(void *)qc,
 						ea->eaMessageDialogName );
