@@ -239,15 +239,72 @@ static int appSpellToolOpenIndices(	SpellCheckContext *	scc,
     return 0;
     }
 
+/************************************************************************/
+/*									*/
+/*  Adapt buttons to different situations..				*/
+/*									*/
+/************************************************************************/
+
+static void appSpellToolSomethingFound(	AppSpellTool *	ast,
+					int		yes_no )
+    {
+    appGuiEnableWidget( ast->astLearnButton, yes_no );
+    appGuiEnableWidget( ast->astIgnoreButton, yes_no );
+
+    return;
+    }
+
+static void appSpellToolGotAlternative(	AppSpellTool *	ast,
+					int		yes_no )
+    {
+    appGuiEnableWidget( ast->astForgetButton, yes_no );
+    appGuiEnableWidget( ast->astCorrectButton, yes_no );
+    appGuiEnableWidget( ast->astGuessButton, yes_no );
+
+    return;
+    }
+
+/************************************************************************/
+/*									*/
+/*  Look for the next unknown word.					*/
+/*									*/
+/************************************************************************/
+
+static void appSpellToolFindNext(	AppSpellTool *	ast )
+    {
+    AppSpellCheckContext *	ascc;
+    SpellCheckContext *		scc;
+
+    if  ( ast->astCurrentDictionary < 0 )
+	{ LDEB(ast->astCurrentDictionary); return;	}
+
+    if  ( ! ast->astFindNext < 0 )
+	{ XDEB(ast->astFindNext); return;	}
+
+    ascc= ast->astDictionaryContexts+ ast->astCurrentDictionary;
+    scc= &(ascc->asccSpellCheckContext);
+
+    appStringToTextWidget( ast->astWordText, "" );
+    appGuiEmptyListWidget( ast->astGuessList );
+
+    if  ( ! scc->sccStaticInd			&&
+	  appSpellToolOpenIndices( scc, ast )	)
+	{ return;	}
+
+    if  ( ! (*ast->astFindNext)( ast->astTarget, scc ) )
+	{ appSpellToolSomethingFound( ast, 1 );	}
+    else{ appSpellToolSomethingFound( ast, 0 );	}
+
+    appSpellToolGotAlternative( ast, 0 );
+    }
+
 /************************************************************************/
 /*									*/
 /*  'Learn' button has been pushed.					*/
 /*									*/
 /************************************************************************/
 
-static void appSpellToolLearnPushed(	APP_WIDGET	w,
-					void *		voidast,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( appSpellToolLearnPushed, w, voidast )
     {
     AppSpellTool *		ast= (AppSpellTool *)voidast;
     unsigned char *		word;
@@ -286,8 +343,12 @@ static void appSpellToolLearnPushed(	APP_WIDGET	w,
 
     fclose( privateDict );
 
+#   if 1
+    appSpellToolFindNext( ast );
+#   else
     appGuiEnableWidget( ast->astLearnButton, 0 );
     appGuiEnableWidget( ast->astForgetButton, 1 );
+#   endif
 
     return;
     }
@@ -298,9 +359,7 @@ static void appSpellToolLearnPushed(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void appSpellToolForgetPushed(	APP_WIDGET	w,
-					void *		voidast,
-					void *		voidpbcs	 )
+static APP_BUTTON_CALLBACK_H( appSpellToolForgetPushed, w, voidast )
     {
     AppSpellTool *		ast= (AppSpellTool *)voidast;
     unsigned char *		word;
@@ -345,64 +404,19 @@ static void appSpellToolForgetPushed(	APP_WIDGET	w,
     return;
     }
 
-/************************************************************************/
-/*  Adapt buttons to different situations..				*/
-/************************************************************************/
-
-static void appSpellToolSomethingFound(	AppSpellTool *	ast,
-					int		yes_no )
-    {
-    appGuiEnableWidget( ast->astLearnButton, yes_no );
-    appGuiEnableWidget( ast->astIgnoreButton, yes_no );
-
-    return;
-    }
-
-static void appSpellToolGotAlternative(	AppSpellTool *	ast,
-					int		yes_no )
-    {
-    appGuiEnableWidget( ast->astForgetButton, yes_no );
-    appGuiEnableWidget( ast->astCorrectButton, yes_no );
-    appGuiEnableWidget( ast->astGuessButton, yes_no );
-
-    return;
-    }
-
 /************************************************************************/
 /*									*/
 /*  'Find Next' button has been pushed.					*/
 /*									*/
 /************************************************************************/
 
-static void appSpellToolFindNext(	APP_WIDGET	w,
-					void *		voidast,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( appSpellToolFindNextPushed, w, voidast )
     {
     AppSpellTool *		ast= (AppSpellTool *)voidast;
-    AppSpellCheckContext *	ascc;
-    SpellCheckContext *		scc;
 
-    if  ( ast->astCurrentDictionary < 0 )
-	{ LDEB(ast->astCurrentDictionary); return;	}
+    appSpellToolFindNext( ast );
 
-    if  ( ! ast->astFindNext < 0 )
-	{ XDEB(ast->astFindNext); return;	}
-
-    ascc= ast->astDictionaryContexts+ ast->astCurrentDictionary;
-    scc= &(ascc->asccSpellCheckContext);
-
-    appStringToTextWidget( ast->astWordText, "" );
-    appGuiEmptyListWidget( ast->astGuessList );
-
-    if  ( ! scc->sccStaticInd			&&
-	  appSpellToolOpenIndices( scc, ast )	)
-	{ return;	}
-
-    if  ( ! (*ast->astFindNext)( ast->astTarget, scc ) )
-	{ appSpellToolSomethingFound( ast, 1 );	}
-    else{ appSpellToolSomethingFound( ast, 0 );	}
-
-    appSpellToolGotAlternative( ast, 0 );
+    return;
     }
 
 /************************************************************************/
@@ -411,9 +425,7 @@ static void appSpellToolFindNext(	APP_WIDGET	w,
 /*									*/
 /************************************************************************/
 
-static void editSpellToolIgnorePushed(	APP_WIDGET	w,
-					void *		voidast,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( appSpellToolIgnorePushed, w, voidast )
     {
     AppSpellTool *		ast= (AppSpellTool *)voidast;
     unsigned char *		word;
@@ -438,7 +450,7 @@ static void editSpellToolIgnorePushed(	APP_WIDGET	w,
     if  ( indMoveWord( scc->sccForgotInd, scc->sccLearntInd, word ) )
 	{ SDEB((char *)word); return;	}
 
-    appSpellToolFindNext( w, voidast, voidpbcs );
+    appSpellToolFindNext( ast );
 
     appFreeStringFromTextWidget( (char *)word );
 
@@ -446,12 +458,13 @@ static void editSpellToolIgnorePushed(	APP_WIDGET	w,
     }
 
 /************************************************************************/
+/*									*/
 /*  The user typed something in the Coreection, Turn on the the		*/
 /*  'Correct' button.							*/
+/*									*/
 /************************************************************************/
-static void editSpellCorrectionTyped(	APP_WIDGET	w,
-					void *		voidast,
-					void *		ignored	 )
+
+static APP_TXTYPING_CALLBACK_H( appSpellCorrectionTyped, w, voidast )
     {
     AppSpellTool *	ast= (AppSpellTool *)voidast;
 
@@ -461,13 +474,13 @@ static void editSpellCorrectionTyped(	APP_WIDGET	w,
     }
 
 /************************************************************************/
+/*									*/
 /*  'Correct' button has been pushed.					*/
 /*  Or a double click on the listbox with guesses.			*/
+/*									*/
 /************************************************************************/
 
-static void editSpellToolCorrect(	APP_WIDGET	w,
-					void *		voidast,
-					void *		ignored	 )
+static APP_BUTTON_CALLBACK_H( appSpellToolCorrect, w, voidast )
     {
     AppSpellTool *	ast= (AppSpellTool *)voidast;
     char *		guess;
@@ -481,7 +494,7 @@ static void editSpellToolCorrect(	APP_WIDGET	w,
 
     appFreeStringFromTextWidget( guess );
 
-    appSpellToolFindNext( w, voidast, ignored );
+    appSpellToolFindNext( ast );
 
     return;
     }
@@ -490,9 +503,7 @@ static void editSpellToolCorrect(	APP_WIDGET	w,
 /*  The 'Guess' button has been pushed.					*/
 /************************************************************************/
 
-static void editSpellGuessButtonPushed(	APP_WIDGET	w,
-					void *		voidast,
-					void *		voidpbcs )
+static APP_BUTTON_CALLBACK_H( appSpellGuessButtonPushed, w, voidast )
     {
     AppSpellTool *	ast= (AppSpellTool *)voidast;
     char *		word;
@@ -510,7 +521,7 @@ static void editSpellGuessButtonPushed(	APP_WIDGET	w,
 /*  A guess in the list has been selected.				*/
 /************************************************************************/
 
-static APP_LIST_CALLBACK( appSpellGuessChosen, w, voidast, voidlcs )
+static APP_LIST_CALLBACK_H( appSpellGuessChosen, w, voidast, voidlcs )
     {
     AppSpellTool *		ast= (AppSpellTool *)voidast;
 
@@ -539,8 +550,8 @@ static void appSpellToolMakeButtonRow(	APP_WIDGET *		pRow,
 					APP_WIDGET *		pRightButton,
 					char *			leftLabel,
 					char *			rightLabel,
-					APP_CALLBACK_FUNC	leftCallback,
-					APP_CALLBACK_FUNC	rightCallback,
+					APP_BUTTON_CALLBACK_T	leftCallback,
+					APP_BUTTON_CALLBACK_T	rightCallback,
 					AppSpellTool *		ast,
 					APP_WIDGET		parent )
 
@@ -611,13 +622,13 @@ static void appSpellMakeButtonRows(	APP_WIDGET		spellForm,
     appSpellToolMakeButtonRow( &buttonRow,
 		&(ast->astIgnoreButton), &(findNextButton),
 		astr->astrIgnore, astr->astrFindNext,
-		editSpellToolIgnorePushed, appSpellToolFindNext,
+		appSpellToolIgnorePushed, appSpellToolFindNextPushed,
 		ast, spellForm );
 
     appSpellToolMakeButtonRow( &buttonRow,
 		&(ast->astGuessButton), &(ast->astCorrectButton),
 		astr->astrGuess, astr->astrCorrect,
-		editSpellGuessButtonPushed, editSpellToolCorrect,
+		appSpellGuessButtonPushed, appSpellToolCorrect,
 		ast, spellForm );
 
     return;
@@ -652,7 +663,7 @@ static void appDestroySpellTool(	AppSpellTool *	ast )
     return;
     }
 
-static APP_CLOSE_CALLBACK( appCloseSpellTool, w, voidast )
+static APP_CLOSE_CALLBACK_H( appCloseSpellTool, w, voidast )
     {
     AppSpellTool *	ast= (AppSpellTool *)voidast;
 
@@ -746,6 +757,7 @@ static int appSpellAddDictionary(	const char *		filename,
     /********************************************/
     if  ( ! strcmp( name, "Czech" )	||
 	  ! strcmp( name, "Polish" )	||
+	  ! strcmp( name, "Hungarian" )	||
 	  ! strcmp( name, "Slovak" )	)
 	{
 	indSpellIso2CharacterKinds( &(fresh->asccSpellCheckContext) );
@@ -754,7 +766,13 @@ static int appSpellAddDictionary(	const char *		filename,
 
     if  ( ! strcmp( name, "Russian" )	)
 	{
-	indSpellKoi8rCharacterKinds( &(fresh->asccSpellCheckContext) );
+	indSpellIso5CharacterKinds( &(fresh->asccSpellCheckContext) );
+	return 0;
+	}
+
+    if  ( ! strcmp( name, "Greek" )	)
+	{
+	indSpellIso7CharacterKinds( &(fresh->asccSpellCheckContext) );
 	return 0;
 	}
 
@@ -804,19 +822,12 @@ static int appSpellGetDictionaries(	AppSpellTool *		ast,
 /*									*/
 /************************************************************************/
 
-static void editSpellDictionaryChosen(	APP_WIDGET	w,
-					void *		voidast,
-					void *		voidpbcs )
+static APP_OITEM_CALLBACK_H( appSpellDictionaryChosen, w, voidast )
     {
     AppSpellTool *	ast= (AppSpellTool *)voidast;
     int			i;
 
-    for ( i= 0; i < ast->astDictionaryCount; i++ )
-	{
-	if  ( ast->astDictionaryContexts[i].asccMenuWidget == w )
-	    { break;	}
-	}
-
+    i= appGuiGetOptionmenuItemIndex( &(ast->astDictionaryOptionmenu), w );
     if  ( i < 0 || i >= ast->astDictionaryCount )
 	{ LLDEB(i,ast->astDictionaryCount); return;	}
 
@@ -845,14 +856,14 @@ static void appSpellFillDictionaryMenu(		AppSpellTool *		ast,
 	ascc->asccMenuWidget= appAddItemToOptionmenu(
 			    &(ast->astDictionaryOptionmenu),
 			    ascc->asccSpellCheckContext.sccDictionaryPrefix,
-			    editSpellDictionaryChosen, (void *)ast );
+			    appSpellDictionaryChosen, (void *)ast );
 	}
 
     if  ( ast->astDictionaryCount == 0 )
 	{
 	(void) appAddItemToOptionmenu( &(ast->astDictionaryOptionmenu),
 			    astr->astrNoDicts,
-			    editSpellDictionaryChosen, (void *)ast );
+			    appSpellDictionaryChosen, (void *)ast );
 
 	appGuiEnableWidget( ast->astTopWidget, 0 );
 	}
@@ -938,6 +949,8 @@ void * appMakeSpellTool(	APP_WIDGET		spellOption,
     ast->astDictionaryCount= 0;
     ast->astCurrentDictionary= -1;
 
+    appInitOptionmenu( &(ast->astDictionaryOptionmenu) );
+
     appSpellGetDictionaries( ast, &astr );
 
     appMakeVerticalTool( &(ast->astTopWidget), &(ast->astMainWidget), ea,
@@ -951,13 +964,13 @@ void * appMakeSpellTool(	APP_WIDGET		spellOption,
 
 #   ifdef USE_MOTIF
     XtAddCallback( ast->astGuessList, XmNdefaultActionCallback,
-					editSpellToolCorrect, (void *)ast );
+					appSpellToolCorrect, (void *)ast );
 #   endif
 
     appMakeTextInColumn( &(ast->astWordText), ast->astMainWidget, 0, 1 );
 
     appGuiSetTypingCallbackForText( ast->astWordText,
-				    editSpellCorrectionTyped, (void *)ast );
+				    appSpellCorrectionTyped, (void *)ast );
 
     appSpellMakeButtonRows( ast->astMainWidget, &astr, ast );
 
