@@ -10,12 +10,12 @@
 #   include	<stdio.h>
 #   include	<stddef.h>
 
-#   include	<appDebugon.h>
-
 #   include	<appSymbolPicker.h>
 #   include	<appEncodingMenu.h>
 #   include	<appDraw.h>
 
+#   include	<appDebugon.h>
+
 /************************************************************************/
 /*									*/
 /*  Resources for the Symbol Picker.					*/
@@ -52,6 +52,7 @@ static AppConfigurableResource APP_SymbolPickerResourceTable[]=
 typedef struct AppSymbolPicker
     {
     EditApplication *		aspApplication;
+    unsigned int		aspCurrentDocumentId;
 
     AppSymbolPickerResources *	aspResources;
 
@@ -69,12 +70,10 @@ typedef struct AppSymbolPicker
     SymbolPickerInsert		aspInsert;
     void *			aspTarget;
 
-    AppFontFamily *		aspFontFamilies;
-    int				aspFontFamilyCount;
-
     int				aspSymbolSelected;
 
-    int				aspFontFamilySet;
+    DocumentFontList		aspDocumentFontList;
+    TextAttribute		aspTextAttribute;
     int				aspFontFamilyChosen;
 
     APP_FONT *			aspFont;
@@ -88,39 +87,43 @@ typedef struct AppSymbolPicker
 
 /************************************************************************/
 /*									*/
-/*  Adapt the symbol Picker to the current font family.			*/
+/*  Adapt the symbol Picker to the current font.			*/
+/*									*/
+/*  9)  Also remember that no symbol has been selected yet, and that	*/
+/*	for that reason, no symbol can be inserted.			*/
 /*									*/
 /************************************************************************/
 
-static int appSymbolAdaptToFamilyEncoding(	AppSymbolPicker *	asp,
-						int			fam,
-						int			enc )
+static int appSymbolAdaptDrawingToFont(		AppSymbolPicker *	asp )
     {
-    AppDrawingData *	add= &(asp->aspDrawingData);
+    AppDrawingData *		add= &(asp->aspDrawingData);
+    const DocumentFontList *	dfl= &(asp->aspDocumentFontList);
+    TextAttribute *		ta= &(asp->aspTextAttribute);
+    int				physicalFont;
 
-    char		scratch[120];
-
-    APP_FONT *		xfs;
-
-    AppFontFamily *	aff;
-
-    const int		twipsSize= 20* 10;
-
-    aff= asp->aspFontFamilies+ fam;
-
-    if  ( appFontXFont( scratch, add, aff, enc, aff->affFaces,
-						twipsSize, DOCfontREGULAR ) )
-	{ SLDEB(aff->affFontFamilyName,enc); return- 1; }
-
-    xfs= appDrawOpenFont( add, scratch );
-    if  ( ! xfs )
-	{ SXDEB(scratch,xfs); return -1;	}
+    if  ( asp->aspFontFamilyChosen < 0 )
+	{ LDEB(asp->aspFontFamilyChosen); return -1;	}
 
     if  ( asp->aspFont )
-	{ appDrawFreeFont( add, asp->aspFont );	}
-    asp->aspFont= xfs;
+	{
+	/*  No! managed by the drawing data
+	appDrawFreeFont( add, asp->aspFont );
+	*/
+	asp->aspFont= (APP_FONT *)0;
+	}
 
-    appDrawSetFont( add, asp->aspFont );
+    physicalFont= appOpenScreenFont( add, dfl, ta );
+    if  ( physicalFont < 0 )
+	{ LDEB(physicalFont);	}
+    else{
+	const DrawScreenFontList *	apfl= &(add->addScreenFontList);
+	DrawScreenFont *		apf;
+
+	apf= apfl->apflFonts+ physicalFont;
+	asp->aspFont= apf->apfFontStruct;
+
+	appDrawSetFont( add, apf->apfFontStruct );
+	}
 
     appExposeRectangle( add, 0, 0, 0, 0 );
 
@@ -130,37 +133,18 @@ static int appSymbolAdaptToFamilyEncoding(	AppSymbolPicker *	asp,
     return 0;
     }
 
-static int appSymbolAdaptToFamily(  AppSymbolPicker *	asp,
-				    int			fontFamilyNumber )
+static int appSymbolAdaptToFamily(  AppSymbolPicker *	asp )
     {
-    AppFontFamily *	aff;
-
-    aff= asp->aspFontFamilies+ fontFamilyNumber;
-
-    appEncodingMenuAdaptToFamilyEncodings( &(asp->aspEncodingMenu), aff );
-
-    if  ( appSymbolAdaptToFamilyEncoding( asp, fontFamilyNumber,
-				asp->aspEncodingMenu.aemFontEncodingChosen ) )
-	{ LDEB(fontFamilyNumber);	}
-
-    return 0;
-    }
-
-static int appSymbolAdaptToEncoding(	AppSymbolPicker *	asp,
-					int			enc )
-    {
-    AppFontFamily *	aff;
+    const DocumentFontList *	dfl= &(asp->aspDocumentFontList);
 
     if  ( asp->aspFontFamilyChosen < 0 )
-	{ LLDEB(asp->aspFontFamilyChosen,enc); return -1;	}
+	{ LDEB(asp->aspFontFamilyChosen); return -1;	}
 
-    aff= asp->aspFontFamilies+ asp->aspFontFamilyChosen;
+    appEncodingMenuAdaptToFamilyEncodings( &(asp->aspEncodingMenu),
+			    dfl->dflFamilies+ asp->aspFontFamilyChosen );
 
-    if  ( ! aff->affSupportedCharsets[enc].scSupported )
-	{ LLDEB(asp->aspFontFamilyChosen,enc); return -1;	}
-
-    if  ( appSymbolAdaptToFamilyEncoding( asp, asp->aspFontFamilyChosen, enc ) )
-	{ LLDEB(asp->aspFontFamilyChosen,enc); return -1;	}
+    if  ( appSymbolAdaptDrawingToFont( asp ) )
+	{ LDEB(asp->aspFontFamilyChosen); return -1;	}
 
     return 0;
     }
@@ -171,28 +155,33 @@ static int appSymbolAdaptToEncoding(	AppSymbolPicker *	asp,
 /*									*/
 /************************************************************************/
 
-static void appSymbolInsertPushed(	APP_WIDGET	w,
-					void *		voidasp,
-					void *		voidpbcs )
+static void appSymbolPickerInsertSymbol(	AppSymbolPicker *	asp )
     {
-    AppSymbolPicker *	asp= (AppSymbolPicker *)voidasp;
-    AppEncodingMenu *	aem= &(asp->aspEncodingMenu);
-    AppFontFamily *	aff;
+    PropertyMask	setMask;
+
+    PROPmaskCLEAR( &setMask );
 
     if  ( asp->aspSymbolSelected < 0 || ! asp->aspInsert )
 	{ LXDEB(asp->aspSymbolSelected,asp->aspInsert); return;	}
 
-    aff= asp->aspFontFamilies+ asp->aspFontFamilyChosen;
+    if  ( asp->aspFontFamilyChosen >= 0 )
+	{ PROPmaskADD( &setMask, TApropDOC_FONT_NUMBER );	}
 
     (*asp->aspInsert)( asp->aspTarget, asp->aspSymbolSelected,
-		    asp->aspFontFamilyChosen,
-		    aff->affFontFamilyName,
-		    aem->aemFontEncodingChosen,
-		    aem->aemFontEncodingChosen == aff->affDefaultEncoding );
+				    &(asp->aspTextAttribute), &setMask );
 
     return;
     }
 
+static APP_BUTTON_CALLBACK_H( appSymbolInsertPushed, w, voidasp )
+    {
+    AppSymbolPicker *	asp= (AppSymbolPicker *)voidasp;
+
+    appSymbolPickerInsertSymbol( asp );
+    
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Redraw the symbol part of the symbol picker.			*/
@@ -243,7 +232,7 @@ static void appSymbolRectangle(		DocumentRectangle *	drSym,
     return;
     }
 
-static APP_EVENT_HANDLER( appSymbolRedraw, w, voidasp, exposeEvent )
+static APP_EVENT_HANDLER_H( appSymbolRedraw, w, voidasp, exposeEvent )
     {
     AppSymbolPicker *			asp= (AppSymbolPicker *)voidasp;
     AppDrawingData *			add= &(asp->aspDrawingData);
@@ -374,7 +363,7 @@ static APP_EVENT_HANDLER( appSymbolRedraw, w, voidasp, exposeEvent )
 /*									*/
 /************************************************************************/
 
-static APP_EVENT_HANDLER( appSymbolMousePress, w, voidasp, downEvent )
+static APP_EVENT_HANDLER_H( appSymbolMousePress, w, voidasp, downEvent )
     {
     AppSymbolPicker *	asp= (AppSymbolPicker *)voidasp;
 
@@ -441,23 +430,7 @@ static APP_EVENT_HANDLER( appSymbolMousePress, w, voidasp, downEvent )
 					    drExpose.drY1- drExpose.drY0+ 1 );
 
     if  ( sameAsPrevious && seq == 2 )
-	{
-	AppEncodingMenu *	aem= &(asp->aspEncodingMenu);
-	AppFontFamily *		aff;
-
-	if  ( ! asp->aspInsert )
-	    { XDEB(asp->aspInsert); return;	}
-
-	aff= asp->aspFontFamilies+ asp->aspFontFamilyChosen;
-
-	(*asp->aspInsert)( asp->aspTarget,
-		    asp->aspSymbolSelected,
-		    asp->aspFontFamilyChosen,
-		    aff->affFontFamilyName,
-		    aem->aemFontEncodingChosen,
-		    aem->aemFontEncodingChosen == aff->affDefaultEncoding );
-
-	}
+	{ appSymbolPickerInsertSymbol( asp );	}
 
     return;
     }
@@ -470,51 +443,97 @@ static APP_EVENT_HANDLER( appSymbolMousePress, w, voidasp, downEvent )
 /************************************************************************/
 
 /*  1  */
-static void appSymbolFontFamilyChosen(		APP_WIDGET	w,
-						void *		voidasp,
-						void *		voidpbcs )
+static APP_OITEM_CALLBACK_H( appSymbolFontFamilyChosen, w, voidasp )
     {
-    int			i;
-    AppSymbolPicker *	asp= (AppSymbolPicker *)voidasp;
+    int				fam;
+    AppSymbolPicker *		asp= (AppSymbolPicker *)voidasp;
+    AppEncodingMenu *		aem= &(asp->aspEncodingMenu);
+    const DocumentFontList *	dfl= &(asp->aspDocumentFontList);
+    const DocumentFontFamily *	dff;
+    int				encoding= aem->aemFontEncoding;
+    int				changed= 0;
 
-    for ( i= 0; i < asp->aspFontFamilyCount; i++ )
+    fam= appGuiGetOptionmenuItemIndex( &(asp->aspFontOptionmenu), w );
+    if  ( fam < 0 || fam >= dfl->dflFamilyCount )
+	{ LLDEB(fam,dfl->dflFamilyCount); return;	}
+
+    dff= dfl->dflFamilies+ fam;
+    if  ( encoding < 0				||
+	  dff->dffFontForEncoding[encoding] < 0	)
 	{
-	if  ( w == asp->aspFontFamilyOptions[i] )
-	    { break;	}
+	int	enc;
+
+	encoding= -1;
+
+	for ( enc= 0; enc < ENCODINGps_COUNT; enc++ )
+	    {
+	    if  ( dff->dffFontForEncoding[enc] >= 0 )
+		{ encoding= enc; break;	}
+	    }
 	}
 
-    if  ( i < 0 || i >= asp->aspFontFamilyCount )
-	{ LLDEB(i,asp->aspFontFamilyCount); return;	}
-
-    if  ( i != asp->aspFontFamilyChosen )
+    if  ( encoding >= 0 )
 	{
-	if  ( appSymbolAdaptToFamily( asp, i ) )
-	    { LDEB(i);	}
+	TextAttribute *		ta= &(asp->aspTextAttribute);
 
-	asp->aspFontFamilyChosen= i;
+	if  ( ta->taFontNumber != dff->dffFontForEncoding[encoding] )
+	    {
+	    ta->taFontNumber= dff->dffFontForEncoding[encoding];
+	    changed= 1;
+	    }
+	}
+
+    if  ( changed )
+	{
+	asp->aspFontFamilyChosen= fam;
+
+	appEncodingMenuSetEncoding( aem, dff, encoding );
+
+	if  ( appSymbolAdaptToFamily( asp ) )
+	    { LDEB(fam);	}
 	}
 
     return;
     }
 
 /*  2  */
-static void appSymbolFontEncodingChosen(	APP_WIDGET	w,
-						void *		voidasp,
-						void *		voidpbcs )
+static APP_OITEM_CALLBACK_H( appSymbolFontEncodingChosen, w, voidasp )
     {
-    int			enc;
-    AppSymbolPicker *	asp= (AppSymbolPicker *)voidasp;
+    int				enc;
+    AppSymbolPicker *		asp= (AppSymbolPicker *)voidasp;
+    int				changed= 0;
 
-    if  ( appEncodingMenuEncodingFromWidget( &enc,
-					    &(asp->aspEncodingMenu), w ) )
-	{ XDEB(w); return;	}
+    enc= appGuiGetOptionmenuItemIndex(
+			&(asp->aspEncodingMenu.aemEncodingOptionmenu), w );
+    if  ( enc < 0 || enc >= ENCODINGps_COUNT )
+	{ LLDEB(enc,ENCODINGps_COUNT); return;	}
 
-    if  ( enc != asp->aspEncodingMenu.aemFontEncodingChosen )
+    if  ( asp->aspFontFamilyChosen >= 0 )
 	{
-	if  ( appSymbolAdaptToEncoding( asp, enc ) )
-	    { LDEB(enc);	}
+	const DocumentFontList *	dfl= &(asp->aspDocumentFontList);
+	const DocumentFontFamily *	dff;
+	TextAttribute *			ta= &(asp->aspTextAttribute);
 
-	asp->aspEncodingMenu.aemFontEncodingChosen= enc;
+	dff= dfl->dflFamilies+ asp->aspFontFamilyChosen;
+
+	if  ( dff->dffFontForEncoding[enc] < 0 )
+	    { LLDEB(enc,dff->dffFontForEncoding[enc]); return;	}
+
+	if  ( ta->taFontNumber != dff->dffFontForEncoding[enc] )
+	    {
+	    ta->taFontNumber= dff->dffFontForEncoding[enc];
+	    changed= 1;
+	    }
+
+	if  ( changed )
+	    {
+	    AppEncodingMenu *		aem= &(asp->aspEncodingMenu);
+
+	    if  ( appSymbolAdaptDrawingToFont( asp ) )
+		{ LDEB(asp->aspFontFamilyChosen);	}
+
+	    appEncodingMenuSetEncoding( aem, dff, enc );
+	    }
 	}
 
     return;
@@ -617,26 +636,22 @@ static APP_WIDGET appSymbolMakeButtonRow( APP_WIDGET		parent,
 static void appSymbolFillFontMenu(	AppSymbolPickerResources *	aspr,
 					AppSymbolPicker *		asp )
     {
-    int			i;
-    AppFontFamily *	aff;
+    int				fam;
+    const DocumentFontList *	dfl= &(asp->aspDocumentFontList);
+    const DocumentFontFamily *	dff;
 
     appEmptyOptionmenu( &(asp->aspFontOptionmenu) );
 
-    aff= asp->aspFontFamilies;
-    for ( i= 0; i < asp->aspFontFamilyCount; aff++, i++ )
+    dff= dfl->dflFamilies;
+    for ( fam= 0; fam < dfl->dflFamilyCount; dff++, fam++ )
 	{
-	char *		labelText;
-
-	if  ( aff->affFontFamilyText )
-	    { labelText= aff->affFontFamilyText;	}
-	else{ labelText= aff->affFontFamilyName;	}
-
-	asp->aspFontFamilyOptions[i]= appAddItemToOptionmenu(
-				    &(asp->aspFontOptionmenu), labelText,
+	asp->aspFontFamilyOptions[fam]= appAddItemToOptionmenu(
+				    &(asp->aspFontOptionmenu),
+				    dff->dffFamilyName,
 				    appSymbolFontFamilyChosen, (void *)asp );
 	}
 
-    if  ( asp->aspFontFamilyCount == 0 )
+    if  ( dfl->dflFamilyCount == 0 )
 	{
 	asp->aspFontFamilyOptions[0]= appAddItemToOptionmenu(
 				&(asp->aspFontOptionmenu), aspr->asprNone,
@@ -644,9 +659,10 @@ static void appSymbolFillFontMenu(	AppSymbolPickerResources *	aspr,
 
 	appGuiEnableWidget( asp->aspTopWidget, 0 );
 	}
+    else{ appGuiEnableWidget( asp->aspTopWidget, 1 );	}
 
-    appSetOptionmenu( &(asp->aspFontOptionmenu), 0 );
     asp->aspFontFamilyChosen= 0;
+    appSetOptionmenu( &(asp->aspFontOptionmenu), 0 );
 
     appOptionmenuRefreshWidth( &(asp->aspFontOptionmenu) );
 
@@ -660,15 +676,19 @@ static void appSymbolFillFontMenu(	AppSymbolPickerResources *	aspr,
 /*									*/
 /************************************************************************/
 
-static APP_CLOSE_CALLBACK( appCloseSymbolPicker, w, voidasp )
+static APP_CLOSE_CALLBACK_H( appCloseSymbolPicker, w, voidasp )
     {
     AppSymbolPicker *	asp= (AppSymbolPicker *)voidasp;
 
     if  ( asp->aspDestroy )
 	{ (*asp->aspDestroy)( asp->aspTarget );	}
 
+    /* No! managed by the drawing data
     if  ( asp->aspFont )
 	{ appDrawFreeFont( &(asp->aspDrawingData), asp->aspFont );	}
+    */
+
+    docCleanFontList( &(asp->aspDocumentFontList) );
 
     appCleanDrawingData( &(asp->aspDrawingData) );
 
@@ -705,9 +725,6 @@ void * appMakeSymbolPicker(	APP_WIDGET		symbolOption,
     double				xfac;
     double				yfac;
 
-    AppFontFamily *			fontFamilies;
-    int					fontFamilyCount;
-
     static AppSymbolPickerResources	aspr;
     static int				gotResources;
 
@@ -723,27 +740,24 @@ void * appMakeSymbolPicker(	APP_WIDGET		symbolOption,
 	gotResources= 1;
 	}
 
-    if  ( psFontCatalog( ea->eaAfmDirectory,
-				    &fontFamilies, &fontFamilyCount ) )
-	{
-	SDEB(ea->eaAfmDirectory);
-	fontFamilies= (AppFontFamily *)0;
-	fontFamilyCount= 0;
-	}
+    if  ( appPostScriptFontCatalog( ea ) )
+	{ SDEB(ea->eaAfmDirectory);	}
 
     appGetFactors( ea, &horPixPerMM, &verPixPerMM, &xfac, &yfac );
 
     asp= (AppSymbolPicker *)malloc( sizeof(AppSymbolPicker)+
-					fontFamilyCount* sizeof(APP_WIDGET) );
+		ea->eaPostScriptFontList.psflFamilyCount* sizeof(APP_WIDGET) );
     if  ( ! asp )
 	{ XDEB(asp); return (void *)0;	}
 
     asp->aspApplication= ea;
-    asp->aspFontFamilies= fontFamilies;
-    asp->aspFontFamilyCount= fontFamilyCount;
+    asp->aspCurrentDocumentId= 0;
+    docInitFontList( &(asp->aspDocumentFontList) );
 
     asp->aspResources= &aspr;
 
+    appInitOptionmenu( &(asp->aspFontOptionmenu) );
+
     asp->aspTopWidget= (APP_WIDGET)0;
     asp->aspSymbolDrawing= (APP_WIDGET)0;
     asp->aspInsertButton= (APP_WIDGET)0;
@@ -753,12 +767,15 @@ void * appMakeSymbolPicker(	APP_WIDGET		symbolOption,
     asp->aspTarget= target;
 
     asp->aspSymbolSelected= -1;
-    asp->aspFontFamilySet= -1;
     asp->aspFontFamilyChosen= -1;
 
     asp->aspFont= (APP_FONT *)0;
     appInitDrawingData( &(asp->aspDrawingData) );
 
+    asp->aspDrawingData.addPostScriptFontList= &(ea->eaPostScriptFontList);
+
+    utilInitTextAttribute( &(asp->aspTextAttribute) );
+
     appMakeVerticalTool( &(asp->aspTopWidget), &(asp->aspMainWidget), ea,
 			    iconPixmap, iconMask, widgetName, userResizable,
 			    symbolOption, appCloseSymbolPicker, (void *)asp );
@@ -812,40 +829,61 @@ void appShowSymbolPicker(	void *	voidasp	)
 /*									*/
 /************************************************************************/
 
-int appAdaptSymbolPickerToFontFamily(	void *		voidasp,
-					int		fontFamilyNumber,
-					int		enc )
+int appAdaptSymbolPickerToFontFamily(
+				void *				voidasp,
+				unsigned int			documentId,
+				const DocumentFontList *	dflFrom,
+				const TextAttribute *		taNew,
+				const PropertyMask *		newMask )
     {
-    AppSymbolPicker *	asp= (AppSymbolPicker *)voidasp;
-    AppEncodingMenu *	aem= &(asp->aspEncodingMenu);
-    int			rval= 0;
-    AppFontFamily *	aff;
+    AppSymbolPicker *		asp= (AppSymbolPicker *)voidasp;
+    DocumentFontList *		dfl= &(asp->aspDocumentFontList);
+    AppEncodingMenu *		aem= &(asp->aspEncodingMenu);
+    int				rval= 0;
+    const DocumentFont *	df= dfl->dflFonts;
 
-    if  ( fontFamilyNumber < 0 )
-	{ fontFamilyNumber=  0; enc= 0;	}
-    if  ( enc < 0 )
-	{ enc= 0;	}
+    PropertyMask		doneMask;
 
-    asp->aspFontFamilySet= fontFamilyNumber;
-    asp->aspFontFamilyChosen= fontFamilyNumber;
+    if  ( asp->aspCurrentDocumentId != documentId )
+	{
+	if  ( docCopyFontList( dfl, dflFrom ) )
+	    { LDEB(1); return -1; }
 
-    aem->aemFontEncodingSet= enc;
-    aem->aemFontEncodingChosen= enc;
+	appSymbolFillFontMenu( asp->aspResources, asp );
 
-    if  ( appSymbolAdaptToFamily( asp, fontFamilyNumber ) )
-	{ LLDEB(fontFamilyNumber,enc); rval= -1;	}
+	asp->aspCurrentDocumentId= documentId;
+	}
 
-    aff= asp->aspFontFamilies+ fontFamilyNumber;
+    PROPmaskCLEAR( &doneMask );
+    utilUpdateTextAttribute( &doneMask, &(asp->aspTextAttribute),
+							    taNew, newMask );
 
-    if  ( appSymbolAdaptToFamilyEncoding( asp,
-					    asp->aspFontFamilyChosen,
-					    aem->aemFontEncodingChosen ) )
-	{ LLDEB(fontFamilyNumber,enc); rval= -1;	}
+    asp->aspTextAttribute.taFontSizeHalfPoints= 20;
+    asp->aspTextAttribute.taFontIsBold= 0;
+    asp->aspTextAttribute.taFontIsSlanted= 0;
+
+    if  ( asp->aspTextAttribute.taFontNumber >= 0 )
+	{
+	df= dfl->dflFonts+ asp->aspTextAttribute.taFontNumber;
+	asp->aspFontFamilyChosen= df->dfDocFamilyIndex;
+	}
+
+    if  ( asp->aspFontFamilyChosen < 0 )
+	{
+	asp->aspFontFamilyChosen= 0; 
+	asp->aspTextAttribute.taFontNumber= 0;
+	df= dfl->dflFonts+ asp->aspTextAttribute.taFontNumber;
+	}
+
+    if  ( appSymbolAdaptToFamily( asp ) )
+	{ LDEB(asp->aspFontFamilyChosen); rval= -1;	}
 
     appSetOptionmenu( &(asp->aspFontOptionmenu),
 					    asp->aspFontFamilyChosen );
-    appSetOptionmenu( &(aem->aemEncodingOptionmenu),
-					    aem->aemFontEncodingChosen );
+
+    appEncodingMenuSetEncoding( aem,
+			    dfl->dflFamilies+ asp->aspFontFamilyChosen,
+			    df->dfEncodingSet );
 
     return rval;
     }
