@@ -6,6 +6,7 @@
 
 #   include	<appWinMeta.h>
 #   include	<utilPs.h>
+#   include	<utilMatchFont.h>
 #   include	<sioEndian.h>
 
 #   define	y0	math_y0
@@ -128,15 +129,16 @@ static void appWinMetaSetTransform(	DeviceContext *		dc,
     return;
     }
 
-int appMetaInitDeviceContext(	DeviceContext *		dc,
-				int			objectCount,
-				int			mapMode,
-				int			xWinExt,
-				int			yWinExt,
-				int			xViewExt,
-				int			yViewExt,
-				int			twipsWide,
-				int			twipsHigh )
+int appMetaInitDeviceContext(	DeviceContext *			dc,
+				const PostScriptFontList *	psfl,
+				int				objectCount,
+				int				mapMode,
+				int				xWinExt,
+				int				yWinExt,
+				int				xViewExt,
+				int				yViewExt,
+				int				twipsWide,
+				int				twipsHigh )
     {
     dc->dcDrawBorders= 1;
     dc->dcFillInsides= 1;
@@ -146,6 +148,7 @@ int appMetaInitDeviceContext(	DeviceContext *		dc,
     dc->dcFillPattern= 0;
 
     docInitFontList( &(dc->x_dcFontList) );
+    dc->dcPostScriptFontList= psfl;
     appInitDrawingData( &(dc->dcDrawingData) );
 
     dc->dcPen.lpStyle= PS_SOLID;
@@ -215,7 +218,7 @@ int appMetaInitDeviceContext(	DeviceContext *		dc,
     dc->dcBlueSet= -1;
     dc->dcPenSet.lpStyle= -1;
 
-    docInitTextAttribute( &(dc->dcTextAttributeSet) );
+    utilInitTextAttribute( &(dc->dcTextAttributeSet) );
 
     return 0;
     }
@@ -497,15 +500,23 @@ int appMetaCreateFontIndirect(	DeviceContext *		dc,
 				int			recordSize,
 				SimpleInputStream *	sis )
     {
-    LogicalFont *	lf;
-    DocumentFont *	df;
+    int				rval= 0;
 
-    char *		familyStyle= "fnil";
+    LogicalFont *		lf;
 
-    int			count;
-    int			done;
-    int			ob;
-    int			i;
+    int				familyStyle= DFstyleFNIL;
+
+    int				count;
+    int				done;
+    int				ob;
+
+    int				fontNum;
+    DocumentFont		dfNew;
+    DocumentFont *		df;
+    DocumentFontList *		dfl= &(dc->x_dcFontList);
+    const PostScriptFontList *	psfl= dc->dcPostScriptFontList;
+
+    docInitDocumentFont( &dfNew );
 
     for ( ob= 0; ob < dc->dcObjectCount; ob++ )
 	{
@@ -551,53 +562,44 @@ int appMetaCreateFontIndirect(	DeviceContext *		dc,
 	{ done= -done;	}
     }
 
+    utilInitTextAttribute( &(lf->lfTextAttribute) );
+
     lf->lfTextAttribute.taFontNumber= 0;
     lf->lfTextAttribute.taFontSizeHalfPoints= ( done+ 5 )/ 10;
     lf->lfTextAttribute.taFontIsBold= lf->lfWeight > 500;
     lf->lfTextAttribute.taFontIsSlanted= lf->lfItalic != 0;
-    lf->lfTextAttribute.taIsUnderlined= lf->lfUnderline != 0;
+    lf->lfTextAttribute.taTextIsUnderlined= lf->lfUnderline != 0;
+    lf->lfTextAttribute.taHasStrikethrough= lf->lfStrikeOut != 0;
 
     switch( lf->lfPitchAndFamily & 0xf0 )
 	{
-	case FF_ROMAN:		familyStyle= "froman";	break;
-	case FF_SWISS:		familyStyle= "fswiss";	break;
-	case FF_MODERN:		familyStyle= "fmodern";	break;
-	case FF_SCRIPT:		familyStyle= "fscript";	break;
-	case FF_DECORATIVE:	familyStyle= "fdecor";	break;
-	case 0x60:		familyStyle= "ftech";	break;
+	case FF_ROMAN:		familyStyle= DFstyleFROMAN;	break;
+	case FF_SWISS:		familyStyle= DFstyleFSWISS;	break;
+	case FF_MODERN:		familyStyle= DFstyleFMODERN;	break;
+	case FF_SCRIPT:		familyStyle= DFstyleFSCRIPT;	break;
+	case FF_DECORATIVE:	familyStyle= DFstyleFDECOR;	break;
+	case 0x60:		familyStyle= DFstyleFTECH;	break;
 
 	case FF_DONTCARE:
 	default:
 	    break;
 	}
 
-    df= dc->x_dcFontList.dflFonts;
-    for ( i= 0; i < dc->x_dcFontList.dflCount; df++, i++ )
-	{
-	if  ( df->dfDocFamilyNumber < 0	 )
-	    { continue;	}
+    dfNew.dfCharset= lf->lfCharSet;
+    if  ( docFontSetFamilyName( &dfNew, lf->lfFaceName ) )
+	{ LDEB(1); rval= -1; goto ready;	}
+    if  ( docFontSetFamilyStyle( &dfNew, familyStyle ) )
+	{ LDEB(1); rval= -1; goto ready;	}
 
-	if  ( lf->lfFaceName[0]			&&
-	      ! df->dfName			)
-	    { continue;	}
+    fontNum= docMergeFontIntoFontlist( dfl, &dfNew );
+    if  ( fontNum < 0 )
+	{ SLDEB(lf->lfFaceName,fontNum); goto ready;	}
 
-	if  ( ! lf->lfFaceName[0]		&&
-	      ! df->dfName			)
-	    { break;	}
+    df= dfl->dflFonts+ fontNum;
+    if  ( utilFindPsFontForDocFont( df, dfl, psfl ) )
+	{ SDEB(df->dfName);	}
 
-	if  ( ! strcmp( df->dfName, lf->lfFaceName ) )
-	    { break;	}
-	}
-
-    if  ( i >= dc->x_dcFontList.dflCount )
-	{
-	df= docInsertFont( &(dc->x_dcFontList), -1,
-						familyStyle, lf->lfFaceName );
-	if  ( ! df )
-	    { SXDEB(lf->lfFaceName,df); return -1;	}
-	}
-
-    lf->lfTextAttribute.taFontNumber= df->dfDocFamilyNumber;
+    lf->lfTextAttribute.taFontNumber= fontNum;
 
     lf->lfPrivateFont= -1;
 
@@ -606,7 +608,11 @@ int appMetaCreateFontIndirect(	DeviceContext *		dc,
 			    lf->lfWidth,
 			    lf->lfFaceName, ob ));
 
-    return 0;
+  ready:
+
+    docCleanDocumentFont( &dfNew );
+
+    return rval;
     }
 
 int appMeta_ExtTextOut(		SimpleInputStream *	sis,
@@ -806,6 +812,16 @@ int appMetaSetWindowExt(	DeviceContext *		dc,
     dc->dcTransform.dct_yWinExt= h;
     dc->dcTransform.dct_xWinExt= w;
 
+    /*
+    Necessary?
+    appWinMetaSetTransform( dc, dc->dcMapMode,
+				    DC_xWinOrg(dc), DC_yWinOrg(dc),
+				    DC_xWinExt(dc), DC_yWinExt(dc),
+				    DC_xViewOrg(dc), DC_yViewOrg(dc),
+				    DC_xViewExt(dc), DC_yViewExt(dc),
+				    dc->dcTwipsWide, dc->dcTwipsHigh );
+    */
+
     return 0;
     }
 
@@ -933,6 +949,40 @@ int appMetaRestoreDC(		DeviceContext *		dc,
     return 0;
     }
 
+/************************************************************************/
+/*									*/
+/*  Collect a series of points.						*/
+/*									*/
+/************************************************************************/
+
+int appWinMetaGetPoints(	DeviceContext *		dc,
+				int			count,
+				SimpleInputStream *	sis )
+    {
+    APP_POINT *		xp;
+
+    int			done;
+
+    xp= (APP_POINT *)realloc( dc->dcPoints, (count+ 1)* sizeof(APP_POINT) );
+    if  ( ! xp )
+	{ LXDEB(count,xp); return -1;	}
+    dc->dcPoints= xp;
+
+    for ( done= 0; done < count; xp++, done++ )
+	{
+	int	x= sioEndianGetLeInt16( sis );
+	int	y= sioEndianGetLeInt16( sis );
+
+	xp->x= DC_xViewport( x, dc );
+	xp->y= DC_yViewport( y, dc );
+	}
+
+    *xp= dc->dcPoints[0];
+
+    return 0;
+    }
+
+
 /************************************************************************/
 /*									*/
 /*  Handle Clipping by ignoring it.					*/
@@ -1144,129 +1194,3 @@ int appMeta_GetCounts(	SimpleInputStream *	sis,
     *pCount= count; *pCounts= counts; return 0;
     }
 
-int appMetaSaveBitmapMetafile(	const BitmapDescription *	bd,
-				const unsigned char *		buffer,
-				SimpleOutputStream *		sos )
-    {
-    int			done;
-    int			bytesWritten= 0;
-    long		bltArgCountPos= -1L;
-    long		bltArgCount;
-
-    long		fileSize= 0L;
-    long		headerOffset= 0L;
-    long		recordSize;
-    long		maxRecordSize= 0L;
-
-    sioEndianPutLeInt16( 1, sos );		/*  fileType		*/
-    sioEndianPutLeInt16( 9, sos );		/*  headerSize		*/
-    sioEndianPutLeInt16( 768, sos );		/*  windowsVersion	*/
-    sioEndianPutLeInt32( fileSize, sos );	/*  fileSize		*/
-    sioEndianPutLeInt16( 0, sos );		/*  objectCount		*/
-    sioEndianPutLeInt32( maxRecordSize, sos );	/*  maxRecordSize	*/
-    sioEndianPutLeInt16( 0, sos );		/*  parameterCount	*/
-
-    bytesWritten += 2+ 2+ 2+ 4+ 2+ 4+ 2;
-
-    recordSize= 2+ 1;
-    sioEndianPutLeInt32( recordSize, sos );
-    sioEndianPutLeInt16( WINMETA_SaveDC, sos );
-
-    bytesWritten += 2* recordSize;
-    if  ( maxRecordSize < recordSize )
-	{ maxRecordSize=  recordSize;	}
-
-    recordSize= 2+ 1+ 1;
-    sioEndianPutLeInt32( recordSize, sos );
-    sioEndianPutLeInt16( WINMETA_SetMapMode, sos );
-    sioEndianPutLeInt16( MM_ANISOTROPIC, sos );
-
-    bytesWritten += 2* recordSize;
-    if  ( maxRecordSize < recordSize )
-	{ maxRecordSize=  recordSize;	}
-
-    recordSize= 2+ 1+ 1+ 1;
-    sioEndianPutLeInt32( recordSize, sos );
-    sioEndianPutLeInt16( WINMETA_SetWindowOrg, sos );
-    sioEndianPutLeInt16( 0, sos );
-    sioEndianPutLeInt16( 0, sos );
-
-    bytesWritten += 2* recordSize;
-    if  ( maxRecordSize < recordSize )
-	{ maxRecordSize=  recordSize;	}
-
-    recordSize= 2+ 1+ 1+ 1;
-    sioEndianPutLeInt32( recordSize, sos );
-    sioEndianPutLeInt16( WINMETA_SetWindowExt, sos );
-    sioEndianPutLeInt16( bd->bdPixelsHigh, sos );
-    sioEndianPutLeInt16( bd->bdPixelsWide, sos );
-
-    bytesWritten += 2* recordSize;
-    if  ( maxRecordSize < recordSize )
-	{ maxRecordSize=  recordSize;	}
-
-    bltArgCountPos= bytesWritten;
-    recordSize= 2+ 1+ 2+ ( 1+ 1+ 1+ 1 )+ ( 1+ 1+ 1+ 1 );
-    sioEndianPutLeInt32( recordSize, sos );
-    sioEndianPutLeInt16( WINMETA_StretchBlt, sos );
-    sioEndianPutLeInt32( 0xcc0020, sos );
-    sioEndianPutLeInt16( bd->bdPixelsHigh, sos );
-    sioEndianPutLeInt16( bd->bdPixelsWide, sos );
-    sioEndianPutLeInt16( 0, sos );
-    sioEndianPutLeInt16( 0, sos );
-    sioEndianPutLeInt16( bd->bdPixelsHigh, sos );
-    sioEndianPutLeInt16( bd->bdPixelsWide, sos );
-    sioEndianPutLeInt16( 0, sos );
-    sioEndianPutLeInt16( 0, sos );
-
-    done= bmBmpSaveDib( bd, buffer, bytesWritten+ 2* recordSize, (void *)sos );
-    if  ( done < 0 || done % 2 )
-	{ LDEB(done); return -1;	}
-
-    recordSize += done/ 2;
-    bltArgCount= recordSize;
-
-    bytesWritten += 2* recordSize;
-    if  ( maxRecordSize < recordSize )
-	{ maxRecordSize=  recordSize;	}
-
-    recordSize= 2+ 1+ 1;
-    sioEndianPutLeInt32( recordSize, sos );
-    sioEndianPutLeInt16( WINMETA_RestoreDC, sos );
-    sioEndianPutLeInt16( -1, sos );
-
-    bytesWritten += 2* recordSize;
-    if  ( maxRecordSize < recordSize )
-	{ maxRecordSize=  recordSize;	}
-
-    recordSize= 2+ 1;
-    sioEndianPutLeInt32( recordSize, sos );
-    sioEndianPutLeInt16( 0, sos );
-
-    bytesWritten += 2* recordSize;
-    if  ( maxRecordSize < recordSize )
-	{ maxRecordSize=  recordSize;	}
-
-    fileSize= bytesWritten;
-
-    if  ( sioOutSeek( sos, headerOffset ) )
-	{ LDEB(headerOffset); return -1; }
-
-    sioEndianPutLeInt16( 1, sos );		/*  fileType		*/
-    sioEndianPutLeInt16( 9, sos );		/*  headerSize		*/
-    sioEndianPutLeInt16( 768, sos );		/*  windowsVersion	*/
-    sioEndianPutLeInt32( fileSize, sos );	/*  fileSize		*/
-    sioEndianPutLeInt16( 0, sos );		/*  objectCount		*/
-    sioEndianPutLeInt32( maxRecordSize, sos );	/*  maxRecordSize	*/
-    sioEndianPutLeInt16( 0, sos );		/*  parameterCount	*/
-
-    if  ( sioOutSeek( sos, bltArgCountPos ) )
-	{ LDEB(headerOffset); return -1; }
-
-    sioEndianPutLeInt32( bltArgCount, sos );
-
-    if  ( sioOutSeek( sos, fileSize ) )
-	{ LDEB(headerOffset); return -1; }
-
-    return 0;
-    }
