@@ -3,6 +3,8 @@
 /*  Simple io streams, Fork input or output.. Usually for debugging	*/
 /*  purposes.								*/
 /*									*/
+/*  Additionally, a pure debugging output stream is provided.		*/
+/*									*/
 /************************************************************************/
 
 #   include	<sioGeneral.h>
@@ -13,3 +15,6 @@ extern SimpleInputStream * sioInDebugOpen(
 extern SimpleOutputStream * sioOutDebugOpen(
 					SimpleOutputStream *	sosOut );
 
+
+extern SimpleOutputStream * sioOutAppDebugOpen( void );
+
