@@ -2,10 +2,15 @@
 /*									*/
 /*  Simple io streams, recursive use for hexadecimal data.		*/
 /*									*/
+/*  Do not forget to manually add the '>' at the end if you use these	*/
+/*  streams in conjuntion with '/ASCIIHexDecode filter' in PostScript.	*/
+/*									*/
 /************************************************************************/
 
 #   include	"appUtilConfig.h"
 
+#   include	<stdlib.h>
+
 #   include	"sioHex.h"
 #   include	<appDebugon.h>
 
@@ -26,8 +31,7 @@ static unsigned char		SioHexIndices[256];
 /*									*/
 /************************************************************************/
 
-
-static int sioHexClose( void *	voids )
+static int sioInHexClose( void *	voids )
     { return 0;	}
 
 static int sioInHexReadBytes(	void *		voidsis,
@@ -94,8 +98,7 @@ SimpleInputStream * sioInHexOpen(	SimpleInputStream *	sisHex )
 	    { SioHexIndices[SioHexDigitsL[i]]= i; i++;	}
 	}
 
-    sis= sioInOpen( (void *)sisHex,
-			    sioInHexReadBytes, (SIOinSEEK)0, sioHexClose );
+    sis= sioInOpen( (void *)sisHex, sioInHexReadBytes, sioInHexClose );
 
     if  ( ! sis )
 	{ XDEB(sis); return (SimpleInputStream *)0; }
@@ -103,44 +106,105 @@ SimpleInputStream * sioInHexOpen(	SimpleInputStream *	sisHex )
     return sis;
     }
 
-static int sioOutHexWriteBytes(	void *			voidsos,
+/************************************************************************/
+/*									*/
+/*  Output.								*/
+/*									*/
+/************************************************************************/
+
+typedef struct HexOutputStream
+    {
+    SimpleOutputStream *	hosSosHex;
+    int				hosWide;
+    int				hosLastNl;
+    int				hosColumn;
+    } HexOutputStream;
+
+static int sioOutHexClose( void *	voidhos )
+    {
+    HexOutputStream *		hos= (HexOutputStream *)voidhos;
+    SimpleOutputStream *	sos= hos->hosSosHex;
+
+    if  ( hos->hosWide > 0 && hos->hosLastNl && hos->hosColumn > 0 )
+	{
+	sioOutPutCharacter( '\n', sos );
+	hos->hosColumn= 0;
+	}
+
+    free( hos );
+
+    return 0;
+    }
+
+static int sioOutHexWriteBytes(	void *			voidhos,
 				const unsigned char *	buffer,
 				int			count )
     {
-    SimpleOutputStream *	sos= (SimpleOutputStream *)voidsos;
+    HexOutputStream *		hos= (HexOutputStream *)voidhos;
+    SimpleOutputStream *	sos= hos->hosSosHex;
     int				done= 0;
 
     while( done < count )
 	{
+	if  ( hos->hosWide > 0 && hos->hosColumn >= hos->hosWide )
+	    {
+	    sioOutPutCharacter( '\n', sos );
+	    hos->hosColumn= 0;
+	    }
+
 	sioOutPutCharacter( SioHexDigitsL[ ( (*buffer) >> 4 ) & 0x0f ], sos );
 	sioOutPutCharacter( SioHexDigitsL[ ( (*buffer) >> 0 ) & 0x0f ], sos );
 
+	if  ( hos->hosWide > 0 )
+	    { hos->hosColumn += 2;	}
+
 	buffer++; done++;
 	}
 
     return count;
     }
 
-static int sioHexSeek(		void *			voidsos,
+static int sioHexSeek(		void *			voidhos,
 				long			pos )
     {
-    SimpleOutputStream *	sos= (SimpleOutputStream *)voidsos;
+    HexOutputStream *		hos= (HexOutputStream *)voidhos;
 
-    if  ( sioOutSeek( sos, 2* pos ) )
+    if  ( hos->hosWide > 0 )
+	{ LDEB(hos->hosWide); return -1;	}
+
+    if  ( sioOutSeek( hos->hosSosHex, 2* pos ) )
 	{ LDEB(0); return -1;	}
 
     return 0;
     }
 
-SimpleOutputStream * sioOutHexOpen(	SimpleOutputStream *	sosHex )
+SimpleOutputStream * sioOutHexOpenFolded(
+				    SimpleOutputStream *	sosHex,
+				    int				wide,
+				    int				lastNl )
     {
+    HexOutputStream *		hos= malloc( sizeof(HexOutputStream) );
     SimpleOutputStream *	sos;
 
-    sos= sioOutOpen( (void *)sosHex, sioOutHexWriteBytes,
-						sioHexSeek, sioHexClose );
+    if  ( ! hos )
+	{ XDEB(hos); return (SimpleOutputStream *)0;	}
+
+    hos->hosSosHex= sosHex;
+    hos->hosWide= wide;
+    hos->hosLastNl= lastNl;
+    hos->hosColumn= 0;
+
+    sos= sioOutOpen( (void *)hos, sioOutHexWriteBytes,
+						sioHexSeek, sioOutHexClose );
 
     if  ( ! sos )
-	{ XDEB(sos); return (SimpleOutputStream *)0; }
+	{ XDEB(sos); free( hos ); return (SimpleOutputStream *)0; }
 
     return sos;
     }
+
+SimpleOutputStream * sioOutHexOpen(	SimpleOutputStream *	sosHex )
+    {
+    return sioOutHexOpenFolded( sosHex, 0, 0 );
+    }
+
