@@ -14,6 +14,8 @@ typedef struct PrintDestination
     char *	pdCommand;
     char *	pdPrinterName;
     int		pdPrintKind;
+    int		pdPercentCount;
+    int		pdCommandLength;
     } PrintDestination;
 
 /************************************************************************/
@@ -22,6 +24,10 @@ typedef struct PrintDestination
 
 extern int utilPrinterGetPrinters(	int *			pPrinterCount,
 					int *			pDefaultPrinter,
-					PrintDestination **	pDestinations );
+					PrintDestination **	pDestinations,
+					const char *		customCommand,
+					const char *		customName,
+					const char *		customCommand2,
+					const char *		customName2 );
 
 #   endif
