@@ -4,13 +4,81 @@
 #   include	<stdlib.h>
 #   include	<string.h>
 #   include	"bmintern.h"
-#   include	"bmp.h"
 #   include	<sioHex.h>
 #   include	<sioEndian.h>
 #   include	<sioStdio.h>
 
 #   include	<appDebugon.h>
 
+/************************************************************************/
+/*									*/
+/*  Data structures.							*/
+/*									*/
+/************************************************************************/
+
+/************************************************************************/
+/*  Describes a BMP file.						*/
+/************************************************************************/
+
+#define COMPRESS_RGB        0L      /* No compression               */
+#define COMPRESS_RLE8       1L      /* 8 bits per pixel compression */
+#define COMPRESS_RLE4       2L      /* 4 bits per pixel compression */
+#define BMP_ID              0x4d42  /* BMP "magic" number           */
+
+/************************************************************************/
+/*  Common information for all kinds of BMP files.			*/
+/************************************************************************/
+typedef struct BmpFileHeader
+    {
+			/************************************************/
+			/*  Common to all variants.			*/
+			/************************************************/
+    short	bfhMagic;
+    long	bfhFileSize;
+    short	bfhReserved1;
+    short	bfhReserved2;
+    long	bfhBitmapDataOffset;
+    long	bfhSizeOfRestOfHeader;	/*  It is the Type of file!	*/
+			/************************************************/
+			/*  Common to all variants.			*/
+			/************************************************/
+    long	bfhPixelsWide;
+    long	bfhPixelsHigh;
+    short	bfhPlanes;
+    short	bfhBitsPerPixel;
+			/************************************************/
+			/*  Common to Windows 3.* and OS/2 2.*		*/
+			/************************************************/
+    long	bfhCompression;
+    long	bfhBufferLength;
+    long	bfhXResolution;
+    long	bfhYResolution;
+    long	bfhColorCount;
+    long	bfhImportantColors;
+    } BmpFileHeader;
+
+/************************************************************************/
+/*  Structures desribing ICO files.					*/
+/************************************************************************/
+typedef struct IcoFileHeader
+    {
+    short	ifhReserved;
+    short	ifhType;
+    short	ifhCount;
+    } IcoFileHeader;
+
+typedef struct IcoDirectoryEntry
+    {
+    unsigned char	ideWidth;
+    unsigned char	ideHeight;
+    unsigned char	ideColorCount;
+    unsigned char	ideReserved;
+    short		idePlanes;
+    short		ideBitCount;
+    long		ideBytesInRes;
+    long		ideImageOffset;
+    } IcoDirectoryEntry;
+
 /************************************************************************/
 /*									*/
 /*  Read a 3 or 4 byte color table.					*/
@@ -354,14 +422,16 @@ static int bmpHeaderToDescription(	BitmapDescription *	bd,
 	else{ bd->bdColorCount= bfh->bfhColorCount; }
 
 	bd->bdColorEncoding= BMcoRGB8PALETTE;
+	bd->bdBitsPerPixel= ( 1+ assumeAlpha )* bfh->bfhBitsPerPixel;
+	}
+    else{
+	bd->bdColorEncoding= BMcoRGB;
 	}
-    else{ bd->bdColorEncoding= BMcoRGB;		}
 
     bd->bdHasAlpha= assumeAlpha;
 
-    bd->bdBitsPerPixel= ( 1+ assumeAlpha )* bfh->bfhBitsPerPixel;
-    bd->bdBytesPerRow= ( bd->bdBitsPerPixel* bd->bdPixelsWide + 7 )/ 8;
-    bd->bdBufferLength= bd->bdBytesPerRow* bd->bdPixelsHigh;
+    if  ( bmCalculateSizes( bd ) )
+	{ LDEB(1); return -1;	}
 
     *pBytesPerRow= ( bfh->bfhBitsPerPixel* bd->bdPixelsWide + 7 )/ 8;
 
@@ -729,14 +799,15 @@ static int bmBmpReadImageBytes(	unsigned char **		pBuffer,
 /*									*/
 /*  1)	Read and check header.						*/
 /*  2)  Translate header to a BitmapDescription.			*/
-/*  3)  Seek to data. ( We are supposed to be there).			*/
+/*  3)  Seek to data. ( We are supposed to be there ).			*/
 /*									*/
 /************************************************************************/
+
 int bmReadBmpFile(	const char *		filename,
 			unsigned char **	pBuffer,
 			BitmapDescription *	bd,
 			int *			pPrivateFormat,
-			double *		pCompressionFactor	)
+			double *		pCompressionFactor )
     {
     SimpleInputStream *	sis;
     BmpFileHeader	bfh;
