@@ -1,3 +1,6 @@
+#   ifndef	BM_INTERN_H
+#   define	BM_INTERN_H
+
 #   include	<stdio.h>
 #   include	<stddef.h>
 #   include	<stdlib.h>
@@ -16,7 +19,9 @@ extern unsigned char	Bmc1Masks[8];
 extern unsigned char	Bmc7Masks[8];
 
 /************************************************************************/
+/*									*/
 /*  Routines.								*/
+/*									*/
 /************************************************************************/
 
 extern int bmReadTiffFile(	const char *		filename,
@@ -41,6 +46,12 @@ extern int bmReadBmpFile(	const char *		filename,
 				int *			pPrivateFormat,
 				double *		pCompressionFactor );
 
+extern int bmReadWbmpFile(	const char *		filename,
+				unsigned char **	pBuffer,
+				BitmapDescription *	bd,
+				int *			pPrivateFormat,
+				double *		pCompressionFactor );
+
 extern int bmReadIcoFile(	const char *		filename,
 				unsigned char **	pBuffer,
 				BitmapDescription *	bd,
@@ -51,6 +62,10 @@ extern int bmCanWriteBmpFile( const BitmapDescription *	bd,
 				int			privateFormat,
 				double			compressionFactor );
 
+extern int bmCanWriteWbmpFile( const BitmapDescription *	bd,
+				int			privateFormat,
+				double			compressionFactor );
+
 extern int bmCanWriteIcoFile( const BitmapDescription *	bd,
 				int			privateFormat,
 				double			compressionFactor );
@@ -61,6 +76,12 @@ extern int bmWriteBmpFile(	const char *		filename,
 				int			privateFormat,
 				double			compressionFactor );
 
+extern int bmWriteWbmpFile(	const char *		filename,
+				const unsigned char *	buffer,
+				const BitmapDescription * bd,
+				int			privateFormat,
+				double			compressionFactor );
+
 extern int bmWriteIcoFile(	const char *		filename,
 				const unsigned char *	buffer,
 				const BitmapDescription * bd,
@@ -148,6 +169,16 @@ extern int bmWriteXbmFile(	const char *		filename,
 				int			privateFormat,
 				double			compressionFactor );
 
+extern int bmCanWriteWmfFile( const BitmapDescription *	bd,
+				int			privateFormat,
+				double			compressionFactor );
+
+extern int bmWriteWmfFile(	const char *		filename,
+				const unsigned char *	buffer,
+				const BitmapDescription *	bd,
+				int			privateFormat,
+				double			compressionFactor );
+
 extern int bmReadPngFile(	const char *		filename,
 				unsigned char **	pBuffer,
 				BitmapDescription *	bd,
@@ -217,31 +248,11 @@ extern int bmFillBlock(	unsigned char *		buffer,
 			int			y0,
 			int			x1,
 			int			y1,
-			int			bytesPerRow,
-			int			colorEncoding	);
+			int			bytesPerRow );
 
 extern unsigned char *	bmBackgroundBuffer(	int	bufferLength,
 						int	colorEncoding	);
 
-extern int bmDrawLine(	unsigned char *		buffer,
-			int			x0,
-			int			y0,
-			int			x1,
-			int			y1,
-			int			wide,
-			int			bytesPerRow,
-			int			colorEncoding	);
-
-extern int bmDrawBox(	unsigned char *		buffer,
-			int			x0,
-			int			y0,
-			int			x1,
-			int			y1,
-			int			width,
-			int			bytesPerRow,
-			int			colorEncoding	);
-
-
 extern int bmMakeGrayPalette(	const BitmapDescription *	bd,
 				int *				pColorCount,
 				RGB8Color *			palette );
@@ -252,11 +263,26 @@ extern int bmInflateTo8bit(		unsigned char *			to,
 					int				trnsC,
 					int				remA );
 
-extern int bmInflateTo8bitGray(		unsigned char *			to,
-					const unsigned char *		from,
-					const BitmapDescription *	bd );
+extern int bmInflateTo8bitGray(	
+			unsigned char *			to,
+			const unsigned char *		from,
+			int				bitsPerPixel,
+			int				pixelsWide );
 
 extern int bmMakeMonochrome(		BitmapDescription *	bd,
 					RGB8Color *		palette,
 					unsigned char *		buffer );
 
+extern int bmSetColorAllocatorForPaletteImage(
+					ColorAllocator *		ca,
+					const BitmapDescription *	bd );
+
+extern int bmSetColorAllocatorForBWImage(
+					ColorAllocator *		ca,
+					const BitmapDescription *	bd );
+
+extern int bmSetColorAllocatorForWBImage(
+					ColorAllocator *		ca,
+					const BitmapDescription *	bd );
+
+#   endif /*  BM_INTERN_H	*/
