@@ -6,11 +6,11 @@
 #   include	"bitmap.h"
 #   include	<appDebugon.h>
 
-int bmWrite(	const char *		filename,
-		const unsigned char *	buffer,
-		BitmapDescription *	bd,
-		int			fileFormat,
-		double			compressionFactor	)
+int bmWrite(	const char *			filename,
+		const unsigned char *		buffer,
+		const BitmapDescription *	bd,
+		int				fileFormat,
+		double				compressionFactor )
     {
     if  ( ! bmFileFormats[fileFormat].bffFileType->bftWrite )
 	{
@@ -26,9 +26,9 @@ int bmWrite(	const char *		filename,
 			compressionFactor			);
     }
 
-int bmCanWrite( BitmapDescription *	bd,
-		int			fileFormat,
-		double			compressionFactor	)
+int bmCanWrite( const BitmapDescription *	bd,
+		int				fileFormat,
+		double				compressionFactor )
     {
     if  ( ! bmFileFormats[fileFormat].bffFileType->bftWrite )
 	{ return -1;	}
@@ -39,11 +39,17 @@ int bmCanWrite( BitmapDescription *	bd,
 			compressionFactor			);
     }
 
+/************************************************************************/
+/*									*/
+/*  Read a bitmap/pixmap file. Derive the type from the file name.	*/
+/*									*/
+/************************************************************************/
+
 int bmRead(	const char *		filename,
 		unsigned char **	pBuffer,
 		BitmapDescription *	bd,
 		int *			pFileFormat,
-		double *		pCompressionFactor	)
+		double *		pCompressionFactor )
     {
     int		fileType;
     int		privateType;
