@@ -256,11 +256,11 @@ static int bmJpegGetReadParameters( struct jpeg_decompress_struct * cinfo )
 
 /************************************************************************/
 /*									*/
-/*  Store one row of pixels.						*/
+/*  Remember one row of pixels.						*/
 /*									*/
 /************************************************************************/
 
-static int bmJpegStoreScanline( JSAMPARRAY			pixel_data,
+static int bmJpegRememberScanline( JSAMPARRAY			pixel_data,
 				struct jpeg_decompress_struct * cinfo	)
     {
     BmJpegInputSource *		bjis= (BmJpegInputSource *)cinfo->src;
@@ -433,7 +433,7 @@ static int read_JPEG_file(	BmJpegInputSource * bjis )
 	{
 	(void)jpeg_read_scanlines( &cinfo, buffer, 1 );
 
-	bmJpegStoreScanline( buffer, &cinfo );
+	bmJpegRememberScanline( buffer, &cinfo );
 	}
 
     (void)jpeg_finish_decompress( &cinfo );
@@ -493,18 +493,20 @@ static int bmJpegFillCompressor(	struct jpeg_compress_struct *	cinfo,
     cinfo->image_width= bd->bdPixelsWide;
     cinfo->image_height= bd->bdPixelsHigh;
 
-    cinfo->input_components= bd->bdSamplesPerPixel;
-
     switch( bd->bdColorEncoding )
 	{
 	case BMcoBLACKWHITE:
 	case BMcoWHITEBLACK:
+	    cinfo->input_components= 1;
 	    cinfo->in_color_space= JCS_GRAYSCALE;
 	    break;
+
 	case BMcoRGB:
 	case BMcoRGB8PALETTE:
+	    cinfo->input_components= 3;
 	    cinfo->in_color_space= JCS_RGB;
 	    break;
+
 	default:
 	    LDEB(bd->bdColorEncoding);
 	    return -1;
@@ -580,7 +582,7 @@ static int bmJpegCompressRow(	struct jpeg_compress_struct *	cinfo,
 	case BMcoRGB8PALETTE:
 	    bmExpandRGB8Palette( bjod->bjodScratchBuffer, from,
 				    bd->bdPixelsWide, bd->bdBitsPerPixel,
-				    bd->bdRGB8Palette, 0 );
+				    bd->bdRGB8Palette, bd->bdHasAlpha );
 
 	    row_pointer[0]= (JSAMPROW)bjod->bjodScratchBuffer;
 	    res= jpeg_write_scanlines( cinfo, row_pointer, 1 );
@@ -647,6 +649,7 @@ int bmJpegWriteJfif(	const BitmapDescription *	bd,
 			const unsigned char *		buffer,
 			SimpleOutputStream *		sos )
     {
+    int				rval= 0;
     BmJpegOutputDestination	bjod;
 
     bjod.bjodBd= bd;
@@ -665,7 +668,7 @@ int bmJpegWriteJfif(	const BitmapDescription *	bd,
 	    if  ( ! bjod.bjodScratchBuffer )
 		{
 		LXDEB(bd->bdBytesPerRow,bjod.bjodScratchBuffer);
-		return -1;
+		rval= -1; goto ready;
 		}
 	    break;
 
@@ -674,7 +677,19 @@ int bmJpegWriteJfif(	const BitmapDescription *	bd,
 	    if  ( ! bjod.bjodScratchBuffer )
 		{
 		LXDEB(bd->bdBytesPerRow,bjod.bjodScratchBuffer);
-		return -1;
+		rval= -1; goto ready;
+		}
+	    break;
+
+	case BMcoRGB:
+	    if  ( bd->bdHasAlpha )
+		{
+		bjod.bjodScratchBuffer= malloc( 3* bd->bdPixelsWide );
+		if  ( ! bjod.bjodScratchBuffer )
+		    {
+		    LXDEB(bd->bdBytesPerRow,bjod.bjodScratchBuffer);
+		    rval= -1; goto ready;
+		    }
 		}
 	    break;
 
@@ -683,19 +698,14 @@ int bmJpegWriteJfif(	const BitmapDescription *	bd,
 	}
 
     if  ( write_JPEG_file( &bjod ) )
-	{
-	LDEB(1);
+	{ LDEB(1); rval= -1; goto ready; }
 
-	if  ( bjod.bjodScratchBuffer )
-	    { free( bjod.bjodScratchBuffer );	}
-
-	return -1;
-	}
+  ready:
 
     if  ( bjod.bjodScratchBuffer )
 	{ free( bjod.bjodScratchBuffer );	}
 
-    return 0;
+    return rval;
     }
 
 int bmWriteJpegFile(	const char *			filename,
