@@ -5,6 +5,7 @@
 #   include	<png.h>
 #   include	<appDebugon.h>
 #   include	<sioStdio.h>
+#   include	<utilEndian.h>
 
 # if 1
 #   ifndef png_jmpbuf
@@ -35,7 +36,7 @@ static int bpPngiToBitmap(	const png_info *		pngi,
 	    bd->bdColorCount= pngi->num_palette;
 	    bd->bdBitsPerSample= 8;
 	    bd->bdSamplesPerPixel= 3;
-	    bd->bdBitsPerPixel= pngi->pixel_depth;
+	    bd->bdBitsPerPixel= pngi->bit_depth;
 
 	    bd->bdRGB8Palette= (RGB8Color *)
 				malloc( bd->bdColorCount* sizeof(RGB8Color) );
@@ -51,12 +52,14 @@ static int bpPngiToBitmap(	const png_info *		pngi,
 		bd->bdRGB8Palette[col].rgb8Alpha= 0;
 		}
 	    break;
+
 	case PNG_COLOR_TYPE_RGB:
 	    bd->bdColorEncoding= BMcoRGB;
 	    bd->bdBitsPerSample= pngi->bit_depth;
 	    bd->bdSamplesPerPixel= pngi->channels;
 	    bd->bdBitsPerPixel= pngi->pixel_depth;
 	    break;
+
 	case PNG_COLOR_TYPE_GRAY:
 	    bd->bdColorEncoding= BMcoWHITEBLACK;
 	    bd->bdBitsPerSample= pngi->bit_depth;
@@ -65,9 +68,17 @@ static int bpPngiToBitmap(	const png_info *		pngi,
 	    break;
 
 	case PNG_COLOR_TYPE_RGB_ALPHA:
+	    bd->bdHasAlpha= 1;
+	    bd->bdColorEncoding= BMcoRGB;
+	    bd->bdBitsPerSample= pngi->bit_depth;
+	    bd->bdSamplesPerPixel= pngi->channels;
+	    bd->bdBitsPerPixel= pngi->pixel_depth;
+	    break;
+
 	case PNG_COLOR_TYPE_GRAY_ALPHA:
 	    bd->bdHasAlpha= 1;
 	    LDEB(pngi->color_type); return -1;
+
 	default:
 	    LDEB(pngi->color_type); return -1;
 	}
@@ -102,15 +113,25 @@ static int bmPngReadContents(	png_info *		pngi,
 				BitmapDescription *	bd,
 				unsigned char **	pBuffer )
     {
-    int			numberOfPasses;
-    int			i;
+    int				numberOfPasses;
+    int				i;
 
-    unsigned char *	buffer;
+    unsigned char *		buffer;
 
     numberOfPasses= 1;
     if  ( pngi->interlace_type )
 	{ numberOfPasses= png_set_interlace_handling( png ); }
 
+    if  ( pngi->color_type == PNG_COLOR_TYPE_RGB	&&
+	  pngi->bit_depth == 16				)
+	{
+	const unsigned short	one= 1;
+	const unsigned char *	testEndian= (const unsigned char *)&one;
+
+	if  ( testEndian[0] )
+	    { png_set_swap( png );	}
+	}
+
     png_start_read_image( png );
     /*
     png_read_update_info( png, pngi );
@@ -219,7 +240,7 @@ int bmPngReadPng(	BitmapDescription *	bd,
 
     if  ( bpPngiToBitmap( pngip, bd ) )
 	{
-	LLDEB(bd->bdColorCount,bd->bdRGB8Palette); 
+	LLLDEB(pngip->color_type,bd->bdColorCount,bd->bdRGB8Palette); 
 	png_destroy_read_struct( &pngp, &pngip, (png_infop *)0 );
 	return -1;
 	}
@@ -308,6 +329,7 @@ static int bpPngiFromBitmap(	png_info *			pngi,
 	    pngi->color_type= PNG_COLOR_TYPE_GRAY;
 	    pngi->sig_bit.gray= bd->bdBitsPerSample;
 	    break;
+
 	case BMcoRGB:
 	    pngi->bit_depth= bd->bdBitsPerSample;
 	    pngi->color_type= PNG_COLOR_TYPE_RGB;
@@ -315,6 +337,7 @@ static int bpPngiFromBitmap(	png_info *			pngi,
 	    pngi->sig_bit.green= bd->bdBitsPerSample;
 	    pngi->sig_bit.blue= bd->bdBitsPerSample;
 	    break;
+
 	case BMcoRGB8PALETTE:
 	    pngi->bit_depth= bd->bdBitsPerPixel;
 	    pngi->color_type= PNG_COLOR_TYPE_PALETTE;
@@ -334,6 +357,7 @@ static int bpPngiFromBitmap(	png_info *			pngi,
 		pngi->palette[i].blue= bd->bdRGB8Palette[i].rgb8Blue;
 		}
 	    break;
+
 	default:
 	    LDEB(bd->bdColorEncoding);
 	    return -1;
@@ -363,13 +387,32 @@ static void bmPngWriteContents(	png_structp			png,
 	    }
 	}
 
+    if  ( pngi->color_type == PNG_COLOR_TYPE_RGB	&&
+	  bd->bdBitsPerSample == 16			)
+	{
+	const unsigned short	one= 1;
+	const unsigned char *	testEndian= (const unsigned char *)&one;
+
+	/* Does not work!
+	if  ( testEndian[0] )
+	    { png_set_swap( png );	}
+	*/
+	if  ( testEndian[0] )
+	    {
+	    scratch= (unsigned char *)malloc( bd->bdBytesPerRow );
+	    if  ( ! scratch )
+		{ LXDEB(bd->bdBytesPerRow,scratch);	}
+	    }
+	}
+
     png_write_info( png, pngi );
 
     for ( row= 0; row < bd->bdPixelsHigh; row++ )
 	{
 	const unsigned char *	from= buffer+ row* bd->bdBytesPerRow;
 
-	if  ( scratch )
+	if  ( bd->bdColorEncoding == BMcoBLACKWHITE	&&
+	      scratch					)
 	    {
 	    int			col;
 	    unsigned char *	to= scratch;
@@ -380,6 +423,23 @@ static void bmPngWriteContents(	png_structp			png,
 	    from= scratch;
 	    }
 
+	if  ( pngi->color_type == PNG_COLOR_TYPE_RGB	&&
+	      bd->bdBitsPerSample == 16			&&
+	      scratch					)
+	    {
+	    int			col;
+	    const BmUint16 *	fr= (const BmUint16 *)from;
+	    unsigned char *	to= scratch;
+
+	    for ( col= 0; col < bd->bdBytesPerRow; col += sizeof(BmUint16) )
+		{
+		utilEndianStoreBeInt16( *(fr++), to );
+		to += sizeof(BmUint16);
+		}
+
+	    from= scratch;
+	    }
+
 	png_write_rows( png, (unsigned char **)&from, 1 );
 	}
 
@@ -463,9 +523,11 @@ int bmPngWritePng(		const BitmapDescription *	bd,
 	{
 	LDEB(1);
 	png_destroy_write_struct( &pngp, &pngip );
+	/*  Crashes:
 	if  ( bd->bdColorEncoding == BMcoRGB8PALETTE	&&
 	      pngip->palette				)
 	    { free( pngip->palette );	}
+	*/
 	return -1;
 	}
 
