@@ -232,14 +232,17 @@ int bmReadXpmFile(	const char *		filename,
 		if  ( transparent && ! bd->bdHasAlpha )
 		    {
 		    bd->bdHasAlpha= 1;
-		    bd->bdSamplesPerPixel= 4;
 		    bd->bdBitsPerPixel= 2* bitsPerPixel;
-		    bd->bdBytesPerRow=
-			    ( bd->bdBitsPerPixel+ 7 )/8* bd->bdPixelsWide;
-		    bd->bdBufferLength= bd->bdPixelsHigh* bd->bdBytesPerRow;
 		    }
 		}
 
+	    if  ( bmCalculateSizes( bd ) )
+		{
+		LXDEB(bd->bdColorCount,bd->bdRGB8Palette);
+		XpmFreeXpmImage( &image );
+		return -1;
+		}
+
 	    buffer= (unsigned char *)malloc( bd->bdBufferLength );
 	    if  ( ! buffer )
 		{
@@ -259,8 +262,6 @@ int bmReadXpmFile(	const char *		filename,
     bd->bdXResolution= bd->bdYResolution= 1;
     bd->bdUnit= BMunPIXEL;
 
-    bd->bdHasAlpha= 0;
-
     switch( bd->bdBitsPerPixel )
 	{
 	case 8:
