@@ -10,6 +10,8 @@
 #   undef	APP_DEB
 #   endif
 
+/*  # include "../mecheck/mecheck.h"  */
+
 #ifdef	__GNUC__
 extern int appDebug(	const char *	format, ... )
 			__attribute__ (( format ( printf, 1, 2 ) ));
@@ -85,6 +87,15 @@ extern int appDebugSetFile(	const char *	filename,
 			    #c1, DEBC(c1),			\
 			    #c2, DEBC(c2)			))
 
+# define LCCDEB(l1,c2,c3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFML				\
+			    DFMC				\
+			    DFMC				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #c2, DEBC(c2),			\
+			    #c3, DEBC(c3)			))
+
 # define SSDEB(s1,s2) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFMS				\
 			    DFMS				\
@@ -187,6 +198,17 @@ extern int appDebugSetFile(	const char *	filename,
 			    #l2, DEBL(l2),			\
 			    #s3, DEBS(s3)			))
 
+# define SLSSDEB(s1,l2,s3,s4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFMS				\
+			    DFML				\
+			    DFMS				\
+			    DFMS				\
+			    "\n", __FILE__, __LINE__,		\
+			    #s1, DEBS(s1),			\
+			    #l2, DEBL(l2),			\
+			    #s3, DEBS(s3),			\
+			    #s4, DEBS(s4)			))
+
 # define XSDEB(l1,s2) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFMX				\
 			    DFMS				\
@@ -201,6 +223,15 @@ extern int appDebugSetFile(	const char *	filename,
 			    #l1, DEBL(l1),			\
 			    #d2, DEBD(d2)			))
 
+# define LFLDEB(l1,d2,l3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFML				\
+			    DFMD				\
+			    DFML				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #d2, DEBD(d2),			\
+			    #l3, DEBL(l3)			))
+
 # define FFDEB(d1,d2) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFMD				\
 			    DFMD				\
@@ -309,6 +340,15 @@ extern int appDebugSetFile(	const char *	filename,
 			    #s2, DEBS(s2),			\
 			    #l3, DEBL(l3)			))
 
+# define LSXDEB(l1,s2,l3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFML				\
+			    DFMS				\
+			    DFMX				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #s2, DEBS(s2),			\
+			    #l3, DEBX(l3)			))
+
 # define LSFDEB(l1,s2,d3) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFML				\
 			    DFMS				\
@@ -357,7 +397,18 @@ extern int appDebugSetFile(	const char *	filename,
 			    DFML				\
 			    DFMS				\
 			    "\n", __FILE__, __LINE__,		\
-			    #l1, DEBS(l1),			\
+			    #l1, DEBL(l1),			\
+			    #l2, DEBL(l2),			\
+			    #l3, DEBL(l3),			\
+			    #s4, DEBS(s4)			))
+
+# define SLLSDEB(s1,l2,l3,s4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFMS				\
+			    DFML				\
+			    DFML				\
+			    DFMS				\
+			    "\n", __FILE__, __LINE__,		\
+			    #s1, DEBS(s1),			\
 			    #l2, DEBL(l2),			\
 			    #l3, DEBL(l3),			\
 			    #s4, DEBS(s4)			))
@@ -371,6 +422,15 @@ extern int appDebugSetFile(	const char *	filename,
 			    #l2, DEBL(l2),			\
 			    #l3, DEBL(l3)			))
 
+# define XLLDEB(l1,l2,l3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFMX				\
+			    DFML				\
+			    DFML				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBX(l1),			\
+			    #l2, DEBL(l2),			\
+			    #l3, DEBL(l3)			))
+
 # define XLXDEB(l1,l2,l3) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFMX				\
 			    DFML				\
@@ -391,6 +451,50 @@ extern int appDebugSetFile(	const char *	filename,
 			    #l3, DEBL(l3),			\
 			    #l4, DEBL(l4)			))
 
+# define LLXXDEB(l1,l2,l3,l4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFML				\
+			    DFML				\
+			    DFMX				\
+			    DFMX				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #l2, DEBL(l2),			\
+			    #l3, DEBX(l3),			\
+			    #l4, DEBX(l4)			))
+
+# define FFLLDEB(d1,d2,l3,l4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFMD				\
+			    DFMD				\
+			    DFML				\
+			    DFML				\
+			    "\n", __FILE__, __LINE__,		\
+			    #d1, DEBD(d1),			\
+			    #d2, DEBD(d2),			\
+			    #l3, DEBL(l3),			\
+			    #l4, DEBL(l4)			))
+
+# define XXXXDEB(l1,l2,l3,l4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFMX				\
+			    DFMX				\
+			    DFMX				\
+			    DFMX				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBX(l1),			\
+			    #l2, DEBX(l2),			\
+			    #l3, DEBX(l3),			\
+			    #l4, DEBX(l4)			))
+
+# define XXXLDEB(l1,l2,l3,l4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFMX				\
+			    DFMX				\
+			    DFMX				\
+			    DFML				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBX(l1),			\
+			    #l2, DEBX(l2),			\
+			    #l3, DEBX(l3),			\
+			    #l4, DEBL(l4)			))
+
 # define LLLLLDEB(l1,l2,l3,l4,l5) APP_DEB(appDebug( "%s(%3d)"	\
 			    DFML				\
 			    DFML				\
@@ -404,6 +508,32 @@ extern int appDebugSetFile(	const char *	filename,
 			    #l4, DEBL(l4),			\
 			    #l5, DEBL(l5)			))
 
+# define LLLLLLDEB(l1,l2,l3,l4,l5,l6) APP_DEB(appDebug( "%s(%3d)" \
+			    DFML				\
+			    DFML				\
+			    DFML				\
+			    DFML				\
+			    DFML				\
+			    DFML				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #l2, DEBL(l2),			\
+			    #l3, DEBL(l3),			\
+			    #l4, DEBL(l4),			\
+			    #l5, DEBL(l5),			\
+			    #l6, DEBL(l6)			))
+
+# define LLLFDEB(l1,l2,l3,d4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFML				\
+			    DFML				\
+			    DFML				\
+			    DFMD				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #l2, DEBL(l2),			\
+			    #l3, DEBL(l3),			\
+			    #d4, DEBD(d4)			))
+
 # define LXXDEB(l1,l2,l3) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFML				\
 			    DFMX				\
@@ -440,7 +570,31 @@ extern int appDebugSetFile(	const char *	filename,
 			    #d2, DEBD(d2),			\
 			    #d3, DEBD(d3)			))
 
-# define LFFFDEB(l1,d2,d3,d4) APP_DEB(appDebug( "%s(%3d)"		\
+# define LFLLDEB(l1,d2,l3,l4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFML				\
+			    DFMD				\
+			    DFML				\
+			    DFML				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #d2, DEBD(d2),			\
+			    #l3, DEBL(l3),			\
+			    #l4, DEBL(l4)			))
+
+# define LLFFFDEB(l1,l2,d3,d4,d5) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFML				\
+			    DFML				\
+			    DFMD				\
+			    DFMD				\
+			    DFMD				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #l2, DEBL(l2),			\
+			    #d3, DEBD(d3),			\
+			    #d4, DEBD(d4),			\
+			    #d5, DEBD(d5)			))
+
+# define LFFFDEB(l1,d2,d3,d4) APP_DEB(appDebug( "%s(%3d)"	\
 			    DFML				\
 			    DFMD				\
 			    DFMD				\
@@ -451,6 +605,19 @@ extern int appDebugSetFile(	const char *	filename,
 			    #d3, DEBD(d3),			\
 			    #d4, DEBD(d4)			))
 
+# define LFFFFDEB(l1,d2,d3,d4,d5) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFML				\
+			    DFMD				\
+			    DFMD				\
+			    DFMD				\
+			    DFMD				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #d2, DEBD(d2),			\
+			    #d3, DEBD(d3),			\
+			    #d4, DEBD(d4),			\
+			    #d5, DEBD(d5)			))
+
 # define LLFDEB(l1,l2,d3) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFML				\
 			    DFML				\
@@ -469,6 +636,15 @@ extern int appDebugSetFile(	const char *	filename,
 			    #l2, DEBL(l2),			\
 			    #s3, DEBS(s3)			))
 
+# define LXSDEB(l1,l2,s3) APP_DEB(appDebug( "%s(%3d)"		\
+			    DFML				\
+			    DFMX				\
+			    DFMS				\
+			    "\n", __FILE__, __LINE__,		\
+			    #l1, DEBL(l1),			\
+			    #l2, DEBX(l2),			\
+			    #s3, DEBS(s3)			))
+
 # define SSXDEB(s1,s2,l3) APP_DEB(appDebug( "%s(%3d)"		\
 			    DFMS				\
 			    DFMS				\
@@ -540,6 +716,17 @@ extern int appDebugSetFile(	const char *	filename,
 			    #l3, DEBL(l3),			\
 			    #l4, DEBL(l4)			))
 
+# define SSLFDEB(s1,s2,l3,d4) APP_DEB(appDebug( "%s(%3d)"	\
+			    DFMS				\
+			    DFMS				\
+			    DFML				\
+			    DFMD				\
+			    "\n", __FILE__, __LINE__,		\
+			    #s1, DEBS(s1),			\
+			    #s2, DEBS(s2),			\
+			    #l3, DEBL(l3),			\
+			    #d4, DEBD(d4)			))
+
 # define LLSLDEB(l1,l2,s3,l4) APP_DEB(appDebug( "%s(%3d)"	\
 			    DFML				\
 			    DFML				\
